*-- Author :    Hartwig Albrecht   05/10/95
*-- changed:    Thomas Lohse  March 14, 1999
*               special hight-pT event weighting or pT-cuts implemented

      SUBROUTINE ALFRIT (LRPART)

************************************************************************
*
* Description: generate one FRITIOF event
*
* Called from: ALEVNT
*
* External references: LUND, PYTHIA
*
* Arguments in/out description
* LRPART    in     required particle
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUJETS.fi"
#include "jets/LUDAT1.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      real evwei
      common / weight / evwei

      real ptmax, pt02
      real RLU

      INTEGER NSRC, I, j, mstj1, LRPART

#include "arte/Arte.ff"

C loop over generated events until requested particle found

      CALL ALSAVE
      MSEL = 1

      if (modemb.gt.1) pt02 = pt0mb**2

      DO 20 NSRC = 1, 1000

        IF (NSRC .EQ. 20)  CALL MEMESC (71345, 10, ILARGEST,
     +    'ALFRIT: more than 20 searches for the requested particle')

        IF (TARGETMAT .EQ. 'N ')  CALL MEERRC (132654,
     +    'ALFRIT: Fritiof with neutron target not supported')

C FRITIOF !

        if (modemb.eq.1) then
C
C normal inelastic interaction
C
          CALL FREVENT ('FIXT', 'P', TARGETMAT, HERAEBEAM)
        elseif (modemb.eq.2) then
C
C normal inelastic events, importance sampling for parton pT>pt0mb
C
          mstj1 = MSTJ(1)
 2        continue
C switch off fragmentation
          MSTJ(1) = 0
          evwei = 1.
          CALL FREVENT ('FIXT', 'P', TARGETMAT, HERAEBEAM)
C get highest pT of final state parton systems
          ptmax = 0.
          do I = 1, N
            if (K(I,1).ne.0.and.K(I,1).lt.21) then
C exclude the nuclear remnant - the decay of which is not simulated
              if (K(I,2).ge.-10000) then
                ptmax = max(P(I,1)**2+P(I,2)**2, ptmax)
                if (ptmax.gt.pt02) goto 3
              endif
            endif
          enddo
          ptmax = sqrt(ptmax)
C perform importance sampling for modemb=2
          evwei = max(exp(6.*(ptmax-pt0mb)),(1.-ptmax))
          if (RLU(1).gt.evwei)                 goto 2
          evwei = 1./evwei
 3        continue
C switch fragmentation back on
          MSTJ(1) = mstj1
C fragment the parton systems
          call LUEXEC
C
        elseif (modemb.eq.3) then
C
C biased high pT event with at least one particle with pT>pt0mb
C
 5        continue
          CALL FREVENT('FIXT', 'P', TARGETMAT, HERAEBEAM)
C get highest pT of final state stable particle
          ptmax = 0.
          do I = 1, N
            if (K(I,1).ne.0.and.K(I,1).lt.21.and.
     +          K(I,1).ne.3.and.K(I,1).ne.13.and.K(I,1).ne.14
     +          .and.K(I,4).eq.0.and.K(I,3).ne.0) then
C exclude the nuclear remnant - the decay of which is not simulated
              if (K(I,2).ge.-10000) then
                ptmax = max(P(I,1)**2+P(I,2)**2, ptmax)
                if (ptmax.gt.pt02) goto 6
              endif
            endif
          enddo
C apply the pT cut here
          goto 5
 6        continue
c
c -- end of inelastic generator options 
c
        endif

C search for the requested particle

        IF (LRPART .EQ. 0)  GO TO 15
        DO 10 I = 1, N
          IF (K(I,1).ne.0.and.K(I,1).lt.21.and.
     +        K(I,2) .EQ. LRPART)              GO TO 15
 10     CONTINUE

 15     continue

C search for at least one particle from the selected particle group

        if (lunsel.lt.1)   goto 30
        do  I = 1, N
          do  j = 1, lunsel
            if (K(I,1).ne.0.and.K(I,1).lt.21.and.
     +          K(I,2) .eq. lusele(j))         GO TO 30
          enddo
        enddo

 20   CONTINUE

      CALL MEMESC (325476, 10, 100,
     +  'ALFRIT: more than 1000 searches for the requested particle')
      GO TO 80

C fill Arte tables

 30   CALL ALMCIN
      CALL ALFSTO
 80   CALL ALREST

      END

************************************************************************
* End of ALFRIT
************************************************************************
