*-- Author : T. Lohse   

      SUBROUTINE ALONIA

************************************************************************
*
* Description: Selection of PYTHIA fragments from Onium decays
*              as preparation of generation of underlying event (FRITIOF)
* 
*-- Authors :  T. Lohse    
*
*   
* Called from: ALPYTH
*
* External references: LUND, PYTHIA
*
* Arguments: none
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUDAT1.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      real evwei
      common / weight / evwei

      integer iniups, inipsi
      save iniups, inipsi
      data iniups /0/
      data inipsi /0/
      integer i, j, j1, j2, im, monium, mtyp 
      integer mess_id_1, mess_id_2
      save mess_id_1, mess_id_2

      real xF, pT
      real*8 beta, gamma, sqrts, sum1, sum2, x, term
      real*8 facups, facpsi
      real*8 a1,a2,a3,a4,a5,a6,b1,b2,b3,pthr
      real*8 p1,p2,p3,p4,q1
      real*8 e789pt, e789xf
      save a1,a2,a3,a4,a5,a6,b1,b2,b3,pthr
      save p1,p2,p3,p4,q1
      save e789pt,e789xf
      save facups, facpsi

#include "arte/Arte.ff"

c     Identify the first line number in buffer LUJETS 
c     which contains a heavy onium state

      monium = 0

      do i=1, N
         if( K(i,1).ne.0 .and. (K(i,1).le.20.or.K(i,1).ge.31) ) then
           j1 = mod(abs(K(i,2)),1000)/100
           j2 = mod(abs(K(i,2)),10000)/1000
           if (j1.eq.4.or.j1.eq.5.or.j2.eq.4.or.j2.eq.5) then
             monium = i
             if (j1.eq.4.or.j2.eq.4) then
               mtyp = 4
             else
               mtyp = 5
             endif
             goto 10
           endif
         endif
      enddo
 10   continue

      if(monium.eq.0) then
         call MEERRC(mess_id_1,'ALONIA: No ONIUM state found')
         goto 900
      endif

c     Compute the event weight if correction to measured
c     inclusive spectra is requested
c
      beta  = HERAEBEAM/(HERAEBEAM+0.9383d0)
      gamma = 1.d0/sqrt(1.-beta**2)
      sqrts = sqrt(.9383d0*(2.d0*HERAEBEAM+.9383d0))
      xF = 2.*gamma/sqrts*(P(monium,3)-beta*P(monium,4))
      pT = SQRT(P(monium,1)**2+P(monium,2)**2)
c
      if (mtyp.eq.4.and.modePsi.eq.2) then
c
c  weighting function in xF and pT for charmonium
        if (inipsi.eq.0) then
          inipsi = 1
          sum1 =  0.
          sum2 =  0.
          pthr =  1.375
          a1   =  1.17905
          a2   =  0.177022
          a3   =  4.44771
          a4   = -5.54442
          a5   =  2.00871
          a6   =  0.928383
          b1   =  1.1276
          b2   = -3.3821
          b3   = 12.8233
          e789pt = 3.00
          e789xf = 4.91
          do  j = 1, 5000
            x = (j-0.5)/1000.
            term = (1.+(x/e789pt)**2)**(-6)
            sum1 = sum1 + term
            if ( x.le.pthr ) then
              sum2 = sum2 + term/(a1*exp(a2*x)*(a3+a4*x+a5*x**2))
            else
              sum2 = sum2 + term/a6
            endif
          enddo
          facpsi = sum2/sum1
          sum1 = 0.
          sum2 = 0.
          do  j = 1, 8000
            x = (j-0.5)/10000.
            term = (1.-x)**e789xf
            sum1 = sum1 + term
            sum2 = sum2 + term/(b1+b2*x+b3*x**2)
          enddo
          facpsi = facpsi*sum2/sum1 
        endif
        if (pT.le.pthr) then
          evwei = a1*exp(a2*pT)*(a3+a4*pT+a5*pT**2)
        else
          evwei = a6
        endif
        evwei = evwei * (b1+b2*abs(xF)+b3*xF**2)
        evwei = evwei * facpsi
      else if (mtyp.eq.5.and.modeY.eq.2) then
c
c  weighting function in xF and pT for bottomonium
        if (iniups.eq.0) then
          iniups = 1
          sum1 = 0.
          sum2 = 0.
          q1 = 3.175
          p1 = 1.571
          p2 = 1.702
          p3 = 4.666
          p4 = 1.175
          do  j = 1, 5000
            x = (j-0.5)/1000.
            sum1 = sum1 + x/(1.+(x/q1)**2)**6
            sum2 = sum2 + x**p1/(1.+(x/p2)**p3)**p4
          enddo
          facups = sum2/sum1
        endif
        evwei = pT/(1.+(pT/q1)**2)**6
        evwei = evwei / pT**p1*(1.+(pT/p2)**p4)**p4
        evwei = evwei * facups
      endif

c discard all particles but the ONIUM and its decay products
      do  j = 1, N
        if (K(j,1).ne.0.and.K(j,1).lt.21) then
          if (j.ne.monium) then
            im = j
 2          continue
            im = K(im,3)
            if (im.eq.monium) goto 3
            if (im.ne.0)                        goto 2
            K(j,1)=0
 3          continue
          endif
        endif
      enddo

c     remove lines with status flag =0. Update event history information.
      call luedit(12)      

c     remove comment lines. Update event history information.
      call luedit(15)      

 900  END

************************************************************************
* End of ALONIA
************************************************************************


