//                              -*- Mode: C++ -*-
//:ECHitCellArray.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 7/4/97
// Last Update ........

//#include <iostream.h>


#ifdef HIST
#  include "ntuple.hh"
#endif
#include "care/care.hh"
#include "care/caredata.hh" 
#include "arte/ArteMessage.hh"
#include "care/ECHitCellArray.hh"
#include "care/ECCellID.hh"
#include "care/ECMCcalib.hh"
#include "care/CmdCareDglev.hh"
#include "care/dgcal.hh"
#include "arte/ArteProto.hh"
#include "care/ECDcalIO.hh"
#include "care/EFCompareHitCells.hh"
#include "care/CmdCareCalerr.hh"
#include "care/CmdCareLevel.hh"
#include "care/ECHF_status.hh"
#include "care/MMBank.hh"

#ifdef _OLD_NAMES_
#  include <algo.h>   
#else
#  include <algorithm>
#endif

#ifdef PERFORMANCES
#  include "arte/Chrono.hh"
#  include "arte/ChronoManager.hh"
   Chrono*  conversion=0;
   Chrono*  boardtime=0;
   Chrono*  remHitC=0;
#endif

#undef WRITE
//#define WRITE

/*
CLASS
  ECHitCellArray
  Store an array of pointers to the Signal cells (HITC bank).
  It provides the basic functions for the first stage of pattern recognition:
  Filling and Sorting.


*/

/////////////////////// ECHitCellArray static data definition


//--
//  Minimum energy to consider: 150 MeV
Energy_t  ECHitCellArray::minEnergy[6]   = {0.0, 0.050, 0.100, 0.0, 0.0, 0.150};


///////////////////   ECHitCellArray CLASS definition   
 
//--
//  Constructor
ECHitCellArray::ECHitCellArray() : 
  arr_pHITC(), entries(0), virtual_end(arr_pHITC.end())  {}

 
//--
//  Destructor
ECHitCellArray::~ECHitCellArray(){}

//--
//  Fill the array
void ECHitCellArray::fill(EcalEvent_t * event){

  int lev = CmdCareDglev::dglevel();  // get the ECAL digi level 

  switch(lev){
  case 0:
    // Standar digitalization for arte-01-05  MCAL -> HITC (Q=energy)
    fill01(lev);
    break;
  case 1:
    // Digitalization for arte >01-05  MCAL -> HITC (Q= ADC channel)
    fill01(lev);
    break;
  case 2:
    // Digitalization till the Second Level Buffer  MCAL -> DCAL
    //
    fill2(event);
    break;
  default:
    // Error: at least one digitization level must be choosen
    arteMsg<<" DGCAL: wrong digitization level !!";
    arteMsg.error();
    fill01(0);
    break;
  }
  //dump();
}

//--
//  Fill the array
void ECHitCellArray::fill01(int lev){
# ifndef ONLINE
  ECMCcalib calibration;
  CmdCareCalerr prec;
  ArteTableIterator<HITC> hitcIt;         // create a table iterator on HITC
  /// cout<<" In fill01 with "<<hitcIt.nrow()<<" cells "<<endl;
  // test if the bank is empty
  if(ArtePointer<HITC>::getLink()()==0) return;
  arr_pHITC.reserve(hitcIt.nrow());   // reserve space for all HITC
  start();
  ECCellID cellID;
  Energy_t  calib;
  int ncl;
  CellIdx_t cell;
  // cout<<" Now looping on HITC: "<<hitcIt.nrow()<<endl;
  for( ; hitcIt.valid(); hitcIt.next()){
    cell = hitcIt->gcce;
    cellID = ECCellID(cell->ceid);
    if(  hitcIt->gcce==0){
	cout<<" Warning: gcce=0 for cell "<<hitcIt->gcce<<endl;
    } else {	
    if( lev!=1) {
      // old digitization routine
      ncl = cellID.modulePointer()->ncl;
      calib = calibration[ncl]*prec.factor();
      hitcIt->q *= calib; // do calibrate in energy
      hitcIt->r *= calib; // calibrate error on energy
      if( hitcIt->q> minEnergy[ncl]){ // skip cells with energy 
	                              // less than threshold
	push_back(hitcIt.index());
      }
    } else {
      float sqerr = 0.0;
      // new digitization routine
      int adc = (int)  hitcIt->q; // convert to int
      hitcIt->q = dgcalche(cell.index(), adc, hitcIt->adc, sqerr);// * prec.factor();
      hitcIt->r = sqerr;
      // cout<<" Energy: "<<hitcIt->q<<" Threshold : "<<echs->thr1<<endl;
      ncl = cellID.modulePointer()->ncl;
	
      if( hitcIt->q >minEnergy[ncl] ){   // skip cells with energy 
	                              // less than threshold
	// cout<<" FIlling "<<hitcIt.index()<<"  "<<nce++<<endl;
	push_back(hitcIt.index());
      }
    }
    }
  }
  // cout<<" End of loop "<<endl;
# endif
}



//--
//  dglev=2 Fill the array and create the HITC bank 
void ECHitCellArray::fill2(EcalEvent_t * event){
  int ix= 0, iy= 0, ndim=0;
  ECCellID  cellid(0);
  //  CmdCareCalerr prec;
  Energy_t ener = 0.0;
  Energy_t enerr = 0.0;
  // Coord_t dx,dy,rdim;
  unsigned short channel;
  unsigned short sfed_id, idmk, bx, firstbx = 0x8000;
  unsigned short dummy1 = 0;
  unsigned short dummy2 =0;
  // initialize
  static Coord_t rdim[6], rdx[6][6];
  static int init=0;
  static vector<int> threshs;
  if(init==0){
    Coord_t tmpdx;
    for(int dim=1; dim<6; dim++){
      rdim[dim] = 1./float(dim);
      tmpdx = ECalInfo.xMsize*rdim[dim];
      for(int jx=1; jx<=dim; jx++){
	rdx[dim][jx] = (jx-0.5)*tmpdx;
      }
    }
    init = 1;
    threshs.push_back(10000);
    for(ArteTableIterator<ECHS> echs; echs.valid(); echs.next()){
      threshs.push_back(echs->thr1);
    }
  }
# ifdef PERFORMANCES
  if( conversion==0){
      ChronoManager CM;
      conversion = CM.createChrono("test filling");
      boardtime = CM.createChrono("Board time");
      remHitC =  CM.createChrono("Conversion");
  }
  conversion->start();
# endif
  // start the array
  if(arr_pHITC.begin()==arr_pHITC.end()){
    arr_pHITC.reserve(ArtePointer<ECBO>::nrow()*32); // reserve space for all 
    int siz = ArtePointer<ECBO>::nrow()*32;
    for(int ii=0; ii<siz; ii++)
      arr_pHITC.push_back(0);
  }

  start();

  if( ArtePointer<DCAL>::getLink()()==0 || ArtePointer<DCAL>::nrow()==0 ){
    static int numwarn = 0;
    numwarn++;
    if( numwarn<100){
      arteMsg<<"ECAL: No DCAL bank to analyze!";
      arteMsg.warn();
    }
  } else {

    // book the maximum number of entries in the HITC bank
# ifndef ONLINE
    MMBank<HITC>::Book(ArtePointer<ECBO>::nrow()*32);
# else
    HITC ht = {0.0,0.0,0.0,0.0,0};
    //  int size = ArtePointer<ECBO>::nrow()*32;
    int size = CmdCareLevel::ncells();
    for(int ixi=0; ixi<size; ixi++)
      ArtePointer<HITC>::append(ht);
# endif
    ArteTableIterator<HITC> hitc;

# ifdef PERFORMANCES
    conversion->stop();
    boardtime->start();
#endif

    char occur[256];
    for(int j=0; j<256; j++)
       occur[j] = 0;

    // loop on info in DCAL
    CellIdx_t cellidx;
    int thresh;
    unsigned short siz;
    ECDcalIO dcal;
    dcal.start(event); 
    // read the bank header
    dcal.getBankHeader(dummy1, dummy2);	
# ifdef WRITE
    cout<<" Bank header: "<<dummy1<<" "<<dummy2<<endl;
# endif
    
    unshort fed_id = dcal.nextFED(); // read the Front End Detector IDentifier
    while( fed_id>0 ) {

#   ifdef WRITE
      cout<<"dcal.nextFED(): "<<fed_id<<endl;
#   endif

      // read the header      
      unsigned int nch = dcal.getFedHeader(siz, sfed_id, idmk, bx);
      fed_id = sfed_id & 255;
#   ifdef WRITE
      cout<<"Fed id:"<< fed_id<<" size:"<<siz<<" ";
#   endif 
      siz = (siz/2)-4;
      siz = (siz >32 ? 32 : siz);
      // find the related ECBO entry given the fed_id
      if( (fed_id<16 || fed_id>ArtePointer<ECBO>::nrow()+15) ||
          ( (sfed_id & 0xFF00)!=0 && (sfed_id & 0xFF00)!=0xEC00)  ||
	  (idmk & 0xC000)!=0 ){
	static int ercount = 0;
	if( ercount<100){
  	  arteMsg<<"ECAL: Wrong fed id:"<<fed_id;
	  arteMsg.warn();
	}
	ercount++;
 	fed_id = 0;
      }
      if( firstbx == 0x8000) firstbx = bx;      ArtePointer<ECHF> echf;
      ArteTableIterator<ECBO> ecbo(fed_id-15);
      if( fed_id!=0 && ecbo->stat>=0 ){
        occur[fed_id]++;
        if( occur[fed_id]>1){
  	  static int erocc = 0;
	  if( erocc<100){
  	    arteMsg<<"ECAL: Multiple fed id record :"<<fed_id;
	    arteMsg.warn();
	  }
	  erocc++;
        }
        if( bx!=firstbx){
  	  static int erbx = 0;
	  if( erbx<100){
  	    arteMsg<<"ECAL: Wrong BX in fed id :"<<fed_id;
	    arteMsg.warn();
	  }
	  erbx++;

        }
	// correct and active ecbo table entry
	for(unsigned int jj=0; jj<nch; jj++){
	  int ofs = 0;
	  // readout channel 
	  dcal.getNextShort(ofs,channel);
	  if( ecbo->chan[ofs]!=-1){ // test if active channel
	    cellidx = ecbo->chan[ofs];
	    echf = cellidx.index();
	    //	  int thresh = ArtePointer<ECHC>(cellidx.index())->thr1;
	    thresh = threshs[cellidx.index()];
	    if( channel > thresh && channel < 4096 && 
                (echf->stat & Echfs::usable) ){
	      // conversion   ADC channel -> energy (GeV)
	      ener = dgcalche(cellidx.index(), channel, hitc->adc, enerr);
	      cellid(cellidx->ceid);
	      cellid.getModuleIndex(ix, iy, ndim);
	      if( ener > minEnergy[ndim]){ // skip cells with energy 
		// less than threshold
		hitc->x = cellid.modulePointer()->xmin+rdx[ndim][ix];
		hitc->y = cellid.modulePointer()->ymin+rdx[ndim][iy];
		hitc->q = ener;                 // energy
		hitc->r = enerr;                // error on energy
		hitc->gcce = cellidx.index();   // cell ID (GCCE numbering);
#             ifdef MCARLO
	        build_HITC_MCAL(hitc);        // build links
#             endif
		push_back(hitc.index());
		hitc.next();
	      }
	    }
	  }
#       ifdef    WRITE
	  //  	  cout<<endl;
#       endif
	}  
      } else {
	if( !ecbo.valid()){
	  static int invfed = 0x0ECA0004;
	  arteMsg<<" Invalid FED ID in DCAL table ";
	  arteMsg.warn(invfed);
	}
      }
      fed_id = dcal.nextFED(); // go and take the next FED
    }
# ifdef    WRITE
    cout<<endl;
# endif
# ifdef PERFORMANCES
    boardtime->stop();
    remHitC->start();
# endif
#ifndef ONLINE
    // remove the unused entries
    if( CmdCareLevel::level()>2){
      MMBank<HITC>::Book(hitc.index()-ArtePointer<ECBO>::nrow()*32-1);
    }
#endif
# ifdef PERFORMANCES
  remHitC->stop();
# endif
  }
}


//--
//  Sort the array
void ECHitCellArray::sortCells(){
  // Sort desceding in energy Hit Cell pointer array with quicksort
  if( size()>0){
    stable_sort(arr_pHITC.begin(), end(), EFCompareHitCells());
  }
}


//-- 
//   For debugging a dump operation is provided. It print on cout
//   all information stored.
void  ECHitCellArray::dump() const{

  int n = size();
 
  cout<<"Dump array ECHitCellArray of "<<size()<<" elements:"<<endl;
  // loop on cells
  cout<<"Index \tCell\tModule \tEnergy    \tPosition"<<endl;
  for(int i=0; i<n; i++){
    ArtePointer<HITC> hitc(arr_pHITC[i]);
    cout<<i+1<<"\t"<<ECCellID(CellIdx_t(hitc->gcce)->ceid)()<<"\t"
	<<hitc->gcce<<"\t"<<hitc->q
	<<"\t"<<hitc->x<<"\t"<<hitc->y<<endl;
  }
  cout<<"-----------------------"<<endl<<endl;
}


////////// Class ECHitCellArray static function definitions


//--
//   Set the minimum energy a cell
//   must have in order to consider for clusterization
void ECHitCellArray::setMinimumEnergy(Energy_t innerCal, 
					 Energy_t middleCal,
					 Energy_t outerCal){
  // set the minimum energy passed
  minEnergy[0] = 0.0;
  minEnergy[1] = outerCal;  // Outer CAL has 1 cell per module
  minEnergy[2] = middleCal; // Middle CAL has 2x2 cells per module
  minEnergy[3] = 0.0;
  minEnergy[4] = 0.0;
  minEnergy[5] = innerCal;  // Inner CAL has 5x5 cells per module
}
			    
//--
//   Printout Information for the current MinimumEnergy data stored
void ECHitCellArray::dumpCutOff(){
 
  arteMsg<<" Cut Off energy for inner, middle and outer ECAL (GeV):";
  arteMsg.allp();
  arteMsg<<minEnergy[5]<<"  "<<minEnergy[2]<<"  "<<minEnergy[1];
  arteMsg.allp();
}


//--
//   Build the links between the current HITC and the MCAL bank
void ECHitCellArray::build_HITC_MCAL(ArtePointer<HITC> hitc){
# ifdef MCARLO
  int ix=0, iy=0, ndim;
  unsigned int ipl;
  CellIdx_t cell = hitc->gcce;
  ECCellID  cellid(cell->ceid);
  cellid.getModuleIndex(ix, iy, ndim);
  ipl = cellid.module();

  ArtePointer<GCAL> gcal(ipl);
  int nmim = 0;
  if((gcal->mim1).index() != 0) 
    nmim = (gcal->miml).index() - (gcal->mim1).index() + 1; //num MCAL in IPL
  if(nmim > 0){
    int ndim = gcal->ncl;
    for(int l = 0; l < nmim; l++){
      unsigned int il = (gcal->mim1).index() + l;
      ArtePointer<MCAL> mcal(il);
      int icmp = ((mcal->gcal)>>24);
      if( icmp==0 || ArtePointer<GESL>(icmp)->cmp==7)
	if( (((mcal->gcal) & 0x00FFFFFF) -ipl*100) == 
	    (unsigned int) (ndim*ix+iy)){
	  createRelation( mcal, hitc, &HITC::mcal);
	  //	  MMBank<HITC_MCAL>::Brel( il, ihitc);
      }
    }
  }
# endif
}

