//                              -*- Mode: C++ -*-
//:ELNearHitCells.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
// Last Update ........
//
//

#include "care/ELNearHitCells.hh"
#include "care/EMClusterTree.hh"
#include "care/EMHitCell.hh"
#include "care/ELLUTsManager.hh"
#include "arte/arte.hh"
#include "care/ELNearNeiCellLUT.hh"
#undef WRITE

/*
CLASS 
  ELNearHitCells
  Get the list of nearby clusterized cells
  and provide access to the list


class ELNearHitCells {
public:
  //--
  //  constructor
  ELNearHitCells();
  //--
  //  destructor
  ~ELNearHitCells();
  //--
  //  access member
  bool more() const;
  //--
  //  start an iteration 
  void begin();
  //--
  //  consider next cell
  void next();
  //--
  //  get the current cell
  EMHitCell* operator*() const;

private:
  //--
  //  Number of store neibours
  unsigned int numNeib;
  //--
  //  Array containing the cells
  EMHitCell*  neibCells[20];
  //--
  //  pointer to current cell
  int curr;
};
*/

///////////////// ELNearHitCells.C member function definitions ////////
//--
//  constructor
ELNearHitCells::ELNearHitCells() : 
   numNeib(0), curr(0) {}
//--
//  destructor
ELNearHitCells::~ELNearHitCells(){}


//--
//--
//  Get the neibours of a given cell
void ELNearHitCells::of(const EMHitCell* hcell, const EMClusterTree & tree){
  //--
  // find the index of the cell pointed by the current cell
  of(hcell->cellIdx(), tree);
}

//--
//  Get neibouring cells of a given cell
void ELNearHitCells::of(CellIdx_t cell, const EMClusterTree & tree){

  //
  ELLUTsManager LM;
  // get the number of nearest neibouring cells of the current one
  int ncell = LM.neiboursTable().howManyNeibouring(cell);  
  // for each neibouring cell find if it is linked with a cluster
  numNeib = 0;
  CellIdx_t nearCell; 
  EMHitCell * nearCluster;

# ifdef WRITE
  cout<<" Looking for neiboouring of cell "<< cell.index()<<endl;
# endif
  for(int i=0; i<ncell; i++){
    nearCell = LM.neiboursTable()[i];// get the index of the neibouring cell i
    
# ifdef WRITE
    cout<<" "<< nearCell.index();
# endif
    nearCluster = tree.whichHitCell(nearCell);
    if(nearCluster!=0){
       
# ifdef WRITE
     cout<<"Stored   ";
# endif
      // found a nearby cell already clusterized. 
      neibCells[numNeib] = nearCluster;
      numNeib++;
    }
  }
# ifdef WRITE
     cout<<endl;
# endif

  curr = 0;
}

