//                              -*- Mode: C++ -*-
//: EMHitCell.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//

#include "care/EMHitCell.hh"
#include "care/EMCluster.hh"
#include "care/EAClusterAnalyzer.hh"
#include"care/EDClusterModifier.hh"
#include "arte/ArteMessage.hh"
#include "care/ECCellID.hh"

//////////////  Class EMHitCell definitions //////////////////////////////

//-- Constructors
//   The construction of EMHitCell is done referring to some
//   parent. The depth level is adjusted accordingly.
EMHitCell::EMHitCell( EMCluster* parent, 
		    ArtePointer<HITC> currCell):
  EMClusterMember(parent),  cell(currCell)
{}

//-- Default Constructors
EMHitCell::EMHitCell():  EMClusterMember(),  cell() {}



//-- Destructor
//   A virtual destructor is needed: but not for this derived class: 
//   it is a dummy one
EMHitCell::~EMHitCell(){}


// Status access functions
//--
// Return a flag to tell that this EMClusterMember is NOT a leaf 
bool EMHitCell::isALeaf() const{
  return true;
}


//--
// Return the Energy of the Cell itself. 
Energy_t  EMHitCell::maxEnergy() const{
  return cell->q;
}


//--
// Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
// that are object with definite analyzing properties. 
// A direct call to the DoAnalisys() member of EAClusterAnalyzer for this
// "leaf" is done.
void EMHitCell::analyzeMe(EAClusterAnalyzer & analyzer) const {
  analyzer.doAnalysis(this);
}

void EMHitCell::modifyMe(EDClusterModifier & modifier){
  modifier.doModify(this);
}

//--
/// Access the energy of a hit cell
Energy_t  EMHitCell::energy() const {
  return cell->q;
}

//--
/// Access the squared energy error of a hit cell
Energy_t  EMHitCell::sqerror() const {
  return cell->r;
}

//--
//  Dump for debugging
void EMHitCell::dump() const {
  for(int i=0; i<depthLevel(); i++)
    arteMsg<<"|";
  arteMsg<<"->Cell: IDX="<<cellIdx()<<"  Energy = "<<cell->q
	 <<"  Pos:("<<x()<<", "<<y()<<" ) ";
  
  if(parentCluster==0) {
    arteMsg<<" VOID parent.";
    arteMsg.allp();
  } else {
    arteMsg.allp();
  }
}


//-- 
//  set internal pointer to hit cell
void EMHitCell::pointToCell(ArtePointer<HITC> currCell){
  cell = currCell;
  depth = TOPLEVEL;
  parentCluster = 0;
  nextClusterMember = 0;
  previousClusterMember = 0;
}



//--
/// Access the type of a hit cell
int  EMHitCell::ncl() const{
  return ECCellID(CellIdx_t(cell->gcce)->ceid).modulePointer()->ncl;
}


//--
/// Access the index of a hit cell
CellIdx_t  EMHitCell::cellIdx() const{
  return CellIdx_t(cell->gcce);
}



//--
/// Access the z coordinate of the front of a hit cell
Coord_t    EMHitCell::z() const{
  return ECCellID(CellIdx_t(cell->gcce)->ceid).modulePointer()->zmin;
}




