//                              -*- Mode: C++ -*-
//: CmdCareCalerr.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef CMDCARECALERR_HH_
#define CMDCARECALERR_HH_

#include "care/CmdBase.hh" 


/**
class  CmdCareCalerr.
  Class to deal with the care/prec kuip command
  Set the precision of ECAL calibration.
*/

class CmdCareCalerr : public CmdBase {
public:
  /// Constructor
  CmdCareCalerr();
  //-- Destructor
  /// Destructor: currently void
  virtual ~CmdCareCalerr();
  
  //-- Behavioural operations
  ///   Reset stored data to default values
  virtual void defValues();
  //--
  ///   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  ///   Read a command from external
  virtual void read();
# endif
  //--
  //    Dump internal data
  virtual void dump();
  //--
  ///   Return the stored precision on calibration
  inline float precfactor();
  //--
  ///   Return a random factor with mean 1 and
  ///   sigma given by the precision on calibration
  float factor();

private:
  ///////////////////// Data Part /////////////////////////////////
  //--
  static  float precision;
};


// INLINES for  CmdCareCalerr

///   Return the stored precision on calibration
float CmdCareCalerr::precfactor() {
  return precision;
}

#endif  //   CMDCARECALERR_HH_

