//                              -*- Mode: C++ -*-
//: CmdCareECDB.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/4/98
// Last Update ........
//

#ifndef CMDCAREECDBMIZ_HH_
#define CMDCAREECDBMIZ_HH_

#include "care/CmdBase.hh" 


/*
CLASS
  CmdCareECDBMIZ
  Class to deal with the care/ECDBMIZ kuip command
  Define the name of the MIZZI file containing the calibration DB

*/

class CmdCareECDBMIZ : public CmdBase {
public:
  enum Tbset {TALL, TSLOW, TFAST};

  //-- Constructors
  CmdCareECDBMIZ();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareECDBMIZ();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--g
  //   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  //   Read acommand from external
  virtual void read();
# endif
  //--
  //   Dump internal data
  virtual void dump();
  //--
  //   Static access member function
  char* calibfile();

  //  Set DB properties
  void setDB(char *nam, int imajor, int iminor);

  //-- 
  //  Get functions
  inline int Major() const;
  inline int Minor() const;

  //--
  // Read the database
  void readDB(Tbset whichtable, int access);
  //--
  // Write the database
  void writeDB(Tbset whichtable, int access);

private:

  ///////////////////// Data Part /////////////////////////////////
  //--
  static  char filename[256];
  enum  {none, dbread, dbwrite} action;
  Tbset tbset;
  int match;
  int major;
  int minor;

};

int CmdCareECDBMIZ::Major() const {
    return major;
}

int CmdCareECDBMIZ::Minor() const {
    return minor;
}


#endif  //   CMDCAREECDBMIZ_HH_

