#include <vector>
#include <arte/arte.hh>

#include "rhp.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>


#define MAXNUMBEROFHITS     10000
#define MAXNUMBEROFTRACKS   1000

extern "C" {
  extern void isplci_(int* n);
  extern void ipl_(int* n, float* xpl, float* ypl);
  extern void iuwk_(int* m, int* n);
}
extern int DRAW;

extern int   base_plane[5];
extern int   shift_coeff[5];
extern int   Ntk, nh_trk[MAXNUMBEROFTRACKS+1];
extern float Yp_trk[MAXNUMBEROFTRACKS+1], Y0_trk[MAXNUMBEROFTRACKS+1];
extern float Zp_trk[MAXNUMBEROFTRACKS+1], Z0_trk[MAXNUMBEROFTRACKS+1];
extern int   hits[MAXNUMBEROFHITS+MAXNUMBEROFTRACKS+10];
extern int   Ntr_mc_proper, Ntr_mc_reco, N_Events;
extern int   Ntr_mc_b_proper, Ntr_mc_b_reco, N_b_Events, N_g_Events;
extern float svd_efficiency, svd_b_efficiency, svd_ghost_rate;
extern float geometrical_acceptance;

extern "C" {

void analyze_MC()
{
  int   nh_mc, ipr, ih, itk, nh;
  int   mtra_ok, il_mc, nl_mc, np_mc;
  int   hits_total, hits_signal;
  int   nh_found, nh_rec;
  int   Ntr_mc_good, Ntr_mc_found, Ntr_reco_ghost;
  int   nl_mc_histo[5];
  float xpl[1000], ypl[1000], SHIFT=10.0, shift;
  int   nn;
  int   n_mc_at_least = 10;
  int   n_ghost_at_least = 8;
  int   Red = 6; 
  int   Blue = 3;
  float yy, zz, x;

  if (DRAW) {


  /*
   * RSEG <-> MTRA identification
   */

    isplci_(&Blue);

    for(ipr=1; ipr <= 4; ipr++){ 
      
      shift = shift_coeff[ipr]*SHIFT;



      for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
	   rseg != ArteTable<RSEG>::end(); ++rseg ) {

	if ((*rseg)->cmp != 1)
	  continue;

  	yy=(*rseg)->xf+(*rseg)->txf*(-4.7-(*rseg)->zf);
	zz=(*rseg)->yf+(*rseg)->tyf*(-4.7-(*rseg)->zf);

	ArtePointer<GEDE> gedebase = ArtePointer<GEDE> (base_plane[ipr]);

	x = yy*gedebase->cos - zz*gedebase->sin; 
	
	xpl[0] = -4.7;
	ypl[0] = x+shift+0.4;
	
  	yy=(*rseg)->xf+(*rseg)->txf*(200.0-(*rseg)->zf);
	zz=(*rseg)->yf+(*rseg)->tyf*(200.0-(*rseg)->zf);
	
	x = yy*gedebase->cos - zz*gedebase->sin; 
	
	xpl[1] = 200.0;
	ypl[1] = x+shift+0.4;

	nn = 2;
	ipl_(&nn,xpl,ypl);
	
      }
    }
    
}






  if (DRAW) 
    isplci_(&Red);


  hits_total  = 0;
  hits_signal = 0;

  for( ArteTable<HITB>::iterator hitb = ArteTable<HITB>::begin();
       hitb!=ArteTable<HITB>::end();
       ++hitb ) {
	ArtePointer<HITB> ihitb = *hitb;
      if (ihitb->gede->cmp==1) hits_total++;
  }

  Ntr_mc_good     = 0; /* no energy cuts */
  Ntr_mc_proper   = 0; /* energy > 1 GeV */
  Ntr_mc_b_proper = 0;

  Ntr_mc_found  = 0;   /* no energy cuts */
  Ntr_mc_reco   = 0;   /* energy > 1 GeV */
  Ntr_reco_ghost = 0;   /* has no corresponding mc track */

  for( ArteTable<MTRA>::iterator mtra = ArteTable<MTRA>::begin();
       mtra!=ArteTable<MTRA>::end();
       ++mtra ) {
    
    if ((*mtra)->topp->chrg == 0) continue;
    ArtePointer<MTRA> itm = *mtra;


    nh_mc = 0;
    np_mc = 0;
    for(ipr=1; ipr <= 4; ipr++){ 

      shift = shift_coeff[ipr]*SHIFT;
      ArtePointer<GEDE> gedebase = ArtePointer<GEDE> (base_plane[ipr]);

      il_mc = 0;
      nl_mc = 0;
      for( ArteTable<HITB>::iterator hitb = ArteTable<HITB>::begin();
	   hitb!=ArteTable<HITB>::end();
	   ++hitb ) {
	
	ArtePointer<HITB> ihitb = *hitb;

	ArtePointer<GEDE> igede = ihitb->gede;
	if (ihitb->gede->cmp ==1) {

	  if (
	      ((fabs(igede->sin - gedebase->sin)<0.00001)&&
	       (fabs(igede->cos - gedebase->cos)<0.00001))
	      || 
	      ((fabs(igede->sin + gedebase->sin)<0.00001)&&
	       (fabs(igede->cos + gedebase->cos)<0.00001)) 
	      ){
	    
	    mtra_ok = 0;

	    for( ArteRelationIterator<MIMP> mimpIt = hitb->mimp.begin();
		 mimpIt != hitb->mimp.end();
		 ++mimpIt ) {
	      if ((*mimpIt)->mtra == itm){

		if ( (*mimpIt)->x > (*mimpIt)->gede->sxmn &&
		     (*mimpIt)->x < (*mimpIt)->gede->sxmx && 
		     (*mimpIt)->y > (*mimpIt)->gede->symn &&
		     (*mimpIt)->y < (*mimpIt)->gede->symx ) {
		  //mimp is inside active area
		  mtra_ok = 1;
		}
	      }
	    }
	    if (mtra_ok == 1){
	      nh_mc++;

	      ArtePointer<GESL> igesl = (ihitb->gede)->gesl;
	      if (igesl->nsl!=il_mc){
		il_mc = igesl->nsl;

		if (
		    ((fabs(igede->sin - gedebase->sin)<0.00001)&&
		     (fabs(igede->cos - gedebase->cos)<0.00001))
		    ){
		  xpl[nl_mc] = ihitb->z;
		  ypl[nl_mc] = ihitb->x+shift+0.3;
		}	      
		else{
		  xpl[nl_mc] = ihitb->z;
		  ypl[nl_mc] = -ihitb->x+shift+0.3;
		}	      

		nl_mc++;
	      }
	    }
	  }
	}
      }
      
      nl_mc_histo[ipr]= nl_mc;
      if (nl_mc>=3)
	np_mc++;
      if ((nl_mc>=2)&&(DRAW)){
	ipl_(&nl_mc,xpl,ypl);
      }
    }

    if (nh_mc>0){ /* track is detected */
      //      hfill(144, (float)(nh_mc),          0.0, 1.0);
      //      hfill(145, (float)(nl_mc_histo[1]), 0.0, 1.0);
      //      hfill(145, (float)(nl_mc_histo[2]), 0.0, 1.0);
      //      hfill(145, (float)(nl_mc_histo[3]), 0.0, 1.0);
      //      hfill(145, (float)(nl_mc_histo[4]), 0.0, 1.0);
      //      hfill(146, (float)(np_mc),          0.0, 1.0);
    }

    if ((np_mc>=2)&&(nh_mc>=n_ghost_at_least)){//more that 3 planes in more than 2 projections
      
      hits_signal = hits_signal+nh_mc;
      Ntr_mc_good++;
      if ((nh_mc>=n_mc_at_least)&&(sqrt(
	       (itm->px)*(itm->px)+(itm->py)*(itm->py)+(itm->pz)*(itm->pz)
	       )>1.0))
	Ntr_mc_proper++;
      /*
      if (itm->topp == ArtePointer<TOPP> (162) ){   
	if ( (itm->btra)->topp == ArtePointer<TOPP> (248) ){ 
	  Ntr_mc_b_proper++;
	}
      }
      
      if ((itm->topp==ArtePointer<TOPP> (18) )|| 
	  (itm->topp==ArtePointer<TOPP> (19) )|| 
	  (itm->topp==ArtePointer<TOPP> (22) )|| 
	  (itm->topp==ArtePointer<TOPP> (23) )){ 
	if ((itm->btra)->topp==ArtePointer<TOPP> (295) ){ 
	  Ntr_mc_b_proper++;
	}
      }
      */
      
      
      // check if the track is reconstructed

      nh_rec = 0;
      nh = 0;
      for (itk=0;itk<Ntk;itk++){
	nh_found = 0;
	for (ih=nh;ih<nh+nh_trk[itk];ih++){
	  ArtePointer<HITB> hitb = ArtePointer<HITB> (hits[ih]);
	  mtra_ok = 0;
	  
	  for( ArteRelationIterator<MIMP> mimpIt = hitb->mimp.begin();
	       mimpIt != hitb->mimp.end();
	       ++mimpIt ) {
	    if ((*mimpIt)->mtra == itm){

	      if ( (*mimpIt)->x > (*mimpIt)->gede->sxmn &&
		   (*mimpIt)->x < (*mimpIt)->gede->sxmx && 
		   (*mimpIt)->y > (*mimpIt)->gede->symn &&
		   (*mimpIt)->y < (*mimpIt)->gede->symx ) {
		//mimp is inside active area
		mtra_ok = 1;
	      }
	    }
	  }
	  if (mtra_ok == 1){
	    nh_found++;
	  }
	}
	nh=nh+nh_trk[itk]+1; //skip (-1)
	if (nh_found>nh_rec){ 
	  nh_rec = nh_found;
	}
      }

      
      if ( ((float)nh_rec/(float)nh_mc)>=0.69 ){
	Ntr_mc_found++;


	if ((nh_mc>=n_mc_at_least)&&(sqrt(
		 (itm->px)*(itm->px)+(itm->py)*(itm->py)+(itm->pz)*(itm->pz)
		 )>1.0))
	  Ntr_mc_reco++;

	if(DRAW){
	  cout << "Track " << itm << " code "<<itm->topp<<
	    " energy "<< sqrt((itm->px)*(itm->px)+(itm->py)*(itm->py)+
			      (itm->pz)*(itm->pz))<<
	    " with "<< nh_mc  << " hits RECO with "<< nh_rec <<" hits " <<
	    ((float)nh_rec/(float)nh_mc) << endl;
	}
      }
      else{
	if(DRAW){
	  cout << "Track " << itm << " code "<<itm->topp<<
	    " energy "<< sqrt((itm->px)*(itm->px)+(itm->py)*(itm->py)+
			      (itm->pz)*(itm->pz))<<
	    " with "<< nh_mc  << " hits -NO- with "<< nh_rec <<" hits " <<
	    ((float)nh_rec/(float)nh_mc) << endl;
	}
      }

    }
  }
  

  
  // determine ghost reconstructed tracks

  for (itk=0;itk<Ntk;itk++){

    nh_rec = 0;
    nh = 0;
    np_mc = 0;

    for( ArteTable<MTRA>::iterator mtra = ArteTable<MTRA>::begin();
	 mtra!=ArteTable<MTRA>::end();
	 ++mtra ) {

      if ((*mtra)->topp->chrg == 0) continue;

      ArtePointer<MTRA> itm = *mtra;

      nl_mc = 0;

      for(ipr=1; ipr <= 4; ipr++){ 

	ArtePointer<GEDE> gedebase = ArtePointer<GEDE> (base_plane[ipr]);

	nh_found = 0;

	for (ih=nh;ih<nh+nh_trk[itk];ih++){
	  ArtePointer<HITB> hitb = ArtePointer<HITB> (hits[ih]);

	  ArtePointer<GEDE> igede = hitb->gede;

	  if (
	      ((fabs(igede->sin - gedebase->sin)<0.00001)&&
	       (fabs(igede->cos - gedebase->cos)<0.00001))
	      || 
	      ((fabs(igede->sin + gedebase->sin)<0.00001)&&
	       (fabs(igede->cos + gedebase->cos)<0.00001)) 
	      ){
	  
	    mtra_ok = 0;
	
	    for( ArteRelationIterator<MIMP> mimpIt = hitb->mimp.begin();
		 mimpIt != hitb->mimp.end();
		 ++mimpIt ) {
	      if ((*mimpIt)->mtra == itm){

		if ( (*mimpIt)->x > (*mimpIt)->gede->sxmn &&
		     (*mimpIt)->x < (*mimpIt)->gede->sxmx && 
		     (*mimpIt)->y > (*mimpIt)->gede->symn &&
		     (*mimpIt)->y < (*mimpIt)->gede->symx ) {
		  //mimp is inside active area
		  mtra_ok = 1;
		}
	      }
	    }
	    if (mtra_ok == 1){
	      nh_found++;
	    }
	  } // in a projection
	} // loop over hits

	if (nh_found>=2) nl_mc++; // projection is "reconstructed"
      } // loop over projections

      if (nl_mc>np_mc) np_mc = nl_mc; // projections "reconstructed"
    } // mtra

    nh=nh+nh_trk[itk]+1; //skip (-1)

    if ( np_mc <2 ) Ntr_reco_ghost++;
  } // loop over reco tracks

  

  if (Ntr_mc_proper>0){
    N_Events++;
    svd_efficiency = svd_efficiency+(float)Ntr_mc_reco/(float)Ntr_mc_proper;
    if (Ntk!=0)
      svd_ghost_rate = svd_ghost_rate+(float)(Ntr_reco_ghost)/(float)Ntk;
  }


  if(DRAW){
    cout << "MC-proper tracks " << Ntr_mc_proper <<  
      "  reco "<< Ntr_mc_reco << " ghost " << Ntr_reco_ghost << 
      " hits signal/total = " << hits_signal<<"/"<<hits_total << endl;
  }
  /*  
  hfill(140, (float)(hits_signal),  (float)(hits_total-hits_signal), 1.0);
  hfill(141, (float)(hits_total),  (float)hits_signal, 1.0);
  hfill(142, (float)(hits_total),  (float)(hits_total-hits_signal), 1.0);
  if (Ntr_mc_proper!=0){
    hfill(143, (float)(Ntr_mc_proper),  
	      (float)(hits_total-hits_signal)/(float)(Ntr_mc_proper), 1.0);
  }
  else{
    hfill(143, (float)(Ntr_mc_proper),  
	      (float)(hits_total-hits_signal), 1.0);
  }
  */

  if(DRAW){
    nn = 1;
    iuwk_(&nn,&nn);
  }

  return;
}
}



