  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Updates: 
   *   01.04.98 Some changes are made by S.Moukhine (JINR, Dubna)
   *            to adapt the package for Arte version 2.
   *   12.08.98 Modifications by A.Semenov (removing of DbList functions)
   *   31.08.98 Modifications by A.Semenov (args change, ierr code)
   *   12.10.98 iofile support, ro/rw flags added
   *   01.11.98 short types supported
   *
   ************************************************************************/


#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"

typedef struct {
	char Name[DB_MAX_NAME_SIZE+1];
	char Comm[DB_MAX_NAME_SIZE+1];
	char Fmt[TABLE_DESC_LEN];
	char Type[64];
	int Dim[64];
	int Entries;
	} TableDesc;

typedef struct _FileDesc {
	int  lun;
	char Name[256];
	char Csym[DB_MAX_NAME_SIZE+1];
	Tables Id;
	int active;
	int rwflag;
	struct _FileDesc *next;
	} FileDesc;



static FileDesc *FileDescList = NULL;

static int read_all_flag=0;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
char *ku_gets(void);
char *ku_getc(void);
int ku_geti(void);
static void DbToUpper( char *Csym, int len );
TableDesc *ArteTableDesc(char *);


int DbMatchType(char *opt)
	
	/*
	 * Converting the string "eq","le",... to MIZZI code. 
	 * Ruturning -1 if the string is not recognized.
	 */
	 
	{
	char topt[32];
	int var;
	
	strncpy(topt,opt,2);
	topt[2]=0;
	
	for(var=0;var<2;var++)
		topt[var]=toupper(topt[var]);
		
	if(strcmp(topt,"EQ")==0) return TABLEeq;
	if(strcmp(topt,"GE")==0) return TABLEge;
	if(strcmp(topt,"LE")==0) return TABLEle;
	if(strcmp(topt,"GT")==0) return TABLEgt;
	if(strcmp(topt,"LT")==0) return TABLElt;
	
	return -1;
	}

void DbOpen(int lun, char *Name, int rwflag, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A.Semenov (JINR, Dubna)
   *
   * Description: Open a MIZZI file/server and connect a symbolic name to it.
   *
   * Called from: anywhere.
   *
   * External references: OpenTables
   *
   * Argument in/out description
   *  lun     in     IO stream ID
   *  Name    in     file/server name
   *  rwflag  in   0 - open database read-only
   *               1 - open database for reading and writing
   *  ierr    out    Operation status:
   *               0  OK
   *               1  LUN already connected to file/server
   *               2  Can not allocate memory for FileDesc cell
   *               3  Can not open database file
   *
   *
   ************************************************************************/
 
  char mess[80]="DBOPEN: ", warn[80]="DBOPEN: ";
  char Csym[DB_MAX_NAME_SIZE+1],NameC[256];
  FileDesc *FD;
  
  int var;

  /* Get symbolic name */
  
  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;
  
  strcpy(NameC,Name);
  
	
  for(FD=FileDescList;FD;FD=FD->next)
  {
	  if(FD->lun==lun)
	  {
	    static int errPtr;
	    
	    /* 
	     * LUN already connected to a file.
	     * Use dbclose before reconnecting to a new file.
	     */

    sprintf(mess+strlen(mess),
       "symbolic name %s already connected to a file/server",Csym);
    mecErrorCount( &errPtr, mess );
    *ierr = 1;
	return;
      }
	  
  }


  sprintf(warn+strlen(warn),
     "opening file/server %s on %s",NameC,Csym );
  if (arteflag_.ibatch == 0)
     mecAllPrint( warn );
  else
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }
	 
  /* Allocate list cell for file descriptor */
	 
  for(FD=FileDescList;FD;FD=FD->next)
	  if(FD->active==0)
		  break;
  
  if(FD==NULL)
  {
	  FD=(FileDesc *)malloc(sizeof(FileDesc));
	  if(FD==NULL)
	  {
		  static int errPtr;
          sprintf(mess+strlen(mess),"cannot allocate memory");
          mecErrorCount( &errPtr, mess );
          *ierr = 2;
		  return;
      }
	  FD->next=FileDescList;
	  FileDescList=FD;
  }
  
  
  /* Open database */
  
  if( ( FD->Id = OpenTables( NameC , rwflag?0:TABLEreadonly ) ) == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "cannot open file/server '%s'",NameC);
    mecErrorCount( &errPtr, mess );
    *ierr = 3;
	return;
  }
  
  
  FD->active=1;
  FD->lun=lun;
  FD->rwflag=rwflag;
  strcpy(FD->Name,NameC);
  strcpy(FD->Csym,Csym);
  
  *ierr=0;
  return;
}

void DbClose(int lun, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Close a MIZZI file/server and disconnect the
   *              symbolic name from it.
   *
   * Called from: anywhere.
   *
   * External references: CloseTables
   *
   * Argument in/out description
   *  lun  in        IO stream ID
   *
   *  ierr    out    Operation status 
   *               0 success
   *               1 error (lun was not connected to file/server)
   *
   *
   ************************************************************************/
  
  char mess[80]="DBCLOS: ", warn[80]="DBCLOS: ";
  char Csym[DB_MAX_NAME_SIZE+1];
  int var;
  FileDesc *FD;
  
  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;


  for(FD=FileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

   if(FD==NULL)
   {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s not connected to a file/server",Csym );
    mecErrorCount( &errPtr, mess );
    *ierr = 1;
	return;
   }

  sprintf(warn+strlen(warn),
     "closing file/server %s on %s",FD->Name,FD->Csym );
  if (arteflag_.ibatch == 0)
     mecAllPrint( warn );
  else
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }

  CloseTables( FD->Id );
  FD->active=0;
  FD->Csym[0]=0;
  FD->Name[0]=0;
  FD->lun=0;

  *ierr = 0;
  return;
}


void DbWriteHeader(int lun, int *Mlink, int major, int minor, int *ierr)
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Write to a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbwtab_
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     header bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  Bank name too long for MIZZI
   *              3  Bank is not the header one
   *              4  Bank and all subs are empty
   *              5  Bank is not marked for output
   *           >100  100*nlink+(err status from dbwtab_ routine)
   *                 dbwtab reported error while writing bank # nlink
   *
   *
   ************************************************************************/

  int n, nlink, *Link, sub, retval=1;
  char mess[80]="DBWRIT: ", warn[80]="DBWRIT: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  int var;
  
  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;


  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
	*ierr=1;
    return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_( MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen > %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr=2;
    return;
  }
  
  /* Check if table is a header bank. */

  if( MEMO(nlink).sup[0] != 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not a header one",Name);
    mecErrorCount( &errPtr, mess );
	*ierr=3;
    return;
  }

  /* 
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* 
   * Check if the bank is marked for output. 
   */

	
	var=mmwstat_( &lun, &nlink );
  		
  	if(var>1)	
		{
		static int errPtr;
		sprintf(mess+strlen(mess),
		"header bank name \"%s\" is not marked for output", Name);
		mecErrorCount( &errPtr, mess );
		*ierr=5;
		return;
		}
		
	/* Check if there are non-empty banks */
		
	if(var==1)
	{
		static int errPtr;
		int i=0;
		for(n=MEMO(nlink).sub1;n <= MEMO(nlink).sub2; n++)
		{
			int sub;
			sub=MEMS(n).sub;
			if(mmwstat_(&lun, &sub)==0)
				i++;
		}
		if(i==0)
		{ 
			sprintf(mess+strlen(mess),
				"header bank \"%s\" and all subs are empty", Name);
			mecErrorCount( &errPtr, mess );
			*ierr=4;
			return;
		}
  	}


  /* Write out header bank ... */

  sprintf(warn+strlen(warn),
     "writing table %s on %s", Name , Csym );
  if (arteflag_.ibatch == 0)
     mecAllPrint( warn );
  else
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }

  DbWriteTable( lun, Mlink, major, minor, &retval );
  if(retval)
  {
	  *ierr=retval+nlink*100;
	  return;
  }

  /* ... and all subs. */
  

  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) {
    int res;
    
    sub = MEMS(n).sub;

    if( sub == nlink ) 
	    continue;

    res = mmwstat_( &lun, &sub );
    
    if( res > 1 ) 
		continue;
	/*
	  {
      static int errPtr;
      var = DB_MAX_NAME_SIZE/4;
      chhtot_(MEMO( sub ).name, Name, &var );
      sprintf(mess+8,
         "subbank %s is %s, skipped", Name, 
         	res==1?"empy":"not marked for output");
      mecWarningCount( &errPtr, mess );
      continue;
      }
	*/

    
    DbWriteTable( lun, (int *)((artelink_.larln1 + sub ) * 4),
    		 major, minor, &retval );
	if(retval)
	{
		*ierr=retval+sub*100;
		return;
	}
    
  }
  
  /* Save tables holding many-many relations */
  {
  int mema, nCol, i;
  mema=MEMO(nlink).mema;
  nCol=MEMO(nlink).ocol;
  for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;

  		DbWriteTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 		major, minor, &retval );
  		}
  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) 
    {
    sub = MEMS(n).sub;
    if( sub == nlink ) 
	    continue;
    mema=MEMO(nlink).mema;
    nCol=MEMO(nlink).ocol;
    for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
  		DbWriteTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 		major, minor, &retval );
  		}
     }
     
  }
  
  *ierr=0;
  return;

}

void DbWriteTable(int lun, int *Mlink, int major, int minor, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Write to a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  Bank name too long for MIZZI
   *              3  LUN is not connected to data file/server
   *              4  Can not book table
   *              5  Table already exists (at interactive mode, user 
    					refused overwriting)
   *              6  Can not allocate memory for buffer
   *              7  field type not implemented 
   *              8  lun is opened read-only
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows, *databuffer;
  short *databuffer1;
  char mess[80]="DBWTAB: ", *warn, what;
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  Tables Id;
  int var;
  TableDesc *TD;
  FileDesc  *FD;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;
  
  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
    *ierr=1;
	return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
    *ierr=2;
	return;
  }
  
  /*
   * Get bank fields.
   */
  
  TD=ArteTableDesc(Name);
   
  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* File id (given by OpenTables). */
  
  
  for(FD=FileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=3;
    return;
    }
    
  if( FD->rwflag == 0 ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "attemt to write to read-only stream %s", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=8;
    return;
    }  

  Id   =    FD->Id;
      
  /*
  ** Book Table T->Name
  */
    
  /* Check Format string */
  
  Tab=0;
  if(SearchTable(Id, 0, TD->Name, TD->Name))
	  Tab  =  BookTable( Id, TD->Name, 0, TD->Comm );
	  
  if (Tab != 0)
     {
     char *oldFmt;
     
     oldFmt = TableDescription( Id, Tab );
     if (strlen(oldFmt) != strlen(TD->Fmt) || strcmp(oldFmt,TD->Fmt) != 0)
        {
        static int errPtr;
        char       warnMess[80];
        
        /* Warning if new description conflicts with the old one */
        if (strlen(TD->Fmt) < strlen(oldFmt) ||
            strncmp(TD->Fmt,oldFmt,strlen(oldFmt)) != 0)
           {
           sprintf(warnMess,
              "DBWTAB: bank %s: new description conflicts with the old one",
				    TD->Name );
           mecWarningCount( &errPtr, warnMess );
           }
        
        UnbookTable( Id, Tab );
        Tab = 0;
        }
     }

  /* Book Table again */
    
  if (Tab == 0)
     Tab  =    BookTable( Id, TD->Name, TD->Fmt, TD->Comm );
  if( Tab == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "cannot book table %s", TD->Name );
    mecErrorCount( &errPtr, mess );
    *ierr=4;
	return;
  }
  
    
  TableStamp( Id, major , minor );
    
    
  /*
  ** Check if Major and Minor revision numbers have already been used.
  */
    
  if( MatchTable( Id, Tab, 0 ) )
    {
    if (arteflag_.ibatch == 0) /* (for interactive application only) */
      {
      sprintf(mess+strlen(mess),
         "table %s with major and minor %d %d already exists",
          Name , major, minor );
      mecAllPrint( mess );
      warn = "Overwrite y/n ?";
      mecAllPrint( warn );
      do
        {
        scanf( "%c", &what );
        what = toupper ( what );
        } while ( what != 'Y' && what != 'N' );
    
      if( what == 'N' )
        {
        UnbookTable( Id, Tab );
		*ierr=5;
        return;
        }
      }

    sprintf(mess, "DBWRIT: overwriting table %s", Name );
    if (arteflag_.ibatch == 0)
      mecAllPrint( mess );
    else
      {
      static int errPtr;
      mecWarningCount( &errPtr, mess );
      }
    }

  SkipTableField( Id, 1 );
  
  if(mmwstat_(&lun, &nlink)==0)
	  Nrows = N_U(Link);
  else
	  Nrows=0;
  
  /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	m=Nrows*(TD->Dim[n]==0?1:TD->Dim[n])*sizeof(int);
  	if(m>var)
  		var=m;
  	}
  	
  
  databuffer=(void *)malloc(var);
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr=6;
      return;
      }
	  
   databuffer1=(short *)databuffer;

    /* Put rows number and fields dimensions into the first field. */


  databuffer[0]=Nrows;
  for(n=0;n<TD->Entries;n++)
  	databuffer[n+1]=TD->Dim[n];
  
  PutTableField( Id, TABLEint, TD->Entries+1, databuffer );
  
  if(Nrows==0)
  {
	StoreTable( Id, Tab, 0 );
  	UnbookTable( Id, Tab );
  	*ierr=0;
  	return;
  }
  
  /* Finally record the bank into the db file. */
  
  /* Cycle on the fields of the bank. */
  
  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = TABLEascii;
      break;
	  
	case 'U':
	  FieldType = TABLEushort;
	  break;
	  
	case 'S':
	  FieldType = TABLEshort;
	  break;
      
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr=7;
      return;
      }

    }
    
    /* Cycle on the dimension of each field. */
    
    for( m = 0 ; m < ( (TD->Dim[n] == 0) ? 1 : TD->Dim[n] ) ; m++ ) 
    {
    int k;
      
      int nCopy;
      
      nCopy=Nrows;
      if(TD->Type[n]=='H')
      	nCopy*=4;
	  
	  if(TD->Type[n]=='S' || TD->Type[n]=='U')
	  	{
	  	for(k=0;k<Nrows;k++)
      		databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m] =
      			*(Link + 2 + j + k*MEMO(nlink).col + m );
  		}
	  else
	    {
        for(k=0;k<Nrows;k++)
      	  databuffer[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m] =
      		*(Link + 2 + j + k*MEMO(nlink).col + m );
	    }
		  
    }
    
    PutTableField(Id, FieldType, 
      Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
      	databuffer);
    
    j += ( (TD->Dim[n] ==0 ) ? 1 : TD->Dim[n] );
    
  }
  StoreTable( Id, Tab, 0 );
  UnbookTable( Id, Tab );
  *ierr=0;
  return;
  
}



static int dbr_last_major, dbr_last_minor, dbr_empty;

void DbReadHeader(int lun, int *Mlink, int major, int minor, 
		char *opt, int *ierr)
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Read from a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbrtab_
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  opt     in     "eq","le",... - applied to major, minor
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  bank name too long for MIZZI
   *              3  'opt' has incorrect value
   *              4  bank is not the header one
   *              5  bank is not empty
   *              6  bank is not marked for input
   *              7  bank and all subs are empty after reading
   *           >100  100*nlink+(err status from dbrtab_ routine)
   *                 dbrtab_ reported error while reading bank # nlink
   *
   ************************************************************************/

  int n, m, nlink, sub, res, maj, min, tbls_read=0;
  char subopt[4]="EQ";
  char mess[80]="DBREAD: ",warn[80]="DBREAD: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  int var;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
	*ierr = 1;
    return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr = 2;
    return;
  }

  /*
  ** Check 'opt' parameter has allowed value.
  */
  
  if( DbMatchType(opt)==-1 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
    "Unknown option '%s'; should use one of \"eq\", \"le\", \"ge\", \"lt\", \"gt\"",
       opt);
    mecErrorCount( &errPtr, mess );
	*ierr = 3;
    return;
  }


  /* Check if the bank is a header one. */

  if( MEMO(nlink).sup[0] != 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not a header one", Name );
    mecErrorCount( &errPtr, mess );
	*ierr = 4;
    return;
  }

  /* 
   * Drop existing tables.
   */

/*  if( *Mlink != 0 ) mmdrop_( Mlink ); */

  var=mmrstat_(&lun, &nlink);
  if(var==1)
  {
	static int errPtr;
    sprintf(mess+strlen(mess),
       "header bank %s is not empty", Name);
    mecWarningCount( &errPtr, mess );
	*ierr = 5;
    return;
  }  
  if(var)
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "header bank %s is not marked for input on %s", Name, Csym );
    mecWarningCount( &errPtr, mess );
    *ierr = 6;
	return;
  }


  /* Start reading. */  

  sprintf(warn+strlen(warn),
     "reading table %s on %s", Name, Csym );
  if (arteflag_.ibatch == 0)
     mecAllPrint( warn );


  DbReadTable( lun, Mlink, major, minor, opt, &var);
  if(var)
  {
	  *ierr=var+100*nlink;
	  return;
  }
  
  if(!dbr_empty)
	  tbls_read++;
  
  maj=dbr_last_major;
  min=dbr_last_minor;

  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) {

    sub = MEMS(n).sub;

/*    printf("Reading sub-bank %d named %60.60s\n",n,MEMO(sub).name);*/

    if( sub == nlink ) continue;

    res=mmrstat_( &lun, &sub );
    
    if( res !=  0 ) {
      static int errPtr;
      var = DB_MAX_NAME_SIZE/4;
      chhtot_(MEMO( sub ).name, Name, &var );
      sprintf(mess+8,
         "subbank %s is %s,skipped", Name, 
         	res==1?"not empy":"not marked for input");
      mecWarningCount( &errPtr, mess );
      continue;
    }

    DbReadTable( lun, (int *)((artelink_.larln1 + sub) * 4),
		       maj, min, subopt, &var);
	if(var)
	{
		*ierr=var+sub*100;
		return;
	}
	
	if(!dbr_empty)
		tbls_read++;

  }
  
    
  /* Load tables holding many-many relations */
  {
  int mema, nCol, i;
  mema=MEMO(nlink).mema;
  nCol=MEMO(nlink).ocol;
  for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
  		DbReadTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 maj, min, subopt, &var);
  		}
  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) 
    {
    sub = MEMS(n).sub;
    if( sub == nlink ) 
	    continue;
    mema=MEMO(nlink).mema;
    nCol=MEMO(nlink).ocol;
    for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
  		DbReadTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 maj, min, subopt, &var);
  		}
     }
     
  }
  

  /* Book parallel tables and relations. */

  if(!tbls_read)
  {
	  *ierr=7;
	  return;
  }
  
  m = 0;
  mmbook_( Mlink, &m );
  mmrupd_( &nlink );

  *ierr=0;
  return;

}

void DbReadTable(int lun, int *Mlink, int major, int minor, 
		char *opt, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Read from a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  opt     in     "eq","le",... - applied to major, minor
   *  ierr    out    operation status
   *              0  OK
   *              1  link not inside link area
   *              2  'opt' has incorrect value
   *              3  bank name too long for MIZZI
   *              4  LUN is not connected to data file/server
   *              5  bank does not exist in file/server
   *              6  Can not find requested major/minor table
   *              7  Can not allocate memory for buffer
   *              8  Field type is not implemented
   *              9  Database table format differs from Arte table
   *
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows, *databuffer;
  short *databuffer1;
  char mess[80]="DBRTAB: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  TableDesc *TD;
  FileDesc  *FD;
  Tables Id;
  int var;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
    *ierr = 1;
	return;
  }
  
  /*
  ** Check 'opt' parameter has allowed value.
  */
  
  if( DbMatchType(opt)==-1 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
    "Unknown option '%s'; should use one of \"eq\", \"le\", \"ge\", \"lt\", \"gt\"",
       opt);
    mecErrorCount( &errPtr, mess );
	*ierr = 2;
    return;
  }


  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr = 3;
    return;
  }

  TD=ArteTableDesc(Name);
 

  /* File id (given by OpenTables). */


  for(FD=FileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr = 4;
    return;
  }
  
  Id   =    FD->Id;

  /* Check that table T->Name exists */
  
  if(SearchTable(Id, 0, TD->Name, TD->Name)==0)
  {
    if( read_all_flag==0 )
    {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "bank %s does not exist in file/server %s", Name, FD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 5;
      return;
    }
    else
    {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "bank %s is not in file/server %s, skipped", Name, FD->Name );
      mecWarningCount( &errPtr, mess );
	  *ierr = 0;
      return;
    }
  }

  /* Book Table T->Name for reading. */

  Tab  =    BookTable( Id, TD->Name, 0, 0 );

  if( Tab == 0 ) 
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s can not be booked in file/server %s", Name, FD->Name );
    mecErrorCount( &errPtr, mess );
	*ierr = 5;
    return;
  }


  TableStamp( Id, major, minor );

  if( FetchTable( Id, Tab, DbMatchType(opt) ) == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "table %s with major and minor %d %d doesn't exist",
       Name, major, minor );
    UnbookTable( Id, Tab );
    mecErrorCount( &errPtr, mess );
	*ierr = 6;
    return;
  }
  
  dbr_last_major=TableMajor(Id);
  dbr_last_minor=TableMinor(Id);
  
  SkipTableField( Id, 1 );

  var=TableFieldSize(Id);
  if(var!=TD->Entries+1)
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "different format for table %s ", Name );
    UnbookTable( Id, Tab );
    mecErrorCount( &errPtr, mess );
	*ierr = 9;
    return;
  }
  
  databuffer=malloc(var*sizeof(int));
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }
  databuffer1 = (short *) databuffer;
      
  /* Get rows number and entries dimensions (first field). */

  GetTableField( Id, TABLEint, var, databuffer );
  
  for(n=0;n<TD->Entries;n++)
  	if(TD->Dim[n]!=databuffer[n+1])
	  {
	    static int errPtr;
	    sprintf(mess+strlen(mess),
	       "different format for table %s ", Name );
	    UnbookTable( Id, Tab );
	    mecErrorCount( &errPtr, mess );
	    *ierr = 9;
	    return;
	  }
  
  Nrows=databuffer[0];

  free(databuffer);

  /* Check if bank exists already. */

  if( *Mlink != 0 ) {
	mmdrop_( Mlink );
    sprintf(mess+strlen(mess),
       "existing bank %s will be overwritten", Name );
    if (arteflag_.ibatch == 0)
       mecAllPrint( mess );
    else
       {
       static int errPtr;
       mecWarningCount( &errPtr, mess );
       }
  }
 
  if(Nrows==0)
  {
	  dbr_empty=1;
	  UnbookTable(Id,Tab);
	  *ierr=0;
	  return;
  }
  
   /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	m=Nrows*(TD->Dim[n]==0?1:TD->Dim[n])*sizeof(int);
  	if(m>var)
  		var=m;
  	}
  	
  
  databuffer=(void *)malloc(var);
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }


  /* Book space in memory. */

  mmbook_( Mlink, &Nrows );


  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* Cycle on the fields of the bank. */

  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = TABLEascii;
      break;
	  
    case 'S':
      FieldType = TABLEshort;
      break; 
	  
    case 'U':
      FieldType = TABLEushort;
      break;
	  
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 8;
      return;
      }
    }

    GetTableField(Id, FieldType, 
      Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
      databuffer);    

    /* Cycle on the dimension of each field (scalar 0, vector n). */

    for( m = 0 ; m < ( ( TD->Dim[n] == 0 ) ? 1 : TD->Dim[n] ) ; m++ ) 
    {
      int k;
      int nCopy;
      
      nCopy = Nrows;
    
	  if(TD->Type[n]=='S' || TD->Type[n]=='U')
	  {
      for(k=0;k<Nrows;k++)
      	*(Link + 2 + j + k*MEMO(nlink).col + m ) =
      		databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m];
      }
	  else
	  {
	  for(k=0;k<Nrows;k++)
      	*(Link + 2 + j + k*MEMO(nlink).col + m ) =
      		databuffer[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m];  
	  }
    
    }
    j += ( ( TD->Dim[n] == 0 ) ? 1 : TD->Dim[n] );


  }
  UnbookTable( Id, Tab );

  /* Free unused rows. */

  mmfree_( Mlink );

  free(databuffer);

  dbr_empty=0;
  *ierr = 0;
  return;
}


void dbinit_(void) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Initialize Db package.
   *
   * Called from: ATINIT
   *
   * External references:
   *
   * Return value: 1 success, 0 error.
   *
   ************************************************************************/

  return;

}

void dbstop_(void) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 31.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Close files before exit.
   *
   * Called from: ATSTOP
   *
   * External references: DbClose
   *
   ************************************************************************/


   FileDesc *FD;
   int ierr;

   for(FD=FileDescList;FD;FD=FD->next)
	   if(FD->active)
		   DbClose(FD->lun,&ierr);

}



/***********************************************************************/

TableDesc *ArteTableDesc(char *tName)

/************************************************************************
**
** PARAMETERS
**	tName	 in    table name 
**
** RETURNS
**	Arte table description to be used in MIZZI
**
** FUNCTION DEFINITION
**	Returns Arte table description to be used in MIZZI.
*************************************************************************
*/

{
int tab;
static TableDesc TD;
char    errMess[256];
char    lastName[90],curName[90];
int	mema;
int	nCol;
int	iCol;
int	col;
int	isFirst;
int	type;
int     i;
char	oneDesc[100];
int	lastType = -1;
int	lastIlow;
int	lastIupp;
float	lastFlow;
float	lastFupp;
float	lastAccu;
int	nEq;
int	iRpt;
int	changed;
int var;


	{ /* Lookup table index */
	int n,nlink,var;
	char mess[80]="ArteTableDesc: ";
	char curName[DB_MAX_NAME_SIZE+1];
	
	  /* Look for bank name in MEMO bank. */

	 for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) 
	 	{
    	 	var = DB_MAX_NAME_SIZE/4;
    		chhtot_(MEMO(n).name, curName, &var );
    		if( strcmp( tName, curName ) == 0 ) 
    			{
      			nlink = n; 
      			break; 
    			} 
		}

	  if( nlink == 0 ) 
	  {
    	  static int errPtr;
    	  sprintf(mess+strlen(mess),
       	        "bank %s is not booked in MEMO bank",tName);
    	  mecErrorCount( &errPtr, mess );
    	  return NULL;
  	  }
  	  
        tab=nlink;
	}

var = DB_MAX_NAME_SIZE/4;
chhtot_(MEMO(tab).name, TD.Name, &var);
chhtot_(MEMO(tab).name, TD.Comm, &var);
TD.Entries=0;

strcpy(TD.Fmt,"I:NROWS-FIELD");
lastName[0]=0;
isFirst = 1;

mema = MEMO(tab).mema;
nCol = MEMO(tab).ocol;

nEq = 0;
changed = 0;

for (iCol=0; iCol<nCol; iCol++)
   {
   int i;
   col = mema+iCol;
   type = MEMA(col).type;
   strncpy(curName,MEMA(col).name,85);
   for(i=0;i<80;i++)
   	{
   	if(curName[i]==' ')
   		{
   		curName[i]=0;
   		break;
   		}
   	curName[i]=toupper(curName[i]);
   	}
   	
   if (type == 1 || type == 2 || type == 3 || type == 4)
      type = 1;

   /* check possible types */
   if (type <= 0 || type >= 9)
      {
      static int errPtr;
      
      sprintf(errMess, "ArteTableDesc: Table %s: unknown type %d in MEMA\n",
         tName,type);
      mecWarningCount(&errPtr, errMess);
      return(NULL);
      }

   /* 
   ** If sequence of equal attributes is empty, save attributes 
   */ 
   if (nEq == 0)
      {
      lastIlow = MEMA(col).ilow;
      lastIupp = MEMA(col).iupp;
      lastFlow = MEMA(col).flow;
      lastFupp = MEMA(col).fupp;
      lastAccu = MEMA(col).accu;
      lastType = type;
      
      strcpy(lastName,curName);
      
      changed = 0;
      nEq = 1;
      if (iCol != nCol-1)
         continue;
      }
   /* 
   ** Otherwize check that previos and current attibutes are the same 
   */ 
   else
      {
      changed = 1;
      if(strlen(lastName)==strlen(curName)
      		&& strcmp(lastName,curName)==0)
      	{
        changed = 0;
        nEq++;
        if (iCol != nCol-1)
           continue;
        }
      }


   /* 
   ** Generate description string
   */
   for (iRpt=0; iRpt<2; iRpt++)
      {
      strcpy(oneDesc,"");
      if (!isFirst)
         strcat(oneDesc,",");
      else
         isFirst = 0;
   
      if (nEq != 1)
         sprintf(oneDesc+strlen(oneDesc),"%d",nEq);

      /* generate */
      switch(lastType)
/*TEMP	ilow, iupp, flow, fupp, accu - not yet implemented: ArteTableDesc */
         {
         case 5:	/* int */
            strcpy(oneDesc,"I1");
            break;
         
         case 6:	/* bit */
            strcpy(oneDesc,"B1");
            break;
         
         case 7:	/* float */
            strcpy(oneDesc,"F");
            break;
         
         case 8:	/* holl */
	 case 11:
            strcpy(oneDesc,"H");
            break;
			
	  case 9:    /* short */
			strcpy(oneDesc,"S");
            break;
	  case 10:    /* ushort */
			strcpy(oneDesc,"U");
            break;
         
         case 1:
         case 2:
         case 3:
         case 4:	/* relation */
            if(lastType==4)
            {
            printf("4444444444: table %20.20s, field %20.20s\n",
            	MEMO(tab).name,MEMA(col).name);
            }
            strcpy(oneDesc,"I1");
            break;
         }

      if (strlen(TD.Fmt)+strlen(oneDesc) > TABLE_DESC_LEN)
         {
         static int errPtr;
      
         sprintf(errMess, "ArteTableDesc: Table %s: too long description\n",
            tName);
         mecWarningCount(&errPtr, errMess);
         return(NULL);
         }
      else
         {
         char MizziType[4];
         MizziType[1]=MizziType[2]=0;
         switch(oneDesc[0])
         	{
         	case 'I': MizziType[0]='I'; break;
         	case 'F': MizziType[0]='F'; break;
         	case 'B': MizziType[0]='U'; MizziType[1]='I'; break;
         	case 'H': MizziType[0]='A'; break;
			case 'S': MizziType[0]='S'; break;
			case 'U': MizziType[0]='U'; MizziType[1]='S'; break;
         	default:  MizziType[0]='?';
         	}
			
		 TD.Type[TD.Entries]=oneDesc[0];
		 TD.Dim[TD.Entries]=nEq==1?0:nEq;
		 TD.Entries++;
		 if(MizziType[0]=='A') nEq*=4;
         /*if(nEq==1)*/
              sprintf(TD.Fmt+strlen(TD.Fmt),
              		" %s:%s",MizziType,lastName);
         /*else
              {
              int i;
              for(i=1;i<=nEq;i++)
              sprintf(TD.Fmt+strlen(TD.Fmt),
              		" %s:%s-%d",MizziType,lastName,i);
              }*/
         }
      
      /* new sequence started */
      lastIlow = MEMA(col).ilow;
      lastIupp = MEMA(col).iupp;
      lastFlow = MEMA(col).flow;
      lastFupp = MEMA(col).fupp;
      lastAccu = MEMA(col).accu;
      lastType = type;
      
      strcpy(lastName,curName);
      
      /* Save the previous too if this is the last item */
      if (!(iCol == nCol-1 && changed == 1))
         break;
      nEq = 1;
      }
   nEq = 1;
   }

   for(i=0;TD.Fmt[i];i++)
        if(TD.Fmt[i]=='_')
	      TD.Fmt[i]='-';

return &TD;
}





void dbopenact_(void) {

  char Name[128];
  char *Csym;
  char mess[80];
  char ropt[32];
  int lun, var;
  int ierr;
  
  Csym = ku_gets();
  strcpy(ropt,ku_gets());
  ropt[2]=0;
  DbToUpper(ropt,2);
  
  DbToUpper(Csym,strlen(Csym));
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "dbopen: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  } 

  if(MEIO(lun).concat!=0)
  {
    static int errPtr;
    sprintf(mess,
       "dbopen: do not concatenate database on %s",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  } 

  strncpy(Name,MEIO(lun).name,120);
  for(var=0;var<120 && Name[var] && Name[var]!=' ';var++);
    Name[var]=0;
    
  DbOpen( lun, Name, strcmp(ropt,"RW")==0, &ierr );

  return;
}

void dbcloseact_(void) {

  char *Csym;
  char mess[80];
  int lun;
  int ierr;
  
  Csym = ku_gets();
  
  DbToUpper(Csym,strlen(Csym));
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "dbclose: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbClose( lun, &ierr );

  return;
}

void dbwriteact_(void) {

  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[80]="DBWRITEACT: ";
  int  Major, Minor, n, nlink;
  char curName[DB_MAX_NAME_SIZE+1];
  int var;
  int lun, ierr;
  
  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();

  DbToUpper( Name, strlen(Name) );
  DbToUpper( Csym, strlen(Csym) );
  
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       " symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  /* Look for bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_(MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank",Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbWriteHeader(lun, (int *)((artelink_.larln1 + nlink) * 4), 
		 Major, Minor, &ierr );

  return;
}


void dbreadact_(void) {

  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[128]="DBREADACT: ";
  int Major, Minor, n, nlink;
  char Opt[16];
  char curName[DB_MAX_NAME_SIZE+1];
  int var, lun, ierr;

  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();
  strcpy(Opt,ku_gets());
  
  DbToUpper( Csym, strlen(Csym) );

  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "DBREADACT: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  DbToUpper( Name, strlen(Name) );

  /* Look for Bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_( MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank", Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbReadHeader( lun, (int *)((artelink_.larln1 + nlink) * 4), 
		Major, Minor, Opt, &ierr );

  return;
}

void dblfilact_(void) {
  char *Csym;

  Csym   =  ku_gets();

  DbListFile( Csym );

  return;
}



void DbListFile(char *CsymIn) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: List the content of a MIZZI file.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *
   *
   ************************************************************************/

  char *tbname, mess[80]="DBLFIL: ";
  char Csym[DB_MAX_NAME_SIZE+1],Csymt[DB_MAX_NAME_SIZE+1];
  int xe, major, minor, len;
  int flag = 1;
  Tables x;
  FileDesc *FD;
  int var;
  
  len=strlen(CsymIn);

  strcpy(Csym,CsymIn);
  DbToUpper( Csym, len );

  var = len/4 + (len%4?1:0);
  chhtot_( Csym, Csymt, &var );
  
  for(FD=FileDescList;FD;FD=FD->next)
	  if(strcmp(FD->Csym,Csymt)==0)
		  break;

  if( FD == NULL ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csymt );
    mecErrorCount( &errPtr, mess );
    return;
  }

  x = FD->Id;

  printf( "\nListTables: Listing tables for file/server %s\n", FD->Name );
  SearchTable(x,0,"","");
  while( (tbname = SearchTable(x,1,0,0)) ) {
    flag = 0;
    xe = BookTable( x, tbname, 0, 0 );

    printf( "\n===>Name    '%s'\n", tbname );
    printf(   "--->Title   '%s'\n", TableTitle(x,xe) );
    printf(   "--->Format  '%s'\n", TableDescription(x,xe) );

    TableStamp( x, 0, 0 );

    printf( "--->Entries\n" );

    for( TableStamp( x, 0, 0 ) ; MatchTable( x, xe, 1 ) ; 
	 TableStamp( x, major, minor ) ) {

      major = TableMajor(x);
      minor = TableMinor(x);

      printf( "    Major = %d          Minor = %d\n", major, minor );

    }
    UnbookTable( x, xe );
  }

  if( flag ) {
    strcat( mess, "file empty\n");
    mecAllPrint( mess );
  }
  return;
}

static void DbToUpper( char *Csym, int len ) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Convert text to upper case.
   *
   * Called from: anywhere.
   *
   * External references: toupper function.
   *
   * Argument in/out description
   *  Csym    in/out the text to be converted
   *  len     in     lenght of Csym.
   *
   ************************************************************************/

  while ( len-- != 0 ) 
    *Csym++ = toupper(*Csym);

  return;

}

