
// *****************************************************************************
//
// source:    faFile.C
//
// created:   30 Jul 97
// updated:   23 Feb 99 Andreas Gellrich
//
// Author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   multiple file input (AG: 27-Nov-98)
//            includes (AG: 11 Jan 99)
//            remove not needed evtNum (AG: 15 Jan 99)
//            polishing (AG: 27 Jan 99)
//            removed faUtil.hh (AG: 23 Feb 99)
//
// *****************************************************************************

// includes
#include "farm/fa.hh"

// externals
extern int  FADEB;        // debug flag
extern int  OLD;          // old format
extern int  SEG_LEN;      // buffer size
extern char *evtBuf;      // buffer
extern char *IN_FILE[50]; // input files
extern FILE *in;          // input streams
extern int  nFiles;       // number of files

//
// program code
//
int
faReadFile()
{
  // inits
  int         rc      = ETB_ok;
  static int  iFile   = 0;
  static int  newFile = 1;
  static FILE *in     = 0;
  static char *name   = 0;
    
  //
  // open file
  //
  while( 1 ){
    if( newFile ){
      
      // close old file
      if( in ){
	faCloseFile( name, in );
      }
      
      // new file
      if( iFile >= nFiles ){
	return ETB_file_err;
      }
      
      // open
      if( ( in = faOpenFile( IN_FILE[iFile] ) ) ){
	name    = IN_FILE[iFile]; // current file name
	newFile = 0;              // flag
	iFile++;                  // increase file counter
#ifdef _DEB2_
	if( FADEB ){
	  cout << "FILE: "
	       << "file="    << name            << " "
	       << "in="      << (void *) in     << " "
	       << "SEG_LEN=" << SEG_LEN         << " "
	       << "evtBuf="  << (void *) evtBuf << endl;
	}
#endif
      }
      // file open error
      else {
	in = 0;
#ifdef _ERR_
	cerr << "FILE: ERROR: file=" << name << endl;
#endif
	rc = ETB_file_err;
	break;
      }
    }
    
    //
    // read event from file
    //
    int rc = ETB_ok;
    if( !OLD ){
      int bc = 0;
      LogHdr hdr;
      if( ( rc = l4aGetEvt( in, (ETB_event *) evtBuf, SEG_LEN, &bc, &hdr ) ) == GRR_OK ){
#ifdef _DEB_
	if( FADEB ){
	  cout << endl;
	  cout << "FILE: read event from file=" << name << endl;
	  cout << "FILE: Log header: "
	       << "bc="     << bc         << " "
	       << "exp="    << hdr.exp    << " "
	       << "run="    << hdr.run    << " "
	       << "evt="    << hdr.evt    << " "
	       << "datime=" << hdr.datime << endl;
	}
#endif
	break;
      }
      // eof
      else {
	newFile = 1;
#ifdef _ERR_
	cerr << "FILE: End-of-file in file=" << name << " "
	     << "rc="                        << rc   << endl;
#endif
	rc = ETB_file_err;
	continue;
      }
    }
    //
    // read old format
    //
    else {
      if( ( rc = ETB_read_event( in, (ETB_event *) evtBuf, SEG_LEN ) ) == ETB_ok ){
#ifdef _DEB_
	if( FADEB ){
	  cout << endl;
	  cout << "FILE: read event from OLD file=" << name << endl;
	}
#endif
	break;;
      }
      // eof
      else if( rc != ETB_ok ){
	newFile = 1;
#ifdef _ERR_
	cout << "FILE: End-of-file in OLD file=" << name << " "
	     << "rc="                            << rc   << endl;
#endif
	continue;
      }
    }
  }
  
  // return value
  return rc;
}

//
// open file
//
FILE *
faOpenFile( char *name )
{
  FILE *in = 0;
  if( ( in = fopen( name, "rb" ) ) != 0 ){
#ifdef _OUT_
    cout << "FILE: Opened file=" << name << endl;
#endif
    return in;
  }
  else {
#ifdef _ERR_
    cerr << "FILE: ERROR: Opening file=" << name << endl;
#endif
    return NULL;
  }
}

//
// close file
//
int
faCloseFile( char *name, FILE *in )
{
  if( in ){
#ifdef _OUT_
    cout << "FILE: Closed file=" << name << endl;
#endif
    fclose ( in );
  }  
  return GRR_OK;
}
