
// *****************************************************************************
//
// source:    faUtil.C
//
// type:      source code
//
// created:   29 Oct 96
// updated:   26 Mar 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   polishing (AG: 9 Feb 99)
//            DTAR handling (AG: 10 Feb 99)
//            DTAR has SLB_SUBHDR inside (AG: 22 Feb 99)
//            removed faUtil.hh (AG: 23 Feb 99)
//            provide target and all daq info (2 Mar 99)
//            Fortran/C callable functions
//            polishing (AG: 11 Mar 99)
//            change default return values of utility functions (AG: 26 Mar 99)
//
// *****************************************************************************

// includes
#include "farm/fa.hh"

// externals
extern int FADEB;

//
// print run/event
//
void
faPrintEvtCount( int run, int event )
{
  if( run > 0 && event > 0 ){
    if( !( event % int( pow( 10, int( log10( event ) ) ) ) ) ){
      cout << "PRINT: "
	   << "run="   << run   << " "
	   << "event=" << event << endl;
    }
  }
}

//
// print rows of tables
//
int
faPrintNrow()
{
  // get first 4-byte word of table
  ArtePointer<EVHD> evhd( 1 );
  ArtePointer<DIHD> dihd( 1 );
  
  ArtePointer<DDAQ> ddaq( 2 );
  ArtePointer<DSVD> dsvd( 1 );
  ArtePointer<DITR> ditr( 1 );
  ArtePointer<DPTC> dptc( 1 );
  ArtePointer<DOTR> dotr( 1 );
  ArtePointer<DRIC> dric( 1 );
  ArtePointer<DTRD> dtrd( 1 );
  ArtePointer<DCAL> dcal( 1 );
  ArtePointer<DMUT> dmut( 1 );
  ArtePointer<DMUX> dmux( 1 );
  ArtePointer<DMUP> dmup( 1 );
  ArtePointer<DFLT> dflt( 1 );
  ArtePointer<DSLT> dslt( 1 );
  ArtePointer<DTLT> dtlt( 1 );
  ArtePointer<DTAR> dtar( 1 );
  ArtePointer<DFCS> dfcs( 1 );

  cout << "NROW: ";
  if( evhd.nrow() ) cout << "EVHD=" << evhd.nrow() << " ";
  if( dihd.nrow() ) cout << "DIHD=" << dihd.nrow() << " ";
  if( ddaq.nrow() ) cout << "DDAQ=" << ddaq.nrow() << " ";
  if( dsvd.nrow() ) cout << "DSVD=" << dsvd.nrow() << " ";
  if( ditr.nrow() ) cout << "DITR=" << ditr.nrow() << " ";
  if( dptc.nrow() ) cout << "DPTC=" << dptc.nrow() << " ";
  if( dotr.nrow() ) cout << "DOTR=" << dotr.nrow() << " ";
  if( dric.nrow() ) cout << "DRIC=" << dric.nrow() << " ";
  if( dtrd.nrow() ) cout << "DTRD=" << dtrd.nrow() << " ";
  if( dcal.nrow() ) cout << "DCAL=" << dcal.nrow() << " ";
  if( dmut.nrow() ) cout << "DMUT=" << dmut.nrow() << " ";
  if( dmux.nrow() ) cout << "DMUX=" << dmux.nrow() << " ";
  if( dmup.nrow() ) cout << "DMUP=" << dmup.nrow() << " ";
  if( dflt.nrow() ) cout << "DFLT=" << dflt.nrow() << " ";
  if( dslt.nrow() ) cout << "DSLT=" << dslt.nrow() << " ";
  if( dtlt.nrow() ) cout << "DTLT=" << dtlt.nrow() << " ";
  if( dtar.nrow() ) cout << "DTAR=" << dtar.nrow() << " ";
  if( dfcs.nrow() ) cout << "DFCS=" << dfcs.nrow() << " ";
  cout << endl;

  return ARTE_GOOD;
}

//
// print EVHD-table
//
int
faPrintEVHD()
{
  ArtePointer<EVHD> evhd( 1 );
  if( evhd.nrow() ){
    cout << "EVHD: "
	 << "evt="  << evhd->evt  << " "
	 << "run="  << evhd->run  << " "
	 << "exp="  << evhd->exp  << " "
	 << "date=" << evhd->date << " "
	 << "time=" << evhd->time << " "
	 << "vfld=" << evhd->vfld << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// print DIHD-table
//
int
faPrintDIHD()
{
  ArtePointer<DIHD> dihd( 1 );
  if( dihd.nrow() ){
    cout << "DIHD: "
	 << "bx=0x"   << hex << dihd->bx   << dec << " "
	 << "flt=0x"  << hex << dihd->flt  << dec << " "
	 << "slt=0x"  << hex << dihd->slt  << dec << " "
	 << "slb=0x"  << hex << dihd->slb  << dec << " "
	 << "cmsk=0x" << hex << dihd->cmsk << dec << " "
	 << "emsk=0x" << hex << dihd->emsk << dec << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// print DDAQ-table
//
int
faPrintDDAQ()
{
  ArtePointer<DDAQ> ddaq( 2 );
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );

    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );

    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );

    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );

    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );

    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );

    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );

    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );

    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );

    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );

    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );

    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );

    cout << "DDAQ: " 
	 << "run_number="            << run_number                              << " "
	 << "event_time="            << event_time                              << " "
	 << "date="                  << faDate( (time_t *) &event_time )        << " "
	 << "time="                  << faTime( (time_t *) &event_time )        << " "
	 << "event_status=0x" << hex << event_status                     << dec << " "
	 << "flt_status=0x"   << hex << flt_status                       << dec << " "
	 << "slp_node="              << slp_node                                << " "
	 << "slt_status=0x"   << hex << slt_status                       << dec << " "
	 << "exp_number="            << exp_number                              << " "
	 << "farm_node="             << farm_node                               << " "
	 << "BX_long="               << BX_long                                 << " "
	 << "BX_upper="              << BX_upper                                << " "
	 << "FCS_tcode="             << FCS_tcode                               << " "
	 << "physBX="                << physBX                                  << " "
	 << "FLT_BX="                << FLT_BX                                  << " "
	 << "fltevt="                << fltevt                                  << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// daq run_number
//
int
faDDAQrun_number()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    return run_number;
  }
  else {
    return -1;
  }
}

//
// daq event_time
//
int
faDDAQevent_time()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    return event_time;
  }
  else {
    return -1;
  }
}

//
// daq event_status
//
int
faDDAQevent_status()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    return event_status;
  }
  else {
    return -1;
  }
}

//
// daq flt_status
//
int
faDDAQflt_status()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );
    return flt_status;
  }
  else {
    return -1;
  }
}

//
// daq slp_node
//
int
faDDAQslp_node()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );
    return slp_node;
  }
  else {
    return -1;
  }
}

//
// daq slt_status
//
int
faDDAQslt_status()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );
    return slt_status;
  }
  else {
    return -1;
  }
}

//
// daq exp_number
//
int
faDDAQexp_number()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );
    return exp_number;
  }
  else {
    return -1;
  }
}

//
// daq farm_node
//
int
faDDAQfarm_node()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );
    return farm_node;
  }
  else {
    return -1;
  }
}

//
// daq BX_long
//
int
faDDAQBX_long()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );
    return BX_long;
  }
  else {
    return -1;
  }
}

//
// daq BX_upper
//
short
faDDAQBX_upper()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );
    return BX_upper;
  }
  else {
    return -1;
  }
}

//
// daq FCS_tcode
//
short
faDDAQFCS_tcode()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );
    return FCS_tcode;
  }
  else {
    return -1;
  }
}

//
// daq physBX
//
short
faDDAQphysBX()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );
    return physBX;
  }
  else {
    return 0;
  }
}

//
// daq FLT_BX
//
short
faDDAQFLT_BX()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );
    return FLT_BX;
  }
  else {
    return -1;
  }
}

//
// daq fltevt
//
short
faDDAQfltevt()
{
  ArtePointer<DDAQ> ddaq( 2 ); 
  if( ddaq.nrow() ){
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );
    return fltevt;
  }
  else {
    return -1;
  }
}

//
// print DTAR-table
//
int
faPrintDTAR()
{
  ArtePointer<DTAR> dtar( 2 );
  if( dtar.nrow() ){
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); 

    cout << "DTAR: " 
	 << "date="   << faDate( (time_t *) &tgts.time ) << " "
	 << "time="   << faTime( (time_t *) &tgts.time ) << " "
	 << "ia="     << tgts.ia                         << "MHz "
	 << "bg="     << tgts.bg * 1000                  << "kHz "
	 << "pos[0]=" << tgts.pos[0]                     << "mm "
	 << "pos[1]=" << tgts.pos[1]                     << "mm "
	 << "pos[2]=" << tgts.pos[2]                     << "mm "
	 << "pos[3]=" << tgts.pos[3]                     << "mm "
	 << "pos[4]=" << tgts.pos[4]                     << "mm "
	 << "pos[5]=" << tgts.pos[5]                     << "mm "
	 << "pos[6]=" << tgts.pos[6]                     << "mm "
	 << "pos[7]=" << tgts.pos[7]                     << "mm"   << endl;

    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// interaction rate
//
float
faDTARia()
{
  ArtePointer<DTAR> dtar( 2 );
  if( dtar.nrow() ){
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); 
    
    return tgts.ia;
  }
  else {
    return -1;
  }
}

//
// interaction rate
//
extern "C" float
faDTARia_()
{
  return faDTARia();
}


//
// background rate
//
float
faDTARbg()
{
  ArtePointer<DTAR> dtar( 2 );
  if( dtar.nrow() ){
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); 
    
    return tgts.bg;
  }
  else {
    return -1;
  }
}

//
// background rate
//
extern "C" float
faDTARbg_()
{
  return faDTARbg();
}

//
// run number
//
int
faEVHDrun()
{
  ArtePointer<EVHD> p( 1 );
  if( p.nrow() ){
    return p->run;
  }
  else {
    return -1;
  }
}

//
// event number
//
int
faEVHDevent()
{
  ArtePointer<EVHD> p( 1 );
  if( p.nrow() ){
    return p->evt;
  }
  else {
    return -1;
  }
}

//
// date
//
int
faEVHDdate()
{
  ArtePointer<EVHD> p( 1 );
  if( p.nrow() ){
    return p->date ;
  }
  else {
    return -1;
  }
}

//
// time
//
int
faEVHDtime()
{
  ArtePointer<EVHD> p( 1 );
  if( p.nrow() ){
    return p->time;
  }
  else {
    return -1;
  }
}

// ****************************************************************************
//
// name:      faNameIDInit
//
// purpose:   initialize Name/ID array
//
// input:     void
// return:    int  lenght of array
//
// **************************************************************************** 

const int iNameIDLen = 16;
int iNameID[iNameIDLen];

int
faNameIDInit()
{
  memset( &iNameID, 0, iNameIDLen );
  iNameID[COMP_ID_DAQ /0x0100] = faName2Int( "DDAQ" );
  iNameID[COMP_ID_SVD /0x0100] = faName2Int( "DSVD" );
  iNameID[COMP_ID_ITR /0x0100] = faName2Int( "DITR" );
  iNameID[COMP_ID_HIPT/0x0100] = faName2Int( "DPTC" );
  iNameID[COMP_ID_OTR /0x0100] = faName2Int( "DOTR" );
  iNameID[COMP_ID_RICH/0x0100] = faName2Int( "DRIC" );
  iNameID[COMP_ID_TRD /0x0100] = faName2Int( "DTRD" );
  iNameID[COMP_ID_ECAL/0x0100] = faName2Int( "DCAL" );
  iNameID[COMP_ID_MUON/0x0100] = faName2Int( "DMUT" );
  iNameID[COMP_ID_MPIX/0x0100] = faName2Int( "DMUX" ); 
  iNameID[COMP_ID_MPAD/0x0100] = faName2Int( "DMUP" );
  iNameID[COMP_ID_FLT /0x0100] = faName2Int( "DFLT" );
  iNameID[COMP_ID_SLT /0x0100] = faName2Int( "DSLT" );
  iNameID[COMP_ID_FARM/0x0100] = faName2Int( "DTLT" );
  iNameID[COMP_ID_TARG/0x0100] = faName2Int( "DTAR" );
  iNameID[COMP_ID_FCS /0x0100] = faName2Int( "DFCS" );
#ifdef _DEB2_
  if( FADEB ){
    for( int i=0; i<iNameIDLen; i++ ){
      cout << "faNameIDInit:"
	   << "iNameID[" << i      << "]="
	   << iNameID[i]           << " "
	   << faName( iNameID[i] ) << endl;
    }
  }
#endif
  return iNameIDLen;
}

// ****************************************************************************
//
// name:      faName2ID
//
// purpose:   convert table name to ID
//
// input:     table name
// return:    ID
//
// **************************************************************************** 

int
faName2ID( int iName )
{
  // initialize array once
  static int b = 1;
  static int l = 0;
  if( b ){
    b = 0;
    l = faNameIDInit();
  }
#ifdef _DEB2_
  if( FADEB ){
    cout << "faID2Name: "
	 << "l=" << l << endl;
    cout << "faID2Name: "
	 << "iName=" << iName << endl;
  }
#endif
  for( int i=0; i<l; i++ ) {
    if( iNameID[i] == iName ) {
#ifdef _DEB2_
      if( FADEB ){
	cout << "faID2Name: "
	     << "ID=" << i * 0x0100 << endl; 
      }
#endif
      return ( i * 0x0100 );
    }
  }
  return ARTE_ERR;
}

// ****************************************************************************
//
// name:      faID2Name
//
// purpose:   convert ID to table name
//
// input:     ID
// return:    table name
//
// **************************************************************************** 

int
faID2Name( int ID )
{
  // initialize array once
  static int b = 1;
  static int l = 0;
  if( b ){
    b = 0;
    l = faNameIDInit();
  }

  int i = ID / 0x0100; // index
  if( i >= l ){
    cerr << "faID2Name: Fatal index i=" << i << endl;
    return ARTE_ERR;
  }
#ifdef _DEB2_
  if( FADEB ){
    cout << "faID2Name: "
	 << "ID=0x" << hex << ID << dec << " " 
	 << "iNameID[" << i << "]=" << iNameID[i] << endl;
  }
#endif
  return iNameID[i];
}

// ****************************************************************************
//
// name:      faName2Link
//
// purpose:   dispose link number of table iName
//
// input:     int iName
// return:    int iLink
//
// **************************************************************************** 

int
faName2Link( int iName ) 
{
  // identify table
  int iLink = 0;
  for( ArteTableIterator<MEMO> memo; memo.valid(); memo.next() ){
    iLink++;
#ifdef _DEB2_
    if( FADEB ){
      int iTable = indexTable( iLink );
      cout << "faName2Link: "
	   << "iLink="  << iLink               << " "
	   << "name="   << memo->name          << " " 
	   << "iTable=" << iTable              << " "
	   << "pTable=" << ptr2Table( iTable ) << " "
	   << "nWords=" << numData( iTable )   << endl;
    }
#endif
    if( !strcmp( faName( iName ), memo->name ) ){
      return iLink;
    }
  }
  return 0;
}

// *****************************************************************************
//
// name:      faDate
//
// purpose:   providing current date
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date yymmdd
//
// *****************************************************************************

int
faDate( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[7];
  strftime( s, 7, "%y%m%d", t );
  return atoi( s );
}

// *****************************************************************************
//
// name:      faTime
//
// purpose:   providing current time
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date hhmmss
//
// *****************************************************************************

int
faTime( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[7];
  strftime( s, 7, "%H%M%S", t );
  return atoi( s );
}

// *****************************************************************************
//
// name:      fatime_
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************

extern "C" float
fatime_()
{
  static int first = 1;
  static clock_t clock0;
  static time_t  time0;
  
  if( first ){
    first  = 0;
    clock0 = clock();
    time0  = time( 0 );
  }
  
  // double user_time = ( clock() - clock0 ) / double( CLOCKS_PER_SEC );
  double real_time = difftime( time( 0 ), time0 );
  
  // return float( user_time );
  return float( real_time );
}

// *****************************************************************************
//
// name:      faCopySlb
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************
int
faCopySlb( int *oldRecID, char *pData , int *newRecID, char *pTable, int nData )
{

#ifdef _DEB2_
  // print data
  cout << "faSparseNNNN: oldRecID=0x" << hex << *oldRecID << dec << endl;
  cout << "faSparseNNNN: nData="             << nData            << endl;
  cout << "faSparseNNNN: Data: ";
  for( int i=0; i<nData; i++ ) {
    cout << "0x" << hex << int( *( pData + i ) ) << dec << " ";
  }
  cout << endl;
#endif

  // pre-set
  int nWrite = 0;

  // no data
  if( !nData ){
    int error = RID_ERROR_SPARSE;
    ESP_put_int( &error, newRecID, 1 );
    return nWrite;
  }

  // rec ID
  ESP_put_int( oldRecID, newRecID, 1 );

  // data
  nWrite = nData;

  // just copy byte-wise
  for( int i=0; i<nData; i++ ) {
    *( pTable + i ) = *( pData + i );
  }

#ifdef _DEB2_
  if( FADEB ){
    int recID = 0;
    ESP_get_int( newRecID, recID, 1 );
    cout << "faSparseNNNN: newRecID=0x" 
	 << hex << *recID << dec << endl;
    cout << "faSparseNNNN: nWrite="
	 << nWrite << endl;
  }
#endif
  
  return nWrite;
}

//
// get record id from table
//
int
ARTE_get_recid_DSVD( int *recid )
{
  ArtePointer<DSVD> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DITR( int *recid )
{
  ArtePointer<DITR> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DPTC( int *recid )
{
  ArtePointer<DPTC> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DOTR( int *recid )
{
  ArtePointer<DOTR> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DRIC( int *recid )
{
  ArtePointer<DRIC> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DTRD( int *recid )
{
  ArtePointer<DTRD> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DCAL( int *recid )
{
  ArtePointer<DCAL> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUT( int *recid )
{
  ArtePointer<DMUT> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUX( int *recid )
{
  ArtePointer<DMUX> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUP( int *recid )
{
  ArtePointer<DMUP> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DTAR( int *recid )
{
  ArtePointer<DTAR> dnnn( 1 );
  ESP_get_int( (int *) dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DSVD( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DSVD> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DITR( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DITR> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DPTC( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DPTC> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DOTR( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DOTR> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DRIC( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DRIC> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DTRD( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DTRD> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DCAL( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DCAL> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUT( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DMUT> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUX( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DMUX> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;

  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUP( int max, ETB_fed *feds, int *n )
{
  // Arte pointer
  ArtePointer<DMUP> dnnn( 1 );
  
  // check length of table
  if( dnnn.nrow() <= 0 ){
    //cerr << "ARTE_get_feds_DNNN(): Table is empty!" << endl;
    return ETB_corrupted_length;
  }
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    //cerr << "ARTE_get_feds_DNNN(): NO FED array!" << endl;
    return ETB_out_of_memory;
  }
  
  // data words in table (first word is recid!)
  int total = ( dnnn.nrow() - 1 ) * sizeof( int );
  
  // recid
  int recid = 0;
  ESP_get_int( (int *) dnnn->bits, &recid, 1 );
  
  // SLB_SUBHDR
  unsigned int  bc   = 0;
  unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
  int j              = 0;
  
  // loop
  while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
    
    // check array size
    if( j >= max ){
      //cerr << "ARTE_get_feds_DNNN(): max reached j=" << j << endl;
      return ETB_out_of_memory;
    }

    // update FED count
    *n = j + 1;
    
    // ETB_fed header
    SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
    ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
    feds[j].slb  = 0; // pointer to slb record (not existent here)
    feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
    
    // increase cunters
    j++; // fed counter
    bc += hdr->length; // byte counter
  }
  return ETB_ok;
}
