
#ifndef _FA_HH_
#define _FA_HH_

// *****************************************************************************
//
// source:    fa.hh
//
// type:      header file
//
// created:   16 Apr 96
// updated:   11 Mar 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   multiple file access (AG: 27-Nov-98)
//            prototype for faDaqLog added (AG: 15-Dec-98)
//            online include path (AG: 11 Jan 99)
//            introduced return code for faDaqLog() (AG: 15 Jan 99) 
//            target raw data include tgts.h (AG: 10 Feb 99)
//            introduce  function faCycle() (AG: 12 Feb 99)
//            include utility protoypes here (AG: 23 Feb 99)
//            polishing (AG: 11 Mar 99)
//
// *****************************************************************************

// defines
#define _OUT_
#define _ERR_
#define _DEB_

// standard includes
#include <iostream>
#include <strstream>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <math.h>

// online includes
#include "rpm.h"
#include "log.h"
#include "grr.h"
#include "emg.h"
#include "etb.h"
#include "esp.h"
#include "event.h"
#include "tgts_raw.h"

// Arte includes
#include "arte/Arte.hh"
#include "arte/ArteFlag.h"
#include "auto/EVHD.hh"
#include "auto/DIG.hh"
#include "auto/MEMO.hh" 

// farm includes
#include "farm.hh"

// constants
const int ERR  = -1; // error occurred
const int GOOD =  0; // okay
const int OK   =  0; // okay
const int EOD  =  4; // end-of-data (no more data, next bc=0)
const int END  =  5; // end-of-file (end reached)

const int ARTE_ERR  = -1; // error occurred
const int ARTE_GOOD =  0; // okay
const int ARTE_OK   =  0; // okay
const int ARTE_EOD  =  4; // end-of-data (no more data, next bc=0)
const int ARTE_END  =  5; // end-of-file (end reached)

// C prototpyes
extern "C" int  fainit_();
extern "C" int  fastop_();
extern "C" int  faread_();
extern "C" float fatime_();

// C++ prototypes
void faCatchSIGINT ( int );
void faCatchSIGQUIT( int );
void faCatchSIGTERM( int );

int faLine();

FILE *faOpenFile( char *name );
int  faCloseFile( char *name, FILE *in );
int  faReadFile ();

int faInitEmg();
int faReadEmg();
int faStopEmg();

int faInitGrr();
int faStopGrr();

void faCatchSIGINT ( int sigNum );
void faCatchSIGQUIT( int sigNum );
void faCatchSIGTERM( int sigNum );

void faCycle();

int faDate( time_t * );
int faTime( time_t * );

void faPrintEvtCount( int, int );

int faName2ID( int );
int faID2Name( int );
int faNameIDInit();
int faName2Link( int iName );

inline char *faName( int );
inline int  faName2Int( char * );

int faCopySlb( int *oldid, char *src, int *newid, char *dest, int size );

// inline functions
inline int
faName2Int( char *c )
{
  return *( (int *) c );
}

inline char *
faName( int i )
{
  static int j;
  j = i;
  return (char *) &j;
}

inline int indexTable( int iLink )
{
  return *( (int *) artelink_.jlink + iLink - 1 );
}

inline int intName( int indexTable )
{
  return *( gcbank_.iq + indexTable - 5 );
}

inline int numData( int indexTable )
{
  return *( gcbank_.iq + indexTable - 2 );
}

inline int numCols( int indexTable )
{
  return *( gcbank_.iq + indexTable + 0 );
}

inline int numRows( int indexTable )
{
  return *( gcbank_.iq + indexTable + 1 );
}

inline int * ptr2Table( int indexTable )
{
  return gcbank_.iq + indexTable;
}

#endif
