
      SUBROUTINE CAFMFM

C************************************************************
C*-- Author :    Olya Igonkina   12/12/95                   *
C* CAFMFM (vsn 1.00)                                        *
C* for    SHOWER parametrization and                        *
C*        calculation of the Energy deposition              *
C*        in the MIDDLE ECAL tower                          *
C*                                                          *
C* IN:                                                      *
C* OUT:   EMOD - energy deposition in the tower             *
C*                                                          *
C* Called by:    CAFMSU                                     *
C************************************************************
      IMPLICIT none

      REAL SCSIZ, RL, RM, RAD_MOL, LEN
      INTEGER NUMZ
      DIMENSION RAD_MOL(3)
C -- 0.5 * transvwerse size of SC-plate
      PARAMETER( SCSIZ = 2.7875)
C -- radiation length
      PARAMETER( RL =1.675)
C - length of the calorimeter NUMZ * DZ
       parameter( LEN = 34.04)
C - number of Sc-plate (z)
      PARAMETER( NUMZ = 37)

      REAL PARE
      DIMENSION PARE(7)

      REAL PARZ
      DIMENSION PARZ(7)
      INTEGER  I
      REAL Rzero, Zzer
      REAL  X1, X2, Y1,Y2, XC, YC, DIST, RzeroOLD
      REAL AX, BX, AY, BY, ZR, VOL, SUMOLD, ET
      REAL VOLSUM, RMAX, RMIN, XHELP, YHELP, DZ

#include "gean/gctrak.fi"
#include "gean/gckine.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGCALO.fi"

#include "gean/cafmcp.fi"
#include "gean/cafmco.fi"
#include "gean/cafmcc.fi"

C -- Moliere radius ( O/M/I)
      DATA RAD_MOL/ 4.1, 4.1, 1.42/

      DO I=1,5
        LL(I,1) = 0.
        LL(I,2) = 0.
      ENDDO

      IF( ABS(VECT(6)).LT.0.1E-8 )RETURN
      PAR_MOD(1) = SCSIZ
      RM = RAD_MOL(2)

C..  calculation of Energy DEPosition for the tower

      EMOD = 0

      PARE(1) = 1045 / (VECT(7)**0.419) - 55.87
      PARE(2) = 59.316 * (VECT(7)**0.00762 ) - 57.374
      PARE(3) = 1.303 / (VECT(7)**0.239) + 5.6266
      PARE(4) = 5.005 * (VECT(7)**0.06) - 3.638
      PARE(5) = 0.0854 * exp(-0.0447*VECT(7)) + 0.02134
      PARE(6) = 1.0952 + 0.5589 * LOG( abs(VECT(7) -1.6462 ))
      PARE(7) = 0.064 / (VECT(7)**0.381 ) + 0.1151

      X1 = PAR_MOD(2) - PAR_MOD(1)
      Y1 = PAR_MOD(3) - PAR_MOD(1)
      X2 = PAR_MOD(2) + PAR_MOD(1)
      Y2 = PAR_MOD(3) + PAR_MOD(1)

      XC = VECT(1) + VECT(4)*(ZZERO+ZACAL(1)-VECT(3))/VECT(6)
      YC = VECT(2) + VECT(5)*(ZZERO+ZACAL(1)-VECT(3))/VECT(6)
      IF( MFL.GT.2.5 )THEN
        XC = XC * RAD_MOL(2) / RAD_MOL(3)
        YC = YC * RAD_MOL(2) / RAD_MOL(3)
      ENDIF

      IF( ABS(VECT(6)).LT.(1.-0.1E-8))THEN
        AX =  vect(4) / VECT(6) / SQRT( 1-VECT(6)*VECT(6) )
        BX = - VECT(5) / SQRT( 1-VECT(6)*VECT(6) )
        AY = vect(5) / VECT(6) / SQRT( 1-VECT(6)*VECT(6) )
        BY = vect(4) / SQRT( 1-VECT(6)*VECT(6) )
      ELSE
        AX = 1.
        BX = 0.
        AY = 0.
        BY = 1.
      ENDIF
C LINES :
      AL(1) = AY
      BL(1) = BY
      CL(1) = Y1 - YC

      AL(2) = AX
      BL(2) = BX
      CL(2) = X1 - XC

      AL(3) = AY
      BL(3) = BY
      CL(3) = Y2 - YC

      AL(4) = AX
      BL(4) = BX
      CL(4) = X2 - XC

      AL(5) = AL(1)
      BL(5) = BL(1)
      CL(5) = CL(1)


C lines interseption

      DIST = AX*BY - AY*BX

C......................................

      VOLSUM = 0.
      DZ = LEN / VECT(6) / NUMZ
      IF( ITRTYP.LT.1.5 )THEN
         ZR = RL + DZ
       IF( VECT(6).GT.0.1E-8.AND.VECT(6).LT.(1-0.1E-8) )THEN
         XHELP = DZ*VECT(4)/VECT(6)
         YHELP = DZ*VECT(5)/VECT(6)
         CL(1) = CL(1) - YHELP
         CL(2) = CL(2) - XHELP
         CL(3) = CL(3) - YHELP
         CL(4) = CL(4) - XHELP
         CL(5) = CL(5) - YHELP
       ENDIF
      ELSE
         ZR = DZ
      ENDIF


      DO WHILE ( ZR.LT.(LEN/VECT(6)) )

        Zzer = ZR / 0.92

C... parametrization :
         PARZ(1) = PARE(1) * ( Zzer** PARE(2) ) / ( 0.3E+08 +
     +   ( Zzer ** 5.46 ) )
         PARZ(2) = 0.35E-06 * ( Zzer ** PARE(3) ) + 0.05
         PARZ(3) = 1
         PARZ(4) = 1+2.79 - PARE(4) * EXP ( -0.127 * Zzer)
         PARZ(5) = PARE(5) * ( Zzer** PARE(6) ) * EXP ( -0.364 *
     +   Zzer )
         PARZ(6) = 21. * EXP( - PARE(7) * Zzer ) * ( 1 - 0.048 *
     +   Zzer + 0.0033 * Zzer * Zzer )

       IF( VECT(6).GT.0.1E-8.AND.VECT(6).LT.(1-0.1E-8) )THEN
         XHELP = DZ*VECT(4)/VECT(6)
         YHELP = DZ*VECT(5)/VECT(6)
         CL(1) = CL(1) - YHELP
         CL(2) = CL(2) - XHELP
         CL(3) = CL(3) - YHELP
         CL(4) = CL(4) - XHELP
         CL(5) = CL(5) - YHELP
       ENDIF
         IF( ABS(DIST).GT.0.1E-8 )THEN
           DO I = 1, 4
             DIST = -DIST
             LL(I,1)  = ( BL(I+1)*CL(I)-BL(I)*CL(I+1)) / DIST
             LL(I,2)  = ( CL(I+1)*AL(I)-AL(I+1)*CL(I)) / DIST
           ENDDO
         ELSE
             RETURN
         ENDIF
          LL(5,1) = LL(1,1)
          LL(5,2) = LL(1,2)

      RMAX= SQRT(LL(1,1)**2 + LL(1,2)**2 )
      R1= SQRT(LL(2,1)**2 + LL(2,2)**2 )
       IF( RMAX.LT.R1 ) RMAX = R1
      R1= SQRT(LL(3,1)**2 + LL(3,2)**2 )
       IF( RMAX.LT.R1 ) RMAX = R1
      R1= SQRT(LL(4,1)**2 + LL(4,2)**2 )
       IF( RMAX.LT.R1 ) RMAX = R1

      IF( (CL(1)*CL(3)).LT.0.AND.(CL(2)*CL(4)).LT.0 )THEN
         RMIN = 0.
      ELSE
          XHELP = VECT(6)/SQRT(1-VECT(4)*VECT(4))
          YHELP = VECT(6)/SQRT(1-VECT(5)*VECT(5))
         RMIN = ABS(CL(1)) * YHELP
         R1 = ABS(CL(2)) * XHELP
          IF( RMIN.GT.R1 ) RMIN = R1
         R1 = ABS(CL(3)) * YHELP
          IF( RMIN.GT.R1 ) RMIN = R1
         R1 =ABS( CL(4)) * XHELP
          IF( RMIN.GT.R1 ) RMIN = R1
      ENDIF
       IF( VECT(6).LT.(1.-0.1E-8) )THEN
        R2 = ZR * VECT(6) / SQRT( 1 - VECT(6)*VECT(6))
        R1 = - ( LEN/VECT(6) - ZR ) * VECT(6)
     +                       / SQRT( 1 - VECT(6)*VECT(6))
       ELSE
        R2 = 1000.
        R1 = -1000.
       ENDIF
       Rzero = 0.05
 100    CONTINUE
        IF( Rzero.LT.RMIN )THEN
          Rzero = Rzero + 0.05 + 0.02*Rzero*Rzero
          GOTO 100
        ENDIF

        SUMOLD = 0
        RZEROOLD = 0.

 200    CONTINUE

          RC = RZERO + (Rzero - RzeroOLD ) / 2.
        IF( (PARZ(6)*RZERO) .LT.25. )THEN
          ET =  PARZ(1) / (PARZ(2) + (Rzero** PARZ(4)))
     +  + PARZ(5) * EXP ( -PARZ(6) *Rzero ) / SQRT( Rzero)
        ELSE
          ET =  PARZ(1) / (PARZ(2) + (Rzero** PARZ(4)))
        ENDIF

          CALL CAFMLO
          VOL = (SUM - SUMOLD) * DZ
          EMOD = EMOD + ET * VOL
          SUMOLD = SUM
          RzeroOLD=Rzero
          Rzero = Rzero + 0.05 + 0.02*Rzero*Rzero
        IF( RC.LE.RMAX )GOTO 200

        ZR = ZR + DZ
      ENDDO

      EMOD = EMOD / 3.1415927 /0.92

      RETURN
      END
