/*************************************************************************
*
* CR_ARTE
* written by Siegmund Nowak
*
* This routines fills the arte NEDE, NWAL and NCAL banks from GDL and GEANT
*
*       called from detgeo
*
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcvolu.h"
#include "gean/zgeom.h"
#include "gean/ZGESL.h"

extern "C" {
    void crlvbk_(int*,int*,int*,int*,char*,int[]);
    void glvolu_(int*,char*,int[],int*);
    void crvtrm_(int*,char*,int[],float[],float[]);    
    void cr_arte_user_(const int,const int,const char*,const int[],const int);}


void cr_arte(int nr_sl) {
/***********************/
    struct NEDE nede;
    struct NWAL nwal;
    struct NCAL ncal;
    zgvol ZGVOL;
    zgpos ZGPOS;
    zgdet ZGDET;
    int ivos[15],in[15],nin[15],lcopy[15],ltype[15],nlevsl,iuser,indt,nr_vol;
    int nlev =0; int module =0;
    float pos[11], rotm[9];
    char  lvols[60];
    union { int ji; float jx;}; 
    union  { int ii; char volname[4];};

//  check whether the superlayer volume already exists
    strncpy(volname,ZGVOL.Get_volname(nr_sl),4);
    int Gvolnum = ZGVOL.Check_Gvol(volname);
    if (Gvolnum >0){
       int found =0; 
       for(unsigned int isl=1;isl<=ArtePointer<NESL>::nrow();isl++) {
           ArtePointer<NESL> ge(isl);
           if(!strncmp(ge->name,volname,4)) { found = isl; break;}}
       if(!found)   { arteMsg << "cr_arte - nonvalid superlayer volume";
                      arteMsg.error();} 
       crlvbk_(&ii,&found,&indt,&nlevsl,lvols,lcopy);
       nlevsl = nlevsl -1;
       nlev = nlevsl;
       ivos[nlev] = Gvolnum;
       in[nlev] = 0; }
    else {
       arteMsg << "cr_arte -nonvalid superlayer volume";   arteMsg.error();}

L10:       //modules and components
//  define the number of subcomponents nin(nlev) for the new level nlev 
    int jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    ji =*(gcbank_.iq + jvos + 2);
    nin[nlev] = int(jx);     

L15:
//  go to the next (first) component of level nlev          
    in[nlev]++;
//  if all components of this level are handled go to one level above or return 
    if(in[nlev] > nin[nlev]) { 
       nlev--;
       if (nlev >= nlevsl) goto L10;
       else
         gcvolu_.nlevel = 0;
         return;           }
//  get the gean banks of the component ivos(nlev) and the name of next subcomp
//  volname or as integer ii
    jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    int jin = *(gcbank_.lq + jvos -1 -in[nlev]);
    ji = *(gcbank_.iq + jin +1);
    int ivot = int(jx);
    ii = *(gcbank_.iq+gclink_.jvolum -1 + ivot);      
    ji = *(gcbank_.iq + jin +2);
    iuser = int(jx);
    int type = iuser>>16;
    if((nlev-nlevsl) == 0) module = iuser&65535; 
    nr_vol = ZGVOL.Get_volnr(volname,type);
    if(nr_vol < 0) goto L15;    

//  increase the level and put the new name and user number in lvols/lcopy
    nlev++;
    strncpy(&lvols[4*nlev],volname,4);
    ltype[nlev] = type;
    lcopy[nlev] = iuser;
//  define the number of subcomponents nin(nlev)
    int jvot = *(gcbank_.lq + gclink_.jvolum -1 -ivot);
    ji = *(gcbank_.iq + jvot +2);
    nin[nlev] = int(jx);

    if(nin[nlev] != 0) {                //    modules only 
       in[nlev] = 0;
       ivos[nlev] = ivot;
       module = iuser&65535;
       goto L10;            }

//  components only
    ji = *(gcbank_.iq + jvot+3); 
    int numed = int(jx);
    int jmd = *(gcbank_.lq + gclink_.jtmed -1 - numed);
    ji = *(gcbank_.iq + jmd + 5);
    int nmat = int(jx);
    int jmt = *(gcbank_.lq + gclink_.jmate -1 - nmat);
    ji = *(gcbank_.iq + jmt +8);
    float xms = jx;

//  prepare and transform positions and rotation matrces
    int nl = nlev+1;
    crvtrm_(&nl,lvols,lcopy,pos,rotm);
    xms = (pos[5]-pos[4])/xms; 

    ArtePointer<NESL> cursl(zgmoth_.CURRSL);
    ArtePointer<GMAT> gm(nmat);
//  fill NEDE table
    if(ZGVOL.Get_sens_flag(nr_vol) > 2) {
        if(cursl->cmp != LGESL_HOD3) {
        int nnede = ArtePointer<NEDE>::nrow() + 1;
        memset(&nede,0,sizeof(nede));  
        if(!cursl->fpl) cursl->fpl =  nnede;
        cursl->npl = cursl->npl +1 ;
        nede.cmp = cursl->cmp;
        ji = *(gcbank_.iq + jvot+1);       
        nede.shp = int(jx); 
        nede.gcop = iuser&65535;
        int in = module<<16;
        nede.gcop = nede.gcop + in;
        nede.lay = ZGPOS.Get_layer(nr_sl,module);
        nede.sec = ZGPOS.Get_sector(nr_sl,module);

        for (int m=0;m<4;m++) nede.gvol[m] = volname[m];
        float *ptpos = &nede.xmin; 
        for(int k=0; k<6; k++)  *(ptpos+k) = pos[k]; 
        ptpos = &nede.xmad;
        for(int k=0; k<3; k++)  *(ptpos+k) = pos[k+7]; 
        ptpos = &nede.rot[0];
        for(int k=0; k<9; k++)  *(ptpos+k) = rotm[k];
        int n_wire[4]; 
        float sens_dx, sens_dy, sdx, sdy,  pitch[4];
        float x_wire[4], y1_wire[4], y2_wire[4], z_wire[4], wire_or[4] ;
        nede.nsl = ZGDET.Get_sides(nr_vol); 
        ZGDET.Get_sens_dim(nr_vol,sens_dx,sens_dy);
        ZGDET.Get_sens_offset(nr_vol,sdx,sdy);
        ZGDET.Get_pitch(nr_vol,pitch,nede.nsl);
        ZGDET.Get_z_wire(nr_vol,z_wire,nede.nsl);
        ZGDET.Get_x_wire(nr_vol,x_wire,nede.nsl);
        ZGDET.Get_n_wire(nr_vol,n_wire,nede.nsl);
        ZGDET.Get_y1_wire(nr_vol,y1_wire,nede.nsl);
        ZGDET.Get_y2_wire(nr_vol,y2_wire,nede.nsl);
        ZGDET.Get_wire_orientation(nr_vol,wire_or,nede.nsl); 

        float sxmn=-1.e+9; float sxmx=1.e+9; float symn=-1.e+9; 
                                             float symx= 1.e+9;
        if(sens_dx>0.) { sxmn = .5*(nede.xmin+nede.xmax -sens_dx) + sdx; 
                        sxmx = .5*(nede.xmin+nede.xmax +sens_dx) + sdx;}
        if(sens_dy>0.) { symn = .5*(nede.ymin+nede.ymax -sens_dy) + sdy; 
                        symx = .5*(nede.ymin+nede.ymax +sens_dy) + sdy;}
        for (int m=0;m<nede.nsl;m++) {
         if(!n_wire[m]) continue; 
         float cosx =cos(AR_DEGRAD*wire_or[m]);
         float sinx =sin(AR_DEGRAD*wire_or[m]);
         if(!m) {nede.cos  = cosx;     nede.sin  = sinx;} 
         float x_center = .5*((pos[0]+pos[1])*cosx - (pos[2]+pos[3])*sinx);
         float y_center = .5*((pos[0]+pos[1])*sinx + (pos[2]+pos[3])*cosx);   
         nede.xfw[m] = x_wire[m] + x_center;
         if(sens_dx <= 0. ||  sxmn <= nede.xmin) 
                            sxmn = nede.xfw[m] - .5*pitch[m];
         if(sens_dx <= 0. ||  sxmx >= nede.xmax) 
                            sxmx = nede.xfw[m] +(n_wire[m]-0.5)*pitch[m]; 
         if(fabs(y2_wire[m]-y1_wire[m]) > 1.e-5) { 
	    nede.y1w[m] = y1_wire[m] + y_center;
            nede.y2w[m] = y2_wire[m] + y_center;  }
         else { 
            if(sens_dy <=0.) {nede.y1w[m]=nede.ymin; nede.y2w[m] = nede.ymax;}
            else             {nede.y1w[m]=symn;      nede.y2w[m] = symx;}}
         if(sens_dy <= 0.) symn = nede.y1w[m];
         if(sens_dy <= 0.) symx = nede.y2w[m];
         nede.zfw[m] = 0.5*(pos[4]+pos[5]) + z_wire[m]; 
	 nede.nwi[m]=  n_wire[m];}

        nede.sxmn = sxmn; nede.sxmx = sxmx; nede.symn = symn; nede.symx = symx;

        nede.pit = pitch[0];
        nede.acc = ZGDET.Get_accuracy(nr_vol);
        nede.eff = ZGDET.Get_effectivity(nr_vol); 
        nede.dod = ZGDET.Get_double_hit_res(nr_vol);
        nede.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);       
        ArtePointer<NEDE>::append(nede);      

        createRelation(cursl,ArtePointer<NEDE>(nnede),&NESL::nede);
        createRelation(gm, ArtePointer<NEDE>(nnede),&GMAT::nede);

//     call possibility to make special arrangements for a special detector
	cr_arte_user_(nnede,0,lvols,lcopy,nlev); 
      }
      else {
        int nncal = ArtePointer<NCAL>::nrow() + 1;
        memset(&ncal,0,sizeof(ncal));  
        if(!cursl->fpl) cursl->fpl =  nncal;
        cursl->npl = cursl->npl +1 ;
        ncal.cmp = cursl->cmp;
        ncal.ncl = 1; 
        for (int m=0;m<4;m++) ncal.gvol[m] = volname[m];
        float *ptpos = &ncal.xmin; 
        for(int k=0; k<6; k++)  *(ptpos+k) = pos[k]; 
        ptpos = &ncal.xmad;
        for(int k=0; k<3; k++)  *(ptpos+k) = pos[k+7]; 
//        ptpos = &ncal.rot[0];
//        for(int k=0; k<9; k++)  *(ptpos+k) = rotm[k];
        ncal.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);       
        ArtePointer<NCAL>::append(ncal);      
        createRelation(cursl,ArtePointer<NCAL>(nncal),&NESL::ncal);
        createRelation(gm,ArtePointer<NCAL>(nncal),&GMAT::ncal);

//      call possibility to make special arrangements for a special detector
        cr_arte_user_(nncal,0,lvols,lcopy,nlev); }
    }
      
//  fill wall table   
    else if(ZGVOL.Get_sens_flag(nr_vol) <=2) {
      if(xms <= 1.e-10) {nlev = nlev-1;   goto L15;} 

      int nnwal = ArtePointer<NWAL>::nrow() + 1;
      memset(&nwal,0,sizeof(nwal));  
      if(!cursl->fwl) cursl->fwl =  nnwal;
      cursl->nwl =cursl->nwl +1 ; 
      ji = *(gcbank_.iq + jvot+1);       
      nwal.shp = int(jx) ;
      for (int m=0;m<4;m++) nwal.gvol[m] = volname[m];
      float *ptpos = &nwal.xmin; 
      for(int k=0; k<6; k++)  *(ptpos+k) = pos[k]; 
      ptpos = &nwal.xmad;
      for(int k=0; k<3; k++)  *(ptpos+k) = pos[k]; 
      ptpos = &nwal.rot[0];
      for(int k=0; k<9; k++)  *(ptpos+k) = rotm[k];       
      nwal.nsl =1; 
      nwal.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);       
      ArtePointer<NWAL>::append(nwal);      
      createRelation(cursl,ArtePointer<NWAL>(nnwal),&NESL::nwal);
      createRelation(gm,ArtePointer<NWAL>(nnwal),&GMAT::nwal);      

//    possibility to change something in the automatically produced tables
      cr_arte_user_(0,nnwal,lvols,lcopy,nlev); 
    }
    nlev = nlev-1;
    goto L15; 
       }
      
/* cr_arte_ */
