/***********************************************************************
* GEOSTAT
* written by Siegmund Nowak
*
* This routines is the main routine for generating the detector geometry
*
*       called from HGINIT
***********************************************************************/
#include "stdio.h"
#include "string.h"
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"
#include "gean/zgvers.hh"


void geostat() {
   char vtext[90];  char vline[90]; char vt[5];
   memset(vline,'-',89);    vline[89]= '\0';
   printf("%s \n",vline);
   sprintf(vtext,"GEOMETRY VERSION %.4f \n",arteflag_.geomvers);
   printf("%s",vtext);
   strcpy(vtext,"Geometry version numbers for subsystems");
   printf("%s\n",vtext);
   strcpy(vtext,
   "SUBDETECTOR   CMP   VERS-Nr   INPUT FILE                         VERSION");
   printf("%s\n",vtext);

   zgvers ZGVERS;
   for (int i=0;  i<32; i++){       
       geovers *vers = ZGVERS.verspointer(i);          
       if(vers->vers_nr >=0) {
         if(zgdetv_.DETV[vers->vers_cmp] == 'L') strncpy(vt,"LONG ",5);
         else if(zgdetv_.DETV[vers->vers_cmp] == 'S')strncpy(vt,"SHORT",5);
         else if(zgdetv_.DETV[vers->vers_cmp] == 'U')strncpy(vt,"DUMMY",5);
         sprintf(vtext,"%.12s   %2d   %7.4f   %-32.32s   %.5s \n", 
                vers->name,vers->vers_cmp,vers->vers_nr,vers->datafile,vt);
         printf("%s",vtext);
       }
   }
   printf("%s\n",vline);

   strcpy(vtext,"Superlayer information");
   printf("%s\n",vtext);

   strcpy(vtext,   
   "SL   cmp mfl    xmin    xmax    ymin    ymax    zmin    zmax    fpl  lpl   fwl   lwl");
   printf("%s\n",vtext);
   printf("%s\n",vline);

   ArteTableIterator<NESL> gs;
   for(; gs.valid(); gs.next()){
      sprintf(vtext,
      "%.4s %3hd %3hd  %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f  %4d %4d %5d %5d \n",
       gs->name,gs->cmp,gs->mfl,gs->x1,gs->x2,gs->y1,gs->y2,gs->z1,gs->z2,
       gs->fpl,gs->fpl+gs->npl-1,gs->fwl,gs->fwl+gs->nwl-1); 
       printf("%s",vtext); 
       if(gs->cmp == LGESL_MOTH || gs->cmp  ==  LGESL_MAG ) {
          sprintf(vtext,"              %7.2f %7.2f %7.2f %7.2f\n",
	       gs->x1-gs->dx, gs->x2+gs->dx, gs->y1-gs->dy, gs->y2+gs->dy);
         printf("%s",vtext);
       }
   }
   printf("%s\n ",vline);
   strcpy(vtext, 
   "All detectors with zero dimensions are not implemented as volumes");
   printf("%s\n",vtext);
   strcpy(vtext, 
   "MOTH and MAGN are trapezoids, first/last rows give the numbers at the front/back side"); 
   printf("%s\n",vtext);
   strcpy(vtext,
   "RICH is implemented as tube with an inner/outer radius given in xmin/xmax");
   printf("%s\n",vtext);
   printf("%s\n",vline);

    
}
