
      SUBROUTINE POTGEO(DPART)

************************************************************************
*
* POTGEO
* written by Siegmund Nowak
*
* Description:  defines roman pot geometry
*
* called from GEOMAIN
*
* External references: CRGWAL,GSVOLU,GSPOS(P),GSATT,GRADLN,MMBOOK
*
* Changes since last version: no more detector region (mother vol)
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*   dpart     in          'RING' definition of ring and conus
*                         'SEND' endplate
************************************************************************

      IMPLICIT NONE

      INTEGER IVOL,MEDNUM,MATNUM,IFIRST,lenocc,len
      REAL PAR(6),ZPOS,ZP,ZEOR,XMS
      CHARACTER*(*) DPART

#include "gean/gcflag.fi"
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGRPOT.fi"
#include "auto/GEO.fi"
      DATA IFIRST /0/
#include "arte/Arte.ff"
#include "auto/GEO.ff"

C--   define overall volumen
      IF(IFIRST.EQ.0) THEN
         CALL POTDAT
         IFIRST = 1
      ENDIF

      CALL CLTOU(DPART(1:LENOCC(DPART)))
      IF(DPART.EQ.'RING') THEN
         ZP = ZZERO - 20.

C--   OUTER TUBE FOR THE SILICON REGION (ROMAN POT)

C--   Cylindrical part

         PAR(1) =  RIPOT(1)
         PAR(2) =  PAR(1) + WDPOT
         PAR(3) = 0.5 * DLPOT(1)
         CALL GSVOLU('BPI1','TUBE',MEDNUM(1,'ALUMINIUM'),PAR,0,IVOL)
         CALL GSATT('BPI1','COLO',1)
         CALL CRPOS('BPI1','MOTH',1,0 ,0.,0.,ZP + PAR(3),0,'ONLY',PAR)

         ZP = ZP + 2.* PAR(3)

C--   Conical part

         PAR(1) = 0.5 * DLPOT(2)
         PAR(2) =  RIPOT(1)
         PAR(3) =  PAR(2) + WDPOT
         PAR(4) =  RIPOT(2)
         PAR(5) =  PAR(4) + WDPOT
         CALL GSVOLU('BPI2','CONE',MEDNUM(1,'ALUMINIUM'),PAR,0,IVOL)
         CALL GSATT('BPI2','COLO',1)
         CALL CRPOS ('BPI2','MOTH',2,0, 0.,0.,ZP + PAR(1),0,'ONLY',PAR)
         GO TO 90

      ELSEIF(DPART.EQ.'SEND') THEN

C---  end wall of roman pot (spheric shell) completely in magnet region

         PAR(1) =  RRPES
         PAR(2) =  RRPES + DLPES
         PAR(3) =  180.-ASIN(ROPES/RRPES)*RADDEG
         PAR(4) =  180.-ASIN(RIPES/RRPES)*RADDEG
         PAR(5) =  0.
         PAR(6) =  360.
         ZPOS = ZAPES - ZMAGM + RRPES + DLPES
         CALL GSVOLU('RPSM','SPHE',MEDNUM(2,'ALUMINIUM'),PAR,6,IVOL)
         CALL GSATT('RPSM','SEEN',0)
         CALL GSPOS('RPSM',1,'MAGN',0.,0.,ZPOS,0,'ONLY')

c        prepare some variables for later use
         ZEOR = ZPOS + RRPES*COS(DEGRAD*PAR(3))


c     since a sphere is momentarily not available in CRGWAL the wall
c     is assumed to be a tube and is put in by hand

         CALL MMBOOK(JNWAL,1)
         IQ(IQNWAL_SHP_L()) = 5
         Q(QNWAL_XMIN_L()) = RIPES
         Q(QNWAL_XMAX_L()) = ROPES
         Q(QNWAL_YMIN_L()) = 0.
         Q(QNWAL_YMAX_L()) = 0.
         Q(QNWAL_ZMIN_L()) = ZAPES
         Q(QNWAL_ZMAX_L()) = ZAPES + DLPES
         Q(QNWAL_XMAD_L())  = 0.
         Q(QNWAL_YMAD_L())  = 0.

         CALL VZERO(Q(QNWAL_ROT_L(1)),9)
         Q(QNWAL_ROT_L(1))  = 1.
         Q(QNWAL_ROT_L(5))  = 1.
         Q(QNWAL_ROT_L(9))  = 1.

         CALL GRADLN(IVOL,XMS)
         XMS = DLPES/XMS
         Q(QNWAL_MS_L())= XMS*(.0136*(1.+.038*ALOG(XMS)))**2
c         IQ(JNWAL+2+(NNWAL()-1)*NCNWAL()+JNWAL_NESL) = CURRSL
         CALL MMBREL(JNSNW,CURRSL,NNWAL())
         CALL MMBREL(JGMNW,MATNUM('ALUMINIUM'),NNWAL())

C---  first end cone  tube of roman pot

         PAR(1) =  0.5*DLPEC(1)
         PAR(2) =  RISPEC(1)
         PAR(3) =  ROSPEC(1)
         PAR(4) =  RIEPEC(1)
         PAR(5) =  ROEPEC(1)
         ZPOS = ZAPES - ZMAGM - PAR(1)
         CALL GSVOLU('RPEC','CONE',MEDNUM(1,'ALUMINIUM'),PAR,0,IVOL)
         CALL CRPOS('RPEC','MAGN',3,0,0.,0.,ZPOS,0,'ONLY',PAR)
         ZPOS = ZPOS - PAR(1)

C---  second end cone  tube of roman pot

         PAR(1) =  DLPEC(2)
         PAR(2) =  RISPEC(2)
         PAR(3) =  ROSPEC(2)
         PAR(4) =  RIEPEC(2)
         PAR(5) =  ROEPEC(2)
         ZPOS = ZPOS - PAR(1)
         CALL CRPOS('RPEC','MAGN',3,0,0.,0.,ZPOS,0,'ONLY',PAR)


C---  outside end ring of roman pot

         PAR(1) =  RIPET
         PAR(2) =  RIPET + WDPET
         PAR(3) =  DLPET
         ZPOS = ZEOR - PAR(3)
         CALL GSVOLU('RPET','TUBE',MEDNUM(2,'ALUMINIUM'),PAR,0,IVOL)
         CALL GSATT('RPET','COLO',1)
         CALL CRPOS('RPET','MAGN',3,0,0.,0.,ZPOS,0,'ONLY',PAR)

      ENDIF

 90   RETURN
      END

************************   END   POTGEO    *****************************
