
/***********************************************************************
*
* int read_dat_
* written by Siegmund Nowak 01/98
*
* This routines reads in standard data cards from file with filepointer 
*
* called from [subdetector]geo 
*
* Arguments  in: filepointer of the file to be read
* Return value:  datagroup 
*                          0 - blank or comment card, no action required
*                          1 - data read in were media
*                          2 - data read in were rotation matrices
*                          3 - data read in were elemetary volumes
*                          4 - data read in were modules
*                          5 - data read in were superlayers
*               
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "gean/hgdat.h"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgeom.h"

// ------------------------------------------------------------------------  

int read_dat(FILE *filepointer, int &nr) {
/******************************/
  int nitem, tagid=0, numcomp= 0, ni = 0 ;
  static int volid=0, datagroup;
  char  buffer[256], tag[80], taga[16]; 
  char *card;
  fpos_t fpos;  
  zgrot ZGROT;
  zgmed ZGMED;
  zgvol ZGVOL;
  zgpos ZGPOS;
  zgdet ZGDET;
  zgcal ZGCAL;

  for(;;) {
     fgetpos(filepointer, &fpos);
     memset(buffer,' ',255);
     buffer[255] = '\n'; 
     if(fgets(buffer,256,filepointer) == NULL) {
//       tagid = 0;
        if(feof(filepointer)) {
          fclose(filepointer);
          tagid = -99; 
          if(volid != 0) goto L10;
          else  return tagid;
        }
        else tagid = 0;
        return tagid; }
     card=&buffer[0];
     nitem=sscanf(card,"%s",tag);
     if (nitem==-1) continue;   
     card+=itemlength(card,nitem);
     ToLower(tag);
     //          printf("buf %s \n",buffer); 

     if (!strncmp(tag,"vers",4)) {
       float vers_nr;
        int nitem=sscanf(card,"%f ",&vers_nr);
        if(nitem!=1) {
           arteMsg <<"check data on version card";
           arteMsg.error();}
     }
          
     if (!strncmp(tag,"datagr",6 )) {
        int nitem=sscanf(card,"%s ",taga);
        if(nitem!=1) {
           arteMsg <<"check data on datagroup card";
           arteMsg.error();}
        fillstr(taga,3);            

        if(volid != 0) goto L10;
        if      (!strncmp(taga,"MED",3)) datagroup = 1;   
        else if (!strncmp(taga,"ROT",3)) datagroup = 2;  
        else if (!strncmp(taga,"COM",3)) datagroup = 3;
        else if (!strncmp(taga,"MOD",3)) datagroup = 4; 
        else if (!strncmp(taga,"SUP",3)) datagroup = 5;  
        tagid = 0;   return tagid; }

     if (datagroup == 1) {
        if (!strncmp(tag,"medium",6)) {
           if(volid !=0 ) goto L10;
           volid = 1;  
           tagid = datagroup;
           ZGMED.Readmed(card,nr);
           ni = 0;
           numcomp = abs(ZGMED.Getncomp(nr));}
 
        else if (!strncmp(tag,"par",5) ) ZGMED.Readparam(card);

        else if (!strncmp(tag,"mcomp",5) && numcomp > 1 ) {
           ni++;
           if(ni > numcomp) {
              arteMsg <<"more components than on card medium";
              arteMsg.error();}
           ZGMED.Readmcomp(card);} 
           
        else if (!strncmp(tag,"quit",4)) {
           if(volid !=0 ) goto L10;
           fclose(filepointer);
           tagid = -99;
           datagroup = 5;
           /*goto L10;*/  return tagid;  } }

     if (datagroup == 2) {       
        if (!strncmp(tag,"rot",3)) {
        ZGROT.Readrot(card,nr);     
        return datagroup;}}


     if (datagroup == 3 || datagroup == 4 || datagroup == 5){

        if (!strncmp(tag,"vol",3))  { 
           if(volid !=0 ) goto L10;
           volid = 1;  
           tagid = datagroup;
           ZGVOL.Read_vol(card,nr); } 

     else if (!strncmp(tag,"shap",4)) ZGVOL.Read_shape(card);
     else if (!strncmp(tag,"dim",3))  ZGVOL.Read_dimension(card);
     else if (!strncmp(tag,"typ",3))  ZGVOL.Read_type(card);
     else if (!strncmp(tag,"sens_fl",7)) ZGVOL.Read_sens_fl(card);
     else if (!strncmp(tag,"mat",3))  ZGVOL.Read_material(card);
     else if (!strncmp(tag,"rel_p",5))ZGVOL.Read_rel_point(card);
     else if (!strncmp(tag,"col",3))  ZGVOL.Read_color(card);

     else if (!strncmp(tag,"sens_dim",8)) ZGDET.Read_sens_dim(card);
     else if (!strncmp(tag,"sens_of",7)) ZGDET.Read_sens_offset(card);
     else if (!strncmp(tag,"sid",3))  ZGDET.Read_sides(card);
     else if (!strncmp(tag,"z_wire",6)) ZGDET.Read_z_wire(card);
     else if (!strncmp(tag,"x_wire",6)) ZGDET.Read_x_wire(card);
     else if (!strncmp(tag,"y1_wire",6)) ZGDET.Read_y1_wire(card);
     else if (!strncmp(tag,"y2_wire",6)) ZGDET.Read_y2_wire(card);
     else if (!strncmp(tag,"n_wire",6)) ZGDET.Read_n_wire(card);
     else if (!strncmp(tag,"wire_or",7)) ZGDET.Read_wire_orientation(card);
     else if (!strncmp(tag,"pol",3)) ZGDET.Read_polarity(card);
     else if (!strncmp(tag,"pit",3))  ZGDET.Read_pitch(card);
     else if (!strncmp(tag,"eff",3))  ZGDET.Read_effectivity(card);
     else if (!strncmp(tag,"acc",3))  ZGDET.Read_accuracy(card);
     else if (!strncmp(tag,"dbl_hit_resol",13))ZGDET.Read_double_hit_res(card);

     else if (!strncmp(tag,"comp",4)) ZGPOS.Read_comp(card);   
     else if(!strncmp(tag,"resolve",7)) ZGPOS.Read_resolve(card);
     else if(!strncmp(tag,"mirror_x",8)) ZGPOS.Read_Mirror_x();
     else if(!strncmp(tag,"mirror_y",8)) ZGPOS.Read_Mirror_y();
     else if(!strncmp(tag,"mirror_z",8)) ZGPOS.Read_Mirror_z();

     else if(!strncmp(tag,"cal_subgroup",12)) ZGCAL.Read_calsubgroup();     
     else if(!strncmp(tag,"ncal",4)) ZGCAL.Read_ncal(card);     
     else if(!strncmp(tag,"pos_cal",7)) ZGCAL.Read_poscal(card);     
     else if(!strncmp(tag,"z_cal",5)) ZGCAL.Read_zcal(card);     
     else if(!strncmp(tag,"zc_wall",7)) ZGCAL.Read_zcwall(card);     
     else if(!strncmp(tag,"dzwall",8)) ZGCAL.Read_dzwall(card);
     else if(!strncmp(tag,"caba_size",9)) ZGCAL.Read_caba_size(card);     
     else if(!strncmp(tag,"cast",4)) ZGCAL.Read_cast(card,filepointer);     

     else if (!strncmp(tag,"quit",4)) {
        fclose(filepointer);
        tagid = -99;
        if(volid !=0 ) goto L10;
        return tagid;  
     } 

     else  {
      if(strncmp(tag,"*",1) && strncmp(tag,"#",1)) {
         arteMsg << "read_dat: - tag " << tag << " notfound ";
         arteMsg.warn();}}}} 
 
 L10:
     if(volid!=0) { 
        fsetpos(filepointer, &fpos);
        volid = 0;
        if (datagroup==3 || datagroup==4 || datagroup==5 ) {    
            if(ZGVOL.Get_ncomp(nr)) ZGPOS.Set_vol_pos_Rel();
        }
        return tagid;}

  return 0;}


