//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcClusPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "marple/marple.hh"

extern ArtePointer<GCAL> maFindGCAL( float x, float y );


RcClusPoint::RcClusPoint( const  ArtePointer <RCCL>  rClusLink)
  : RcPoint(), fClusLink(rClusLink)  
{
  if( rClusLink == 0 || rClusLink.index() > rClusLink.nrow() )
    { 
      arteMsg << " MARPLE: RCCL entry number " << rClusLink 
	      << "does not exist" << endl;
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }
  setX(   rClusLink->x  );
  setY(   rClusLink->y  );
  setZ(   rClusLink->z  );
  //setZ( 1330);
  settx(  rClusLink->tx );
  setty(  rClusLink->ty );
  HepDouble eClus = rClusLink->esum;
  setpe(  1/eClus );
  setcov( rClusLink->cve );
  if( pe() ) 
    {
      if( fabs(rClusLink->cve[c_matrix_pack[4][4]]) < 0.1e-10 )
	{
	  // set energy resolution using TDR values 
	  switch ( maFindGCAL(x(), y())->ncl  ) 
	    {
	    case 1 :
	      // 9.5%/sqrt(E) + 1%
	      setcov(5,5, fabs(eClus)*sqrt( 0.0001   + .009025/fabs(eClus) )); 
	    case 2 :
	      setcov(5,5, fabs(eClus)*sqrt( 0.0001   + .009025/fabs(eClus) ));
	    case 5 :
	      // 17%/sqrt(E) + 1.6% 
	      setcov(5,5, fabs(eClus)*sqrt( 0.000256 + 0.0289 /fabs(eClus) ));
	    }
	}
      HepDouble correction = 1/eClus;
      correction *=correction; // ( 1/ESUM )**2
      setcov(1,5, cov()(1,5)*correction );
      setcov(2,5, cov()(2,5)*correction );
      setcov(3,5, cov()(3,5)*correction );
      setcov(4,5, cov()(4,5)*correction );
      correction *=correction; // ( 1/ESUM )**4
      setcov(5,5, cov()(5,5)*correction );
    }
  if( cov()(1,1) < MARPLE_NEGLIGIBLE ||
      cov()(2,2) < MARPLE_NEGLIGIBLE ) 
    {
      // for space resolution the values from note Hera-V 97-150 are used
      
      switch ( maFindGCAL(x(), y())->ncl  ) 
	{
	case 1 :
	  setcov(1,1, .4356   + 4.9284/fabs(eClus)  ); 
	  setcov(2,2, .4356   + 4.9284/fabs(eClus)  ); 
	case 2 :
	  setcov(1,1, .04     + 3.7636/fabs(eClus)  ); 
	  setcov(2,2, .04     + 3.7636/fabs(eClus)  ); 
	case 5 :
	  setcov(1,1, .0256   + 1.2769/fabs(eClus)  ); 
	  setcov(2,2, .0256   + 1.2769/fabs(eClus)  ); 
	}
      int iErr;
      estimateAngles(2, iErr);
    }
  checkAndCorrectCov();
}

//--------------------------------------------------------------------
// returns relation with MTRA 
vector<ArtePointer<MTRA> > RcClusPoint::mtraList() const
{
  vector<ArtePointer<MTRA> > mList;
  for(ArteRelationIterator<HITC> riHitc = link()->hitc.begin();
      riHitc != link()->hitc.end();
      ++riHitc )
    {
      for(ArteRelationIterator<MCAL> riMcal = (*riHitc)->mcal.begin();
	  riMcal != (*riHitc)->mcal.end();
	  ++riMcal )
	{
	  ArtePointer<MTRA> mEntry( (*riMcal)->mtra);
	  //  RcClusPoint::mtraList  returns only particle with P > 1GeV !!!
	  //float Pmc = sqrt( mEntry->px*mEntry->px + mEntry->py*mEntry->py 
	  //	    + mEntry->pz*mEntry->pz );
	  //if( !RcMtraOk( mEntry, mList ) && Pmc > 1 )
	  if( !RcMtraOk( mEntry, mList ) )
	    mList.push_back(mEntry); 
	  
	}
    }
  return mList;
}

// returns relation with HITC 
vector<ArtePointer<HITC> > RcClusPoint::hitcList() const
{
  vector<ArtePointer<HITC> > hList;
  for(ArteRelationIterator<HITC> riHitc = link()->hitc.begin();
      riHitc != link()->hitc.begin();
      ++riHitc )
    {
      ArtePointer<HITC> hEntry( riHitc);
      hList.push_back(hEntry);      
    }
  return hList;
}

// update RCCL bank by RcClusPoint information
void RcClusPoint::updateARTE( int & iErr) const
{
  iErr = 0;
  ArtePointer<RCCL> rccl = link();
  if( rccl== 0 || rccl.index() > rccl.nrow() )
    {
      arteMsg << " MARPLE: updateARTE: RCCL link " 
	      << rccl << " does not exist";
      arteMsg.allp();
      iErr = 1;
      return;
    }
  static int idMess = 762873456;
  arteMsg << "MARPLE:  updateARTE: RcClusPoint is saved as RCCL";
  arteMsg.allp(idMess,1);
  
  rccl->x  = x();
  rccl->y  = y() ;
  rccl->z  = z() ;
  rccl->tx = tx() ;
  rccl->ty = ty();
  if( fabs(pe() )<1.e-10 ) rccl->esum  = 1/pe();

  for( int i=0; i<5; i++){
    for(int j=i; j<5; j++){
      rccl->cve[c_matrix_pack[i][j]] = cov()[i][j]; 
      // start numbering from [0][0] as in struct RCCL
    }
  }
  if( cov()(5,5) && fabs(pe()) > 1.e-8 ) 
    {
      HepDouble correction = 1/pe();
      correction *=correction; // ( 1/pe() )**2
      rccl->cve[c_matrix_pack[1][5]] = cov()(1,5)*correction ;
      rccl->cve[c_matrix_pack[2][5]] = cov()(2,5)*correction ;
      rccl->cve[c_matrix_pack[3][5]] = cov()(3,5)*correction ;
      rccl->cve[c_matrix_pack[4][5]] = cov()(4,5)*correction ;
      correction *=correction; // ( 1/ESUM )**4
      rccl->cve[c_matrix_pack[5][5]] = cov()(5,5)*correction ;
    }
}

//------------------------------------------------------------
// estimate the incident angle using target information
void RcClusPoint::estimateAngles( HepDouble tarX, HepDouble tarY, 
				  HepDouble tarZ,
				  HepDouble covTarXX, HepDouble covTarYY, 
				  HepDouble covTarXY,
				  int nIter, 
				  int & iErr )
{
  setty( (y()-tarY)/(z()-tarZ) );
  HepDouble dty = (sqrt(fabs(cov()(2,2))) - sqrt(fabs(covTarYY)) ) / 
    ( z()-tarZ);
  setcov(2,2, dty*dty);
  if( maMagnetON() )
    {
      estimatePARfromTAR( 3, tarX, tarY, tarZ, 
			  covTarXX, covTarYY, covTarXY,
			  nIter, iErr );
    }
  else
    {
      settx( (x()-tarX)/(z()-tarZ) );
      HepDouble dtx = (sqrt(fabs(cov()(1,1))) - sqrt(fabs(covTarXX)) ) / 
	( z()-tarZ);
      setcov(2,2, dtx*dtx);
    }
}


void RcClusPoint::estimateAngles( int nIter, 
				  int & iErr )
{
  setty( y()/z() );
  if( !maMagnetON() )
    {
      settx( x()/z() );
      estimatePARfromTAR( 3, 1, iErr );
      estimatePARfromTAR( 4, 1, iErr );
    }else{
      estimatePARfromTAR( 3, 1, iErr );
      estimatePARfromTAR( 4, 1, iErr );
      if( nIter > 1 )  estimatePARfromTAR( 3, nIter-1, iErr );
      estimatePARfromTAR( 4, 1, iErr );
    }
}


ostream & operator << (ostream & os, const RcClusPoint &rp)
{
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RCCL link: " << rp.link() << endl ;
  return os ;
}

