// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/Geom.hh"
#include "rang/GeWa.hh"
#include "rang/Gede.hh"
#include "rang/Gwal.hh"
#include "rang/sortzv.hh"
#undef SHOWSORT

int rangerGeom::nVolume = 0;               // Initialize static members
rangerVolume** rangerGeom::volume = 0;
rangerVolume** rangerGeom::PGede = 0;
rangerVolume** rangerGeom::PGwal = 0;
rangerVolume* rangerGeom::firstVolume = 0;
rangerVolume* rangerGeom::lastVolume = 0;

void rangerGeom::update() {
  reset();
  build();
  sort();
  arteMsg << "rangerGeom::update: geometry table (re)built" << 
    " with " << nVolume << " volumes" << endl;
  arteMsg.warn();
};

void rangerGeom::condUpdate() {
  int nVol = ArtePointer<GEDE>::nrow() + ArtePointer<GWAL>::nrow();
  if (nVol != nVolume) {
    update();
  }
};

void rangerGeom::reset() {
  if (volume) {
    for (int i=0; i<nVolume; i++) delete volume [i];
  }
  nVolume = 0;
  delete PGede;
  delete PGwal;
  delete volume;
};

void rangerGeom::build() {
  PGede = new rangerVolume* [ArtePointer<GEDE>::nrow()];
  PGwal = new rangerVolume* [ArtePointer<GWAL>::nrow()];
  nVolume = ArtePointer<GEDE>::nrow() + ArtePointer<GWAL>::nrow();
  volume = new rangerVolume* [nVolume];
  for (int i=0; i<nVolume; i++) volume [i] = 0;
  rangerVolume** volumeIt =  volume;    // iterator for volume array
  for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	gede != ArteTable<GEDE>::end(); ++gede ) {
    rangerGede* Gede = new rangerGede( gede ); // allocate new Gede
    rangerGeWa* GeWa = Gede;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );
    //cout << "build: made volume from gede=" << gede 
    //	 << " ms=" << vol->ms() << endl;
    *volumeIt = vol;
    int ge = gede.index();
    PGede[ge-1] = *volumeIt;
    //printf("build: volume %d stored at %p, pointers address is %p\n",
    //	   vol->plNumber(),*volumeIt,volumeIt);
    volumeIt++;
  };
  for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
	gwal != ArteTable<GWAL>::end(); ++gwal ) {
    rangerGwal* Gwal = new rangerGwal( gwal ); // allocate new Gwal
    rangerGeWa* GeWa = Gwal;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );
    //cout << "build: made volume from gwal=" << gwal 
    //	 << " ms=" << vol->ms() << endl;
    *volumeIt = vol;
    PGwal[gwal.index()-1] = *volumeIt;
    //printf("build: volume %d stored at %p, pointers address is %p\n",
    //	   vol->plNumber(),*volumeIt,volumeIt);
    volumeIt++;
  };
};

void rangerGeom::sort() {
  float* zArray = new float [nVolume];                  // array for z values
  rangerVolume** vArray = new rangerVolume* [nVolume];  // array to hold
                                                        // active volumes
  long int* index = new long int [nVolume];             // array for index
  float* zIt = zArray;                             // iterator for zArray
  rangerVolume** vIt = vArray;                     // iterator for vArray
  rangerVolume** volumeIt = volume;                // iterator for volume
  long int nElem = 0;
  for (int i=0; i<nVolume; i++) {
    if (*volumeIt) {
      *zIt = (*volumeIt)->zMean();
      *vIt = *volumeIt;
//    if (((*volumeIt)->getGeWa())->shp() == 6) {
// 	rangerGeWa* geeee = (*volumeIt)->getGeWa();
// 	cout << "rangerGeom::sort(): gewa " << geeee->plNumber()
// 	     << "stored with z=" << *zIt << endl;
//    }
      zIt++;
      vIt++;
      nElem++;
    }
    volumeIt++;
  }
  int mode = 1;
  int way = 0;
  int nsort = 0;
  sortzv_(zArray,index,nElem,mode,way,nsort);
  for (int j=0; j<nElem; j++) {
    if (j>0) {
      vArray[index[j]-1]->getPrevious() = vArray[index[j-1]-1];
    }
    if (j<(nElem-1)) {
      vArray[index[j]-1]->getNext() = vArray[index[j+1]-1];
    }
  }
  if (nElem >= 1) {
    firstVolume = vArray[index[0]-1];
    lastVolume = vArray[index[nElem-1]-1];
  }
#ifdef SHOWSORT
  for (rangerVolume* volIt=firstVolume; 
       volIt; volIt=volIt->next()){
    cout << "  z=" << volIt->zMean()
	 << " plane=" << volIt->plNumber() << endl;
  }
  cout << endl << endl << endl << endl;
  for (rangerVolume* volIt=lastVolume; 
       volIt; volIt=volIt->previous()){
    cout << "  z=" << volIt->zMean()
	 << " plane=" << volIt->plNumber() << endl;
  }
#endif
}
  

