// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "rang/State.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include "rang/FitMode.hh"


//-- Constructor
rangerState::rangerState() {
  zState = 0;
  qPEstimate = 0.1;
}

rangerState::rangerState(ArtePointer<RHIT> rhit) {
  zState = rhit->hitb->z;
  qPEstimate = rhit->p;
#ifdef FULL
  cout << "rangerState rhit constructor" << endl;
  cout << "rangerState: the state vector is now " << endl;
  cout << p << endl;
#endif
}

rangerState::rangerState(ArtePointer<HITB> hitb) {
  zState = hitb->z;
  qPEstimate = 0.1;
#ifdef FULL
  cout << "rangerState rhit constructor" << endl;
  cout << "rangerState: the state vector is now " << endl;
  cout << p << endl;
#endif
}



rangerState::rangerState(ArtePointer<RCCL> rccl) {
  zState = rccl->z;
  qPEstimate = 1/rccl->esum;  // beware of charge ambiguity
}

rangerState::rangerState(const rangerState& s)  {
#ifndef FULL
  cout << "rangerState::rangerState: here is the copy constructor" << endl;
#endif
  zState = s.zState;
  qPEstimate = s.qPEstimate;
#ifdef FULL
  cout << "rangerState::rangerState: Copy constructor done " << endl;
#endif
}

rangerState::~rangerState(){
//  cout << "rangerState destructor has been called" << endl;
}

#ifdef DEFASS
rangerState& rangerState::operator=(const rangerState& s) {

  /////#ifndef FULL
  cout << "rangerState::operator=: here is the assignment operator" << endl;
  cout << "   the given z is " << s.z() << endl;
  /////#endif
  int d1=s.dim();
  //int d1=-100;
  int d2=dim();
  cout << "rangerState::operator=: old state dim=" 
       << d1 << " new " << d2 << endl;
  zState = s.zState;
  qPEstimate = s.qPEst();
  //p = s.p;
  //c = s.c;
}
#endif


rangerState* rangerState::makeState(const rangerFitMode fm) {
  rangerState* s;
  switch (fm) {
  case ModeP:
    s = new rangerStateP();
    break;
  case ModePI:
    s = new rangerStatePI();
    break;
  case ModeL:
    s = new rangerStateL();
    break;
  case ModeLI:
    s = new rangerStateLI();
    break;
  default:
    cout << "rangerState::makeState: improper mode " << int(fm) << endl;
    exit(1);
  }
  return s;
}

rangerState* rangerState::makeState( ArtePointer<RHIT>& rhit,
				const rangerFitMode fm ) {
  rangerState* s;
  switch (fm) {
  case ModeP:
    s = new rangerStateP( rhit );
    break;
  case ModePI:
    s = new rangerStatePI( rhit );
    break;
  case ModeL:
    s = new rangerStateL( rhit );
    break;
  case ModeLI:
    s = new rangerStateLI( rhit );
    break;
  default:
    cout << "rangerState::makeState(): improper mode " << int(fm) << endl;
    exit(1);
  }
  return s;
}


rangerState* rangerState::makeState( ArtePointer<HITB>& hitb,
				const rangerFitMode fm ) {
  rangerState* s;
  switch (fm) {
  case ModeP:
    s = new rangerStateP( hitb );
    break;
  case ModePI:
    s = new rangerStatePI( hitb );
    break;
  case ModeL:
    s = new rangerStateL( hitb );
    break;
  case ModeLI:
    s = new rangerStateLI( hitb );
    break;
  default:
    cout << "rangerState::makeState(): improper mode " << int(fm) << endl;
    exit(1);
  }
  return s;
}


rangerState* rangerState::makeState( ArtePointer<RCCL>& rccl,
				     int charge,const rangerFitMode fm ) {
  rangerState* s;
  switch (fm) {
  case ModeP:
    s = new rangerStateP( rccl,charge );
    break;
  case ModePI:
    s = new rangerStatePI( rccl,charge );
    break;
  case ModeL:
    s = new rangerStateL( rccl );
    break;
  case ModeLI:
    s = new rangerStateLI( rccl );
    break;
  default:
    cout << "rangerState::makeState(): improper mode " << int(fm) << endl;
    exit(1);
  }
  return s;
}


void rangerState::mergeRhit(ArtePointer<RHIT> rhit,float weight){
                                      // merge parameters to RHIT
  float w1 = 1 - weight;
  rhit->x  = w1 * rhit->x  + weight * x();
  rhit->y  = w1 * rhit->y  + weight * y();
  rhit->tx = w1 * rhit->tx + weight * tx();
  rhit->ty = w1 * rhit->ty + weight * ty();
  rhit->p  = w1 * rhit->p  + weight * qP();
}


   

