// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerStatePI
// state class for tracking with momentum and interaction time

#ifndef rangerStatePI_HH
#define rangerStatePI_HH

#include "CLHEP/Matrix/SymMatrix.h"
#include "CLHEP/Matrix/Vector.h"
#include "arte/arte.hh"
#include "rang/ScatThick.hh"
#include "rang/FitMode.hh"
#include "rang/State.hh"
#include "rang/HitBase.hh"

class rangerStatePI : public rangerState {

public:
  rangerStatePI();
  rangerStatePI(ArtePointer<RHIT> rhit);
  rangerStatePI(ArtePointer<HITB> hitb);
  rangerStatePI(ArtePointer<RCCL> rccl,int charge);
  rangerStatePI(const rangerStatePI& s);  // copy constructor
  ~rangerStatePI();
  rangerState& operator=(const rangerStatePI& s);  // assignment
  rangerState& operator=(const rangerState& s);
  rangerState& operator=(const rangerStateP& s);
  rangerState& operator=(const rangerStateL& s);
  rangerState& operator=(const rangerStateLI& s);

  const double x() const { return p(1); } // parameter lookup accessors
  const double y() const { return p(2); }
  const double tx() const { return p(3); }
  const double ty() const { return p(4); }
  const double qP() const { return p(5); }
  const double  I()  const { return p(6); }

  inline double& setX() { return p(1); }    // parameter update accessors
  inline double& setY() { return p(2); }
  inline double& setTx() { return p(3); }
  inline double& setTy() { return p(4); }
  inline double& setQp() { return p(5); }
  inline double& setI() { return p(6); }

  int dim() const {return 6;};   // dimension of state vector

  HepVector& vec() { return p; }
  const HepVector& vec() const { return p; }
  HepSymMatrix & cov() { return c; } 
  const HepSymMatrix & cov() const { return c; } 

//   HepVector& vec();    // lookup function for state vector
//   const HepVector& vec() const; // const lookup function for state vector
//   HepSymMatrix& cov(); //  lookup function for covariance matrix
//   const HepSymMatrix& cov() const; //  lookup function for covariance matrix

  rangerState* clone() const { return new rangerStatePI(*this); };
                                   // produce copy
  void covInit();
  int predict(const double& zDestination,
	      const double& qPExp,rangerState*& targetState,
	      HepMatrix*& FTransport);  // perform Kalman filter prediction
  void project(HepVector& resid,HepMatrix& HH,rangerHitBase* hit)
    { hit->projectPI(resid,HH,this); } ;

  void applyScat(const rangerScatThin& scat);
                                        // apply m.s. correction 
  void applyScat(const rangerScatThick& scat);
                                        // apply m.s. correction 
  void applyRadEl(const rangerRadEl& rad);
                                 // apply continuous radiative energy loss correction 
  void applyDeDx(const float rl);// apply continuous ionization energy loss correction
  void applyRadPhoton(const float e,const float dE);
                                     // apply resolved radiated photon
  void storeRseg(ArtePointer<RSEG> rseg,int firstLast); 
                                          // store track parameters into RSEG
  void storeRtra(ArtePointer<RTRA> rtra,int firstLast); 
                                          // store track parameters into RTRA
  void storeRhit(ArtePointer<RHIT> rhit); // store parameters as RHIT

  void copyVec( const rangerState& s); // copy state vector from another state
                                       // (do not copy cov matrix)
  void linExtrapVec( const rangerState& s, float z); 
                                       // extrapolate state vector from 
                                       //another state
  inline rangerFitMode fitMode() { return ModePI; }
  void dump();                         // dump private members to screen

private:
  HepVector p;
  HepSymMatrix c;
};

#endif
