// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/Fit.hh"
#include "rang/Geom.hh"
#include "rang/rginit.hh"


extern "C" {
  int rgfitsv_(const int* segVec,   // integer array of RSEG row numbers
	       const int& nSeg,     // length of this array
	       const int& pType     // particle type identifier:
	                            //    0 - MUON
	                            //    1 - PION
	                            //    2 - KAON
                                    //    3 - PROTON
	                            //    4 - ELECTRON
	       )  {
    if (ArtePointer<RSEG>::nrow() <= 0) return 0;

    // geometry initialization
    rginit_();
    rangerGeom::condUpdate();

    vector<ArtePointer<RSEG> > rsegVec;

    // translate particle type
    rangerParticleType Type = MUON;

    switch (pType) {
    case 0:
      Type = MUON;
      break;
    case 1:
      Type = PION;
      break;
    case 2:
      Type = KAON;
      break;
    case 3:
      Type = PROTON;
      break;
    case 4:
      Type = ELECTRON;
      break;
    default:
      Type = MUON;
      break;
    }

    for (int ii=0; ii<nSeg; ii++) {
      if (segVec[ii] <=0 || segVec[ii] > int(ArtePointer<RSEG>::nrow())) {
	arteMsg << "rgfitsv_: invalid segment " << segVec[ii] << endl;
	arteMsg.warn();
	return -13;
      }
      ArtePointer<RSEG> rseg(segVec[ii]);
      rsegVec.push_back(rseg);
    }

    int fitStatus = rangerFit(rsegVec,Type);

    return fitStatus;
  }
}

