// 
//  Author:  Rainer Mankel                     Created:  15-Nov-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//

// Fortran callable interface to rangerSeg2track for segment vector
 
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/seg2track.hh"

extern "C" {
  int rgsv2t_(const int* segVec,  // segVec = integer array of RSEG row numbers
	      const int& nSeg     // length of this array
	      ) {    
    if (ArtePointer<RSEG>::nrow() <= 0) return 0;
    
    vector<ArtePointer<RSEG> > rsegVec;

    for (int ii=0; ii<nSeg; ii++) {
      if (segVec[ii] <=0 || segVec[ii] > (int) ArtePointer<RSEG>::nrow()) {
	arteMsg << "rgsv2t_: invalid segment " << segVec[ii] << endl;
	arteMsg.warn();
	return -13;
      }
      ArtePointer<RSEG> rseg(segVec[ii]);
      rsegVec.push_back(rseg);
    }

    ArtePointer<RTRA> rtra = rangerSeg2track(rsegVec);
    return rtra.index();
  }
}

