*CMZ :          23/09/96  16.35.54  by  T. Oest
*-- Author :    T. Oest   28/05/96

      SUBROUTINE RKSTEP(H, PIN, FAC, POUT, DELTA)

************************************************************************
*
* Description: one step for Runge Kuta
*
* Called from:
*
* External references:
*
* Argument in/out description
* H        in     step size
* PIN(*)   in     input values for (x,y,vx,vy,rho,z)
* FAC      in     e/p * sign(dz)
* POUT(*)  out    output values for (x,y,vx,vy,rho,z)
* DELTA(4) out    estimated error for (x,y,vx,vy)
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"

      LOGICAL          ZMODE
      DOUBLE PRECISION PIN(*), H, POUT(*), DELTA(4), FAC
      DOUBLE PRECISION PWORK(5),F(5), ADDP(5), HCORR
      DOUBLE PRECISION K(6,5)
      INTEGER I,J, L, M
      LOGICAL IFIRST
      REAL*4           X(3), FIELD(3)

      DOUBLE PRECISION B(15), C(6), CST(6), PSTEPS(6,6)
      DOUBLE PRECISION BFIELD(6,3), FSTEPS(6,2)
      COMMON /CRKUT2/  PSTEPS, B, C, CST, K, BFIELD, FSTEPS
      DATA IFIRST /.TRUE./
      SAVE IFIRST

#include "arte/Arte.ff"
************************************************************************
* initialization
************************************************************************
      IF (IFIRST) THEN
       IFIRST = .FALSE.
       B(1)   =     1.D0 /      5.D0
       B(2)   =     3.D0 /     40.D0
       B(3)   =     9.D0 /     40.D0
       B(4)   =     3.D0 /     10.D0
       B(5)   =    -9.D0 /     10.D0
       B(6)   =     6.D0 /      5.D0
       B(7)   =   -11.D0 /     54.D0
       B(8)   =     5.D0 /      2.D0
       B(9)   =   -70.D0 /     27.D0
       B(10)  =    35.D0 /     27.D0
       B(11)  =  1631.D0 /  55296.D0
       B(12)  =   175.D0 /    512.D0
       B(13)  =   575.D0 /  13824.D0
       B(14)  = 44275.D0 / 110592.D0
       B(15)  =   253.D0 /   4096.D0

       C(1)   =    37.D0 /    378.D0
       C(2)   =     0.D0
       C(3)   =   250.D0 /    621.D0
       C(4)   =   125.D0 /    594.D0
       C(5)   =     0.D0
       C(6)   =   512.D0 /   1771.D0

       CST(1) =  2825.D0 /  27648.D0
       CST(2) =     0.D0
       CST(3) = 18575.D0 /  48384.D0
       CST(4) = 13525.D0 /  55296.D0
       CST(5) =   277.D0 /  14336.D0
       CST(6) =     1.D0 /      4.D0
      ENDIF
************************************************************************
* calculate k first
************************************************************************
      L = 0
      DO I=1,6

C new coordinate
       DO M=1,5
        PWORK(M) = PIN(M)
       ENDDO
       DO J=1,I-1
        L = L + 1
        DO M=1,5
         PWORK(M) = PWORK(M) + B(L) * K(J,M)
        ENDDO
       ENDDO

************************************************************************
* get k positions
************************************************************************
       X(1) = SNGL(PWORK(2))
       X(2) = SNGL(PWORK(3))
       X(3) = SNGL(PWORK(1))
       CALL UTFELD(X,FIELD)

       F(1) = 1.0
       F(2) = PWORK(4)
       F(3) = PWORK(5)
       FSTEPS(I,1) = SQRT(1.+PWORK(4)**2+PWORK(5)**2)*
     +               ( PWORK(4)*PWORK(5) * FIELD(1)
     +               - (1.+PWORK(4)**2)  * FIELD(2)
     +               + PWORK(5)          * FIELD(3))
       F(4) = FAC * FSTEPS(I,1)
       FSTEPS(I,2) = SQRT(1.+PWORK(4)**2+PWORK(5)**2)*
     +               ( (1.+PWORK(5)**2)  * FIELD(1)
     +               - PWORK(4)*PWORK(5) * FIELD(2)
     +               - PWORK(4)          * FIELD(3))
       F(5) = FAC * FSTEPS(I,2)

       DO J=1,5
        K(I,J)      = H * F(J)
        PSTEPS(I,J) = PWORK(J)
       ENDDO
       DO J=1,3
        BFIELD(I,J) = FIELD(J)
       ENDDO
       PSTEPS(I,6) = PIN(6)

      ENDDO

************************************************************************
* increment
************************************************************************


      DO I=2,5
       DELTA(I-1)  = 0.0D0
       ADDP(I)     = 0.0D0
       DO J=1,6
        ADDP(I)    = ADDP(I)    +  C(J)         * K(J,I)
        DELTA(I-1) = DELTA(I-1) + (C(J)-CST(J)) * K(J,I)
       ENDDO
      ENDDO

************************************************************************
* FINAL PARAMETER
************************************************************************

      POUT(1) = PIN(1) + H
      DO I=2,5
       POUT(I)  = PIN(I)  +  ADDP(I)
      ENDDO



      END

************************************************************************
* End of RKSTEP
************************************************************************
