#include "rich/PhotonDetection.h"

extern int RICHdebug;


// =========================================================================
//
//  calculate efficiency and decide if it is detected.
//

int PM::dodetect(int k,const Photon& p) {
  double eff=efficiency(k)*pmtype->efficiency(p.lambda());
  if (rnd_normal()>eff) return PMT_LOSS_EFFICIENCY;
  return NO_LOSS;
}

float& PM::efficiency(int i) {
  if (a->efv0==-1) {
    if (!pmtype) {
      fprintf(stderr,"PM::efficiency: pmtype not initialized. Abort.\n");
      Exit(0);
      }
    a->efv0=ArtePointer<CEFF>::nrow()+1;
    CEFF eff;
    int i;
    for (i=0;i<numchannel();i++) ArteTable<CEFF>::push_back(eff);
    }
  return ArtePointer<CEFF>(a->efv0+i)->eff;
  }
  
// -------------------------------------------------------------------------
//  initialisation & input/ouput for class pm
//

PM::PM(ArtePointer<CPMD> aa,List<PMType>& PMTypeList,List<LensSystem>& LensList)  {
  a=aa;

  int i;
  
  int lens=a->lens;
  for (i=sizeof(a->type)-1;i>=0;i--) if (a->type[i]!=' ') break;
  char *type=(char*) malloc(i+2);
  memcpy(type,a->type,i+1);
  type[i+1]='\0';
  
  for (i=sizeof(a->seno)-1;i>=0;i--) if (a->seno[i]!=' ') break;
  memcpy(seno,a->seno,i+1);
  seno[i+1]='\0';
  
  pmtype=0;
  for (PMTypeList.newiterator();(pmtype=PMTypeList.iterate());) {
    if (!strcmp(pmtype->name(),type)) break;
    }
  free(type);
  if (!pmtype) {
    fprintf(stderr,"class PM: Unknown PMType: %s\n",type);
    Exit(0);
    }
  ls=0;
  for (LensList.newiterator();(ls=LensList.iterate());) {
    if (ls->type()==lens) break;
    }
  }  
 
PM::PM(ScriptFile& f,List<PMType>& PMTypeList,List<LensSystem>& LensList) {
  CPMD pm;

  int lens=1;
  char n[256],tag[256];
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    int err=0;
    if      (!strcmp(tag,"type")){ 
      err=(sscanf(s,"%s",tag)!=1);
      for (PMTypeList.newiterator();(pmtype=PMTypeList.iterate());) {
        if (!strcmp(pmtype->name(),tag)) break;
        }
      if (!pmtype) {
        fprintf(stderr,"class PM: Unknown PMType: %s\n",tag);
        Exit(0);
        }
      int len=(strlen(tag)<sizeof(pm.type))?strlen(tag):sizeof(pm.type);
      memset(pm.type,' ',sizeof(pm.type));
      strncpy(pm.type,tag,len);
      }
    else if (!strcmp(tag,"serial_no")) {
      err=(sscanf(s,"%s",tag)!=1);
      int len=(strlen(tag)<sizeof(seno)-1) ? strlen(tag) : sizeof(seno)-1;
      strncpy(seno,tag,len);
      seno[len]='\0';
      memset(pm.seno,' ',sizeof(pm.seno));
      len=(strlen(tag)<sizeof(pm.seno)) ? strlen(tag) : sizeof(pm.seno);
      memcpy(pm.seno,tag,len);
      }
    else if (!strcmp(tag,"position"))   
      err=(sscanf(s,"%d %d %d",&pm.isup,&pm.irow,&pm.icol)!=3);
    else if (!strcmp(tag,"lenssystem"))   
      err=(sscanf(s,"%d",&lens)!=1);
    else if (!strcmp(tag,"gain_variation")) {  
      int i,j;
      if (pmtype->numchannel()>16) {
        fprintf(stderr,"PM:PM(ScriptFile& f, ...): Too many channels %d %s\n",
	                pmtype->numchannel(),
	               "Code is limited to 16 channels pmt's. Sorry. Abort.\n");
        Exit(1);
	}
      float eff[16];
      for (i=pmtype->numrow()-1;i>=0;i--) {
        f.getline(n);
        char *s=n;
        for (j=pmtype->numcol()-1;j>=0;j--) {
          sscanf(s,"%f",&eff[channelindex(i,j)]);s+=f.itemlength(s,1);
          }
        }
      CEFF e;
      pm.efv0=ArtePointer<CEFF>::nrow()+1;
      for (i=0;i<pmtype->numchannel();i++) {
        e.eff=eff[i];
        ArteTable<CEFF>::push_back(e);
        }
      }
    else if (!strcmp(tag,"pm"))  continue;
    else if (!strcmp(tag,"end")) break;
    else {
      fprintf(stderr,
         "PM(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      Exit(0);
      }
    }   

  pm.lens=lens;
  for (LensList.newiterator();(ls=LensList.iterate());) {
    if (ls->type()==lens) break;
    }
  a=ArteTable<CPMD>::push_back(pm);
  }
 
  
//
//  print data
//
void PM::print(FILE* fp) {
  if (!pmtype) {
    fprintf(stderr,"PM::print: pmtype not set.Abort.\n");
    Exit(0);
    }
  fprintf(fp,"#\n pm\n");
  fprintf(fp,"  type      %s\n",pmtype->name());
  fprintf(fp,"  serial_no %s\n",serialno());
  fprintf(fp,"  position  %2d %2d %2d\n",SuperModule(),Row(),Col());
  int lens=0;
  if (ls) lens=ls->type();
  fprintf(fp,"  lenssystem %d\n",lens);
  fprintf(fp,"  gain_variation\n");
  int i,j;
  for (i=pmtype->numrow()-1;i>=0;i--) {
    fprintf(fp,"      ");  
    for (j=pmtype->numcol()-1;j>=0;j--) {
      fprintf(fp,"%5.3f ",efficiency(channelindex(i,j)));  
      }
    fprintf(fp,"\n");
    }
  fprintf(fp,"  end\n");
  }


