#include "rich/PhiLambda.h"

// ---------------------------------------------------------------------- //
// --------- Transformation: RICH hit -> ray direction ------------------ //
// ---------------------------------------------------------------------- //

LambdaPhi *RICH_LambdaPhi=0;

LambdaPhi::LambdaPhi(Vec3 start) {
  if (RICH_LambdaPhi) memcpy(this,RICH_LambdaPhi,sizeof(LambdaPhi));
  else {
    //cout<<"LambdaPhi::LambdaPhi(Vec3 start)"<<endl;
    Vertex=start;

    if (!RICH_Detector) {
      fprintf(stderr,"LambdaPhi::LambdaPhi:\n");
      fprintf(stderr,"RICH_Detector not initialized. Exiting...\n");
      Exit(1);
    }
  
    if (!RICH_Vessel) {
      fprintf(stderr,"LambdaPhi(char*):\n");
      fprintf(stderr,"RICH_Vessel not initialized. Exiting...\n");
      Exit(1);
    }
  
    RICHDetector &rd=*RICH_Detector;

    pphi=new float[rd.NumChannel()];
    llam=new float[rd.NumChannel()];

    SingleSphericalMirror& usm=RICH_Vessel->spherical_mirror->UpperMirror();
    SingleSphericalMirror& lsm=RICH_Vessel->spherical_mirror->LowerMirror();
    SinglePlaneMirror&     upm=RICH_Vessel->plane_mirror->UpperMirror();
    SinglePlaneMirror&     lpm=RICH_Vessel->plane_mirror->LowerMirror();

    Vec3 detpos,x1,x2,coc;
    double R;
  
    for (rd.NewChannelLoop();rd.ChannelLoop();) {

      detpos=rd.pos();
      if (detpos.y()>=0) {
	coc=usm.CoC();
	R=usm.radius();
	detpos=upm.VirtualPoint(detpos);
      }
      else {
	coc=lsm.CoC();
	R=lsm.radius();
	detpos=lpm.VirtualPoint(detpos);
      }
        
      x1=detpos-coc;            // vector coc-object
      x2=Vertex-coc;            // vector coc-image
      d1=norm(x1)/R;           // distance coc-object
      d1d1=d1*d1;
      d2=norm(x2)/R;           // distance coc-image
      d2d2=d2*d2;
      d12=norm(x1-x2)/R;     // distance object-image
      d12d12=d12*d12;

      // FindReflectionAngle(uInit);
      
      double uOld,u=0;
      int nIter=0;
      do {
	uOld=u;
	u-=f(u)/df(u);
	nIter++;
      } while(nIter<=1);//(fabs(uOld-u)/u>.1);

      double A1=a1(u);
      double A2=a2(u);
      double sin_alpha=sqrt(d2d2-u*u);
      double alpha=asin(sin_alpha);

      double epsilon=asin(A1*sin(2*alpha)/d12);
      double omega=alpha+epsilon;

      double segment=A2*sin_alpha/sin(omega);
      Vec3   segVector=segment/d12*(x1-x2);
      Vec3   radius=R*normalize(x2+segVector);
      Vec3   iniDir=1/(A2*R)*(radius-x2);

#ifdef DEBUG
      cout<<"a1="<<a1(u)*R<<"  R="<<R<<endl;
      cout<<"a2="<<a2(u)*R<<"  R="<<R<<endl;
      cout<<"segment="<<segment<<" "<<segVector<<endl;
      cout<<"radius="<<norm(radius)<<" "<<radius<<endl;
      cout<<"iniDir="<<initialDirection<<endl;
      cout<<"SMhit(reco)="<<x0+x2+R*initialDirection<<endl;
#endif
    
      int  i=rd.ChannelIndex();
      pphi[i]=atan(iniDir.x()/iniDir.z());
      llam[i]=atan(iniDir.y()/iniDir.z()/cos(pphi[i]));
    }
  }

  corr=new Corrections;
}






// ---------------------------------------------------------------------- //
// --------- Corrections for RICH angle reconstruction ------------------ //
// ---------------------------------------------------------------------- //


Corrections::Corrections() {
  //##  remove hard coded data
  R    = 11.4;         // spherical mirror radius
  s    = .226/R;       // detector plane - focal surface
  T    = 2.8/R;        // vessel length
  //lam0 = .05236;       // vertical center of acceptance
  //lam0 = .060912;       // vertical center of acceptance
  lam0 = .066912;       // vertical center of acceptance
  th0  = .15708;       // spherical mirror tilt angle
  z    = 10./R;        // average radiation z-coordinate
  zz   = z*z+T*T/12;   // average radiation z^2-coordinate
  zzz  = z*(z*z+T*T/4);// average radiation z^3-coordinate  
}


Corrections::Corrections(float r0,float s0,float T0,float l0,float t0,float z0) {
  R    = r0;           // spherical mirror radius
  s    = s0/R;         // detector plane - focal surface
  T    = T0/R;         // vessel length
  lam0 = l0;           // vertical center of acceptance
  th0  = t0;           // spherical mirror tilt angle
  z    = z0/R;         // average radiation z-coordinate
  zz   = z*z+T*T/12;   // average radiation z^2-coordinate
  zzz  = z*(z*z+T*T/4);// average radiation z^3-coordinate  
}


void Corrections::newRing(float p,float l,float r) {
  float Y=0,X; 
  if (RichRun::magnet) X=-p*4.5;
  else X=0;
  newRing(p,l,r,X,Y);
}


void Corrections::newRing(float p,float l,float r,float X,float Y) {

  phi=p; lam=l; rad=r; Yt=Y/R; Xt=X/R;
  lam0=fabs(lam0); th0=fabs(th0); dlam=lam-lam0;

  calcH();
  Avh_u=calcA();
  verticalD_u=verticalDistortion();
  horizontalD_u=horizontalDistortion();
  verticalP_u=verticalPosition();
  horizontalP_u=horizontalPosition();
  ddaa_u=1/(verticalD_u*horizontalD_u-Avh_u*Avh_u);

#ifdef DEBUG
  cout<<"0: phi="<<phi
      <<" lam="<<lam
      <<" rad="<<rad
      <<" Tx="<<Xt
      <<" Ty="<<Yt<<endl;
  cout<<"1: Avh="<<Avh_u
      <<" verticalD="<<verticalD_u
      <<" horizontalD="<<horizontalD_u
      <<" verticalP="<<verticalP_u
      <<" horizontalP="<<horizontalP_u<<endl;
#endif

  lam0=-lam0; th0=-th0; dlam=lam-lam0;
  calcH();
  Avh_d=calcA();
  verticalD_d=verticalDistortion();
  horizontalD_d=horizontalDistortion();
  verticalP_d=verticalPosition();
  horizontalP_d=horizontalPosition();
  ddaa_d=1/(verticalD_d*horizontalD_d-Avh_d*Avh_d);
#ifdef DEBUG
  cout<<"2: Avh="<<Avh_d
      <<" verticalD="<<verticalD_d
      <<" horizontalD="<<horizontalD_d
      <<" verticalP="<<verticalP_d
      <<" horizontalP="<<horizontalP_d<<endl;
#endif

}


void Corrections::calcH() {
  h=s+.25*(dlam*dlam+phi*phi)+th0*dlam;
  Hlam=4*h+(rad*rad-3*th0*th0);
  Hphi=Hlam+2*th0*th0;
}


float Corrections::calcA() {
  return 2*z*Xt*(Yt-th0) + Yt*phi + Xt*(2*th0+dlam);
}


float Corrections::verticalPosition() {
  return Yt*(-Hlam + 3*zz*rad*rad + Xt*Xt + Yt*Yt)
    +2*z*rad*rad*(2*th0 + dlam)
    -th0*(3*zz*rad*rad + Xt*Xt + 3*Yt*Yt);
}


float Corrections::horizontalPosition() {
#ifdef DEBUG
  cout<<"-Hphi="<<-Hphi
      <<" 3*zz*rad*rad="<<3*zz*rad*rad
      <<" Xt*Xt="<<Xt*Xt
      <<" -2*th0*Yt="<<-2*th0*Yt
      <<" 2*z*rad*rad*phi="<<2*z*rad*rad*phi<<endl;
#endif
  return Xt*(-Hphi+3*zz*rad*rad+Xt*Xt+Yt*Yt-2*th0*Yt)+2*z*rad*rad*phi;
}


float Corrections::verticalDistortion() {
  return 1 - z*(Hlam+4*rad*rad) + 2*Yt*(2*th0+dlam)
    +z*(Xt*Xt+3*Yt*Yt-6*th0*Yt) + zzz*rad*rad;
}


float Corrections::horizontalDistortion() {
  return  1 - z*(Hphi+4*rad*rad) + 2*Xt*phi
    +z*(3*Xt*Xt+Yt*Yt-2*th0*Yt) + zzz*rad*rad;
}










