#include "rich/vessel.h"
#include "rich/Losses.h"

// ===========================================================================
//

int PhotonDetectorEntryWindow::trace(Photon* p) {
  int ret;
  double path,t;
  Rectangle *m;
  if (p->x().y()>0) {
    t=uwthickness; 
    m=uw; 
    }
  else { 
    t=lwthickness; 
    m=lw;
    }
  if (!m->intersection(*p,&path)) return PDW_LOSS_OUTOFBOUNDS;
  if ((ret=RICH_Vessel->trace(p,path))) return ret;
  return mat->trace(p,fabs(t/(m->ez()*p->n())));
  }


int PhotonDetectorEntryWindow::traceonly(Photon* p) {
  int ret;
  double path,t;
  Rectangle *m;
  if (p->x().y()>0) {
    t=uwthickness; 
    m=uw; 
    }
  else { 
    t=lwthickness; 
    m=lw;
    }
  if (!m->intersection(*p,&path)) return PDW_LOSS_OUTOFBOUNDS;

  if ((ret=RICH_Vessel->traceonly(p,path))) return ret;
  p->x()+=fabs(t/(m->ez()*p->n()))*p->n();
  return NO_LOSS;
  }


int PhotonDetectorEntryWindow::upper_trace(StraightLine* t) {
  double path;
  uw->intersection(*t,&path);
  t->x()+=fabs(path+uwthickness/(uw->ez()*t->n()))*t->n();
  return NO_LOSS;
  }

int PhotonDetectorEntryWindow::lower_trace(StraightLine* t) {
  double path;
  lw->intersection(*t,&path);
  t->x()+=fabs(path+lwthickness/(lw->ez()*t->n()))*t->n();
  return NO_LOSS;
  }

 
void PhotonDetectorEntryWindow::print(FILE *fp) {
  fprintf(fp,"#\nPhotonDetectorEntryWindow\n");
  fprintf(fp,"#\n upper_window\n");
  fprintf(fp,"  thickness  %f\n",UpperWindowThickness());
  UpperWindow().print(fp,0);
  fprintf(fp,"#\n lower_window\n");
  fprintf(fp,"  thickness  %f\n",LowerWindowThickness());
  LowerWindow().print(fp,0);
  fprintf(fp,"#\n# material\n");
  Material().print(fp);
  fprintf(fp,"  end\n");
  }

PhotonDetectorEntryWindow::PhotonDetectorEntryWindow(ScriptFile& f) {
  char n[256],tag[256];

  splitted=1;
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    int err=0;
    if      (!strcmp(tag,"upper_window")) {
      f.getline(n);
      err=(sscanf(n,"%s %lf",tag,&uwthickness)!=1);  
      uw=new Rectangle(f);
      }
    else if (!strcmp(tag,"lower_window")) {
      f.getline(n);
      err=(sscanf(n,"%s %lf",tag,&lwthickness)!=1);  
      lw=new Rectangle(f);
      }
    else if (!strcmp(tag,"dielectric")) {
      mat=new DiElectric(f);
      }
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "PhotonDetectorEntryWindow(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      Exit(1);
      }
    }  
  if (ArtePointer<CPDW>::nrow()==0) {
    CPDW win;
    ArteTable<CPDW>::push_back(win);
    ArteTable<CPDW>::push_back(win);
    }

  ArtePointer<CPDW> uwin(1);
  ArtePointer<CPDW> lwin(2);

  uwin->orig[0]=UpperWindow().x0().x();
  uwin->orig[1]=UpperWindow().x0().y();
  uwin->orig[2]=UpperWindow().x0().z();
  uwin->xvec[0]=UpperWindow().xsize()*UpperWindow().ex().x();
  uwin->xvec[1]=UpperWindow().xsize()*UpperWindow().ex().y();
  uwin->xvec[2]=UpperWindow().xsize()*UpperWindow().ex().z();
  uwin->yvec[0]=UpperWindow().ysize()*UpperWindow().ey().x();
  uwin->yvec[1]=UpperWindow().ysize()*UpperWindow().ey().y();
  uwin->yvec[2]=UpperWindow().ysize()*UpperWindow().ey().z();
  uwin->t      =UpperWindowThickness();
  uwin->imat   =Material().ArteTableIndex();

  lwin->orig[0]=LowerWindow().x0().x();
  lwin->orig[1]=LowerWindow().x0().y();
  lwin->orig[2]=LowerWindow().x0().z();
  lwin->xvec[0]=LowerWindow().xsize()*LowerWindow().ex().x();
  lwin->xvec[1]=LowerWindow().xsize()*LowerWindow().ex().y();
  lwin->xvec[2]=LowerWindow().xsize()*LowerWindow().ex().z();
  lwin->yvec[0]=LowerWindow().ysize()*LowerWindow().ey().x();
  lwin->yvec[1]=LowerWindow().ysize()*LowerWindow().ey().y();
  lwin->yvec[2]=LowerWindow().ysize()*LowerWindow().ey().z();
  lwin->t      =LowerWindowThickness();
  lwin->imat   =Material().ArteTableIndex();

  }

PhotonDetectorEntryWindow::PhotonDetectorEntryWindow(ArtePointer<CPDW> a) {
  if (a.nrow()!=2) {
    fprintf(stderr,
      "RICH: PhotonDetectorEntryWindow: Arte Table CPDW is not filled.\n");
    return;
    }
  ArtePointer<CPDW> uwin(1);
  ArtePointer<CPDW> lwin(2);
  uw=new Rectangle(Vec3(uwin->orig),Vec3(uwin->xvec),Vec3(uwin->yvec));
  lw=new Rectangle(Vec3(lwin->orig),Vec3(lwin->xvec),Vec3(lwin->yvec));
  UpperWindowThickness()=uwin->t;
  LowerWindowThickness()=lwin->t;
  mat=new DiElectric(ArtePointer<CDIM>(uwin->imat));
  }

PhotonDetectorEntryWindow::~PhotonDetectorEntryWindow() {
  delete mat;
  delete uw;
  delete lw;
  }



