#include "rich/PhotonDetection.h"

extern int RICHdebug;

//===========================================================================
//
//   determine the cell, which the photon hits: no physics here
//

int PhotonDetectorSuperModule::dodetect(Photon* pp,int *daqid) {
  Photon& p=*pp; // synonym
  double l;
  if (!intersection(p,&l)) return SUP_LOSS_OUTOFAREA;
// r.hit() = intersection point in coordinates of "r"
  int r=row(localhit().y());
  int c=col(localhit().x());
  int k=index(r,c);
  if (k<0||k>=numcell) {
    fprintf(stderr,"RICH: code error in Rectangle\n lasthit: ");
    localhit().print("%f %f %f\n",stderr);
    fprintf(stderr,"xsize %f ysize %f\n",xsize(),ysize());
    return 0;
    }
  if (!pdcell[k]) return SUP_LOSS_CELLEMPTY;
  else {
    p.x()=localhit()-Vec3(x(c),y(r),0);  
    p.n()=RotateFromOld(p.n());
    p.polarisation()=RotateFromOld(p.polarisation());
//
    return pdcell[k]->dodetect(p,daqid);
    }
  }

Vec3 PhotonDetectorSuperModule::pos(int row,int col,int channel) const {
  if (Cell(row,col)) {
    return TransformToOld(Cell(row,col)->pos(channel)+
                            Vec3(x(col),y(row),0));
    }
  else return Vec3(-1000,-1000,0);
  }
  
  
// --------------------------------------------------------------------------
//   initialisation and input/output
//

PhotonDetectorSuperModule::PhotonDetectorSuperModule(
     const ArtePointer<CSUP>& aa,List<PM>& PMList) :
  Rectangle(Vec3(aa->x0[0],aa->x0[1],aa->x0[2]),
    Vec3(aa->xvec[0],aa->xvec[1],aa->xvec[2]),
    Vec3(aa->yvec[0],aa->yvec[1],aa->yvec[2]),
    aa->nx,aa->xdiv,aa->ny,aa->ydiv),
   numcell(nx()*ny()) {

  xsize()=nx()*xdiv();
  ysize()=ny()*ydiv();
  
  iid=aa->id;

  pdcell = new  PhotonDetectorCell*[numcell]; 

  int i;
  for (i=0;i<numcell;i++) pdcell[i]=0;
 
  PM* pm;
  for (PMList.newiterator();(pm=PMList.iterate());) {
    if (pm->SuperModule()==id()) {
      PhotonDetectorCell* pc=new PhotonDetectorCell(pm);
      pdcell[index(pm->Row(),pm->Col())]=pc;
      }
    }
  }

void PhotonDetectorSuperModule::print(FILE* fp) {
  fprintf(fp,"#\n#\n supermodule\n");
  fprintf(fp,"  id          %d\n",id());
  fprintf(fp,"  origin      ");x0().print("%12f %12f %12f\n",fp);
  fprintf(fp,"  x_vector    ");(ex()*xsize()).print("%12f %12f %12f\n",fp);
  fprintf(fp,"  y_vector    ");(ey()*ysize()).print("%12f %12f %12f\n",fp);
  fprintf(fp,"  numcol      %2d\n",nx());  
  fprintf(fp,"  coldivision %f\n",xdiv());  
  fprintf(fp,"  numrow      %2d\n",ny());  
  fprintf(fp,"  rowdivision %f\n",ydiv());  
  fprintf(fp,"  end\n");  
  }

PhotonDetectorSuperModule::PhotonDetectorSuperModule(
     ScriptFile& f,List<PM>& PMList)  {
  char n[256],tag[256];

  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    if (RICHdebug) {printf("tag %s %s",tag,n);fflush(stdout);}
    int err=0;
    if      (!strcmp(tag,"supermodule")) continue;  
    else if      (!strcmp(tag,"origin"))   
      err=(sscanf(s,"%lf %lf %lf",&x0()[0],&x0()[1],&x0()[2])!=3);
    else if (!strcmp(tag,"x_vector")) 
      err=(sscanf(s,"%lf %lf %lf",&ex()[0],&ex()[1],&ex()[2])!=3);
    else if (!strcmp(tag,"y_vector"))   
      err=(sscanf(s,"%lf %lf %lf",&ey()[0],&ey()[1],&ey()[2])!=3);
    else if (!strcmp(tag,"numcol"))   
      err=(sscanf(s,"%d",&nx())  !=1);
    else if (!strcmp(tag,"coldivision")) 
      err=(sscanf(s,"%lf",&xdiv())!=1);
    else if (!strcmp(tag,"numrow"))   
      err=(sscanf(s,"%d",&ny())  !=1);
    else if (!strcmp(tag,"rowdivision")) 
      err=(sscanf(s,"%lf",&ydiv())!=1);
    else if (!strcmp(tag,"id")) {
      err=(sscanf(s,"%d",&iid)  !=1);
      }
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "PhotonDetectorSuperModule(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      Exit(0);
      }
    }   
    
  CSUP sm;
  sm.x0[0]=x0().x();
  sm.x0[1]=x0().y();
  sm.x0[2]=x0().z();
  sm.xvec[0]=ex().x();
  sm.xvec[1]=ex().y();
  sm.xvec[2]=ex().z();
  sm.yvec[0]=ey().x();
  sm.yvec[1]=ey().y();
  sm.yvec[2]=ey().z();
  sm.nx=nx();
  sm.xdiv=xdiv();
  sm.ny=ny();
  sm.ydiv=ydiv();
  sm.id=iid;

  ArtePointer<CSUP>::append(sm);

  Rectangle::operator=(Rectangle(x0(),ex(),ey(),nx(),xdiv(),ny(),ydiv()));
  
  numcell=sm.nx*sm.ny;  
  pdcell = new  PhotonDetectorCell*[numcell]; 

  int i;
  for (i=0;i<numcell;i++) pdcell[i]=0;
 
  PM* pm;
  for (PMList.newiterator();(pm=PMList.iterate());) {
    if (pm->SuperModule()==sm.id) {
      PhotonDetectorCell* pc=new PhotonDetectorCell(pm);
      pdcell[index(pm->Row(),pm->Col())]=pc;
      }
    }
  }

PhotonDetectorSuperModule::~PhotonDetectorSuperModule() {
  if (pdcell) delete pdcell;
  }

