
// ===========================================================================
//
//    Spherical Mirror
//

#include "rich/vessel.h"
#include "rich/Losses.h"

extern int RICHdebug;

// -------------------------------------------------------------------------
//
//  reflect: search single mirror and execute reflexion
//

int SphericalMirror::reflect(Photon* pp) {

  if (speed==1) return fullmirror_reflect(pp);

  Photon& p=*pp;
  int ret;

  double l;
  if ((ret=fullmirror_intersection(p,&l))) return ret;

  Vec3 x=p.x()+l*p.n();


  SingleSphericalMirror *sm;
  if (!(sm=GetSingleMirror(x))) return SMI_LOSS_OUTOFMIRROR;


  if ((ret=sm->reflect(pp))) return ret;

  x=RICH_Vessel->ProtonBeamShroud().TransformFromOld(pp->x());
  if (x.x()*x.x()+x.y()*x.y()<RadiusProtonHole()*RadiusProtonHole()) {
    return SMI_LOSS_PBEAMCUTOUT;
    }
  x=RICH_Vessel->ElectronBeamShroud().TransformFromOld(pp->x());
  if (x.x()*x.x()+x.y()*x.y()<RadiusElectronHole()*RadiusElectronHole()) {
    return SMI_LOSS_EBEAMCUTOUT;
    }
   return NO_LOSS;
  }

// -------------------------------------------------------------------------
//
//  hit is a position of a hit one of the full mirrors
//

SingleSphericalMirror* SphericalMirror::GetSingleMirror(const Vec3& hit) {
  Vec3 vv;
  float frow;
  int row=-1;
  if (hit.y()<0) {
    vv=hit-lsm[0]->CoC();
    frow=(+alphav0+asin(-vv.y()/r))/alphav;
    if (frow<7-alphav0/alphav) row=3-int(frow);
  }
  else {
    vv=hit-usm[0]->CoC();
    frow=(+alphav0+asin(vv.y()/r))/alphav;
    if (frow<7-alphav0/alphav) row=4+int(frow);
  }
  float fcol=(atan(vv.x()/vv.z())-alphah0)/alphah;
  int   col=-1;
  if (fcol>=0) col=int(fcol);
  SingleSphericalMirror* sm=SingleMirror(row,col); 


// check special cases
    
  if (sm&&!sm->inside(sm->TransformFromOld(hit))) {
  
    if (hit.y()>0) { // for upper half
      if (sm->row()%2==0) {
	if (sm->top&&sm->top->inside(sm->top->TransformFromOld(hit))) {
	  sm=sm->top;
	}
	else {
          if (!(sm=sm->right)) return 0;
	  if (!(sm=sm->top))   return 0;
	}
      }
      else {
        if (!sm->top||!sm->top->inside(sm->top->TransformFromOld(hit))) {
          if (!(sm=sm->right)) return 0;  
	}
        else sm=sm->top;
      }
    }
 
    else {  // for lower half
      if (sm->row()%2==1) {
        if (sm->bottom&&sm->bottom->inside(sm->bottom->TransformFromOld(hit))){
          sm=sm->bottom;
	}
        else {
          if (!(sm=sm->right))  return 0;
          if (!(sm=sm->bottom)) return 0;
	}
      }
      else {  
        if (!sm->bottom ||
            !sm->bottom->inside(sm->bottom->TransformFromOld(hit))) {
          if (!(sm=sm->right)) return 0;  
	}
        else sm=sm->bottom;
      }
    }
      
  }  // endif !sm->inside(...)

  return sm;
  
}

// -------------------------------------------------------------------------
//
//  find intersection with full mirror
//

int SphericalMirror::fullmirror_intersection(const Photon& p,double *path) {
  if (upper->intersection(p,path) ||
      p.x().y()+*path*p.n().y()<0) return lower->intersection(p,path);
  return NO_LOSS;

/*
  int ret=usm[0]->intersection(p,path);
  double hy=p.x().y()+*path*p.n().y();
  if (ret&&hy>=0) return ret;
  if (hy<0&& (ret=lsm[0]->intersection(p,path))) return ret;
  return NO_LOSS;
  */
}
  
// -------------------------------------------------------------------------
//
//  used to estimate the geometrical efficiency on track by track basis
//  and as part of the fast simulation
//

int SphericalMirror::fullmirror_reflectonly(Photon* p) {
  int ret;
  double path;
  if ((ret=fullmirror_intersection(*p,&path))) return ret;
  
  if ((ret=RICH_Vessel->traceonly(p,path))) return ret;
  
  Vec3 y=RICH_Vessel->ProtonBeamShroud().TransformFromOld(p->x());
  if (y.x()*y.x()+y.y()*y.y()<RadiusProtonHole()*RadiusProtonHole()) {
    return SMI_LOSS_PBEAMCUTOUT;
    }
  y=RICH_Vessel->ElectronBeamShroud().TransformFromOld(p->x());
  if (y.x()*y.x()+y.y()*y.y()<RadiusElectronHole()*RadiusElectronHole()) {
    return SMI_LOSS_EBEAMCUTOUT;
    }
  
// small error: outer mirror boundary check is missing
//
//   if (GetSingleMirror(pp->x())) return SMI_LOSS_OUTOFMIRROR;
//

// reflexion

  Vec3 coc;
  if (p->x().y()>0) coc=upper->CoC();
  else coc=lower->CoC();

  Vec3 ns=normalize(p->x()-coc);// normal vector at hit position
  p->n()-=double(2)*(ns*p->n())*ns;     // short hand formula for the reflection

  return NO_LOSS;
  }

  
// -------------------------------------------------------------------------
//
//  reflexion on full mirror used for fast simulation: no losses in mirror gaps
//

int SphericalMirror::fullmirror_reflect(Photon* pp) {
  int ret;
  if ((ret=fullmirror_reflectonly(pp))) return ret;
  if (rnd_normal()>reflectivity().f(pp->lambda())) return SMI_LOSS_REFLECTIVITY;
  return NO_LOSS;
  }  
 

// -------------------------------------------------------------------------
//
//  print data
//

void SphericalMirror::print(FILE *fp) {
  fprintf(fp,"#\nSphericalMirror\n");
  fprintf(fp,"  horizontal_tiling    %f %f\n",alphah,alphah0);
  fprintf(fp,"  vertical_tiling      %f %f\n",alphav,alphav0);
  fprintf(fp,"  radiusofcurvature    %f\n",r);
  fprintf(fp,"  radius_proton_hole   %f\n",radiusprotonhole);
  fprintf(fp,"  radius_electron_hole %f\n",radiuselectronhole);
  fprintf(fp,"  numberofmirrors      %d\n",nummir);
//  fprintf(fp,"  splitted          %d\n",split);
  fflush(fp);
  fprintf(fp,"#\n reflectivity\n");
  reflectivity().print(fp,0);
  fprintf(fp,"  end\n");
  fflush(fp);
  int i;
  for (i=0;i<nummir/2;i++) lsm[i]->print(fp);
  for (i=0;i<nummir/2;i++) usm[i]->print(fp);
  }


// -------------------------------------------------------------------------
void SphericalMirror::SingleMirrorNeighbours() {
  int i;
//
//  define neighbourhood in local coordinate:
//    pay attention: for the lower mirror the coordinatesystem is rotated
//
  int j;
  for (j=0;j<10;j++) {
    SingleMirror(0,j)->bottom=0;
    SingleMirror(0,j)->top   =SingleMirror(1,j);  
    SingleMirror(1,j)->bottom=SingleMirror(0,j);
    SingleMirror(1,j)->top   =SingleMirror(2,j);  
    SingleMirror(2,j)->bottom=SingleMirror(1,j);
    SingleMirror(2,j)->top   =SingleMirror(3,j);  
    SingleMirror(3,j)->bottom=SingleMirror(2,j);
    SingleMirror(3,j)->top   =0;  
    for (i=0;i<4;i++) {
      SingleMirror(i,j)->right=0;
      if (j==0) {
        SingleMirror(i,j)->left=0;
        }
      else {
        SingleMirror(i,j-1)->right=SingleMirror(i,j);
        SingleMirror(i,j)  ->left =SingleMirror(i,j-1);
        }
      }
    SingleMirror(4,j)->bottom=0;
    SingleMirror(4,j)->top   =SingleMirror(5,j);  
    SingleMirror(5,j)->bottom=SingleMirror(4,j);
    SingleMirror(5,j)->top   =SingleMirror(6,j);  
    SingleMirror(6,j)->bottom=SingleMirror(5,j);
    SingleMirror(6,j)->top   =SingleMirror(7,j);  
    SingleMirror(7,j)->bottom=SingleMirror(6,j);
    SingleMirror(7,j)->top   =0;  
    for (i=4;i<8;i++) {
      SingleMirror(i,j)->right=0;
      if (j==0) {
        SingleMirror(i,j)->left=0;
        }
      else {
        SingleMirror(i,j-1)->right=SingleMirror(i,j);
        SingleMirror(i,j)  ->left =SingleMirror(i,j-1);
        }
      }
    }

  if (RICHdebug) {
    int r,c;
    for (r=0;r<8;r++) {
      for (c=0;c<10;c++) {
        SingleSphericalMirror* s=SingleMirror(r,c);
        printf("mirror %d %d %d %d\n",r,s->row(),c,s->col());
        if (s->top)    printf("top %d %d\n",s->top->row(),s->top->col());
        if (s->bottom) printf("bot %d %d\n",s->bottom->row(),s->bottom->col());
        if (s->left)   printf("lef %d %d\n",s->left->row(),s->left->col());
        if (s->right)  printf("rig %d %d\n",s->right->row(),s->right->col());
//        getchar();
        }
      }
    xyview();
    printf("SphericalMirror end\n");fflush(stdout);  
    }
  }


// -------------------------------------------------------------------------
//
//  initialisation from input file
//

SphericalMirror::SphericalMirror(ScriptFile& f) {
  char n[256],tag[256];

  FullSimulation();
  loss=-1;
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    if (RICHdebug) { printf("%s %s",tag,n);fflush(stdout);}
    int err=0;
    if      (!strcmp(tag,"horizontal_tiling"))   
      err=(sscanf(s,"%lf %lf",&alphah,&alphah0)!=2);  
    else if (!strcmp(tag,"vertical_tiling"))   
      err=(sscanf(s,"%lf %lf",&alphav,&alphav0)!=2);  
    else if (!strcmp(tag,"radiusofcurvature"))   
      err=(sscanf(s,"%lf",&r)!=1);  
    else if (!strcmp(tag,"radius_proton_hole")) {
      err=(sscanf(s,"%lf",&radiusprotonhole)!=1); 
      } 
    else if (!strcmp(tag,"radius_electron_hole")) {
      err=(sscanf(s,"%lf",&radiuselectronhole)!=1); 
      } 
    else if (!strcmp(tag,"numberofmirrors"))  
      err=(sscanf(s,"%d",&nummir)!=1); 
    else if (!strcmp(tag,"reflectivity")) refl=Table(f);
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "SphericalMirror(ScriptFile):  Unknown tag: %s err=%d\n",tag,err);
      Exit(0);
      }
    }   

  dx=r*sin(alphah0);
  dy=2*r*sin((4.*alphav)/2.)*cos(M_PI-alphav0-(M_PI/2-(4.*alphav)/2.));
  
  CSMI sm;
  sm.ah =alphah;
  sm.ah0=alphah0;
  sm.av =alphav;
  sm.av0=alphav0;
  sm.r  =r;
  sm.rpc=radiusprotonhole;
  sm.rec=radiuselectronhole;
  sm.iref=reflectivity().firstpoint();
  sm.nref=reflectivity().numpoints();

  ArteTable<CSMI>::push_back(sm);
  
  usm=new SingleSphericalMirror*[nummir/2];
  lsm=new SingleSphericalMirror*[nummir/2];

  int i;
  SingleSphericalMirror* m;
  for (i=0;i<nummir+2;i++) {
    f.getline(n);sscanf(n,"%s",tag);
    if (RICHdebug) { printf("%s %s",tag,n);fflush(stdout);}
    if (strcmp(tag,"SingleSphericalMirror")) {
      fprintf(stderr,"SphericalMirror(ScriptFile): Not enough single spherical "
              "mirrors are defined in the input file. Abort\n");
      Exit(1);
      }
  if (RICHdebug) {printf("start scan\n");fflush(stdout);}
    m=new SingleSphericalMirror(f,refl);
    
    if (!strcmp(m->name(),"full_upper")) upper=m;
    else if (!strcmp(m->name(),"full_lower")) lower=m;
    else 
    {
      int k=index(m->row(),m->col());

      if (RICHdebug) { printf(" %d %d %d\n",m->row(),m->col(),k);fflush(stdout); }
 
      if (k<nummir/2) { k=index(3-m->row(),m->col());lsm[k]=m;}
      else usm[k-nummir/2]=m;
      }
    }

  SingleMirrorNeighbours();
  }
  

// -------------------------------------------------------------------------
//
//  init from ARTE table
//

SphericalMirror::SphericalMirror(ArtePointer<CSMI> sm) {
  FullSimulation();
  loss=-1;

  alphah =sm->ah;
  alphah0=sm->ah0;
  alphav =sm->av;
  alphav0=sm->av0;
  r      =sm->r;
  radiusprotonhole =sm->rpc;
  radiuselectronhole=sm->rec;
  
  if (RICHdebug) { printf("refl:  %d %d\n",sm->iref,sm->nref);fflush(stdout);}
 
  refl=Table(sm->iref,sm->nref);
  nummir=ArtePointer<CSSM>::nrow()-2;
 
  dx=r*sin(alphah0);
  dy=2*r*sin((4.*alphav)/2.)*cos(M_PI-alphav0-(M_PI/2-(4.*alphav)/2.));

  if (RICHdebug) { printf(" num mirror %d \n",nummir);fflush(stdout); }
  usm=new SingleSphericalMirror*[nummir/2];
  lsm=new SingleSphericalMirror*[nummir/2];

  int i;
  SingleSphericalMirror* m;
  for (i=1;i<=(int)ArtePointer<CSSM>::nrow();i++) {
    ArtePointer<CSSM> a(i);
    m=new SingleSphericalMirror(a);
    
    if (!strcmp(m->name(),"full_upper")) upper=m;
    else if (!strcmp(m->name(),"full_lower")) lower=m;
    else {
      int k=index(m->row(),m->col());

      if (RICHdebug) { printf(" %d %d %d\n",m->row(),m->col(),k);fflush(stdout); }
 
      if (k<nummir/2) { k=index(3-m->row(),m->col());lsm[k]=m;}
      else usm[k-nummir/2]=m;
      }
    }

  SingleMirrorNeighbours(); 
  }


SphericalMirror::~SphericalMirror() {
  int i;
  for (i=0;i<nummir;i++) {      
    if (i<nummir/2) delete usm[i];
    else delete  lsm[i-nummir/2];
    }
  delete usm;
  delete lsm;
  }



void SphericalMirror::xyview() {
  FILE* fp=fopen("mirror.kumac","w");
  int i,j;
  for (i=0;i<8;i++) {
    for (j=0;j<10;j++) {
      SingleMirror(i,j)->xyview(fp);
      }
    }
  fclose(fp);
  }

