#include "rich/ricutil.h"
#include <stdarg.h>

extern int RICHdebug;

int RichRun::type=0;
ArtePointer<GEDE> RichRun::gede=0;
int RichRun::magnet=1;
int RichRun::cleanHot=1;

//
//  get intersection point
//
int Tube::intersection(const StraightLine& p,double *ll) {
  Vec3 x=TransformFromOld(p.x());
  Vec3 n=RotateFromOld(p.n());
  
  double sinthe=1-n.z()*n.z();
  if (sinthe<1E-14) {
    if (radius()*radius()>x[0]*x[0]+x[1]*x[1]) return 1;
    return 0;
    }
  sinthe=sqrt(sinthe);

  double xn=(x[0]*n[0]+x[1]*n[1])/sinthe;
  double a=radius()*radius()-x[0]*x[0]-x[1]*x[1]+xn*xn;

  if (a<0) return 0;
  a=sqrt(a);
  double l=-xn-a;

  if (l<0) {
    l=-xn+a;
    if (l<0) return 0;
    }
  *ll=l/sinthe;
  double z=x[2]+*ll*n[2];
  if (fabs(z)>0.5*length()) return 0;

  return 1;  
  }
  
// -------------------------------------------------------------------------
//
//

void Tube::print(FILE *fp,char *name) {
  if (name) fprintf(fp,"#\n%s\n",name);
  fprintf(fp,"  center     ");x0().print("%12f %12f %12f\n",fp);
  fprintf(fp,"  axis       ");(length()*ez()).print("%12f %12f %12f\n",fp);
  fprintf(fp,"  radius     %12f\n",radius()); 
  fprintf(fp,"  end\n"); 
  }

// -------------------------------------------------------------------------
//
//

Tube::Tube(ScriptFile& f) {
  char n[256],tag[256];
  Vec3 axis;
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    if (RICHdebug) printf("line %s tag   %s\n",n,tag);
    int err=0;
    if (!strcmp(tag,"Tube")) {
      continue;
      }
    else if (!strcmp(tag,"center")) {
      err=sscanf(s,"%lf %lf %lf",&x0()[0],&x0()[1],&x0()[2]);
      }
    else if (!strcmp(tag,"axis")) {
      err=sscanf(s,"%lf %lf %lf",&axis[0],&axis[1],&axis[2]);
      }
    else if (!strcmp(tag,"radius")) {
      err=sscanf(s,"%lf ",&r);
      }
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "Tube(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      Exit(0);
      }
    }   
  l=norm(axis);
  setrotation(axis); // x,y arbitrary, but perpendicular to axis.
  }

// ===========================================================================

Rectangle::Rectangle(const Vec3& c1,const Vec3& c2,const Vec3& c3,
                       int nc,double xc,int nr,double yc) {

//
//  coordinate system for boundary check
//
  x0()=c1;  
  ex()=normalize(c2);
  ey()=normalize(c3);
  ez()=normalize(cross(ex(),ey()));
  
  dd=-(x0()*ez());

  xdim=norm(c2);
  ydim=norm(c3);
  nnx=nc;
  if (nnx==1) xcell=xdim;
  else xcell=xc;
  if (xcell!=0) xdim=nnx*xcell;
  nny=nr;
  if (nnx==1) ycell=ydim;
  else ycell=yc;
  if (ycell!=0) ydim=nny*ycell;

  if (RICHdebug) {
    CoordinateSystem::print();
    printf("xdim: %e ydim: %e\n",xdim,ydim);
    }
  }




int Rectangle::intersection(const StraightLine& p,double *ll) {
//
//  get intersection point
//
  double s=p.n()*ez();
  if (fabs(s)<1E-14) return 0;
  double l=-(dd+ez()*p.x())/s;
  lasthit=p.x()+l*p.n();        // intersection point in global coordinates
  *ll=l;
  lastlhit=TransformFromOld(lasthit);// intersection point in local  coordinates
  if (lastlhit.x()<0||lastlhit.x()>xdim) return 0;
  if (lastlhit.y()<0||lastlhit.y()>ydim) return 0;
  return 1;  
  }


/*
int Rectangle::reflect(Photon* pp) {
  Photon& p=*pp;
  if (!intersection(pp)) return 0;
  p.n()-=2*(p.n()*ez())*ez();
  return 1;
  }
 */ 
  
void Rectangle::print(FILE* fp,char *name) const {
  if (name) fprintf(fp,"#\n%s\n",name);
  fprintf(fp,"  origin      ");x0().print("%f %f %f\n",fp);
  fprintf(fp,"  x_vector    ");(xdim*ex()).print("%f %f %f\n",fp);
  fprintf(fp,"  y_vector    ");(ydim*ey()).print("%f %f %f\n",fp);
  if (xcell>0) {
    fprintf(fp,"  numcol      %d\n",nnx);
    fprintf(fp,"  coldivision %f\n",xcell);
    fprintf(fp,"  numrow      %d\n",nny);
    fprintf(fp,"  rowdivision %f\n",ycell);
    }
  fprintf(fp,"  end\n");
  }  
  
Rectangle::Rectangle(ScriptFile& f) {
  char n[256],tag[256];
  nnx=nny=1;
  xcell=ycell=0;
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag); s+=f.itemlength(s,1);
    if      (!strcmp(tag,"rectangle")) {
      continue;
      }
    else if      (!strcmp(tag,"origin")) {
      sscanf(s,"%lf %lf %lf",&x0()[0],&x0()[1],&x0()[2]);
      }
    else if (!strcmp(tag,"x_vector")) {
      sscanf(s,"%lf %lf %lf",&ex()[0],&ex()[1],&ex()[2]);
      }
    else if (!strcmp(tag,"y_vector")) {
      sscanf(s,"%lf %lf %lf",&ey()[0],&ey()[1],&ey()[2]);
      }
    else if (!strcmp(tag,"numcol")) {
      sscanf(s,"%d",&nnx);
      }
    else if (!strcmp(tag,"coldivision")) {
      sscanf(s,"%lf",&xcell);
      }
    else if (!strcmp(tag,"numrow")) {
      sscanf(s,"%d",&nny);
      }
    else if (!strcmp(tag,"rowdivision")) {
      sscanf(s,"%lf",&ycell);
      }
    else if (!strcmp(tag,"end")) break;
    else {
      fprintf(stderr,"Rectangle(ScriptFile& f): Unknown tag: %s\n",tag);
      Exit(1);
      }
    }
  xdim=norm(ex());
  ydim=norm(ey());
  ex()/=xdim;
  ey()/=ydim;
  ez()=cross(ex(),ey());
  dd=-(x0()*ez());  
  if (nnx>1) xdim=xcell*nnx;
  if (nny>1) ydim=ycell*nny;
  
  }
  
  



    
// -----------------------------------------------------------------------

  
Polygon::~Polygon() {
  if (num) {
    free(xx);
    free(yy);
    if (nnx) {
      free(nnx);
      free(nny);
      free(dd);
      }
    }
  }
  
void Polygon::set(double x,double y) {
  num++;
  xx=(double*)realloc(xx,num*sizeof(double));
  yy=(double*)realloc(yy,num*sizeof(double));
  xx[num-1]=x;
  yy[num-1]=y;
  }
  

int Polygon::inside(double x,double y) { 
  if (!nnx) {
    nnx=(double*)malloc(num*sizeof(double));
    nny=(double*)malloc(num*sizeof(double));
    dd =(double*)malloc(num*sizeof(double));
    
    int i;
    for (i=0;i<num;i++) {
      int j= (i==0) ? num-1 : i-1;
      nnx[i]= (yy[i]-xx[j]);
      nny[i]=-(xx[i]-yy[j]);
      dd [i]=-(xx[i]*nnx[i]+yy[i]*nny[i]);
      } 
    }
  int i;
  int is=1,side=0;
  for (i=0;i<num;i++) {
    float s = nnx[i]*x+nny[i]*y+dd[i];
    is= (s<0) ? -1 : 1;
    if (side==0) side=is;
    else if (!(is==side)) break;
    }
  return (is==side);
  
  }
 
// -------------------------------------------------------------------------

ScriptFile::ScriptFile(char *fn) {
  fp=fopen(fn,"r");
  if (!fp) {
    fprintf(stderr,"Cannot open ScriptFile: %s\n",fn);
    Exit(0);
    }
  bfr=0;
/*
  fseek(fp,0,SEEK_END);
  bufsize=ftell(fp)+1;
  fseek(fp,0,SEEK_SET);
  
  bfr=(char*)malloc(bufsize);
  fread(bfr,1,bufsize-1,fp);
  fclose(fp);
  bfr[bufsize-1]='\0';
  s=bfr;

  if (*s=='#') {
    do {
      while (*s++!='\n') ;
      } while (*s=='#');
    }
*/
  }

ScriptFile::~ScriptFile() {
  if (bfr) free(bfr);
  fclose(fp);
  }

int ScriptFile::itemlength(char *s,int nitem) {
  int i;
  char* s0=s;
  for (i=0;i<nitem;i++) {
    while (*s==' '||*s=='\t') s++;
    while (*s!=' '&&*s!='\t'&&*s!='\n') s++;
    }
//
//  skip comment lines
//
  if (*s=='\n') s++;
  while (*s=='#') {
    while (*s++!='\n');
    }
  return s-s0;
  }
  

int ScriptFile::scan(char *fmt, ...) {
  if (s[0]=='\0') return 0;

  char f[20];
  va_list ap;
  va_start(ap,fmt);
  int nitem=0;
  int i=0;
  do {
    while (*fmt==' '||*fmt=='\t') fmt++;
    while (fmt[i]!=' '&&fmt[i]!='\t'&&fmt[i]!='\0') i++;
    if (i==0) break;
    memcpy(f,fmt,i);
    f[i]='\0';
      
    char *n=va_arg(ap,char*);
    if (!sscanf(s,f,n)) return nitem;
    s+=itemlength(s,1);
    nitem++;
    if (s[0]=='\0') break;
    } while((fmt+=i)!='\0');
  va_end(ap);
  return nitem;
  }


int ScriptFile::getline(char s[256]) {
  while (!feof(fp)) {
    if (!fgets(s,256,fp)) return 0;
    if (!(s[0]=='#'||s[0]=='\n')) return 1;
    };
  return 0;
  }

int ScriptFile::skipline(char s[256]) {
  while (!feof(fp)) {
    if (!fgets(s,256,fp)) return 0;
    if (s[0]!='#') return 1;
    };
  return 0;
  }

int ScriptFile::gettag(char s[256]) {
  while (!feof(fp)) {
    if (fscanf(fp,"%s",s)!=1) return 0;
    if (s[0]!='#') return 1;
    else fgets(s,256,fp);
    };
  return 0;
  }

//=========================================================================
  
Table::~Table() {
  delete h;
  if (num) {
// remove table ??
    }
  }
 
void Table::append(float x,float y) {
  if (num==0) ifirst=ArtePointer<CPOI>::nrow()+1;
  num++;
  CPOI p;
  p.x[0]=x;
  p.x[1]=y;
  ArteTable<CPOI>::push_back(p);
  ffl=1;
  }
  
double Table::LinearInterpolation(const double xx) const {
  int i;
  for(i=0;i<num;i++) {
    if(x(i)>=xx) break;
    } 
  if (i==0)   return y(0);
  if (i==num) return y(num-1);
  return y(i)+(y(i)-y(i-1))/(x(i)-x(i-1))*(xx-x(i));
  }


double Table::CubicInterpolation(const double xx) const {
  int i;
  for(i=0;i<num;i++) {
    if(x(i)>=xx) break;
    } 
  if (i==0)   return y(0);
  if (i==num) return y(num-1);
  if (i==num-1) i-=3;
  else i-=2;

  float f[4],a[4];
  int j;
  for (j=0;j<4;j++) {
    a[j]=x(i+j);
    f[j]=y(i+j);
    }
  return polint(f,a,4,xx);
  }

void Table::finit() {
  ffl=0;
  if (h) delete h;
  nx=100*numpoints();
  xxmin=xmin();
  xxmax=xmax();
  dx=(xxmax-xxmin)/(nx-1);
  h=new double[nx];
  int i;
  for (i=0;i<nx;i++) h[i]=LinearInterpolation(xxmin+i*dx);
  }

 double  Table::f(const double xx)   { 
 // return LinearInterpolation(xx);
    if (ffl) finit();
    double ret;
    if (xx<=xxmin) ret=h[0];
    else if (xx>=xxmax) ret=h[nx-1];
    else {
      int i=int((xx-xxmin)/dx);
      ret= (h[i+1]-h[i])/dx*(xx-(i*dx+xxmin))+h[i];
      }
    return ret;
    }
  

Table::Table(ScriptFile& f) {
  char n[256],tag[256];
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    int err=0;
    if      (!strcmp(tag,"table")) {
      continue;
      }
    else if      (!strcmp(tag,"numpoints")) {   
      err=(sscanf(s,"%d",&num)!=1);
      if (num==0) {
        fprintf(stderr,"Table without points makes no sense. Remove table from input.\n");
        Exit(1);
        }
      ifirst=ArtePointer<CPOI>::nrow()+1;
      CPOI p;
      int i;
      for (i=0;i<numpoints();i++) {
        int err=!f.getline(n);
        err|=(sscanf(n,"%e %e\n",&p.x[0],&p.x[1])!=2);
        if (RICHdebug) printf("%e %e\n",p.x[0],p.x[1]);
        if (err) {
          fprintf(stderr,
            "Table(ScriptFile):  Error during scanning values  %d\n",err);
          Exit(1);
          }
        ArteTable<CPOI>::push_back(p);
        }
      break;
      }
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "Table(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      Exit(1);
      }
    } 
  h=0;
  finit();
  }
 
void Table::print(FILE *fp,char *name) const {
  if (name) fprintf(fp,"#\n%s\n",name);
  fprintf(fp,"  numpoints  %d\n",numpoints());
  int i;
  for (i=0;i<numpoints();i++) {
    fprintf(fp,"    %12e %12e\n",x(i),y(i));
    }
  if (name) fprintf(fp,"  end\n");
  }

