
//**********************************************************************/
// Created: 08.03.99, by Ignacio Ari\~no   for the RICH reconstruction 
//         from tracks
//
// implementation of RichTrack class
//
//**********************************************************************/


#include <arte/arte.hh>

#include "rire/richreco.hh"

#include "rich/PhiLambda.h"

#include "CLHEP/Hist/HBookTuple.h"

//float geomaccept(Hep3Vector xin,Hep3Vector dir,double costhe,int n);
//float geomeff(float *x,float *p,float *costhe,int *n);

RichTrack::RichTrack()
{
  track= new RcSegPoint();
  Readconstantsfromfile();
    //default constructor
}

RichTrack::RichTrack(RcSegPoint rcsp)
{  
  Readconstantsfromfile();
  track= new RcSegPoint(rcsp);
  track->prolongate(ZMidRich()); //prolongate to middle of RICH
}


RichTrack::RichTrack(RcMatchPoint rcmp)
{
  Readconstantsfromfile();
  track = new RcSegPoint(rcmp);
  track->prolongate(ZMidRich()); //prolongate to middle of RICH
}

RichTrack::RichTrack(Hep3Vector point, Hep3Vector direction, float momentum )
{
  Readconstantsfromfile();
  HepSymMatrix cov(5,0); //make zero cov matrix

  RcPoint rcp(point,direction.x(),direction.y(),1/momentum,cov,0);
  //track = new RcSegPoint(point,direction.x(),direction.y(),1/momentum,cov,0);
  track = new RcSegPoint(rcp);

  //track->setcov(3,3,0.000001);
  //track->setcov(4,4,0.000001);
  track->setcov(3,3,0.00000025);  //0.0005^2
  track->setcov(4,4,0.00000025);

  track->prolongate(ZMidRich()); //prolongate to middle of RICH

  //beampipe shadow
  SetShadow();

  //corrections
  float maxradius=acos(1/Meanrefrindex);  //max radius in rad
  
  float x0=(point.x()-direction.x()*point.z())*0.01; //displacemnts at z=0 (m)
  float y0=(point.y()-direction.y()*point.z())*0.01;

  //old
  //corr.newRing(atan(direction.x()),atan(direction.y()),maxradius,x0,y0);
  
  RICH_LambdaPhi->Correct()->newRing(atan(direction.x()),atan(direction.y())
				     ,maxradius,x0,y0);
}


//constructor from cluster (straight line to target) (only for magnet off now)
RichTrack::RichTrack(ArtePointer<RCCL> aprccl)
{
  Readconstantsfromfile();
  
  //provisional!!!!!!!!  
  float targetx=0.67; //provisional run 1335
  float targety=0;
  float targetz=-4.7;
  
//   float targetx=0.2; //MC
//   float targety=0;
//   float targetz=-0.6;
  
  float tx=(aprccl->x-targetx)/(aprccl->z-targetz);
  float ty=(aprccl->y)/(aprccl->z-targetz);
  
  Hep3Vector point(aprccl->x,aprccl->y,aprccl->z);

  RcPoint rcp(point,tx,ty,1/aprccl->esum,aprccl->cve,0);
  track = new RcSegPoint(rcp);
  //error in cluster energy to error in 1/p
  track->setcov(5,5,track->cov()(5,5)/pow(aprccl->esum,2));

  track->prolongate(ZMidRich());
  
  //beampipe shadow
  SetShadow();

  //corrections
  double maxradius=acos(1/Meanrefrindex);//max radius in rad
  //  corr.newRing(atan(tx),atan(ty),maxradius,0,0); //old
           
  RICH_LambdaPhi->Correct()->newRing(atan(tx),atan(ty),maxradius,0,0);

}

RichTrack::RichTrack(ArtePointer<RTRA> aprtra)
{
  Readconstantsfromfile();

  //look for RPNT closer to the begin of RICH
  ArtePointer<RPNT> closerpnt;
  for(ArteRelationIterator<RPNT> rpntit=aprtra->rpnt.begin();
      rpntit!=aprtra->rpnt.end();rpntit++)
    {
      if((*rpntit)->zf>closerpnt->zf && (*rpntit)->zf<zRichmin)
	closerpnt=*rpntit;
    }
  Hep3Vector point(closerpnt->xf,closerpnt->yf,closerpnt->zf);

  //if there is no point after magnet give momentum 0
  //so that no likelihoods are calculated (provisional)
  if(closerpnt->zf>300){
    RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,closerpnt->pf,
		closerpnt->cvf,0);
    track = new RcSegPoint(rcp);
    track->prolongate(ZMidRich());
  }
  else{
    RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,10000001,
		closerpnt->cvf,0);
    track = new RcSegPoint(rcp);
    //  track->prolongate(ZMidRich());
  }

  //beampipe shadow
  SetShadow();
  
  float maxradius=acos(1/Meanrefrindex);//max radius in rad
  float x0=(point.x()-closerpnt->txf*point.z())*0.01;//displacements at z=0 (m)
  float y0=(point.y()-closerpnt->tyf*point.z())*0.01;

  //old
  //  corr.newRing(atan(closerpnt->txf),atan(closerpnt->tyf),maxradius,x0,y0);

  RICH_LambdaPhi->Correct()->newRing(atan(closerpnt->txf),atan(closerpnt->tyf)
				     ,maxradius,x0,y0);
}


RichTrack::~RichTrack()
{
  delete track;
}





void RichTrack::Readconstantsfromfile()
{
  //find vessel dimensions from ARTE tables.
  for(ArteTable<GESL>::iterator geslit=ArteTable<GESL>::begin();
      geslit!=ArteTable<GESL>::end();geslit++){
    if((*geslit)->cmp==5){
      zRichmin=(*geslit)->z1;
      zRichmax=(*geslit)->z2;
      break;
    }
  }

  //provisional: constants set here
  //Meanrefrindex=1.00028; //air (aprox)
  Meanrefrindex=1.00133;  //to be read from a database (some day)
  //Nphotbeta1=7; //air (aprox)
  Nphotbeta1=35;
  Sigma_spherical=0.00016;//not fully understood 
  Sigma_cromatic=0.0005;
  Sigma_cellsize=2/(1140*sqrt(12)); //to be multiplied by the cellsize
}




void RichTrack::SetShadow()
{
  float azi0=atan(track->y()/track->x());
  if(track->x() < 0) azi0+=M_PI;
  if(track->x() > 0 && track->y() < 0) azi0+=2*M_PI;

  //only for tracks
  if(track->y()>0) 
    azi0+=M_PI;
  else
    azi0-=M_PI;

  float dist=sqrt(track->x()*track->x() + track->y()*track->y());
  ArtePointer<CVES> apcves(1);
  float bprad=(apcves->prad)/10;

  float theta=asin(bprad/dist);
    
  if(dist>bprad){
    minazi=azi0-theta;
    maxazi=azi0+theta;
  }
  else
    {
      minazi=10;
      maxazi=10;
    }
}




double RichTrack::NumExpPhotons(int hyp)
{

  //beam pipe shadow
  if(minazi==10 && maxazi==10) return 0;

  float shadow=(maxazi-minazi)/(2*M_PI);

  double beta=sqrt(1-pow(masshyp(hyp)*track->pe(),2));
  if(beta<1/Meanrefrindex) return 0;
  //  double ctheta=1/(beta*Meanrefrindex);

  //geometrical efficiency
  //  float geomacpt=geomaccept(RichEntryPoint(),RichDir(),ctheta,20);

  double lrad=300; //length of radiator the track crosses
  //provisional!!!

  double nphotons= lrad/Radlength() * Nphotbeta1 * (1-shadow) *
    (1-1/pow(Meanrefrindex*beta,2))/(1-1/pow(Meanrefrindex,2));
  return nphotons;  
}




int RichTrack::NumCandidates(int nhyp, RichHits* richhits)
{
  int numcand=0;
  double sigma2=pow(SigmaPhiTrack(),2)+pow(SigmaLambdaTrack(),2)+
    pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);

  double beta=sqrt(1-pow(masshyp(nhyp)*track->pe(),2));
  if(beta<1/Meanrefrindex) return 0;
  double theta_calc=acos(1/(beta*Meanrefrindex));
			
  for(int i=0;i<richhits->numhits();i++)
    {
      double hitcell=richhits->Cellsize(i);
      double sigma=sqrt(sigma2 + pow(Sigma_cellsize*hitcell,2));

      //beam pipe shadow
      int shadow=0;
      float azi=atan((richhits->Lambda(i)-TrackLambda())/
		     (richhits->Phi(i)-TrackPhi()));
      if(richhits->Phi(i)-TrackPhi() < 0) azi+=M_PI;
      if(richhits->Phi(i)-TrackPhi() > 0 && 
	 richhits->Lambda(i)-TrackLambda() <0) azi+=2*M_PI;
      if(azi>minazi && azi<maxazi) shadow=1;

      if(fabs(RelTheta(richhits->Phi(i),richhits->Lambda(i),richhits->Up(i))
	      -theta_calc)<3*sigma && !shadow)
	{
	  numcand++;
	}
    }
  return numcand; 
}


 

double RichTrack::FindLikelihood(int ihyp,RichHits *richhits)
{
  //likelihood=Ppoisson*Prod_i=1,Ncand(Pcand)
  //Ppoisson=(Nexp+Nback)**Ncand * exp(-(Nexp+Nback))/Ncand!
  //Pcand=fs * 1/(sqrt(2*pi))*exp(-(theta_i-theta_calc)**2/2*sigma_i**2)
  //      + fb * theta_i/(6*theta_calc)
  // fs= Nexp/(Nexp+Nback)
  // fb= Nback/(Nexp+Nback)
  //Nback is a parameter to be found maximizing likelihood

  //sigmas
  //(consider both directions as independent?)
  double sig_center=sqrt(pow(SigmaPhiTrack(),2)+pow(SigmaLambdaTrack(),2)); 
  double sig2=pow(sig_center,2)+pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);

  double beta=sqrt(1-pow(masshyp(ihyp)*track->pe(),2));
  if(beta<1/Meanrefrindex){
    lnlikelihoods[ihyp]=0;
    return 0;
  }
  double theta_calc=acos(1/(beta*Meanrefrindex));


  //in case another track has been initialised before the likelihood has 
  //been evaluated (provisional)
  //displacements at z=0 (m)
  float x0=(RichEntryPoint().x()-RichDir().x()*RichEntryPoint().z())*0.01;
  float y0=(RichEntryPoint().y()-RichDir().y()*RichEntryPoint().z())*0.01;
  RICH_LambdaPhi->Correct()->newRing(atan(RichDir().x()),atan(RichDir().y())
				     ,theta_calc,x0,y0);
  
			 
  //find the candidate hits:
  vector<RHit*> VecCand;
  for(int i=0;i<richhits->numhits();i++)
    {
      double sigma=sqrt(sig2 + pow(Sigma_cellsize*richhits->Cellsize(i),2));

      //beam pipe shadow
      int shadow=0;
      float azi=atan((richhits->Lambda(i)-TrackLambda())/
		     (richhits->Phi(i)-TrackPhi()));
      if(richhits->Phi(i)-TrackPhi() < 0) azi+=M_PI;
      if(richhits->Phi(i)-TrackPhi() > 0 && 
	 richhits->Lambda(i)-TrackLambda() <0) azi+=2*M_PI;
      if(azi>minazi && azi<maxazi) shadow=1;

      if(fabs(RelTheta(richhits->Phi(i),richhits->Lambda(i),richhits->Up(i))
	      -theta_calc)<3*sigma && !shadow)
	{
	  RHit *rh=new RHit(richhits->rhit(i));
	  VecCand.push_back(rh);
	}
    }
  double nexpp=NumExpPhotons(ihyp);

  //try to fit a(background parameter) maximizing the likelihood:
  double a=0;  //check the first assumption
  if(nexpp<VecCand.size()) a=VecCand.size()-nexpp;
  double aold=-1000;
  int niter=0;
  double likelihood=0;
  double d1lnlike=0;
  double d2lnlike=0;
  double likeback=0;
   
  if(VecCand.size()!=0){
    while(fabs(a-aold)>0.01 && niter<20)
      {
	likelihood=lnLike(VecCand,nexpp, a, ihyp, &d1lnlike, &d2lnlike);
	if(fabs(a)>5000) break;
	aold=a;
	a-=(d1lnlike/d2lnlike)*0.99;
	if (a<0) a=0;
	niter++;
      }

     //likelihood for background (Nexp=0)
     for(unsigned int i=0;i<VecCand.size();i++)
       {
	 double thetai=RelTheta(VecCand[i]->Phi(),VecCand[i]->Lambda(),
				VecCand[i]->Up());
 	 likeback+=log(thetai/(6*ThetaCer(ihyp)));
       }
   }
   else{
     likelihood=-nexpp;
   }

  //delete VecCand
  for(vector<RHit*>::iterator vcit=VecCand.begin();
      vcit!=VecCand.end();vcit++)
    {
      delete *vcit;
    }
  VecCand.erase(VecCand.begin(),VecCand.end());

  if(niter<20)
    {
//       cout<<"converged in "<<niter<<" to a="<<a<<"nexpp "<<nexpp<<
// 	" ncand="<<ncand<<
//       	" lnlike= "<<likelihood<<" lnlikeback="<<likeback<<endl;

      lnlikelihoods[ihyp]=likelihood-likeback;
      return likelihood-likeback;
    }
  else
    {
//       printf("not converged, a=%f aold=%f lnlike=%flnlikeback=%f\n",
// 	     a,aold,likelihood,likeback);
      lnlikelihoods[ihyp]=10000;
      return 10000;
    }
}



 
double RichTrack::RelTheta(double hitphi,double hitlambda,int up=0)
{

  //   return sqrt(pow(TrackPhi()-hitphi,2)+pow(TrackLambda()-hitlambda,2));
  //corrections for spherical aberrations
  /* old
     float D_horiz=(up) ? corr.upperDh() : corr.lowerDh();
     float D_verti=(up) ? corr.upperDv() : corr.lowerDv();
     float P_horiz=(up) ? corr.upperPh() : corr.lowerPh();
     float P_verti=(up) ? corr.upperPv() : corr.lowerPv();
     
     float dx=(TrackPhi()-hitphi-P_horiz)/D_horiz;
     float dy=(TrackLambda()-hitlambda-P_verti)/D_verti;
  */
  
  float dphi= RICH_LambdaPhi->Correct()->dPhi(hitphi,hitlambda,up);
  float dlam=RICH_LambdaPhi->Correct()->dLambda(hitphi,hitlambda,up);

  //alignment corrections?
  //  float dy=(TrackLambda()-hitlambda-P_verti+0.001)/D_verti;

  return sqrt(dphi*dphi + dlam*dlam);
}




double RichTrack::lnLike(vector<RHit*> VecCand,double nexpp,double a,int ihyp, 
			 double *d1lnlike,double *d2lnlike)
{

  //(consider both directions as independent?)
  double sig_center=sqrt(pow(SigmaPhiTrack(),2)+pow(SigmaLambdaTrack(),2));
  double sig2=pow(sig_center,2)+pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);

  double ncand=VecCand.size();
  
  double fs=nexpp/(nexpp+a);
  double fb=a/(nexpp+a);
  double dfs=-fs/(nexpp+a);
  double d2fs=2*fs/pow((nexpp+a),2);
  
  double lnlike=ncand*log(nexpp+a)-(nexpp+a)-(ncand*log(ncand)-ncand);
  *d1lnlike=(ncand/(nexpp+a)-1);  
  *d2lnlike=-ncand/pow((nexpp+a),2);

  for(unsigned int i=0;i<VecCand.size();i++)
    {
      double thetai=RelTheta(VecCand[i]->Phi(),VecCand[i]->Lambda(),
			     VecCand[i]->Up());
      double sigmai=sqrt(sig2+pow(Sigma_cellsize*VecCand[i]->Cellsize(),2));
      double expi=exp(-0.5*pow((thetai-ThetaCer(ihyp))/sigmai,2))/sqrt(2*M_PI);

      double probi=fs*expi + fb*thetai/(6*ThetaCer(ihyp));

      lnlike+=log(probi);
      *d1lnlike+=(dfs*expi - dfs*thetai/(6*ThetaCer(ihyp)))/probi;
      *d2lnlike+=(d2fs*expi - d2fs*thetai/(6*ThetaCer(ihyp)))/probi -
	pow((dfs*expi - dfs*thetai/(6*ThetaCer(ihyp)))/probi,2);
    } 
  
  return lnlike;
}








