#include "arte/arte.hh"
#include "rich/PhiLambda.h"
#include "rise/RICHFit.h"



extern "C" {

  void myhfithn_(int const &id,
		 int const &np,
		 float *par,
		 float const *step,
		 float const *pmin,
		 float const *pmax,
		 float *sigpar,
		 float &chi2);


  void myhbook1_(int   const &id,
		 int   const &nx,
		 float const &xmi,
		 float const &xma);

  void hfill_(int const &id,
	      float const &x,
	      float const &y,
	      float const &weight);

  void hprint_(int const &id);
  void hdelet_(int const &id);

  float hx_(int   const &id,
	    float const &x);

  void myhreset_(int const &id);

	   }



RICHLikelihood::~RICHLikelihood() {
  hdelet_(id);
}



RICHLikelihood::RICHLikelihood() : id(1696), nx(24), xmi(-7), xma(7) {
  lnL=0;
  sigma=0.7;
  n=0;
  b0=0;
  b1=0;
  b=0;
  limit=20;


  // don't do this for every event and in many places
  rich_gede=0;

  if (!rich_gede)   
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin(); 
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if (igede->cmp == 5) {
	rich_gede=igede.index();
	break;
      }
    }

  // -- create histo
  myhbook1_(id,nx,xmi,xma);

}


int RICHLikelihood::collect(float phi,float lam,float rad) {
  tPhi=phi;
  tLam=lam;
  tRad=rad;
  float invSigma=1/sigma;

  reset();
  RICH_LambdaPhi->Correct()->newRing(tPhi*1e-3,tLam*1e-3,tRad*1e-3);


  ArtePointer<GEDE> gede(rich_gede);

  int nup=0,nlo=0;
  for (ArteTable<HITB>::iterator ihitb=gede->hit1;
       ihitb!=gede->hitl; ++ihitb) {
    int plane=(*(*ihitb)->digb.begin())->id/7000000;

    float dPhi=RICH_LambdaPhi->Correct()->dPhi   ((*ihitb)->q,(*ihitb)->r,plane)*1e3;
    float dLam=RICH_LambdaPhi->Correct()->dLambda((*ihitb)->q,(*ihitb)->r,plane)*1e3;
    float dRad=( tRad - sqrt(dPhi*dPhi + dLam*dLam) )*invSigma;
    if (fabs(dRad)<limit) {
      append( hitAngle(dRad,(*ihitb)->q*1000,(*ihitb)->r*1000,float(plane)) );
      //cout<<tRad<<" - "<<sqrt(dPhi*dPhi + dLam*dLam)<<" = "<<dRad<<" "<<limit<<" "<<list.size()<<endl;
      if (dRad<3) {
	if (plane) nlo++;
	else nup++;
      }
    }
  }
  int ret=0;
  s3s=0;
  f=2;
  if (nlo || nup) {
    ret=1;
    f=float(nup-nlo)/float(nup+nlo);
    s3s=nlo+nup;
  }
  return ret;
}



void RICHLikelihood::print() {

  int num=0;
  for (vector<hitAngle>::iterator it=list.begin(); it!=list.end(); ++it) {
    cout<<"photon "<< num++ <<" : "<< (*it).dRad<<endl; 
  }
}



int RICHLikelihood::fillRadiusHistogram() {

  const float weight=1;
  const float y=0;
  int num=0;
  for (vector<hitAngle>::iterator it=list.begin(); it!=list.end(); ++it) {
    const float dRad=(*it).dRad;
    if (dRad>xmi && dRad<xma) {
      hfill_(id,dRad,y,weight);
      num++;
    }
  }
  return num;
}

void RICHLikelihood::printRadiusHistogram() {
  hprint_(id);
}

void RICHLikelihood::resetRadiusHistogram() {
  myhreset_(id);
}



/*void RICHLikelihood::fitRadiusHistogram() {

  // -- fit histo
  const int np=4;
  float par[]={10,0,0.7,0,0},sigpar[np];
  const float step[np],pmin[np],pmax[np];
  myhfithn_(id,np,par,step,pmin,pmax,sigpar,chi2);

  n=par[0];
  mu=par[1];
  sigma=fabs(par[2]);
  b0=par[3];
  //b1=par[4];

  s=0;
  b=0;
  float dx=(xma-xmi)/float(nx);
  float x0=xmi+.5*dx;
  for (int i=0; i<nx; i++) {
    float x=x0+i*dx;
    float t=(x-mu)/sigma;
    float signal=n*exp(-.5*t*t);
    float backgr=b0;//+b1*x;
    if (signal) s+=signal/(signal+backgr)*hx_(id,x);
    //cout<<signal/(signal+backgr)<<" "<<backgr/(signal+backgr)<<" "<<hx_(id,x)<<" "<<x<<endl;
    if (backgr) b+=backgr/(signal+backgr)*hx_(id,x);
  }
  myhreset_(id);
  }*/


void RICHLikelihood::fitRadiusHistogram() {

  n=mu=0;
  sigma=0.7;

  float dx=(xma-xmi)/float(nx);
  float x0=xmi+.5*dx;
  // -- count background
  b=0;
  int cntbg=0;
  for (int i=0; i<nx; i++) {
    float x=x0+i*dx;
    if (x>-3 && x<3) continue;
    b+=hx_(id,x);
    cntbg++;
  }
  float aveb=0;
  if (cntbg) aveb=b/float(cntbg);

  // -- count signal and background under peak
  s=0;
  b=0;
  for (int i=0; i<nx; i++) {
    float x=x0+i*dx;
    if (x<-3 || x>3) continue;
    s+=hx_(id,x)-aveb;
    b+=aveb;
  }

  //hprint_(id);
  //cout<<"+++++++++++++++++++++++++++++++"<<endl;
  //cout<<"s="<<s<<" b="<<b<<" average_b="<<aveb<<endl;
  //cout<<"+++++++++++++++++++++++++++++++"<<endl;

  myhreset_(id);
}


float RICHLikelihood::likelihood() {
  lnL=0;
  for (vector<hitAngle>::iterator it=list.begin(); it!=list.end(); ++it) {
    float dRad2 = (*it).dRad * (*it).dRad;
    lnL-=log( n*exp(dRad2 * .5) + background(dRad2) );
  }
  return lnL;
}



float RICHLikelihood::ChiSquare() {
  chi2=0;
  for (vector<hitAngle>::iterator it=list.begin(); it!=list.end(); ++it) {
    chi2+=(*it).dRad * (*it).dRad;
  }
  return chi2; 
}


float RICHLikelihood::ChiSquareWithShift(float upperPhi,float upperLambda) {

  corr.newRing(tPhi*1e-3,tLam*1e-3,tRad*1e-3);


  float invSigma=1/0.7;
  chi2=0;
  int n=0;
  for (vector<hitAngle>::iterator it=list.begin(); it!=list.end(); ++it) {

    float dPhi = RICH_LambdaPhi->Correct()->dPhi   ((*it).phi*1e3, (*it).lambda*1e3, (*it).plane);
    float dLam = RICH_LambdaPhi->Correct()->dLambda((*it).phi*1e3, (*it).lambda*1e3, (*it).plane);

    float dRad= ( sqrt(dPhi*dPhi + dLam*dLam) - tRad ) * invSigma;
    //cout<<sqrt(dPhi*dPhi + dLam*dLam)<<" "<<tRad<<" "<<dRad<<" "<<dRad*dRad<<endl;
    chi2+=dRad*dRad;
    n++;
  }
  return chi2/float(n); 
}

float RICHLikelihood::fit() {
  // --- vary N,b0,b1
 

  return chi2;
}















