/*** This version allows for a single event only, no checking is performed ***/
/*** Furthermore it only works with a single slp in simulation mode        ***/
/* To make it work with more than one slp, top, trigs, and tracks would
   have to be attached to MOD.
*/
#include "CLHEP/Hist/HBookTuple.h"
#include <math.h>
#include "rich/RICHDAQ.h"
#include "rise/ring.h"
 
extern "C" {
#define _BOOL   // nice.h complains during compilation

#ifndef L2_FARM_MODE
#define rpm_addr int
#else
#include "pce.h"
#include "rhp.h"
#endif


#ifndef L2_FARM_MODE
  typedef struct {
    float x1,y1,z1;
    float x1_err,y1_err,z1_err;
    float x2,y2,z2;
    float x2_err,y2_err,z2_err;
    float energy;
    int   type,mother;
  } FLT_TRACK;
#else
#include "slp_events.h"
#include "slp_error.h"
#include "flte.h"
#endif


int RICH_getTracks(int eventno,EVENT_HDR* evt,FLT_TRACK* track,int ntrack);
int RICH_cut();
int RICH_init(char *dir,float radius,int threshold,int magnetflag);
void rich_stop();

#ifndef L2_FARM_MODE
void RICH_check(HBookTuple *tuple);
#endif

}

static int flte_pref = (COMP_N_RICH << 24);
#define FLTE_PRE(ID) (flte_pref | (ID))
 
static Hits      h;
static DualSpace d;


typedef struct {
  float tx,ty,tx_err,ty_err;
  } RICHtrack;
  
RICHtrack rtrack[100];

// -----------------------------------------------------------------------------------------

int RICH_init(char *dir,float radius,int thresh,int mflag) {  

  RICH_DAQ=new RICHDAQ(dir);
  if (!RICH_DAQ) return 0;
  
  d.define(radius*1000,thresh);
  
/************  book histogramms *******************************/
/*
  int rc;
  if ((rc = rhp_hbook1(FLTE_PRE(1),"rich: number of hit",50,0.,1000)) != GRR_OK)
    SLP_logError(rc ,"RICH_startup: rhp_hbook1 is not pleased with id 1");
*/
  return 1;
  }

// -----------------------------------------------------------------------------------------
 
int RICH_getTracks(int eventno,EVENT_HDR *event, FLT_TRACK *track,int ntracks) {

  RICHDAQ& daq=*RICH_DAQ;


#ifndef L2_FARM_MODE
  daq.NextEvent();
#else
  daq.NextEvent(event);
#endif
 

  //if (daq.NumHitChannel()>1500) return 0;
  printf("numhit1 %d %d\n",ntracks,daq.NumHitChannel());

  float r=51E-3;
  float dr=5*0.3E-3;
  
  float ri    = r-dr;
  float ro    = r+dr;
  float ri2   = ri*ri;
  float ro2   = ro*ro;
  float root2 = sqrt(2);


  for (int k=0;k<ntracks;k++) {
    h.reset();

    float tx=(track[k].x2-track[k].x1)/(track[k].z2-track[k].z1);
    float ty=(track[k].y2-track[k].y1)/(track[k].z2-track[k].z1);
    float dx=3*(track[k].x2_err+track[k].x1_err)/(track[k].z2-track[k].z1);

    rtrack[k].ty=0;
    rtrack[k].tx=0;
    rtrack[k].ty_err=0;
    rtrack[k].tx_err=0;
      
    for (int i=0;daq.HitChannelExist(i);i++) {
      float phi=RICH_LambdaPhi->phi(daq.ChannelIndex());
      float lam=RICH_LambdaPhi->lambda(daq.ChannelIndex());
      int plane=(daq.sm()>6) ? 1 : 0;

      float x=phi-tx;
      float y=lam-ty;
      // inner and outer rectangular bound
      if ((fabs(x)<ri/root2-dx&&fabs(y)<ri/root2)||(fabs(x)>ro+dx&&fabs(y)>ro)) {
        continue;
      }
      // detailed circular cut
      float rp2=(x-dx)*(x-dx)+y*y;
      float rm2=(x+dx)*(x+dx)+y*y;
      if (x<0&&rp2>ri2||x>0&&rm2>ri2) {
        int ok=0;
        if (x>dx) {
          if (rp2<ro2) ok=1;
	}
        else if (x<-dx) {
          if (rm2<ro2) ok=1;
	}
        else if (fabs(y)<ro) ok=1;
        if (ok) { 
	  cout<<".";fflush(stdout);
          h.append(d.dualx(phi*1000),d.dualy(lam*1000),plane);
	}
      }
    } // hit loop
    
    d.nextevent();
    d.ringsearch(h);
    float dold=1;
    float phi=1E37,lam=1E37;
    cout<<"Found rings numring()="<<d.numring()<<endl;
    for (int n=0;n<d.numring();n++) {
      Ring r=d.getRing(n);
      float d2=(r.phi()-tx)*(r.phi()-tx)+(r.lambda()-ty)*(r.lambda()-ty);
      if (d2<dold) {
        phi=r.phi();
	lam=r.lambda();
	dold=d2;
      }
    }
    //if (dold<0.34E-6) {  
    if (dold<1E-5) {
      cout<<"*";fflush(stdout);
      rtrack[k].ty_err=0.3E-3;
      rtrack[k].tx_err=0.5E-3;
      rtrack[k].ty =lam;
      rtrack[k].tx =phi;
    }
  } // track loop
  return 0;
}

// -----------------------------------------------------------------------------------------

int RICH_cut(FLT_TRACK *track,int ntracks) {
  RICHDAQ& daq=*RICH_DAQ;

/*  
  int rhpActive = rhp_active(FLTE_PRE(0));
  if( rhpActive ) {
    rhp_hf1(FLTE_PRE(1), daq.NumHitChannel(), 1);
    }
*/
  printf("numhit %d %d\n",ntracks,daq.NumHitChannel());
  
  if (daq.NumHitChannel()<15000) {
    for (int i=0;i<ntracks;i++) {
      if (rtrack[i].tx_err==0) continue;
      float ei=track[i].energy;
      float tx=rtrack[i].tx;
      float ty=rtrack[i].ty;
      float niz=1./sqrt(1+tx*tx+ty*ty);
      float nix=niz*tx;
      float niy=niz*ty;
       
      for (int j=i+1;j<ntracks;j++) {
        if (rtrack[j].tx_err==0) continue;
        float ej=track[j].energy;
	float tx=rtrack[j].tx;
	float ty=rtrack[j].ty;
	float njz=1./sqrt(1+tx*tx+ty*ty);
        float njx=njz*tx;
        float njy=njz*ty;
	float m2=ei*ej*(1-nix*njx-niy*njy-niz*njz);
	
 	printf("mass: %f %f %f %f %f %f %f %f %f %f %f\n",m2,ei,ej,
          nix,niy,niz,sqrt(nix*nix+niy*niy+niz*niz),
          njx,njy,njz,sqrt(njx*njx+njy*njy+njz*njz));
	
	if (m2>4) { printf("ACCCCCEPPTPTTT\n"); return 1; }
        }
      }
    }
  return 0;
  }
// -----------------------------------------------------------------------------------------

void rich_stop() {
  if (RICH_DAQ)       { delete RICH_DAQ;        RICH_DAQ=0; }
  if (RICH_Detector)  { delete RICH_Detector;   RICH_Detector=0;} 
  if (RICH_Vessel)    { delete RICH_Vessel;     RICH_Vessel=0; }
  if (RICH_LambdaPhi) { delete RICH_LambdaPhi;  RICH_LambdaPhi=0; }
  }


#ifndef L2_FARM_MODE

extern "C" void printrich_();

void RICH_check(HBookTuple *tuple) {

  if (!RICH_DAQ)  {

    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp==5) { RichRun::gede=(*igede); break; }
    }

    for (ArteTable<GESL>::iterator igesl=ArteTable<GESL>::begin();
	 igesl!=ArteTable<GESL>::end(); ++igesl) {
      if ((*igesl)->cmp==12) { RichRun::magnet=(*igesl)->mfl; break; }
    } 

    RICH_init("/afs/desy.de/group/hera-b/ARTE/geo/RICH/",.052,7,0);
    printrich_();
  }


  static myEventCount=1;

  FLT_TRACK TriggerTrack[100];
  int countTracks=0;
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin(); 
       irccl!=ArteTable<RCCL>::end(); ++irccl) {

    // --- check pt
    float tx=(*irccl)->tx;
    float ty=(*irccl)->ty;
    float pt=(*irccl)->esum*sqrt(tx*tx+ty*ty);
    if (pt<0.5) continue;

    // --- find mother
    TriggerTrack[countTracks].mother=(*(*irccl)->mtra.begin())->btra->topp->lund;
    // --- find type
    TriggerTrack[countTracks].type=(*(*irccl)->mtra.begin())->topp->lund;

    // --- track parameters
    TriggerTrack[countTracks].x1=(*irccl)->x;
    TriggerTrack[countTracks].y1=(*irccl)->y;
    TriggerTrack[countTracks].z1=(*irccl)->z;
    TriggerTrack[countTracks].x1_err=.1;
    TriggerTrack[countTracks].y1_err=.1;
    TriggerTrack[countTracks].z1_err=.1;
    TriggerTrack[countTracks].energy=(*irccl)->esum;

    // --- target
    ArtePointer<GTAR> gtar(1);
    TriggerTrack[countTracks].x2=gtar->x;
    TriggerTrack[countTracks].y2=gtar->y;
    TriggerTrack[countTracks].z2=gtar->z;
    TriggerTrack[countTracks].x2_err=.003;
    TriggerTrack[countTracks].y2_err=.003;
    TriggerTrack[countTracks].z2_err=.003;

    if (++countTracks > 9) break;
  }
  cout<<"Track candidates="<<countTracks<<endl;
  
  // --- call ring reconstruction
  if (!countTracks) return;
  EVENT_HDR *event=0;
  int eventno=0;

  RICH_getTracks(eventno,event,TriggerTrack, countTracks);

  for (int i=0; i<countTracks; i++) {

    tuple->column("event",myEventCount);
    tuple->column("richtx",rtrack[i].tx);
    tuple->column("richty",rtrack[i].ty);
    tuple->column("ecaltx",(TriggerTrack[i].x1-TriggerTrack[i].x2)/(TriggerTrack[i].z1-TriggerTrack[i].z2));
    tuple->column("ecalty",(TriggerTrack[i].y1-TriggerTrack[i].y2)/(TriggerTrack[i].z1-TriggerTrack[i].z2));
    tuple->column("p",TriggerTrack[i].energy);
    tuple->column("type",TriggerTrack[i].type);
    tuple->column("mother",TriggerTrack[i].mother);
    tuple->dumpData();

    //cout<<"ECAL: "<<TriggerTrack[i].x1/TriggerTrack[i].z1<<" "
    //<<TriggerTrack[i].y1/TriggerTrack[i].z1
    //<<"    RICH: "<<rtrack[i].tx<<" "<<rtrack[i].ty<<endl;
  }

  myEventCount++;
}

#endif




