#ifndef _RICHFIT_H_
#define _RICHFIT_H_

#include <vector.h>


class hitAngle {
public:
  inline hitAngle(float drad,float p,float l,float pl) {
    dRad=drad;
    phi=p;
    lambda=l;
    plane=pl;
  }
  float dRad;
  float phi;
  float lambda;
  float plane;
};



class RICHLikelihood {
private:
  vector<hitAngle> list;  // photon list
  float lnL;              // ring likelihood log
  float chi2;             // ring chi2
  float sigma;            // ring sigma
  float tRad;             // ring radius
  float tPhi,tLam;        // ring center
  float n;                // ring photons
  float mu;               // ring radius shift
  float s;                // signal photons
  float s3s;              // signal photons inside 3 sigma
  float b;                // background photons
  float b0;               // background parameter 0
  float b1;               // background parameter 1
  int   rich_gede;        // RICH line in GEDE table
  Corrections corr;       // corrections
  float limit;            // dRad/sigma<limit to take photon into calculation
  const int id;           // hbook histogram id
  const int nx;           // number of bins
  const float xmi;        // lower histo limit
  const float xma;        // upper histo limit
  float f;                // photons: (up-lo)/(up+lo)

public:
  // --- constructor
  RICHLikelihood();
  // --- destructor
  ~RICHLikelihood();
 
  // --- hit container
  inline void  append(hitAngle a) { list.push_back(a); } 
  inline void  reset()            { if (list.size()) list.erase(list.begin(),list.end()); } 
  int          collect(float phi, float lam,float rad);
  void         print();

  // --- fitting procedures
  float likelihood();
  float  ChiSquare();
  float  ChiSquareWithShift(float phiShift,float lamShift);
  float  fit();
  inline float background(float dRad2) { return b0+b1*dRad2; }
  int    fillRadiusHistogram();
  void   fitRadiusHistogram();
  void   resetRadiusHistogram();
  void   printRadiusHistogram();


  // --- parameters
  inline float  &Sigma()  { return sigma; }
  inline float  &Mu()     { return mu; }
  inline float  &Radius() { return tRad; }
  inline float  &Phi()    { return tPhi; }
  inline float  &Lambda() { return tLam; }
  inline float  &N()      { return n; }
  inline float  &Signal()       { return s; }
  inline float  &Signal3Sigma() { return s3s; }
  inline float  &Background()   { return b; }
  inline float  &B0()     { return b0; }
  inline float  &B1()     { return b1; }
  inline float  &Limit()  { return limit; }
  inline float  &LnL()    { return lnL; }
  inline float  &Chi2()   { return chi2; } 
  inline float  &F()      { return f; } 

};


#endif







