*CMZ :          20/05/96  12.58.00  by  Unknown
*-- Author :   T.Lohse
c
c-------------------------------------------------------------
c
c       subroutine:   VTSEC(itB,flag)
c
c       input:    itB  track no for a B             Int
c                 reconstructed in VTPSIK from
c                 a J/Psi Ks decay
c                 = 0 if no such decay exists
c
c       output:   flag (<0 if fail)                 Int
c
c       action:   loop over all unassigned tracks and
c                 find other vertices not constrained by
c                 wires (to be called AFTER VTPRIM !!).
c                 If itB>0: Search exclusively with tracks
c                 which are in the vicinity of the reconstructed
c                 B vertex (impact parameter < Bvimpc).
c                 In this case VTPSIK has to be called
c                 before VTPRIM
c
c-------------------------------------------------------------
c
      subroutine  VTSEC(itB,flag)

      implicit none

      integer itB,flag

#include "vt/vtdat.fi"
#include "vt/vtwire.fi"
#include "vt/vtpatt.fi"

      integer it,i,j,k,l,iv,flag1,imax,kmax,it1,it2

      double precision 
     1  x(3), cov(3,3)
      double precision 
     1  di, dmin, sum

      double precision 
     1  dtt(ntrtot,ntrtot), dcl(ntrtot)
      logical secass(ntrtot), cluass(ntrtot), Bsel(ntrtot)
      integer trlist(ntrtot), lislen
      integer trclu(ntrtot), clulen
      integer ipoint(0:ntrtot), kpoint
      integer nprsec

      double precision 
     1  VTDTT, VTDVW, VTDTP

      logical unass

      data cov / 10000.d0,     0.d0,     0.d0,
     1               0.d0, 10000.d0,     0.d0,
     1               0.d0,     0.d0, 10000.d0 /

      flag  = 0
      nvsec = 0

      do  i = 1, trmult
        secass(i) = .false.
        cluass(i) = .false.
        Bsel(i)   = .true.
      enddo
      if ( itB.gt.0.and.itB.le.trmult.and.ivB.gt.0) then
        do  i = 1, trmult
          if ( VTDTP(ivB,i) .gt. Bvimpc ) Bsel(i) = .false.
          if ( jv(ivB,i).ne.0 )           Bsel(i) = .true.
          if ( jv(ivpsik,i).ne.0 )        Bsel(i) = .false.
        enddo
        Bsel(itB)=.false.
      endif

c
c---> prepare distances of track pairs
c
      do  i = 1, trmult-1
        do  j = 1, trmult
          dtt(i,j) = -1.d0
          dtt(j,i) = -1.d0
        enddo
      enddo

c
c---> find largest cluster of unassigned tracks
c
  100 continue
      lislen = 0
      do  it = 1, trmult
        unass = iwitr(it).eq.0.and.nv(it).eq.0
     1        .and..not.cluass(it).and.Bsel(it)
        if (unass) then
          lislen = lislen+1
          trlist(lislen) = it
        endif
      enddo
      if (lislen.eq.0) goto 600
      kpoint    = 0
      ipoint(0) = 0
      dmin = 1.d30
      di   = 0.d0
      clulen = 0
    1 continue
      kpoint = kpoint + 1
      if (kpoint.gt.lislen)  goto 3
      ipoint(kpoint) = ipoint(kpoint-1)
      dcl(kpoint) = 0.d0
    2 continue
      ipoint(kpoint) = ipoint(kpoint) + 1
      if (ipoint(kpoint).gt.lislen) goto 3
      it = trlist(ipoint(kpoint))
      sum = 0.d0
      do  i = 1, kpoint-1
        j = trlist(ipoint(i))
        if (dtt(j,it).lt.0.) then
          dtt(j,it) = VTDTT(j,it)
          dtt(it,j) = dtt(j,it)
        endif
        if (dtt(j,it).ge.ch2stt) goto 2
        sum = sum+dtt(j,it)
      enddo
      dcl(kpoint) = sum
      di = di + sum
      if ( kpoint.gt.clulen .or.
     1   ( kpoint.eq.clulen .and. di.lt.dmin ) ) then
        clulen = kpoint
        dmin   = di
        do  l = 1, kpoint
          trclu(l) = trlist(ipoint(l))
        enddo
      endif
      goto 1
    3 continue
      kpoint = kpoint - 1
      di = di - dcl(kpoint)
      if (kpoint.gt.0) goto 2
      do  i = 1, clulen
        secass(trclu(i)) = .true.
        cluass(trclu(i)) = .true.
      enddo
c
c---> if cluster has length 1, try to find another
c---> cluster which can be split
c
      if (clulen.eq.1) then
        it1 = trclu(1)
        dmin = 1.d30
        imax = 0
        kmax = 0
        do  i = 1, nvsec
          j = ivsec(nvsec)
          if (j.gt.0) then
            if (ntr(j).gt.2) then
              do k = 1, ntr(j)
                it2 = ktr(j,k)
                if (dtt(it1,it2).lt.0.) then
                  dtt(it1,it2) = VTDTT(it1,it2)
                  dtt(it2,it1) = dtt(it1,it2)
                endif
                di = dtt(it1,it2)
                if (di.lt.dmin) then
                  imax = j
                  kmax = k
                  dmin = di
                endif
              enddo
            endif
          endif
        enddo
        if (dmin.lt.ch2stt) then
          it1 = ktr(imax,kmax)
          call VTKILL(imax,kmax,flag1)
          if (flag1.lt.0) then
            flag=-10
            fit(imax)=0
          endif
c
c---> check if split vertex becomes compatible with a wire
c
          do  j = 1, nw
            if (VTDVW(imax,j).lt.ch2svw) then
              fit(imax) = 0
              do  i = 1, ntr(imax)
                secass(ktr(imax,i))=.false.
              enddo
              goto 10
            endif
          enddo
   10     continue
          clulen = 2
          trclu(2) = it1
        endif
      endif
c
c---> allow for nmix non-secondary tracks to be added to cluster
c     (not the signal B or its decay products)
c
      nprsec = 0
  200 continue
      if ( nprsec .ge. nmix ) goto 310
      k    = 0
      dmin = 1.d30
      do  it1 = 1, trmult
        if ( .not.Bsel(it1) ) goto 300
        if (.not.secass(it1)) then
          sum = 0.d0
          do  j = 1, clulen
            it2 = trclu(j)
            if (dtt(it1,it2).lt.0.) then
              dtt(it1,it2) = VTDTT(it1,it2)
              dtt(it2,it1) = dtt(it1,it2)
            endif
            di = dtt(it1,it2)
            if (di.ge.ch2stt) goto 300
            sum = sum + di
          enddo
          if (sum.lt.dmin) then
            k    = it1
            dmin = sum
          endif
        endif
  300   continue
      enddo
      if (k.ne.0) then
        nprsec = nprsec + 1
        secass(k) = .true.
        cluass(k) = .true.
        clulen = clulen+1
        trclu(clulen) = k
        goto 200
      endif
  310 continue
c
c---> open a new secondary vertex with this cluster
c
      if (clulen.ge.1) then
        x(1) = 0.d0
        x(2) = 0.d0
        x(3) = 0.d0
        call VTINIT(x,cov,iv,flag1)
        nvsec = nvsec + 1
        ivsec(nvsec) = iv
        if (flag1.lt.0) then
          flag = -5
          goto 999
        endif
        do  i = 1, clulen
          it = trclu(i)
          call VTADD(iv,it,flag1)
          if (flag1.lt.0) then
            do  j = 1, clulen
              secass(trclu(j)) = .false.
            enddo
            flag = -4
            goto 100
          endif
        enddo
      endif
c
c---> fit this vertex
c
      if (ntr(iv).ge.2) then
        call VTFIT(iv,0,flag1)
        if (flag1.lt.0) then
c---> fit failed, leave the vertex
          flag = -2
          fit(iv) = 0
          do  j = 1, clulen
            secass(trclu(j)) = .false.
          enddo
          goto 100
        endif
      endif
c
c---> take out vertex compatible with a wire
c
      if (fit(iv).eq.0) goto 100
      do  j = 1, nw
        if (VTDVW(iv,j).lt.ch2svw) then
          fit(iv) = 0
          do  i = 1, ntr(iv)
            secass(ktr(iv,i))=.false.
          enddo
          goto 100
        endif
      enddo

      goto 100

  600 continue

c
c---> take out all garbage vertices
c
      do  i = 1, nvsec
        iv = ivsec(i)
c
c---> garbage if fit was set to dummy (fit(iv)=0) or not enough tracks
c
        if (iv.ne.0) then
          if ( fit(iv).eq.0 .or. ntr(iv).lt.2 ) then
            call VTDEL(iv,flag1)
            if (flag1.eq.0) ivsec(i)=0
          endif
        endif
      enddo
      k = 0
      do  i = 1, nvsec
        if (ivsec(i).ne.0) then
          k = k+1
          ivsec(k) = ivsec(i)
        endif
      enddo
      nvsec = k

      return

  999 continue
      flag = -1

      return
      end
