#include <iostream.h>
#include <iomanip.h>

#include "OTrChannel.hh"
#include "OTrEvent.hh"
#include "OTrDaq.hh"
#include "OTrWire.hh"
#include "OTrSection.hh"
#include "OTrGede.hh"
#include "OTrChannelHistogrammer.hh"

//__________________________________________________________________
// OTrChannel
//
//
// Author: Ivar Siccama && Wouter Hulsbergen
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(OTrChannel) ;
#endif

OTrChannel::TdcCount_t OTrChannel::fgMaximumTime05mm=254 ;
OTrChannel::TdcCount_t OTrChannel::fgMinimumTime05mm=50 ;
OTrChannel::TdcCount_t OTrChannel::fgMaximumTime10mm=254 ;
OTrChannel::TdcCount_t OTrChannel::fgMinimumTime10mm=5 ;
OTrChannel::Mask_t OTrChannel::fgMaximumMask=10;

OTrParameterizedRTRelation* OTrChannel::fgRTRelation05mm=0;
OTrParameterizedRTRelation* OTrChannel::fgRTRelation10mm=0;
OTrRTRelation* OTrChannel::fgMCRTRelation05mm=0;
OTrRTRelation* OTrChannel::fgMCRTRelation10mm=0;

OTrChannel::OTrChannel() 
  : OTrDaqComponent(),fWire(0),fMask(0),fTZero(0)
{
#ifdef __OTR_HISTOGRAMMING__
  fHistogrammer = 0 ;
#endif //__OTR_HISTOGRAMMING__
} 

OTrChannel::OTrChannel(ComponentId_t id) 
  : OTrDaqComponent(id), fWire(0),fMask(0),fTZero(0)
{
#ifdef __OTR_HISTOGRAMMING__
  fHistogrammer = new OTrChannelHistogrammer(this) ;
#endif //__OTR_HISTOGRAMMING__
}

OTrChannel::OTrChannel(const OTrChannel& rhs)
  : OTrDaqComponent(rhs), fWire(0),fMask(0),fTZero(0)
{
#ifdef __OTR_HISTOGRAMMING__
  fHistogrammer = 0;
#endif
  operator=(rhs) ;
} ;

OTrChannel::~OTrChannel() 
{
#ifdef __OTR_HISTOGRAMMING__
  if(fHistogrammer) delete fHistogrammer ;
#endif //__OTR_HISTOGRAMMING__
}

OTrChannel&
OTrChannel::operator=(const OTrChannel& rhs) 
{
  //cout << "OTrChannel::operator= " ;
  OTrDaqComponent::operator=(rhs) ;
  fWire = rhs.fWire;
  fMask = rhs.fMask ;
  fTZero = rhs.fTZero ;
#ifdef __OTR_HISTOGRAMMING__
  if(rhs.fHistogrammer) {
    // do not copy the histogrammer, just create it
    if(!fHistogrammer) fHistogrammer = new OTrChannelHistogrammer(this) ;
    *fHistogrammer = *(rhs.fHistogrammer) ;
    fHistogrammer->SetChannel(this) ;
  } else {
    if(fHistogrammer) {
      delete fHistogrammer ;
      fHistogrammer=0;
    }
  }
#endif //__OTR_HISTOGRAMMING__
  return *this ;
}

OTrChannel::TdcCount_t
OTrChannel::GetMaximumTime() const {
  if(GetWire() && 
     GetWire()->GetSection()->GetEPitch()
     == OTrGede::kFiveMM)
    return fgMaximumTime05mm ;
  return fgMaximumTime10mm ;
};

OTrChannel::TdcCount_t
OTrChannel::GetMinimumTime() const {
  if(GetWire() &&
     GetWire()->GetSection()->GetEPitch()
     == OTrGede::kFiveMM)
    return fgMinimumTime05mm ;
  return fgMinimumTime10mm ;
};

OTrParameterizedRTRelation*
OTrChannel::GetRTRelation() const {
  if(GetWire() &&
     GetWire()->GetSection()->GetEPitch()
     == OTrGede::kFiveMM)
    return fgRTRelation05mm ;
  return fgRTRelation10mm ;
};

OTrRTRelation*
OTrChannel::GetMCRTRelation() const {
  if(GetWire() &&
     GetWire()->GetSection()->GetEPitch()
     == OTrGede::kFiveMM)
    return fgMCRTRelation05mm ;
  return fgMCRTRelation10mm ;
};

// Now in OTrDaq.C ?!?
OTrHit*
OTrChannel::ProcessChannelBuffer( const TdcCount_t count, OTrEvent* event)
{
  //cout << "OTrChannel::ProcessChannelBuffer("<<(int)count<<","<<event<<")" << endl ;
  if (ValidHit(count)) {
#ifdef __OTR_HISTOGRAMMING__
    fHistogrammer->ProcessChannelBuffer(count) ;
#endif
    if(GetWire()!=0 && GoodMask() ) {
      return event->Add( new OTrHit(this,count) ) ;
    }
  } 
  return 0 ;
} 

void OTrChannel::Reset() { 
#ifdef __OTR_HISTOGRAMMING__
  fHistogrammer->Reset() ; 
#endif
} ;

#ifdef USE_ROOT
TString OTrChannel::GetName() const
{
  //cout << "OTrChannel::GetName(): " << GetId() << " " << GetTdcId() << endl ;
  char name[14];
  sprintf(name,"Chan_%d.%d.%d",GetTdcId(),GetAsdId(),GetChannelId()) ;
  return TString(name) ;
}

#endif //USE_ROOT


#ifdef __OTR_HISTOGRAMMING__
float OTrChannel::GetOccupancy() const { return fHistogrammer->GetOccupancy() ; } ;

float OTrChannel::GetAverageTime() const { return fHistogrammer->GetAverageTime() ; };

float OTrChannel::GetAverageTime2() const { return fHistogrammer->GetAverageTime2() ; };

float OTrChannel::GetSigma() const { return fHistogrammer->GetSigma() ; } ;

int OTrChannel::GetNHit() const { return fHistogrammer->GetNHit() ; } ;

void OTrChannel::SetTimeHistogramming(bool b) { 
  OTrChannelHistogrammer::SetTimeHistogramming(b) ; } ;

int OTrChannel::GetNTimeBins() { 
  return OTrChannelHistogrammer::GetNTimeBins() ;} ;

int OTrChannel::GetMaximumTimeAxis() { 
  return OTrChannelHistogrammer::GetMaximumTimeAxis() ;} ;

int OTrChannel::GetMinimumTimeAxis() { 
  return OTrChannelHistogrammer::GetMinimumTimeAxis() ;}  ;

int OTrChannel::GetHistogrammingMode() { 
  return OTrChannelHistogrammer::GetHistogrammingMode() ; } ;

int OTrChannel::Bin2Count( const int bin ) { 
  return OTrChannelHistogrammer::Bin2Count( bin ) ; } ;

void OTrChannel::PerformHistogramming(TdcCount_t tdc) { 
  fHistogrammer->PerformHistogramming(tdc) ; } ;

const OTrChannelHistogrammer::TimeHistogramBin_t* 
OTrChannel::GetTimeHistogram() {
  return fHistogrammer->GetTimeHistogram() ; } ;

int OTrChannel::GetTZeroBin() const { 
  return fHistogrammer->GetTZeroBin() ; } ;

float OTrChannel::GetAverageFromTimeH1() const { 
  return fHistogrammer->GetAverageFromTimeH1() ; } ;

ostream & operator << (ostream& os, const OTrChannel& w)
{ 
  os << " aver, sigma: " 
     << setprecision(3) << w.GetAverageTime() << " " 
     << setprecision(3) << w.GetSigma();
  os << " nproc: " << w.GetHistogrammer()->GetNProcessed() << " nhit: " << w.GetNHit();
  os << " occup: " << setprecision(3) << w.GetOccupancy() 
     << " mask = " << w.GetMask();
  return os;
}

#ifdef USE_ROOT
TH1D 
OTrChannel::GetTimeH1() const { 
  return fHistogrammer->GetTimeH1() ; } ;

TH1D* 
OTrChannel::GetTimeH1_B( float tshift = 0 ) const { 
  return fHistogrammer->GetTimeH1_B() ; } ;
#endif //USE_ROOT

#endif //__OTR_HISTOGRAMMING__
