#include "OTrDaq.hh"
#include "OTrTdcBoard.hh"
#include "OTrAsdBoard.hh"
#include "OTrEventBuffer.hh"
#include "OTrEvent.hh"
#include "OTrParameterizedRTRelation.hh"

#ifdef USE_ROOT
 #include "TH1.h"
#endif

#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
//#include "grr.h"

#ifdef USE_ARTE
// #ifdef USE_NEWARTE
   #include "online/etb.h"
//#else
// #include "etb.h"
//#endif
#else
 #include "etb.h"
#endif

int OTrDaq::fgDefaultNTdcs = 80 ;

OTrDaq::OTrDaq() : OTrDaqComponent(),  OTrComponentMother(fgDefaultNTdcs), fNProcessedEvents(0) {} ;

OTrDaq::OTrDaq(int nTdcs) :
  OTrDaqComponent(),  OTrComponentMother(nTdcs), fNProcessedEvents(0) {
    cout << "OTrDaq::OTrDaq(" << nTdcs << ")" << endl;
};

enum TableType {
  kMask=0,
  kT0=1,
  kQMask=2
};

OTrDaq::~OTrDaq() {
  WriteTable("caldata/masks/occup_last.dat",kMask) ;
  WriteTable("caldata/tzero/tzero_last.dat",kT0) ;
  OTrRTRelation* rtr;
  if((rtr=OTrChannel::GetRTRelation5mm()))
    rtr->Write(ofstream("caldata/rtrelation/rtr05mm_last.dat")) ;
  if((rtr=OTrChannel::GetRTRelation10mm())) 
    rtr->Write(ofstream("caldata/rtrelation/rtr10mm_last.dat")) ;
} ;

OTrTdcBoard*
OTrDaq::AddTdcBoard(TdcId_t tdcId)
{
  OTrTdcBoard* tdcIt = FindChild(tdcId) ;
  if(!tdcIt) 
    tdcIt =  AddChild(OTrTdcBoard(SetTdcId(GetId(),tdcId))) ;
  //else cout << "WARNING: tdc " << tdcId << " already added." << endl ;
  return tdcIt ;
};

OTrEvent* 
OTrDaq::ProcessEvent(OTrEventBuffer* eventBuffer)
{
  OTrEvent* event = new OTrEvent(eventBuffer->GetEventHdr()->sltevt) ;
  for (OTrEventBuffer::FedConstIterator fedIt(*eventBuffer);
       fedIt.valid(); fedIt.next())
    {
      int tdcId = (*fedIt).GetId() ;
      OTrTdcBoard* tdcIt = GetTdcBoard(tdcId) ;
      if(tdcIt) tdcIt->ProcessFedBuffer(*fedIt,event) ;
      // else cout << "OTrDaq::ProcessEvent(): could not find fedId " 
      // << tdcId << endl ;
    };
  fNProcessedEvents++ ;
  return event ;
}


void OTrDaq::WriteAsciMask(char * filename) const {
  WriteTable(filename,0) ;
};

void OTrDaq::ReadAsciMask(char * filename) {
  ReadTable(filename) ;
};

void OTrDaq::WriteTable(char * filename, int type) const {
  cout << "OTrDaq::WriteTable(\"" << filename << "\")" << endl ;
  ofstream ofs(filename) ;
  ofs << type << " " << GetNChildren() << endl ;
  for(TdcConstIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() ) {
    ofs << endl << (*tdcIt).GetTdcId() << endl ;
    for(int asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(int chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++)
	switch(type) 
	  {
	  case kMask: {
#ifdef USE_ROOT
	    double occupancy = 
	      (*tdcIt).GetAsdBoards()[asdId].GetChannels()[chanId].GetOccupancy() ;
#else
	    double occupancy = 0 ;
#endif
	    ofs << setw(5) << setprecision(3) << (int)(occupancy*100) ;
	    break ;
	  };
	  case kT0: {
	    ofs << setw(9) << setprecision(3) 
		<< (*tdcIt).GetAsdBoards()[asdId].GetChannels()[chanId].
	      GetTZero().GetValue();
	    break ;
	  };
	  default:
	    ofs << setw(5) << setprecision(3) << "huh" ;
	  };
      ofs << endl ;
    };
  };
};

/*
Sergey Solunin's quality definition 
0 -- channels are supposed to be good;
2 -- low efficiency (50% of neighbours);
3 -- high background in spectra;
4 -- noisy channel;
5 -- oscillations;
6 -- dead channel or one with VERY low efficiency;
7 -- large beginning of spectrum (not so bad);
8 -- two peaks in spectrum (most in sector 2');
9 -- noise 100%;
*/

void OTrDaq::ReadTable(char * filename)
{
  cout << "OTrDaq::ReadTable(\"" << filename << "\")" << endl ;
  ifstream ifs(filename) ;
  if(!ifs) {
    cout << "*** ERROR in OTrDaq::ReadTable : \"" 
	 << filename << "\" does not exist"  << endl ;
    return ;
  };
  
  int type,nTdc, tdcId ;
  ifs >> type >> nTdc ;
  while(ifs >> tdcId) {
    OTrTdcBoard* tdcIt = GetTdcBoard(tdcId) ;
    double tableEntry ;
    for(int asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(int chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++)
	{
	  ifs >> tableEntry ;
	  if(tdcIt) {
	    switch (type)
	      {
	      case kMask: {
		tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId].SetMask(tableEntry);
		break ;
	      };
	      case kT0: {
		OTrFObservable t0(tableEntry,0) ;
		tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId].SetTZero(t0) ;
		break ;
	      };
	      case kQMask: {
		OTrChannel& chan = tdcIt->
		  GetAsdBoards()[asdId].GetChannels()[chanId];
		int mask = chan.GetMask();
		//if ( mask != 0 && tableEntry != 0 ) { 
		//cout << "new,old = " << tableEntry << " " << mask << endl;
		//}
		if ( tableEntry > 0 ) {
		  chan.SetMask( 100 + tableEntry );
		  //  cout << "mask = " << chan.GetMask() << endl;
		}
		break ;
	      }
	      default: ;
	      };
	  };
	};
    };
  };
}; 

void OTrDaq::PrintSummary() const
{
  int n = 0;
  for(TdcConstIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() ) {
    n++;
    cout << (*tdcIt).GetTdcId() << " : " 
	 << (*tdcIt).GetNProcessed();
    if ( GetNProcessedEvents() != (*tdcIt).GetNProcessed() ) {
      cout << "** not OK **";
    }
    if ( n%10 == 0 ) {
      cout << endl;
      n = 0;
    }
  }
};

void OTrDaq::Reset(bool b)
{
  for(TdcIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() )
    (*tdcIt).Reset() ;
}; 

#ifdef USE_ROOT
ClassImp(OTrDaq) ;

TString OTrDaq::GetName() const {
  return TString("Staring ... DAQ") ;
}

TH1* OTrDaq::GetTimeH1() const
{ 
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  TdcConstIterator tdcIt(*this) ;
  TH1* timeH1 = (*tdcIt).GetTimeH1() ;
  timeH1->SetName(timeH1Name) ;
  for(tdcIt.next(); tdcIt.valid(); tdcIt.next()) {
    TH1* histo = (*tdcIt).GetTimeH1() ;
    timeH1->Add(histo,1) ;
    delete histo ;
  };
  return timeH1;
}

#endif //USE_ROOT

