//#include "OTrEvent.hh"
#include "OTrEventBuffer.hh"
#include <iostream>

#if defined(USE_ARTE) 
//&& defined(USE_NEWARTE)
 #include "online/etb.h"
#else
 #include "etb.h"
 #include "log.h"
#endif

#if defined(USE_ARTE) 
//&& defined(USE_NEWARTE)
// USE_NEWARTE because ARTE_get_recid_DOTR is undefined in ARTE-01-08
 #include "arte/Arte.hh"
 #include "auto/DOTR.hh"
 #include "farm/farm.hh"
#endif

const int OTrEventBuffer::fgMaxNFedBuffers = 100 ;
const int OTrEventBuffer::fgMaxEventLength = 128*1024 ;

//OTrEventBuffer::OTrEventBuffer(FILE* inputFile) : fNFedBuffers(0), 
OTrEventBuffer::OTrEventBuffer() : fNFedBuffers(0), 
  fInputFile(0), fComponentId(0x400)
{
  fFedBuffers = new ETB_fed[GetMaxNFedBuffers()];
  fEvtBuffer = new char[fgMaxEventLength] ;
  fEvtHdr = (EVENT_HDR *) fEvtBuffer ;
  //fEvent = new OTrEvent() ;
};

OTrEventBuffer::~OTrEventBuffer() {
  delete[] fFedBuffers ;
  delete[] fEvtBuffer ;
  fEvtHdr = 0;
  //delete fEvent ;
};

#ifndef USE_ARTE
int OTrEventBuffer::NextEventHdr()
{
  //cout << "OTrEventBuffer::NextEventHdr()" << endl ;
  int rc;
  rc=ETB_read_event(fInputFile, fEvtHdr, fgMaxEventLength);
  if (rc!=ETB_ok) {
    cout << "*** OTrEventBuffer::NextEventHdr() "
	 << " error reading event from file" << endl;
  };
  return rc;
}

int OTrEventBuffer::NextEventHdrFarm()
{
  int D1;
  LogHdr D2;
  int rc = l4aGetEvt(fInputFile, fEvtHdr, fgMaxEventLength, &D1, &D2 );
  if ( rc ) {
    cout << "*** OTrEventBuffer::NextEventHdrX() "
	 << " error reading event from file" << endl;
  };
  return rc;
}

int OTrEventBuffer::UpdateFedBuffers()
{ 
  return ETBUpdateFedBuffers();
};

int OTrEventBuffer::ETBUpdateFedBuffers()
{ 
  int nFEDs = fNFedBuffers;
  //cout << "nFEDs = " << nFEDs << endl;
  int rc = ETB_get_feds(fEvtHdr, GetComponentId(), GetMaxNFedBuffers(),
                        fFedBuffers, &fNFedBuffers);
  if (rc!=ETB_ok) {
    printf("Error getting FEDs for *event= %x rc= %d \n", (int) fEvtHdr, rc);
  } else {
    if ( nFEDs != 0 && fNFedBuffers != nFEDs ) {
      cout << "** ERROR in OTrEventBuffer::ETBUpdateFedBuffers(), "
	   << " number of FEDs was " << nFEDs 
	   << " and now is " << fNFedBuffers << endl;
    } 
  }
  //printf("Getting FEDs for *event= %x rc= %d \n", (int) fEvtHdr, rc);
  return rc ;
};

int
OTrEventBuffer::NextEvent( int DaqFileType )
{ 
  //cout << "OTrEventBuffer::NextEvent()" << endl ;
  int rc = 0;
  if ( DaqFileType != 0 ) {
    rc = NextEventHdrFarm() ;
  } else {
    rc = NextEventHdr() ;
  }
  if(rc!=ETB_ok) return rc ;
  return UpdateFedBuffers() ;
};
#endif

// Changed to USE_NEWARTE since ARTE_get_recid_DOTR is undefined in ARTE-01-08
#if defined(USE_ARTE) 
//&& defined(USE_NEWARTE)
int OTrEventBuffer::DOTRUpdateFedBuffers()
{
  int recid = 0;
  int rc;
  //  cout << "calling ARTE_get_recid_DOTR" << endl; 
  rc = ARTE_get_recid_DOTR( &recid );
  //cout << "OTrEventBuffer(): recid=0x" << hex << recid << dec << endl;

  int nFEDs = fNFedBuffers;
  rc = ARTE_get_feds_DOTR( GetMaxNFedBuffers(), fFedBuffers, &fNFedBuffers );
  //cout << "got feds rc = " << rc << endl;
  if ( rc != ETB_ok ) {
    cout << "** ERROR in OTrEventBuffer::DOTRUpdateFedBuffers, rc = "
	 << rc << endl;
  } else {
    if ( fNFedBuffers != nFEDs ) {
      cout << "** ERROR in OTrEventBuffer::DOTRUpdateFedBuffers, "
	   << "number of FEDs was " << nFEDs 
	   << " and now is " << fNFedBuffers << endl; 
    }
    //DumpFedBuffers();
  }
  return rc; 
}
#endif

void 
OTrEventBuffer::DumpFedBuffers()
{
  cout << "fNFedBuffers = " << fNFedBuffers << endl;
  for( int i=0; i < fNFedBuffers; i++ ){
    cout << "usevnt_(): i=" << i << " "
	 << "length="         << fFedBuffers[i].hdr.length        << " "
	 << "fedid=0x" << hex << fFedBuffers[i].hdr.fedid  << dec << " "
	 << "recid=0x" << hex << fFedBuffers[i].hdr.recid  << dec << " "
	 << "bxid=0x"  << hex << fFedBuffers[i].hdr.bxid   << dec << endl;
    
    // FED data
    int           l  = fFedBuffers[i].hdr.length - sizeof(  SLB_SUBHDR );
    unsigned char *p = (unsigned char *) fFedBuffers[i].data;
    for( int j=0; j<l; j++ ){
      unsigned char adc = *( p + j );
      if( adc != 0xff ){
	cout << "  j=" << j << " adc=0x" << hex << int( adc ) << dec << endl;
      }
    }
  }
}



