#include <CLHEP/Geometry/Point3D.h>
typedef HepPoint3D Point3D ;

//#include <Geometry/Point3D.h>
#include "OTrHit.hh"
#include "OTrWire.hh"
#include "OTrSection.hh"
#include "OTrParameterizedRTRelation.hh"
#include "OTrRTResult.hh"
#include "OTrGede.hh"
#include "OTrDaq.hh"

#ifdef USE_ROOT
 #include <TNode.h>
 #include <TTUBE.h>
 #include <TRotMatrix.h>
 #include <TPolyMarker3D.h>
 #include <TBuffer.h>
#endif

#ifdef USE_ARTE
//#include "arte/arte.hh"
//#include "auto/HITB.hh"
#endif


OTrHit::~OTrHit()
{
#ifdef USE_ROOT
  if ( fNode ) { 
    if(fNode->GetShape()) delete fNode->GetShape() ; // fNode is owner but does not know
    TCollection::StartGarbageCollection();
    TCollection::GarbageCollect(fNode);
    delete fNode; 
    fNode =  0;
  }
#endif
}

OTrHit::OTrHit( const OTrHit& h ) 
  : fChannel(h.fChannel), fTdcCounts(h.fTdcCounts), fNode(0), fHITB(h.fHITB)
{ 
  assert(0);
}

OTrHit&
OTrHit::operator=(  const OTrHit& h )
{
  cout << "OTrHit::operator=(  const OTrHit& h ) DO NOT USE " << endl ;
  assert(0);
  return *this;
};

#ifdef USE_ROOT
ClassImp(OTrHit) ;

void
OTrHit::Draw(int color) {
  if(!fNode) {
    float drift_dist = 0.2 ;    
    const OTrWire* w = GetWire() ;
    
    static char shapeName[20]; 
    sprintf( shapeName, "%s%i", "wireShape_", w->GetId() );
    TShape* wireShape = new TTUBE(shapeName,shapeName,"void", (drift_dist-0.0002),
				  (drift_dist+0.0002), w->GetLength()*0.5);
    
    static char nodeName[20];  
    sprintf( nodeName, "%s%i", "wireNode", w->GetId() );
    fNode = new TNode(nodeName,nodeName,wireShape,
		      w->GetCentralX(),w->GetCentralY(),w->GetRawZ(),
		      w->GetGede()->GetWireRotMatrix()) ;
  } ;
  
  if(color==1)
    switch(GetWire()->GetGede()->GetEStereoAngle())
      {
      case OTrGede::kPlusFiveDegrees :
	fNode->SetLineColor(3); break ;
      case OTrGede::kMinusFiveDegrees :
	fNode->SetLineColor(5); break ;
      default:
	fNode->SetLineColor(4);
      }
  else fNode->SetLineColor(color);
}

void
OTrHit::Streamer(TBuffer &R__b)
{
  // Stream an object of class OTrHit
  if (R__b.IsReading()) {
    OTrComponent::ComponentId_t componentId ;
    R__b >> componentId >> fTdcCounts ;
    fChannel =
      OTrDaqComponent::GetDaq()->
      GetTdcBoard(OTrDaqComponent::GetTdcId(componentId))->
      GetAsdBoard(OTrDaqComponent::GetAsdId(componentId)).
      FindChannel(OTrDaqComponent::GetChannelId(componentId)) ;
  } else {
    R__b << fChannel->GetId() << fTdcCounts ;
  } ;
} ;

#endif


double
OTrHit::GetZ() const {
  return GetChannel()->GetWire()->GetRawZ() ;
};

OTrDObservable
OTrHit::GetDriftDistance() const {
  OTrRTRelation* rtr = GetChannel()->GetRTRelation() ;
  double R = 0;
  double sigmaR = 0;
  if ( rtr ) {
    R = rtr->GetR(double(GetDriftTime())) ; // !!!!!!!!!! Do not forget the (double) !!!!
    sigmaR = //rtr->CalcRTResult(R).GetSigmaR().GetValue() ;
      rtr->CalcRTResult(R).GetSigma().GetValue() ;
  } else {
    cout << "ERROR in OTrHit::GetDriftDistance(), GetRTRelation() == 0"
	 << endl; 
  }
  OTrDObservable o;
  o.SetValue(R);
  o.SetError(sigmaR);
  return o;
};

#ifdef USE_ARTE
ArtePointerHITB
OTrHit::AppendHITB() {
  // Do not do anything if fHITB is set (e.g. in Monte Carlo).
  if(fHITB.index()) return fHITB ;

  HITB hit;
  if ( GetWire() == 0 ) {
    cout << "** ERROR in OTrHit::AppendHITB(), GetWire() = 0" 
	 << endl;
    return 0;
  }
  hit.z    = GetWire()->GetRawZ() ;
  hit.x    = GetWire()->GetRawX() ; 
  hit.y    = GetDriftTime();
  OTrDObservable d = GetDriftDistance() ;
  hit.q    = d.GetValue() ; 
  hit.r    = d.GetError() ;
  if ( GetWire()->GetGede() == 0 ) {
    cout << "** ERROR in OTrHit::AppendHITB(),  GetWire()->GetGede() == 0" 
	 << endl;
    hit.gede = 0;
  } else {
    hit.gede = GetWire()->GetGede()->GetGEDEIndex();
  }
  // ArteRelationPointer<HITB_RTRA> rtra;
  // ArteRelationPointer<HITB_MIMP> mimp;
  // ArteRelationPointer<HITB_DIGB> digb;
  // ArteRelationPointer<HITB_RHIT> rhit;
  // ArteRelationPointer<HITB_RSEG> rseg;
  //fHITB = ArtePointer<HITB>::append( hit );
  //fHITB = ArteTable<HITB>::end()-1 ;
  fHITB = ArteTable<HITB>::push_back( hit );
  return fHITB ;
}

void
OTrHit::DumpHITB( ostream& os ) const
{  
  os << "z = " << fHITB->z << " ";
  os << "x = " << fHITB->x << " ";
  os << "y = " << fHITB->y << " ";
  os << "q = " << fHITB->q << " ";
  os << "r = " << fHITB->r << " ";
  os << "gede = " << fHITB->gede << endl;
}
#endif
