#include "OTrLayer.hh"
#include "OTrHit.hh"
#include "OTrSuperLayer.hh"
#include "OTrEvent.hh"
#include "OTrSector.hh"
#include <math.h>

#ifdef USE_ROOT
 #include <TCanvas.h>
 #include <TStyle.h>
 #include <TText.h>
 #include <TPaveLabel.h>
 #include <TH2.h>
#endif

//__________________________________________________________________
// OTrLayer
//
// Container for:
// * Section (gede's)
// Functionality:
// * None
//
// Author: Wouter Hulsbergen && Ivar Siccama 
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(OTrLayer) ;
//void OTrLayer::Streamer(TBuffer &) {} ;
//void OTrLayer::ShowMembers(TMemberInspector &, char *) {} ;
#endif

double fitf(double *x, double *par)
{
  double fitval = exp(par[0])*pow(par[1]-x[0],par[2]);
  return fitval;
}

double rsquared(double* x, double* par)
{
  double normaliz = par[0]; 
  double xco      = x[0] - par[1];
  double ymax     = par[2];

  double numer = xco*xco*sqrt(xco*xco+ymax*ymax);
  double fitval = 0;
  if ( numer != 0 ) {
    fitval = normaliz*(ymax/numer);
  }
  return fitval;
}

double x2y2(double* x, double* par)
{
  double normaliz = par[0]; 
  double xco      = x[0] - par[1];
  double ymax     = par[2];

  double fitval = 0;
  if ( xco != 0 ) 
    fitval = (normaliz/xco)*atan(ymax/xco);
  return fitval;
}

double x2y2bound(double* x, double* par)
{
  double normaliz = par[0]; 
  double xco      = x[0] - 64 - 17.29;
  double ymax     = 55.4;

  double fitval = 0;
  if ( xco != 0 ) 
    fitval = (normaliz/xco)*atan(ymax/xco);
  return fitval;
}

OTrLayer::~OTrLayer()
{  
  //cout << fCombinedHistoTop << " " << fCombinedHistoBot << endl;
#ifdef USE_ROOT
  if ( fCombinedHistoBot != 0 ) { 
    TCollection::StartGarbageCollection();
    TCollection::GarbageCollect(fCombinedHistoBot);
    delete fCombinedHistoBot;
    fCombinedHistoBot = 0; 
  }
  if ( fCombinedHistoTop != 0 ) { 
    TCollection::StartGarbageCollection();
    TCollection::GarbageCollect(fCombinedHistoTop);
    delete fCombinedHistoTop; 
    fCombinedHistoTop = 0;
  }
#endif
  // must delete sections ...
};

OTrLayer::OTrLayer() : fSuperLayer(0), fStereoAngle(0)
{ 
  cout << "OTrLayer::OTrLayer() Please do not use !!" << endl ;
#ifdef USE_ROOT
  fCombinedHistoTop = 0 ;
  fCombinedHistoBot = 0 ;
  fCombinedMC = 0;
  fCombinedProcessed = 0;
#endif
}

OTrLayer::OTrLayer(OTrSuperLayer* sul, ComponentId_t id) :
  OTrDetectorComponent(id), fSuperLayer(sul),
  fStereoAngle(zeroDegrees)
{ 
  //cout << "You created an OTrLayer with id: " << GetId() << endl ;
#ifdef USE_ROOT
  fCombinedHistoTop = 0 ;
  fCombinedHistoBot = 0 ;
  fCombinedMC = 0;
  fCombinedProcessed = 0;
#endif
  fStereoAngle = EStereoAngle(GetLayerId()) ;
} 

OTrSection*
OTrLayer::AddSection( OTrGede* gede ) {
  //  cout << "OTrLayer::AddSection(OTrGede* gede) " << gede << endl;
  SectionId_t secId = gede->determineSection();
  OTrSection* childIt = FindChild(secId) ;
  if (childIt) {
    cout << "OTrLayer::AddSection(), trying to add gede for section"
	 << " that already exists !!! " 
	 << gede->superLayerId() << " "
	 << gede->layerId() << " " << secId << endl;
     return childIt ;
  };
  return AddChild(OTrSection(gede,this,SetSectionId(GetId(),secId))) ;
}

void
OTrLayer::Connect(OTrSuperLayer* sul)
{
  cout << "OTrLayer::Connect(OTrSuperLayer* sul)" << endl ;
  cout << "*** do not call " << endl;
  fSuperLayer=sul;
  //for(SectionContainer::iterator secIt = GetSections().begin() ;
  //    secIt != GetSections().end() ; secIt++) {
  // (*secIt).Connect(this) ;
  //}
  return ;
}

void 
OTrLayer::Translate(const ThreeVector& vec) 
{
  for(SectionContainer::iterator secIt=GetSections().begin();
      secIt!=GetSections().end() ; secIt++) secIt->Translate(vec) ;
}

OTrSector* 
OTrLayer::GetSector(SectorId_t secId) {
  CreateSectors() ;
  OTrSector* sector=0;
  for(SectorContainer::iterator secIt=GetSectors().begin() ;
      secIt!=GetSectors().end() ; secIt++) {
    cout << (*secIt)->GetSectorId() << endl ;
    if((*secIt)->GetSectorId()== secId) {
      sector=*secIt;
      break ;
    }
  }
  return sector ;
}
 
HepPoint3D
OTrLayer::GetPMin() const
{
  Point3D thepmin(0,0,0) ;
  SectionContainer::const_iterator secIt=GetSections().begin() ;
  if(secIt!=GetSections().end() ) {
    thepmin = secIt->GetPMin() ;
    secIt++ ;
    for(;secIt!=GetSections().end(); secIt++) {
      Point3D pmin(secIt->GetPMin()) ;
      if(thepmin.x() > pmin.x()) thepmin.setX(pmin.x()) ;
      if(thepmin.y() > pmin.y()) thepmin.setY(pmin.y()) ;
      if(thepmin.z() > pmin.z()) thepmin.setZ(pmin.z()) ;
    }
  }
  return thepmin ;
}

HepPoint3D
OTrLayer::GetPMax() const
{
  Point3D thepmax(0,0,0) ;
  SectionContainer::const_iterator secIt=GetSections().begin() ;
  if(secIt!=GetSections().end() ) {
    thepmax = secIt->GetPMax() ;
    secIt++ ;
    for(;secIt!=GetSections().end(); secIt++) {
      Point3D pmax(secIt->GetPMax()) ;
      if(thepmax.x()<pmax.x()) thepmax.setX(pmax.x()) ;
      if(thepmax.y()<pmax.y()) thepmax.setY(pmax.y()) ;
      if(thepmax.z()<pmax.z()) thepmax.setZ(pmax.z()) ;
    }
  }
  return thepmax ;
}

void
OTrLayer::CreateSectors()
{
  if(GetSectors().empty()) {
    map<int,int,less<int> > sectorIds ;
    for(SectionContainer::const_iterator sectionIt=GetSections().begin() ;
	sectionIt!=GetSections().end() ; sectionIt++)
      sectorIds[sectionIt->GetSectorId()]++ ;
    for( map<int,int,less<int> >::const_iterator sectorIdIt=sectorIds.begin() ;
	 sectorIdIt!=sectorIds.end() ; sectorIdIt++)
      fSectors.push_back(new OTrSector(this, 
				       SetSectionId(GetId(),(*sectorIdIt).first*100))) ; 
    //cout << "OTrLayer::CreateSectors() : created " << fSectors.size() 
    //	 << " sectors ... :) " << endl ;
  }
}


#ifdef USE_ROOT

void
OTrLayer::Plot( vector<TPad*> pads, const int choice, char* option,
		const OTrSuperLayerEvent* slEvent, const bool use_mask )
{
  if(pads.size() != GetSectors().size()) 
    cout << "Error in OTrLayer::Plot(...) " << endl
	 << "number of sectors not equal to number of pads. " << endl ;
  
  if ( strcmp(option,"") == 0 ) {
    //    cout << "option: " << option << endl;
    for(vector<TPad*>::iterator padIt=pads.begin(); padIt!=pads.end(); padIt++)
      if(*padIt) {
	(*padIt)->cd() ;
	(*padIt)->Clear();
      }
  }
  
  int farbe = (int)GetEStereoAngle() + 1 ;
  vector<TPad*>::iterator padIt = pads.begin() ;
  for(SectorContainer::iterator sectorIt = GetSectors().begin() ;
      sectorIt != GetSectors().end() && padIt != pads.end() ; sectorIt++, padIt++)
    (*sectorIt)->Plot(*padIt,choice, option, farbe, slEvent, use_mask) ;
}

#endif

#ifdef USE_ROOT
int
OTrLayer::OccupancySummary( ostream& ofs, bool useMask )
{
//   OTrSector* s;
//   float occ;

//   ofs << endl;
//   ofs << "the 10mm modules:" << endl;
//   s = FindSector(9);
//   s->OccupancySummary( ofs, useMask );
//   s = FindSector(10);
//   s->OccupancySummary( ofs, useMask );

//   ofs << endl;
//   ofs << "the 5mm modules: " << endl;
//   occ = 0;
//   s = FindSector(1);
//   occ += s->OccupancySummary( ofs, useMask ); 
//   s = FindSector(5);
//   occ += s->OccupancySummary( ofs, useMask );
//   ofs << "total occ = " << occ << endl;

//   occ = 0;
//   s = FindSector(2);
//   occ += s->OccupancySummary( ofs, useMask ); 
//   s = FindSector(6);
//   occ += s->OccupancySummary( ofs, useMask );
//   ofs << "total occ = " << occ << endl;

//   ofs << endl;
//   ofs << "the short modules: " << endl;
//   s = FindSector(15);
//   s->OccupancySummary( ofs, useMask );
//   s = FindSector(25);
//   s->OccupancySummary( ofs, useMask );
  return 0;
}

int 
OTrLayer::BookCombinedOccupancy()
{
  // plots for the combined occupancy of the 5 and 10 mm cells:
  char name1[20];
  char name2[20];
  sprintf(name1,"%s%d","combined_top",GetId());
  sprintf(name2,"%s%d","combined_bot",GetId());
  cout << "############## booking the histos" << endl;
  fCombinedHistoTop = new TH1F(name1,"",64,0,64);
  fCombinedHistoBot = new TH1F(name2,"",64,0,64);
  return 0;
}
#endif

#ifdef USE_ROOT
int 
OTrLayer::FillCombinedOccupancy(const OTrSuperLayerEvent* slEvent,
				const int use_mask )
{ 
  // this function ORs the 5 mm cells (ignoring the short modules)
  fCombinedProcessed++;

  int  nmaxcell = 64;
  int cellhit[2][nmaxcell];
  for ( int j = 0; j < 2; j++ ) {
    for ( int i = 0; i < nmaxcell; i++ ) cellhit[j][i] = 0;
  }

  for(OTrSuperLayerEvent::HitContainer::const_iterator hitp = 
	slEvent->GetHits().begin();
       hitp != slEvent->GetHits().end(); hitp++) {
    const OTrWire* wire = (*hitp)->GetWire();
    if ( wire->GetLayerId() != GetLayerId() )
      continue;
    int secid = wire->GetSectionId();
    //    cout << "secid = " << secid << endl;
    if ( secid == 15 || secid == 25 ) continue;
    if ( secid != 10 && secid != 20 && secid != 50 && secid != 60 &&
	 secid != 90 && secid != 100 ) {
      cout << "** ERROR in OTrLayer::FillCombinedOccupancy, secid = "
	   << secid << endl;
      return 1;
    }
    int cellnr;
    if ( secid == 10 || secid == 50 || 
	 secid == 20 || secid == 60) {
      cellnr = (wire->GetWireNumber()/2) + 32;
    } else {
      cellnr = wire->GetWireNumber();
    }
    int top;
    if ( secid == 90 || secid == 10 || secid == 50 ) {
      top = 1;
    } else {
      top = 0;
    }
    if ( use_mask == 1 ) {
      if ( !wire->GetChannel()->GoodMask() ) cellhit[top][cellnr] = 2;
    }

    // if the cell is bad or it is already hit then continue
    if ( cellhit[top][cellnr] == 2 || cellhit[top][cellnr] == 1) continue;
    cellhit[top][cellnr] = 1;
  }

  for ( int i = 0; i < nmaxcell; i++ ) {
    if ( cellhit[0][i] == 1 ) fCombinedHistoBot->Fill( float(i) , 1 );
    if ( cellhit[1][i] == 1 ) fCombinedHistoTop->Fill( float(i) , 1 );
  }
  return 0;
}
#endif

#ifdef USE_ROOT
int OTrLayer::PlotCombinedOccupancy( TCanvas* c, TPad* p, 
                                       char* option )
{
  if ( fCombinedHistoTop == 0 || fCombinedHistoBot == 0 ) {
    cout << "ERROR in OTrLayer::PlotCombinedOccupancy, " 
	 << " combined histograms do not exist." << endl;
    return 1;
  }
 
  p->cd();
  p->Clear();
  cout << "PlotCombinedOccupancy fCombinedProcessed = " 
       << fCombinedProcessed << endl;

  // scale with the total number of event to get the occupancy:
  float scaling_factor = 1/float(fCombinedProcessed);
  fCombinedHistoTop->Scale(scaling_factor);
  fCombinedHistoBot->Scale(scaling_factor);

  if ( fCombinedHistoTop->GetXaxis() != 0 ) {
    fCombinedHistoTop->GetXaxis()->SetLabelSize(0.03);
    fCombinedHistoTop->GetYaxis()->SetLabelSize(0.03);
  }
  if ( fCombinedHistoBot->GetXaxis() != 0 ) {
    fCombinedHistoBot->GetXaxis()->SetLabelSize(0.03);
    fCombinedHistoBot->GetYaxis()->SetLabelSize(0.03);
  }
  fCombinedHistoTop->SetLineColor(2);  // red
  fCombinedHistoTop->SetMaximum(0.35);
  fCombinedHistoTop->Draw();
  fCombinedHistoBot->SetLineColor(4);  // blue
  fCombinedHistoBot->Draw("same");

  // Creates a Root function based on function fitf 
  //TF1 *func = new TF1("fitf",fitf,0,64,3);

  //double pitch = 0.87;
  //  double rmin = 17.29; // nearest wire 15.04 cm / 0.87 = 17.29 cell widths
  //double ymax = 55.4;

  //TF1 *f1 = new TF1("f1","([0]/(x-[1]))*atan([2]/(x-[1]))",0,64);
  //f1->SetParameters(4.5,rmin+64,ymax); 

  //  TF1 *f1 = new TF1("rsquared",rsquared,0,64,3);
  //f1->SetParameters(120,rmin+64,ymax) ;
  //f1->Draw("same");

  //TF1 *f1 = new TF1("x2y2",x2y2,0,64,3);
  //f1->SetParameters(4.5,rmin+64,ymax) ;
  //f1->Draw("same");

  TF1 *fbnd = new TF1("x2y2bound",x2y2bound,0,64,1);
  fbnd->SetParNames("norm");
  fbnd->SetLineColor(2);
  fCombinedHistoTop->Fit("x2y2bound","E","same");

  /*
  if ( fCombinedMC ) { delete fCombinedMC; }
  fCombinedMC = new TH1F("fCombinedMC","fCombinedMC",64,0,64);
  TH2F* generated = new TH2F("generated","generated",100,-100,0,100,0,100);
  int nent = 0;
  // double rmax = sqrt(2)*79.54;  // furthest 69.2 cm / 1.74 = 79.54 cells
   double rmax = sqrt(2)*(rmin+64);  // furthest 69.2 cm / 1.74 = 79.54 cells
  TH1F* gen2 = new TH1F("gen2","gen2",100,0,64) ;
  //double rmax = sqrt(2)*81;

  for ( int i = 0; i < 320000; i++ ) {
 
  // generate a random radius between rmin and rmax
  //    double r = (rmax - rmin)*drand48() + rmin;
  // generate a value between 0 and the maximum r**2 value
  //  double r2 = rmax*rmax*drand48();

    //cout << "r2 = " << r << " " << r2 << " " << r*r << endl;
    double r = rmin/(1-drand48()*(1-rmin/rmax)) ;

    //if ( r2 > 4.5*r*r ) {
    double phi = 0.5*M_PI*( 1 + drand48() );
    float xco = r*cos(phi);
    float yco = r*sin(phi);
    generated->Fill( xco, yco );
    //cout << " xco = " << xco << " ";
    if ( yco < ymax) {// length in y: 48.2 cm / 0.87 = 55.40 cells 
      gen2->Fill(-xco) ;
      double x = xco + rmin + 64 ;
      if (0<x && x<64) {
	fCombinedMC->Fill( x , 1 );
      };
      nent++;
    }  
  }
  cout << "scaling " << 1./float(nent) << endl;
  fCombinedMC->Scale(scaling_factor*fCombinedHistoTop->GetEntries()/
		     (double)(fCombinedMC->GetEntries()));
  fCombinedMC->Draw("same");
*/

  // Sets initial values and parameter names
  //  func->SetParameters(1,100,-2);
  //  func->SetParNames("normalization","constant","power value");
  //  func->SetParLimits(1, 80, 105);
  //func->SetParLimits(2, -0.1, -10);
  //func->SetLineColor(2);
  //fCombinedHistoTop->Fit("fitf","EB","same");

  fCombinedHistoTop->GetXaxis()->SetTitle("10 mm cell bins");   

  fCombinedHistoTop->GetXaxis()->SetTitleSize(0.04);
  fCombinedHistoTop->GetYaxis()->SetTitle("occupancy");
  fCombinedHistoTop->GetYaxis()->SetTitleOffset(1.20);

  TText* t = new TText();
  t->SetTextFont(1);
  t->SetTextColor(12);
  t->SetTextSize(0.06);
  t->SetTextAlign(12);
  t->DrawText(22,0.21,"transition");

  TLine* l = new TLine(32,0,32,0.195);
  l->SetLineColor(1);
  l->SetLineWidth(2);
  l->Draw();

  //cout << "Chi squared = " <<  func->GetChisquare() << endl;
  c->cd();
  TPaveLabel* lab = new TPaveLabel(0.1,0.95,0.9,0.99,
                       "5 and 10 mm combined at 40 MHz, 0 degree, red = top");
  lab->SetFillColor(18); 
  lab->SetTextSize(0.8); 
  lab->Draw();
  c->Modified();
  c->Update();
  return 0;
}
#endif

