#include "OTrRTCalib.hh"
#include "OTrParameterizedRTRelation.hh"
#include "OTrHit.hh"
#include "OTrWire.hh"
#include "OTrSection.hh"
#include "OTrRTProjection.hh"
#include "OTrChannel.hh"
#include "OTrTrack.hh"
#include "OTrTrackCandidate.hh"
#include "OTrSuperCluster.hh"
#include "OTrReconstructedHit.hh"
#ifdef USE_ROOT
#include <TCanvas.h>
#include <TH1.h>
#endif

//__________________________________________________________________
// OTrRTCalib
//
// The RT calibration class
// Container for:
// Functionality:
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(OTrRTCalib) ;
//const char * OTrRTCalib::DeclFileName() { return 0 ;} ;
//short int OTrRTCalib::Class_Version() { return 0 ;} ;
#endif

int OTrRTCalib::fgUpdateFrequency=20000 ; // tracks

OTrRTCalib::OTrRTCalib() 
  : fCanvas(0)
{
  cout << "OTrRTCalib::OTrRTCalib(....) " << endl ;
  fRTProjection = new OTrRTProjection*[2] ;
  fRTProjection[0] = new 
    OTrRTProjection((OTrParameterizedRTRelation*)OTrChannel::GetRTRelation5mm()) ;
  fRTProjection[1] = new 
    OTrRTProjection((OTrParameterizedRTRelation*)OTrChannel::GetRTRelation10mm()) ;
  
#ifdef USE_ROOT
  //ROOTSTUFF 
  fCanvas = new TCanvas("OTrRTCalibCanvas", "OTrRTCalibCanvas",2); 
  fCanvas->Show(); fCanvas->Clear();
  fChi2ProbH1 = new TH1F("CorrectedChi2Prob","",50,0,1) ;
  fCanvas->cd() ; fChi2ProbH1->Draw() ;
  fCanvas->Update();
#endif //USE_ROOT
  cout << "Leaving OTrRTCalib::OTrRTCalib(....) " << endl ;
  Reset() ;
  OTrParameterizedRTRelation::SetCorrectionLevel(1) ;
} ;

void
OTrRTCalib::Reset()
{
  // Initialisation
  fRTProjection[0]->Reset() ;
  fRTProjection[1]->Reset() ;
  fNTrackCandidates = 0 ;
  fNTracks = 0 ;
  fNHitsTotal = 0 ;
  fNHitsTrack = 0 ;
  fNReconstructedHits = 0;
  fNDofs = 0 ;
  fChiSquare = 0;
#ifdef USE_ROOT
  fChi2ProbH1->Reset();
#endif
} ;

void
OTrRTCalib::UpdateCanvas()
{
#ifdef USE_ROOT
  if(fCanvas) {
    fCanvas->Modified() ;
    fCanvas->Update();
    fRTProjection[0]->UpdateCanvas() ;
    fRTProjection[1]->UpdateCanvas() ;
  };
#endif
};

void
OTrRTCalib::Add(const OTrTrackCandidate* trackCandidate)
{
  fNTrackCandidates++ ;
  //fNHitsTotal += (*trackCandidate).GetHits().size() ;
  OTrReconstructedTrackInterface* trackp ;
  if((trackp=(*trackCandidate).GetTrack())) {
    fNTracks++ ;
    fNHitsTrack += (*trackp).GetHits().size() ;
    fNDofs    += (*trackp).GetNDof() ;
#ifdef USE_ROOT
    fChi2ProbH1->Fill((*trackp).GetChi2Prob());
#endif
    fChiSquare += (*trackp).GetChi2() ;
    
    for(vector<const OTrSuperCluster*>::const_iterator clIt=trackCandidate->GetClusters().begin() ;
	clIt!=trackCandidate->GetClusters().end(); clIt++)
      for(vector<OTrSectionCluster*>::const_iterator secIt=(*clIt)->GetSectionClusters().begin() ;
	  secIt!=(*clIt)->GetSectionClusters().end() ; secIt++) 
	for(OTrSectionCluster::HitIterator hitIt=(*secIt)->begin(); hitIt!=(*secIt)->end() ; hitIt++) {
	  OTrGede* gede = (OTrGede*)((*hitIt)->GetWire()->GetSection());
	  if(fRTProjection[gede->GetEPitch()]->Add(*trackp,*hitIt))
	    fNReconstructedHits++ ;
	  fNHitsTotal++ ;
	};
  };
  //if(!((++fCycle)%2000)) UpdateCanvas() ;
};

void
OTrRTCalib::Add(const OTrReconstructedTrackInterface* trackp)
{
  fNTracks++ ;
  fNHitsTrack += (*trackp).GetHits().size() ;
  fNDofs    += (*trackp).GetNDof() ;
#ifdef USE_ROOT
  fChi2ProbH1->Fill((*trackp).GetChi2Prob());
#endif
  fChiSquare += (*trackp).GetChi2() ;
  //   cout << (*trackp).GetHits().size() << " " 
  //        <<sqr((*trackp).GetHits().front()->GetMeasuredHit()->GetWire()->GetRawZ())*
  //     (*trackp).GetHits().size()*trackp->GetCov() << endl ;
  //if((*trackp).GetHits().size()==3 &&
  //  trackp->GetHits()[0]->GetMeasuredHit()->GetWire()->GetGede()->GetEPitch()==
  // trackp->GetHits()[1]->GetMeasuredHit()->GetWire()->GetGede()->GetEPitch() &&
  // trackp->GetHits()[0]->GetMeasuredHit()->GetWire()->GetGede()->GetEPitch()==
  // trackp->GetHits()[2]->GetMeasuredHit()->GetWire()->GetGede()->GetEPitch()) {
  //    cout << sqr(trackp->GetHits().front()->GetMeasuredHit()->GetWire()->GetRawZ()) /
//       trackp->GetHits().front()->GetMeasuredHit()->GetChannel()->GetRTRelation()->GetSigma2(0.1).GetValue() * trackp->GetCov() ;
  for(OTrReconstructedTrackInterface::HitContainer::const_iterator recHitIt=trackp->GetHits().begin() ;
      recHitIt!=trackp->GetHits().end() ; recHitIt++) {
    const OTrHit* hitIt =  (*recHitIt)->GetMeasuredHit() ;
    OTrSection* gede = hitIt->GetWire()->GetSection() ;
    if(fRTProjection[gede->GetEPitch()]->Add(*trackp,hitIt))
      fNReconstructedHits++ ;
    fNHitsTotal++ ;
  };
  if(!(fNTracks+1)%fgUpdateFrequency) Fit(true,true,true,true) ; 
  //};
  //if(!((++fCycle)%2000)) UpdateCanvas() ;
};

bool
OTrRTCalib::Fit(bool fitT, bool fitSigma2, bool fit05mm, bool fit10mm)
{
  cout << "Number of selected tracks: " << fNTracks << endl 
       << "chi2tot       " << fChiSquare << endl 
       << "chi2totbar:   " << fChiSquare/fNDofs << endl
       << "nHitsTotal:   " << fNHitsTotal << endl
       << "nHitsTrack:   " << fNHitsTrack << endl
       << "totRecHits:   " << fNReconstructedHits << endl ;
  
  bool converged(true) ;

  if(fitT && fit05mm) converged *= fRTProjection[0]->FitT() ;
  if(fitT && fit10mm) converged *= fRTProjection[1]->FitT() ;
  if(fitSigma2 && fit05mm) converged *= fRTProjection[0]->FitSigma2() ;
  if(fitSigma2 && fit10mm) converged *= fRTProjection[1]->FitSigma2() ;

  if(fCanvas) { 
    fRTProjection[0]->Draw() ;
    fRTProjection[1]->Draw() ;
    UpdateCanvas() ;
  };
  return converged ;
};
