#include "OTrRTRelation.hh"
//#include "ReconstructedHit.hh"
//#include "TrackDescription.hh"
//#include "FunctionValue.hh"
//#include "TrackChi2.hh"
//#include "Observable.hh"
#include <iostream.h>
#include <iomanip.h>

OTrRTRelation::OTrRTRelation() :
  tNoTableBins(1000), tTTable(tNoTableBins+1), tSigma2Table(tNoTableBins),
  tRTable(tNoTableBins) {
};

OTrRTRelation::OTrRTRelation(double rmin, double rmax) : 
  tNoTableBins(1000), fRMin(rmin), fRMax(rmax), fT0(0.),
  tTTable(tNoTableBins+1), tSigma2Table(tNoTableBins),
  tRTable(tNoTableBins)
{
  tDeltaR = (fRMax-fRMin)/tNoTableBins ;
  cout << "OTrRTRelation::OTrRTRelation()" << endl ;
};

OTrRTRelation::~OTrRTRelation() {} ;

void
OTrRTRelation::Read(istream& ifs) {
  //cout << "OTrRTRelation::Read(istream& ifs)" << endl ; 
  ifs >> fRMin >> fRMax >> fT0 ;
  tDeltaR = (fRMax - fRMin)/tNoTableBins ;
  cout << "  RMin: " << setw(8) << setprecision(6) << fRMin
       << "  RMax: " << setw(8) << setprecision(6) << fRMax
       << "  T0:   " << setw(8) << setprecision(6) << fT0 << endl;
};

void
OTrRTRelation::Write(ostream& ofs) const {
  cout << "OTrRTRelation::Write(ostream& ofs)" << endl ; 
  ofs << setw(15) << setprecision(10) << fRMin
      << setw(15) << setprecision(10) << fRMax
      << setw(15) << setprecision(10) << fT0 << endl;
};

OTrDObservable OTrRTRelation::GetObservedT(const HepDouble R)
{
  return OTrDObservable(GetT(R).value + GetT0(), sqrt(GetSigma2(R).value)) ;
};

void OTrRTRelation::FillSigma2Table()
{
  //cout << "OTrRTRelation::FillSigma2Table()" << endl ;
  for(int rbin=0; rbin < tNoTableBins; rbin++)
    tSigma2Table[rbin] = CalcFunctionSigma2(GetR(rbin)) ;
};

void OTrRTRelation::FillTTable()
{
  //cout << "OTrRTRelation::FillTTable()" << endl ;
  for(int rbin=0; rbin< tNoTableBins ; rbin++)
    tTTable[rbin] = CalcFunctionT(GetR(rbin)) ;
  tTTable[tNoTableBins] = tTTable[tNoTableBins-1]; //Overflow bin
  tTTable[tNoTableBins].d2dx2 = 0;

  // Fill inverse table
  tTRMax = tTTable[tNoTableBins].GetValue() ;
  tTRMin = tTTable[0].GetValue() ;
  tDeltaT = (tTRMax-tTRMin)/tNoTableBins ;
  bool positive = tDeltaT > 0 ;
  for(int tbin(0); tbin<tNoTableBins; tbin++) {
    HepDouble t = tTRMin + tbin*tDeltaT ;
    tRTable[tbin] = GetRMax() ;
    for(int rint = 0; rint<tNoTableBins ; rint++) {
      //cout << t << " " << tTTable[rint].GetValue() << endl ;
      if( (positive && tTTable[rint].GetValue()>=t) ||
	  (!positive && tTTable[rint].GetValue()<=t)) {
	tRTable[tbin] = GetR(rint) ;
	//- (tTTable[rint].value-t)/tTTable[rint].ddx 
	//firstrbin = rint ;
	break ;
      };
    };
  };
} ;

void
OTrRTRelation::DumpTable()
{
  for(int rbin=0; rbin<10; rbin++)
    cout << rbin << " " << GetR(rbin) << " " << tTTable[rbin] << endl ;
//  cout << tbin << " " << setprecision(10) << tTRMin + tbin*tDeltaT << " " 
//       << setprecision(10) << tRTable[tbin] << endl ;
//  tbin = tNoTableBins-1;
//  cout << tbin << " " << setprecision(10) << tTRMin + tbin*tDeltaT << " " 
//       << setprecision(10) << tRTable[tbin] << endl ;
//  float t=tTRMin-0.001 ;
//  cout << setprecision(10) << t << " " << int((t-tTRMin)/tDeltaT)<< " " << GetR(t) << endl;
//  t=tTRMin+0.001 ;
//  cout << setprecision(10) << t << " " << int((t-tTRMin)/tDeltaT)<< " " << GetR(t) << endl;
 // for(tbin=0; tbin<tRTable.size(); tbin++)
 //  cout << tbin << " " << tRTable[tbin] << endl ;
}
// Observable OTrRTRelation::CalcTime(HepDouble D, HepDouble Phi)
// {
//   HepDouble R = fabs(D);
//   HepDouble dPsidPhi, psi ;
//   CalcPsi(Phi,psi,dPsidPhi) ;
//   FunctionValue* C = GetC(psi) ;
//   FunctionValue* T = GetT(R) ;
//   FunctionValue* W = GetPosW() ;
//   return Observable(T->value,sqrt(W->value));
// }

// inline RTResult GetTime(const TrackDescription& track, const WirePosition&) {
//   const HepDouble& phi = track.GetPhi() ;
//   const HepDouble& d0  = track.GetD0() ;
//   HepDouble dPsidPhi, psi ;
//   CalcPsi(phi,psi,dPsidPhi) ;
//   HepDouble  
//   inline void OTrRTRelation::CalcR(const HepDouble& d0, const HepDouble& phi,
//                               const HepDouble& x, const HepDouble& y,
// 			      HepDouble& R, HepDouble& dRdD0, 
//                               HepDouble& dRdPhi, HepDouble& d2RdPhi2) {


