#include "OTrReconstructedHit.hh"
#include "OTrWire.hh"
#include "OTrTrackDescription.hh"
#include "OTrHit.hh"

//__________________________________________________________________
// OTrReconstructedHit
//
// Container for:
// * left-right ambiguity
// Functionality:
//
// Author: Wouter Hulsbergen 
//__________________________________________________________________

#ifdef USE_ROOT
//ClassImp(OTrReconstructedHit) ;
#endif

OTrReconstructedHit::OTrReconstructedHit() : 
  fHit(0), fLRSign(0), fLRSignIsFixed(false), fProjection(0) {} ;

OTrReconstructedHit::OTrReconstructedHit(const OTrHit* hit, LRSign_t lrSign) :
  fHit(hit), fLRSign(0), fLRSignIsFixed(false), fProjection(0) {
    SetLRSign(lrSign) ;
} ;

OTrReconstructedHit::OTrReconstructedHit(const OTrReconstructedHit& rhs) :
  fHit(rhs.fHit), fLRSign(rhs.fLRSign), fLRSignIsFixed(rhs.fLRSignIsFixed),
  fProjection(0) 
{
  if(rhs.fProjection) fProjection = new OTrNodeProjection(*(rhs.fProjection)) ;
} ;

OTrReconstructedHit::~OTrReconstructedHit() 
{
  if(fProjection) {
    delete fProjection ;
    fProjection=0;
  };
};

bool
OTrReconstructedHit::UpdateInternal() 
{
  bool changed=false ;
  if(!fLRSignIsFixed && fProjection && GetLRSign()*fProjection->GetD()<=0) {
    FlipLRSign() ;
    changed = true ;
    //SetLRSign(fProjection->GetD()) ;
  };
  return changed ;
};

#ifdef USE_ARTE
ArtePointer<RHIT>
OTrReconstructedHit::AppendRHIT(const OTrTrackDescription* track)
{
  RHIT rhit;
  fRHIT = ArteTable<RHIT>::push_back(rhit) ;
  ArtePointer<HITB> hitb = GetMeasuredHit()->GetHITB() ;
  float dz = GetMeasuredHit()->GetWire()->GetRawZ() -
    track->GetZ() ;
  fRHIT->x  = dz*track->GetTx() + track->GetX() ; 
  fRHIT->y  = dz*track->GetTy() + track->GetY() ; 
  fRHIT->tx = track->GetTx() ;
  fRHIT->ty = track->GetTy() ;
  fRHIT->q  = fabs(hitb->q)*GetLRSign() ; //GetNodeProjection()->GetD() ;
  fRHIT->p  = 1E-6; // Set 1/p to very high value (no multiple scatt.)
  //hitb->q = fabs(hitb->q)*GetLRSign() ;
  createRelation(hitb,fRHIT,&HITB::rhit) ;
  return fRHIT ;
}
#endif //USE_ARTE
