#include "OTrReconstructedTrackInterface.hh"
#include "OTrReconstructedHit.hh"
#ifdef USE_ARTE
#include "OTrHit.hh"
#include "OTrWire.hh"
#include <algorithm>
#endif

float 
OTrReconstructedTrackInterface::fgMinChi2Probability=1 ;

OTrReconstructedTrackInterface::OTrReconstructedTrackInterface() 
  : fHits(), fChi2(0), fError(0), fRSEG(0)
{
  return ;
} ;

OTrReconstructedTrackInterface::OTrReconstructedTrackInterface(const OTrReconstructedTrackInterface& rhs) 
  : fHits(), fChi2(0), fError(0), fRSEG(0)
{
  OTrReconstructedTrackInterface::operator=(rhs) ;
  return ;
};

OTrReconstructedTrackInterface::~OTrReconstructedTrackInterface()
{
  EraseHits() ;
};

OTrReconstructedTrackInterface& 
OTrReconstructedTrackInterface::operator=(const OTrReconstructedTrackInterface& rhs) {
  OTrTrackDescription::operator=(rhs) ;
  fChi2  = rhs.GetChi2() ;
  fError = rhs.GetError() ;
  fRSEG  = rhs.GetRSEG() ;
  // delete old hits
  EraseHits() ;
  // create copies
  for(HitContainer::const_iterator rhsHitIt=rhs.fHits.begin() ;
      rhsHitIt!=rhs.fHits.end() ; rhsHitIt++)
    fHits.push_back(new OTrReconstructedHit(**rhsHitIt)) ;
  return *this ;
};

void
OTrReconstructedTrackInterface::EraseHits() 
{
  for(HitContainer::const_iterator hitIt=fHits.begin() ;
      hitIt!=fHits.end() ; hitIt++) delete *hitIt ;
  fHits.erase(fHits.begin(),fHits.end()) ;
}
const OTrReconstructedHit* 
OTrReconstructedTrackInterface::FindHit(const OTrHit* hit) const 
{
  const OTrReconstructedHit* rv(0) ;
  for(HitContainer::const_iterator hitIt=fHits.begin() ;
      hitIt!=fHits.end() ; hitIt++)
    if((*hitIt)->GetMeasuredHit()==hit) {
      rv = *hitIt ;
      break ;
    };
  return rv ;
} ;

#ifdef USE_ARTE

inline bool 
CompareReconstructedHits(const OTrReconstructedHit* lhs,
			 const OTrReconstructedHit* rhs)
{
  return lhs->GetMeasuredHit()->GetWire()->GetRawZ() >
    rhs->GetMeasuredHit()->GetWire()->GetRawZ() ;
} ;

ArtePointerRSEG
OTrReconstructedTrackInterface::AppendRSEG()
{
  // Append this reconstructed track to the RSEG bank.

  //cout << "OTrReconstructedTrackInterface::AppendRSEG()" << endl ; 
  //cout << "This track: " << GetX() << " " << GetY() << " "
  //     << GetTx() << " " << GetTy() << " " << GetDim() << endl ;
  RSEG rseg ;
  fRSEG = ArteTable<RSEG>::push_back(rseg) ;
  fRSEG->ze  = fRSEG->zf  = GetZ();
  fRSEG->xe  = fRSEG->xf  = GetX();
  fRSEG->ye  = fRSEG->yf  = GetY();
  fRSEG->cmp = 2 ;   // that's us
  fRSEG->fit = 0 ;
  fRSEG->chi = GetChi2() ;
  fRSEG->pe  = fRSEG->pf  = 1E-6 ;
 
  fRSEG->txe = fRSEG->txf = GetTx();
  fRSEG->tye = fRSEG->tyf = GetTy() ;
  //cout << "fRSEG= " << fRSEG << endl ;
  int i=0;
  for(int row=0; row<GetCov().num_row(); row++)
    for(int col=0; col<=row; col++, i++)
      fRSEG->cve[i] = fRSEG->cvf[i] = GetCov()[row][col] ;
  sort(fHits.begin(), fHits.end(), CompareReconstructedHits) ;
  //fHits.sort( not2( compareCall( &OTrReconstructedHit::GetZ ) ) ) ;
  for(HitContainer::iterator hitIt=fHits.begin();
      hitIt!=fHits.end(); hitIt++) {
    ArtePointer<RHIT> rhit = (*hitIt)->AppendRHIT(this) ;
    //cout << rhit << " " << rhit->hitb << endl ;
    createRelation(fRSEG,rhit,&RSEG::rhit) ;       // rseg<->rhit
    createRelation(fRSEG,rhit->hitb,&RSEG::hitb) ; // rseg<->hitb
  };

  // Maarten prefers these settings:
  fRSEG->xe = fHits.front()->GetRHIT()->x ;
  fRSEG->ye = fHits.front()->GetRHIT()->y ;
  fRSEG->ze = fHits.front()->GetMeasuredHit()->GetWire()->GetRawZ() ;
  fRSEG->xf = fHits.back()->GetRHIT()->x ;
  fRSEG->yf = fHits.back()->GetRHIT()->y ;
  fRSEG->zf = fHits.back()->GetMeasuredHit()->GetWire()->GetRawZ() ;
  
  return fRSEG ;
};

#endif
