#include <iomanip.h>
#include "OTrLayer.hh"
#include "OTrHit.hh"
#include "OTrGede.hh"
#include "OTrEvent.hh"
#include "OTrSection.hh"
#include "OTrAlignmentProjection.hh"

#ifdef USE_ROOT
 #include <TH1.h>
 #include <TPad.h>
#endif

//__________________________________________________________________
// OTrSection
//
// Container for:
// * OTrWires
// Functionality:
// * OTrSection is derived from OTrGede ... :-)
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(OTrSection) ;
//void OTrSection::Streamer(TBuffer& b) {} ;
//void OTrSection::ShowMembers(TMemberInspector &, char *) {} ;
#endif

bool OTrSection::fgAlignmentOn=true ;

OTrSection::OTrSection( OTrGede* gede, OTrLayer* lay, 
			const ComponentId_t id ) :
  OTrGede(*gede), OTrDetectorComponent(id),  
  fWires(), fLayer(lay), fAlignmentProjection(0)
{
  //CreateWires(); create the wires later, when the pointer to section is fixed
  //cout << "OTrSection, creating a section with id " << GetId() << endl ;
  //  fHitList = list<OTrHit*>();
} 

OTrSection::~OTrSection() 
{ 
} ;

OTrSection::OTrSection() :
  OTrGede(), OTrDetectorComponent(), 
  fWires(), fLayer(0), fAlignmentProjection(0)
{
  cout << "** OTrSection::OTrSection(): do not call !!" << endl ;
  //fHitList = list<OTrHit*>();
}

//OTrSection::OTrSection(OTrLayer* layer, ComponentId_t id) :
//OTrDetectorComponent(id), OTrGede(), fWires(), fLayer(layer),  
//fSectorPlot(0)
//{
//CreateWires();
  //cout << "1: creating a section with id " << GetId() << endl ;
  //fHitList = list<OTrHit*>();
//}

void OTrSection::CreateWires()
{
  for ( int w = 0; w < GetNWires(); w++ ) {
    fWires.push_back( OTrWire(w, this, SetWireId(GetId(),w)) ) ;
  }
  if(fgAlignmentOn) 
    fAlignmentProjection = new OTrAlignmentProjection(this) ;
}

// const OTrWire*
// OTrSection::FindWire( const int wirenumber ) const 
// {
//   const OTrWire* w = 0;
//   for ( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//         wireIt != GetWires().end() ; wireIt++) {
//     //cout << wireIt->GetWireNumber() << " ";
//     if ( wireIt->GetWireNumber() == wirenumber ) { 
//       w = wireIt;
//       break;
//     }
//   } 
//   return w;
// }

#ifdef USE_ROOT

TString OTrSection::GetName() const
{
  char name[16];
  sprintf(name,"Sec_%.3d%.3d%.3d",GetSuperLayerId(),GetLayerId(),GetSectionId()) ;
  return TString(name) ;
}

// float
// OTrSection::OccupancySummary( ostream& ofs, bool useMask )
// {
//   const int nWires = fWires.size() ;
//   float occup = 0;
//   int   ntot  = 0;
//   ofs << "###############" << GetId() << " " << GetSectionId() 
//       << "###############" << endl;
//   //if ( useMask ) {
//   //  cout << "using the mask" << endl;
//   //} else {
//   //  cout << "not using the mask" << endl;
//   //}
//   int nprinted = 0;
//   for( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//        wireIt != GetWires().end() ; wireIt++) {
//     const OTrChannel* chan = wireIt->GetChannel() ;
//     float occ;
//     if (useMask && ! chan->GoodMask()) {
//       occ = -1;
//     } else {
//       occ = chan->GetOccupancy();
//     }
//     ofs << setw(3) << wireIt->GetWireNumber() << "  " 
// 	<< setw(6) << setprecision(4) << occ; 
//     nprinted++;
//     if ( nprinted%8 == 0 ) { ofs << endl; nprinted = 0; }
//     if ( occ < 0.01 || occ > 0.30 ) continue;
//     ntot++;
//     occup += chan->GetOccupancy();
//     //cout << wireIt->GetWireNumber() << ": " << chan->occupancy() << endl;
//   } 

//   if ( ntot > 0 ) {
//     occup = occup/float(ntot);
//   } else {
//     occup = 0;
//   }  
//   if ( GetEStereoAngle() == 0 ) ofs << "-5 degrees: ";
//   if ( GetEStereoAngle() == 1 ) ofs << " 0 degrees: ";
//   if ( GetEStereoAngle() == 2 ) ofs << "+5 degrees: "; 
//   ofs << GetSectionId() << " ntot, occup = " << ntot << " " 
//       << setprecision(2) << occup << endl;
//   const OTrWire* firstwire = FindWire(0);
//   const OTrWire* lastwire = FindWire(nWires-1);
//   if ( firstwire == 0 || lastwire == 0 ) {
//     ofs << "*** ERROR wire not found" << endl;
//   } else {
//     ofs << "position of 1st wire: " << setprecision(4) 
// 	<< firstwire->GetCentralX() << " " 
// 	<< firstwire->GetCentralY() << " " 
// 	<< firstwire->GetRawZ() << endl 
// 	<< "position of " << nWires << "th wire: "
// 	<< lastwire->GetCentralX() << " " 
// 	<< lastwire->GetCentralY() << " " 
// 	<< lastwire->GetRawZ() << endl ;
//   }
//   return occup;
// }

// TH1D
// OTrSection::GetTZeroH1(bool useMask) const
// {
//   const int nWires = fWires.size() ;
//   TString h1Name(GetName());
//   h1Name.Append("_TZero") ;
//   TH1D histo( h1Name, "", nWires, 0, nWires );
//   for( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//        wireIt != GetWires().end() ; wireIt++) {
//     const OTrChannel* chan = wireIt->GetChannel() ;
//     if(useMask && ! chan->GoodMask()) continue ;
//     histo.SetBinContent(wireIt->GetWireNumber()+1,chan->GetTZero().GetValue()) ;
//     histo.SetBinError(wireIt->GetWireNumber()+1,chan->GetTZero().GetError()) ;
//   } 
//   return histo ;
// };

// TH1D
// OTrSection::GetTimeH1() const {
//   TString timeH1Name(GetName());
//   timeH1Name.Append("_TimeH1") ;
//   TH1D timeH1(timeH1Name,timeH1Name,OTrChannel::GetNTimeBins(),
//               OTrChannel::GetMinimumTimeAxis(),
// 	      OTrChannel::GetMaximumTimeAxis()) ;
//   for( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//        wireIt != GetWires().end() ; wireIt++) {
//     const OTrChannel* chan = wireIt->GetChannel() ;
//     if(chan) {
//       TH1D histo = chan->GetTimeH1() ;
//       timeH1.Add(&histo,1) ;
//     };
//   };
//   return timeH1;
// } ;

// TH1D
// OTrSection::GetAvTimeH1( bool useHistogram, bool useMask )
// {
//   const int nWires = fWires.size() ;
//   TString h1Name(GetName());
//   h1Name.Append("_AvTime") ;
//   TH1D histo( h1Name, "", nWires, 0, nWires );

//   /*
//   if ( useHistogram && !OTrChannel::fgTimeHistogramming ) {
//     cout << "** ERROR in OTrSection::GetAvTimeH1, time histogramming " << endl;
//     cout << "   is off. Cannot use histogram to determine average." << endl;
//     return histo;
//   }
//   */

//   for( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//        wireIt != GetWires().end() ; wireIt++) {
//     const OTrChannel* chan = wireIt->GetChannel() ;
//     if(useMask && ! chan->GoodMask()) continue ;
//     float average = 0;
//     if ( !useHistogram ) { 
//       average = chan->GetAverageTime() ;
//     } else {
//       average = chan->GetAverageFromTimeH1();
//       //cout << " av = " << average << " " 
//       //   << chan->GetAverageTime() << endl;
//     };
//     histo.SetBinContent( wireIt->GetWireNumber()+1, average );
//   } ;
//   return histo ;
// }

// TH1D
// OTrSection::GetSigmaTimeH1( bool useHistogram, bool useMask )
// {
//   const int nWires = fWires.size() ;
//   TString h1Name(GetName());
//   h1Name.Append("_SigmaTime") ;
//   TH1D histo( h1Name, "", nWires, 0, nWires );
//   for( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//        wireIt != GetWires().end() ; wireIt++) {
//     const OTrChannel* chan = wireIt->GetChannel() ;
//     if(useMask && ! chan->GoodMask()) continue ;
//     float sigma = 0;
//     if ( !useHistogram ) { 
//       sigma = chan->GetSigma() ;
//     } else {
//       sigma = chan->GetTimeH1().GetRMS();
//     }
//     histo.SetBinContent( wireIt->GetWireNumber()+1, sigma ) ; 
//   } ;
//   return histo ;
// }

// TH1D
// OTrSection::GetOccH1( bool useMask ) const
// {
//   if ( useMask ) {
//     cout << "using the mask" << endl;
//   } else {
//     cout << "not using the mask" << endl;
//   }
//   const int nWires = fWires.size() ;
//   TString h1Name(GetName());
//   h1Name.Append("_Occupancy") ;
//   TH1D histo( h1Name, "", nWires, 0, nWires );
//   int nskipped = 0;
//   for( vector<OTrWire>::const_iterator wire = GetWires().begin() ;
//        wire != GetWires().end() ; wire++) {
//     const OTrChannel* chan = wire->GetChannel() ;
//     if ( chan == 0 ) { continue; }
//     if ( useMask && !chan->GoodMask()  ) {
//       nskipped++;
//       continue; 
//     }
//     histo.Fill( wire->GetWireNumber()+1, chan->GetOccupancy() );
//   }
//   //cout << "number of skipped channels = " << nskipped << endl;
//   return histo;
// }

// int OTrSection::Plot(TPad* p, const int choice, char* option, const int cl,
// 	  	     const OTrSuperLayerEvent* slEvent, const bool use_mask )
// {
//   p->cd();
//   // Delete existing plot
//   if ( fSectorPlot != 0 ) {
//     delete fSectorPlot;
//     fSectorPlot = 0;
//   }
  
//   //cout << "in OTrSection::plot()" << endl;

//   const int nWires = fWires.size() ;
//   switch (choice)
//     {
//     case 1: // occupancy for each channel
//       fSectorPlot = new TH1D( GetOccH1( use_mask ) ) ;
//       //fSectorPlot->SetMaximum(1.2);
//       fSectorPlot->SetMaximum(0.05);
//       //fSectorPlot->SetMaximum(0.15);
//       fSectorPlot->SetFillColor(10);
//       break ;
      
//     case 2: // tdc count for each hit channel
//       char name[30];
//       sprintf( name, "%s%d", "hit_channels_", GetId() );
//       fSectorPlot = new TH1D( name, "", nWires, 0, nWires );
//       if ( slEvent ) {
// 	const OTrSuperLayerEvent::HitContainer& hits = slEvent->GetHits();
// 	//cout << "OTrSection::plot(), hits.size() = " << hits.size() << endl;
// 	for( OTrSuperLayerEvent::HitContainer::const_iterator hitp = hits.begin();
// 	     hitp != hits.end(); hitp++) {
// 	  const OTrWire* wire = (*hitp)->GetWire();

// 	  if ( wire->GetSection() != this ) continue; 
// 	  OTrChannel* chan = wire->GetChannel() ;
// 	  if ( use_mask && !chan->GoodMask()  ) continue; 
// 	  fSectorPlot->Fill( wire->GetWireNumber()+1, (*hitp)->GetTdcCounts() );
// 	}
//       }
//       fSectorPlot->SetMaximum(260);
//       break ;
  
//     case 3: // average tdc value for each channel
//       fSectorPlot = new TH1D(GetAvTimeH1(true, false)) ;
//       //fSectorPlot->GetYaxis()->SetLimits(120,200);
//       if ( GetEPitch() == OTrGede::kFiveMM ) {
// 	fSectorPlot->SetMinimum(160);
// 	fSectorPlot->SetMaximum(210); 
//       } else {
// 	fSectorPlot->SetMinimum(120);
// 	fSectorPlot->SetMaximum(170);
//       }
//       break ;
  
//     case 4: // tdc spectrum for this section
//       fSectorPlot = new TH1D(GetTimeH1()) ;
//       break ;
      
//     case 5: // RMS tdc count for each channel
//       fSectorPlot = new TH1D(GetSigmaTimeH1( true, false )) ;
//       //fSectorPlot->SetMinimum(100);
//       //fSectorPlot->SetMaximum(200);
//       break ;
      
//     case 6: 
//       fSectorPlot = new TH1D(GetTZeroH1( use_mask )) ;
//       fSectorPlot->SetMinimum(-30);
//       fSectorPlot->SetMaximum(30);
//       break ;

//     case 7:
//       if(fgAlignmentOn)
// 	fSectorPlot = new TH1F(*(fAlignmentProjection->GetDeltaXH1())) ;
//       else 
// 	fSectorPlot = new TH1F("EMPTY","Sorry, alignment is off",1,0,1) ;
//       break ;
      
//     default:
//       fSectorPlot = new TH1F("EMPTY","NO SUCH CHOICE!",1,0,1) ;
//       break ;
//     }
    
//   fSectorPlot->SetLabelSize(0.05);
//   fSectorPlot->SetFillColor(0);
//   fSectorPlot->SetMarkerStyle(21);
//   fSectorPlot->SetMarkerSize(0.4);
//   fSectorPlot->SetMarkerColor(cl); 
//   fSectorPlot->SetLineColor(cl);

//   // histogram drawing option
//   char opt[20];
//   if ( choice == 2 ) {
//     sprintf(opt,"%s%s",option,"");
//     //    fSectorPlot->SetFillColor(cl);
//   } else {
//     sprintf(opt,"%s%s",option,"");
//   }  
//   fSectorPlot->Draw(opt) ;
//   return 0;
// }
#endif // USE_ROOT

void OTrSection::DumpBadChannels( ostream& os ) 
{
  os << "superlayer: " << GetSuperLayerId() << " "
     << "layer: " << GetLayerId() << " " 
     << "module: " << GetModuleId() << " "
     << "sector: " << GetSectorId() << " " 
     << "section: " << GetSectionId() << " " 
     << endl;
  for ( WireContainer::const_iterator wire = GetWires().begin();
	wire != GetWires().end(); wire++ ) {
    OTrChannel* chan = (*wire).GetChannel();
    if ( chan ) {
      float occ = chan->GetOccupancy();
      if ( occ < 0.01 || occ > 0.25 ) {
	os << (*wire).GetId() << " " << setprecision(3) 
	   << (*wire).GetChannel()->GetOccupancy() << " "; 
      }
    }
  }
  os << endl;
}

ostream & operator << (ostream & os, const OTrSection& s )
{
  return os << "section: " << s.GetId() << " " << s.GetSectionId();
}
