//#include <iomanip.h>
#include <algorithm>
#include "OTrSuperLayerEvent.hh"
#include "OTrHit.hh"
#include "OTrSuperCluster.hh"
#include "OTrSection.hh"

#ifdef USE_ARTE
 #include <arte/arte.hh>
 #include <arte/artefunc.h>
#endif

#ifdef USE_ROOT
 #include <TH2.h>
 #include <TPad.h>
#endif

//__________________________________________________________________
// OTrSuperLayerEvent
//
// Container for:
// * Hits
// * Section clusters
// * Super clusters
// Functionality
// * Reconstruction of clusters
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(OTrSuperLayerEvent) ;
//void OTrSuperLayerEvent::ShowMembers(TMemberInspector &, char *) {} ;
#endif

//#define __OTR_RECOINFO__

OTrSuperLayerEvent::OTrSuperLayerEvent() : fHits(), fClusters() {} ;

OTrSuperLayerEvent::~OTrSuperLayerEvent() 
{  
  //cout << "in the destructor of OTrSuperLayerEvent" << endl;
  // for(ClusterContainer::const_iterator clusterp= fClusters.begin() ;
  //    clusterp != fClusters.end() ; clusterp++)
  //  delete (*clusterp);

  for(HitContainer::const_iterator hitp=fHits.begin();
      hitp != fHits.end(); hitp++)
    delete (*hitp);

  for(ClusterContainer::const_iterator clIt=fClusters.begin() ;
      clIt!=fClusters.end(); clIt++)
    delete *clIt ;

  //fHits.erase( fHits.begin(), fHits.end() );
  //fClusters.erase( fClusters.begin(), fClusters.end() );
} ;

OTrHit*
OTrSuperLayerEvent::Add(OTrHit* hit) {
  if(hit)
    fHits.push_back(hit) ;
  else
    cout << "*** ERROR in OTrSuperLayerEvent::Add(OTrHit* hit)" << endl; 
  return fHits.back() ;
};

void 
OTrSuperLayerEvent::DumpHits(ostream& os) const
{
  os << "OTrSuperLayerEvent::DumpHits(...) dumps " << fHits.size() 
     << " hits." << endl ;
  int i=0;
  for(HitContainer::const_iterator hitp=fHits.begin();
      hitp != fHits.end(); hitp++, i++)
    if ( *hitp ) os << *hitp << endl ;
  //      os << (*hitp)->GetHITB().index() << " "
  //	 << (*hitp)->GetHITB()->q << endl ;
  //       os << setw(5) << i << " " << *hitp 
  // 	 << " channelId: " << (*hitp)->GetChannel()->GetId() 
  // 	 << " wireId: " << (*hitp)->GetWire()->GetId()
  // 	 << " layerId: " << (*hitp)->GetWire()->GetLayerId()
  // 	 << " tdcvalue:  " << (int)(*hitp)->GetTdcCounts() << endl ;
  return ;
}

#ifdef USE_ROOT
void
OTrSuperLayerEvent::Streamer(TBuffer& R__b)
{
  if (R__b.IsReading()) {
    int nHits ;
    R__b >> nHits ;
    for(int i=0; i<nHits; i++) {
      OTrHit* hitIt = new OTrHit() ;
      hitIt->Streamer(R__b) ;
      fHits.push_back(hitIt) ;
    }
  } else {
    R__b << fHits.size() ;
    for(HitContainer::iterator hitIt=fHits.begin();
	hitIt!=fHits.end(); hitIt++) (*hitIt)->Streamer(R__b) ;
  };
};

void 
OTrSuperLayerEvent::DrawHits()
{
  cout << "OTrSuperLayerEvent::DrawHits(), nr of hits = " << fHits.size()
       << endl;
  for(HitContainer::iterator hitp=fHits.begin();
      hitp != fHits.end(); hitp++)
    (*hitp)->Draw(2) ;
};

void 
OTrSuperLayerEvent::DrawClusters()
{
  cout << "OTrSuperLayerEvent::DrawClusters() " << fClusters.size() << endl ;
  int color=3 ;
  for(OTrSuperLayerEvent::ClusterContainer::iterator clusterp= fClusters.begin() ;
      clusterp != fClusters.end() ; clusterp++, color++)
    (*clusterp)->Draw(color) ;
};

#endif

void 
OTrSuperLayerEvent::DumpClusters(ostream& os) const
{
  os << "OTrSuperLayerEvent::DumpClusters(...) dumps " << fClusters.size()
     << " clusters. " << endl ;
  for(OTrSuperLayerEvent::ClusterContainer::const_iterator clusterp= fClusters.begin() ;
      clusterp != fClusters.end() ; clusterp++)
    (*clusterp)->Dump() ;
};

#ifdef USE_ROOT
void
OTrSuperLayerEvent::PerformHistogramming()
{
  if(OTrChannel::GetHistogrammingMode()==1)
    {
      for(ClusterContainer::iterator clusterIt=fClusters.begin();
	  clusterIt!=fClusters.end(); clusterIt++)
	(*clusterIt)->PerformHistogramming() ;
    } else {
      for(HitContainer::iterator hitIt = fHits.begin();
	  hitIt!=fHits.end(); hitIt++)
	(*hitIt)->PerformHistogramming() ;
    };
};
#endif

bool compareHit( OTrHit* lhs, OTrHit* rhs )
{
  if ( lhs == 0 || rhs == 0 ) cout << "lhs, rhs = " << lhs << " " << rhs << endl ;
  return (lhs->GetWire()->GetGede()->GetGEDEIndex() < rhs->GetWire()->GetGede()->GetGEDEIndex()) ||
    ((lhs->GetWire()->GetGede()->GetGEDEIndex()==rhs->GetWire()->GetGede()->GetGEDEIndex()) 
     && (lhs->GetWire()->GetPRWireNumber() < rhs->GetWire()->GetPRWireNumber()));
} ;

void
OTrSuperLayerEvent::SortHits()
{
  //cout << "OTrSuperLayerEvent::SortHits()" << endl ;
  sort( fHits.begin(), fHits.end(), compareHit );
  //fHits.sort( compareHit );
}

#ifdef USE_ARTE
void
OTrSuperLayerEvent::AppendHITB()  
{
  // Append all hits in this superlayer to the HITB bank. Before
  // calling this function make sure that the hits are sorted with SortHits().
  ArtePointer<GEDE> previousGEDE = 0;
  ArtePointer<HITB> firstHITB = 0;
  ArtePointer<HITB> previousHITB = 0;
  for(HitContainer::iterator hitIt = fHits.begin() ;
      hitIt != fHits.end() ; hitIt++) {
    ArtePointer<GEDE> currentGEDE = (*hitIt)->GetWire()->GetGede()->GetGEDE();
    //cout << "in OTrSuperLayerEvent appending .." << endl;
    ArtePointer<HITB> currentHITB = (*hitIt)->AppendHITB();
    if ( previousHITB == 0 ) {
      firstHITB = currentHITB;
      previousGEDE = currentGEDE;
    } else {
      if ( currentGEDE != previousGEDE ) {
        previousGEDE->hit1 = firstHITB ;
	previousGEDE->hitl = previousHITB ;
	//cout << "first, last = " << firstHITB << " " 
	//<< previousHITB << endl;
	previousGEDE = currentGEDE;
	firstHITB = currentHITB;
      }
    }
    previousHITB = currentHITB;
  }
}

void 
OTrSuperLayerEvent::AppendRSEG()
{
  // Append OTrSuperClusters to the RSEG bank. I added this routine
  // for FLT studies. They are not yet interested in tracks, only in
  // clusters.
  for(ClusterContainer::iterator clusIt = fClusters.begin() ;
      clusIt != fClusters.end() ; clusIt++) (*clusIt)->AppendRSEG();
}
#endif


void
OTrSuperLayerEvent::SearchSectionClusters()
{
  //This is the cluster search algorithm for clusters in one OTrSection only.

#ifdef USE_ROOT
  static TH1F* hitMultH1 = 
    new TH1F("hitMultH1","",200,-0.5,199.5) ;
  static TH1F* sectionClusterSizeSLMinusH1 = 
    new TH1F("sectionClusterSizeSLMinusH1","",20,-0.5,19.5) ; 
  static TH1F* sectionClusterSizeSLPlusH1 = 
    new TH1F("sectionClusterSizeSLPlusH1","",20,-0.5,19.5) ;
  static TH1F* sectionClusterSizeDLMinusH1 = 
    new TH1F("sectionClusterSizeDLMinusH1","",20,-0.5,19.5) ;
  static TH1F* sectionClusterSizeDLPlusH1 = 
    new TH1F("sectionClusterSizeDLPlusH1","",20,-0.5,19.5) ;
  static TH1F* sectionClusterMultH1 = 
    new TH1F("sectionClusterMultH1","",100,-0.5,99.5) ;
#endif USE_ROOT
  // Cleanup current stuff
  if(fPlusClusters.size()>0 || fZeroClusters.size()>0 || fMinusClusters.size()>0) {
    cout << "OTrSuperLayerEvent::CreateSectionClusters() has already been called !" << endl ;
    return ;
  };
  fPlusClusters.reserve(100) ;
  fZeroClusters.reserve(100) ;
  fMinusClusters.reserve(100) ;
  
  HitContainer::const_iterator hitIt(fHits.begin()), nextHitIt(hitIt) ;
  nextHitIt++;
  OTrSectionCluster* currentCluster = new OTrSectionCluster() ;
  for(hitIt=fHits.begin() ; hitIt!=fHits.end() ; hitIt++, nextHitIt++) {
    currentCluster->push_back(*hitIt) ;
    //     cout << (*hitIt)->GetWire()->GetGede()->index() << "("
    //     	 << (*hitIt)->GetWire()->GetWireNumber() << ","
    // 	 << (*hitIt)->GetWire()->GetPRWireNumber()<< ")" ;
    bool clusterIsComplete = (nextHitIt==fHits.end()) || 
      !OTrSectionCluster::SameSectionCluster((*hitIt)->GetWire(),
					     (*nextHitIt)->GetWire()) ;
    if(clusterIsComplete) {
      if(currentCluster->ValidNHits()) {
	switch (currentCluster->GetGede()->GetEStereoAngle())
	  {
	  case OTrGede::kPlusFiveDegrees :
	    fPlusClusters.push_back(*currentCluster) ; break ;
	  case OTrGede::kMinusFiveDegrees :
	    fMinusClusters.push_back(*currentCluster) ; break ;
	  default :
	    fZeroClusters.push_back(*currentCluster) ; break ;
	  }
      }
#ifdef USE_ROOT
      if(currentCluster->GetSection()->GetSuperLayerId()==2) {
	switch (currentCluster->GetGede()->GetNMonoLayers()+
		currentCluster->GetGede()->GetXSide())
	  {
	  case 2:
	    sectionClusterSizeSLMinusH1->Fill(currentCluster->size()) ; break ;
	  case 3:
	    sectionClusterSizeSLPlusH1->Fill(currentCluster->size()) ; break ;
	  case 4:
	    sectionClusterSizeDLMinusH1->Fill(currentCluster->size()) ; break ;
	  case 5:
	    sectionClusterSizeDLPlusH1->Fill(currentCluster->size()) ; break ;
	  default:
	    break ;
	  }
      }
#endif
      //      cout << "|" ;
      delete currentCluster;
      currentCluster = new OTrSectionCluster() ;
    }
  }
  delete currentCluster;
#ifdef USE_ROOT
  hitMultH1->Fill(fHits.size()) ;
  sectionClusterMultH1->Fill(fPlusClusters.size()+fZeroClusters.size()+
			     fMinusClusters.size()) ;  
#endif
#ifdef __OTR_RECOINFO__
  cout << "  NHits : " << fHits.size() 
       << "  NSectionClusters (+0-): "  << fPlusClusters.size() << " "
       << fZeroClusters.size() << " " << fMinusClusters.size() << endl ;
#endif
};

// inline bool 
// CompareXDif(const OTrSuperCluster& lhs, const OTrSuperCluster& rhs) {
//   return fabs(lhs.GetXDif()) < fabs(rhs.GetXDif()) ;
// } ;

inline bool 
CompareXDif(const OTrSuperCluster* lhs, const OTrSuperCluster* rhs) {
  return fabs(lhs->GetXDif()) < fabs(rhs->GetXDif()) ;
} ;

inline bool 
CompareChi2(const OTrSuperCluster* lhs, const OTrSuperCluster* rhs) {
  return fabs(lhs->GetChi2OverNDof()) < fabs(rhs->GetChi2OverNDof()) ;
} ;

int
OTrSuperLayerEvent::SearchClusters()
{
  // Ths routine implements the cluster finding algorithm in three
  // stereo layers. It creaes co called OTrSuperClusters, which are
  // made up of OTrSectionClusters, defined in the
  // SearchSectionCluster memberfunction.
#ifdef USE_ROOT
  static TH1F* xdifH1 = new TH1F("xdifH1","",500,-3,3) ;
  static TH1F* superClusterMultH1 = new TH1F("superClusterMultH1","",100,-0.5,99.5) ;
  static TH1F* superClusterSizeH1 = new TH1F("superClusterSizeH1","",20,-0.5,19.5) ;
#endif

  SearchSectionClusters() ;

  // Make all combinations of one plus and one minus cluster. Find a
  // zero cluster that is close enough in the Tx projection. If more
  // than one cluster is close enough, choose the best one. This is
  // not completely unbiased but pays a bit in speed.
#define OTR_SEARCHCLUSTERS_A
#ifdef OTR_SEARCHCLUSTERS_A
  OTrSuperCluster* currentCluster ;
  for(vector<OTrSectionCluster>::iterator minusIt=fMinusClusters.begin();
      minusIt!=fMinusClusters.end(); minusIt++) 
    for(vector<OTrSectionCluster>::iterator plusIt=fPlusClusters.begin();
	plusIt!=fPlusClusters.end(); plusIt++)
      {
	currentCluster = new OTrSuperCluster(plusIt,minusIt) ;
	if(currentCluster->WithinAcceptance()) {
	  for(vector<OTrSectionCluster>::iterator zeroIt=fZeroClusters.begin();
	      zeroIt!=fZeroClusters.end(); zeroIt++)
	    currentCluster->AddZeroIfValid(zeroIt) ;
	  if(currentCluster->Valid()) {
	    fClusters.push_back(currentCluster) ;
	  } else {
	    delete currentCluster ;
	  };
	} else {
	  delete currentCluster ;
	} 
      }
#else
  // Make ALL valid combinations of one plus, one minus and one zero cluster.
  OTrSuperCluster* currentCluster ;
  for(vector<OTrSectionCluster>::iterator plusIt=fPlusClusters.begin();
      plusIt!=fPlusClusters.end(); plusIt++)
    for(vector<OTrSectionCluster>::iterator minusIt=fMinusClusters.begin();
  	minusIt!=fMinusClusters.end(); minusIt++) { 
      currentCluster = new OTrSuperCluster(plusIt,minusIt) ;
      if(currentCluster->WithinAcceptance()) {
  	for(vector<OTrSectionCluster>::iterator zeroIt=fZeroClusters.begin();
  	    zeroIt!=fZeroClusters.end(); zeroIt++)
  	  if(currentCluster->AddZeroIfValid(zeroIt)) {
  	    fClusters.push_back(currentCluster) ;
  	    currentCluster = new OTrSuperCluster(plusIt,minusIt) ;
  	  };
      };
      delete currentCluster ;
    };
#endif
#ifdef __OTR_RECOINFO__
  cout << "Found " << fClusters.size() << " valid clusters, left " ;
#endif
  
  // Or better ??!! : sort on chi2 ....
  //  sort(fClusters.begin(),fClusters.end(),CompareChi2) ;

  // Fit all clusters in the Tx-Ty plane. If the fit yields an invalid
  // chi2, remove the cluster.
#ifdef OTR_SEARCHCLUSTERS_SLOW
   for(ClusterContainer::iterator clusterIt=fClusters.begin() ;
      clusterIt!=fClusters.end(); clusterIt++) {
    (*clusterIt)->Connect() ; // This call is essential (and before Fit)!!!!
    (*clusterIt)->CreateReconstructedHits() ;
    if(!(*clusterIt)->ValidChi2()) {
      delete *clusterIt ;
      fClusters.erase(clusterIt--) ;
    };
  } ;
#endif
#ifdef __OTR_RECOINFO__
  cout << fClusters.size() << " after chi2 cut, left" ;
#endif

  // Now sort on xdiff
  //fClusters.sort(CompareXDif) ;
  //sort(fClusters.begin(),fClusters.end(),CompareXDif) ;
   // Or better ??!! : sort on chi2 ....
  sort(fClusters.begin(),fClusters.end(),CompareChi2) ;
 
  /** REMOVE OR MERGE ALL OVERLAPPING CLUSTERS **/
  /** (Be careful with order: this is the only correct one ?!) **/
  //   for(ClusterContainer::reverse_iterator tClusterIt=fClusters.rbegin();
  //       tClusterIt!=fClusters.rend(); tClusterIt++) 
  //     for(ClusterContainer::reverse_iterator tClusterIt2=fClusters.rbegin();
  // 	tClusterIt2!=tClusterIt; tClusterIt2++) 
  //       if(tClusterIt->Merge(*tClusterIt2) || !tClusterIt2->Valid()) {
  // 	fClusters.erase(&(*tClusterIt2)) ;
  //       }; 

  #ifdef OTR_SEARCHCLUSTERS_A
  /** This seems the optimal solution, but it is also slow ...! Is it finite ? **/
  bool merged ;
  for(ClusterContainer::iterator clusterIt=fClusters.begin() ;
      clusterIt!=fClusters.end(); clusterIt++) 
    if(*clusterIt) {
      ClusterContainer::iterator clusterIt2= fClusters.end();
      clusterIt2--;
      for(;clusterIt2!= clusterIt; clusterIt2--)
	if(*clusterIt2 && 
	   ((merged=(*clusterIt)->Merge(**clusterIt2)) || !(*clusterIt2)->Valid())) {
	  delete *clusterIt2;
	  *clusterIt2 = 0;
	  fClusters.erase(clusterIt2) ;
	  //#ifdef OTR_SEARCHCLUSTERS_SLOW
	  if(merged) clusterIt2=fClusters.end();
	  //#endif
	};
    };
  #endif
#ifdef __OTR_RECOINFO__
  cout << fClusters.size() << " after cleanup. " << endl ;
#endif
  for(ClusterContainer::iterator clusterIt=fClusters.begin();
      clusterIt!=fClusters.end(); clusterIt++) {
    (*clusterIt)->Connect() ; // This call is essential (and before Fit)!!!!
    //(*clusterIt)->RawWireFit() ;
    (*clusterIt)->CreateReconstructedHits() ;
#ifdef USE_ROOT
    superClusterSizeH1->Fill(float((*clusterIt)->GetSectionClusters().size())) ;
    xdifH1->Fill(float((*clusterIt)->GetXDif()/ (*clusterIt)->GetZeroClusters().front()->
		       GetGede()->GetWirePitch())) ;
#endif
    if(!(*clusterIt)->ValidChi2()) {
      delete *clusterIt ;
      fClusters.erase(clusterIt--) ;
    };
  };  
#ifdef USE_ROOT
  superClusterMultH1->Fill(fClusters.size()) ;
#endif
  return fClusters.size() ;
};


//   for(ClusterContainer::iterator tClusterIt=fClusters.begin();
//       tClusterIt!=fClusters.end(); tClusterIt++) {
//     ClusterContainer::iterator tClusterIt2 = tClusterIt ;
//     tClusterIt2++;
//     for( ;tClusterIt2!=fClusters.end(); tClusterIt2++)
//       for(OTrSuperCluster::ClusterContainer::const_iterator secClusIt=
// 	    tClusterIt->GetSectionClusters().begin() ; 
// 	  secClusIt!=tClusterIt->GetSectionClusters().end() ; secClusIt++)
// 	for(OTrSuperCluster::ClusterContainer::const_iterator secClusIt2=
// 	      tClusterIt2->GetSectionClusters().begin() ; 
// 	    secClusIt2!=tClusterIt2->GetSectionClusters().end() ; secClusIt2++)
// 	  if((*secClusIt2)==(*secClusIt))
// 	    cout << "SCHEISSE!!! " << tClusterIt->GetSectionClusters().size() << " "
// 		 <<  tClusterIt2->GetSectionClusters().size() << " " << *secClusIt << endl ;
//   };
  
  // int
// OTrSuperLayerEvent::SearchClusters()
// {
// #ifdef USE_ROOT
//   /*
//   static TH2F* xzeroVSxcross = new TH2F("xzeroVSxcross","",
// 					25,-45,-10,25,-45,-10);
//   static TH1F* xzeroMINxcross = new TH1F("xzeroMINxcross","",
// 					100,-30,30);
// 					*/
// #endif
//   //cout << "OTrSuperLayerEvent::SearchClusters()" << endl ;
//   //cout << "OTrSlEvent: number of hits = " << fHits.size() << endl;
//   //cout << "OTrEvent fClusters.size() = " << fClusters.size() << endl;
//   //fClusters.erase(fClusters.begin(),fClusters.end()) ;
//   //cout << "ok" << endl;

//   if (GetNHits() > 200 ) return 0; // skip the largest events

//   /** SORT HITS IN STEREOLAYERS **/

//   vector<OTrHit*> tPlusHits ;
//   vector<OTrHit*> tMinusHits ;
//   vector<OTrHit*> tZeroHits ;
//   for(HitContainer::iterator hitp=fHits.begin();
//       hitp != fHits.end(); hitp++)
//     switch ( (*hitp)->GetWire()->GetSubSection()->GetSection()->
// 	     GetGede()->GetEStereoAngle())
//       {
//       case OTrGede::kPlusFiveDegrees :
// 	tPlusHits.push_back( (*hitp) ) ; break ;
//       case OTrGede::kMinusFiveDegrees :
// 	tMinusHits.push_back( (*hitp) ) ; break ;
//       default :
// 	tZeroHits.push_back( (*hitp) ) ; break ;
//       };

//   //cout << "OTrEvent: zero, plus, minus = " 
//   //   << tZeroHits.size() << " " 
//   //   << tPlusHits.size() << " " 
//   //   << tMinusHits.size() << endl; 
//   if(!tZeroHits.size() || !tPlusHits.size() || !tMinusHits.size())
//     return 0;
  
//   /** MAKE ALL VALID CLUSTERS **/
//   for(vector<OTrHit*>::iterator plusHitIt = tPlusHits.begin();
//       plusHitIt!=tPlusHits.end(); plusHitIt++)
//     for(vector<OTrHit*>::iterator minusHitIt = tMinusHits.begin();
// 	minusHitIt!=tMinusHits.end(); minusHitIt++)
//       if((*plusHitIt)->SameCluster(**minusHitIt))
// 	for(vector<OTrHit*>::iterator zeroHitIt = tZeroHits.begin();
// 	    zeroHitIt !=tZeroHits.end() ; zeroHitIt++) {

// #ifdef USE_ROOT
// 	  /*
// 	  if ( (*zeroHitIt)->GetWire()->GetSuperLayerId() == 10 ) {
// 	    if ( (*zeroHitIt)->GetWire()->GetSectionId() == 50 && 
// 		 (*plusHitIt)->GetWire()->GetSectionId() == 50 && 
// 		 (*minusHitIt)->GetWire()->GetSectionId() == 50 ) {
// 	      int ntotal = 0;
// 	      for(vector<OTrHit*>::iterator hit = tZeroHits.begin();
// 		  hit !=tZeroHits.end() ; hit++) {
// 		if ( (*hit)->GetWire()->GetSectionId() ==
// 		     (*zeroHitIt)->GetWire()->GetSectionId() ) {
// 		  ntotal++;
// 		}
// 	      }  
// 	      if ( ntotal > 5 ) {
// 		//cout << "ntotal = " << ntotal << endl;
// 	      } else {
// 		OTrHitCluster* aCluster = new OTrHitCluster(*plusHitIt) ;
// 		aCluster->Add(*minusHitIt) ;
// 		aCluster->Add(*zeroHitIt) ;
// 		float mu;
// 		double xCross = aCluster->GetXcross( mu );
// 		double xZero = aCluster->GetXzero();
// 		//cout << "xCross, xZero, mu = " 
// 		//   << xCross << " " << xZero << " " << mu << endl;
// 		if ( mu > 0 && mu < 1 ) {
// 		  //cout << "filling" << endl;
// 		  xzeroVSxcross->Fill( float(xZero), float(xCross) );
// 		  xzeroMINxcross->Fill( xZero - xCross );
// 		}
// 		delete aCluster;
// 	      }
// 	    }
// 	  }
// 	  */
// #endif

// 	  if((*plusHitIt)->SameCluster(**zeroHitIt) &&
// 	     (*minusHitIt)->SameCluster(**zeroHitIt))
// 	    {
// 	      OTrHitCluster* newCluster = new OTrHitCluster(*plusHitIt) ;
// 	      newCluster->Add(*minusHitIt) ;
// 	      newCluster->Add(*zeroHitIt) ;

// 	      if( newCluster->ValidXDiff()) {
// 		newCluster->Valid();
// 		fClusters.push_back(newCluster);
// 	      } else {
//                 newCluster->Valid();
// 		//cout << "OTrEvent: deleting a new cluster" << endl;
// 		delete newCluster;
// 	      }
// 	    }
// 	};
//   //cout << "OTrSlEvent: found " << fClusters.size() 
//   //    << " cluster candidates."  << endl;
//   /** SORT CLUSTERS **/
//   fClusters.sort(CompareChi2) ;
  
//   /** REMOVE OR MERGE ALL OVERLAPPING CLUSTERS **/

//   for(list<OTrHitCluster*>::iterator tClusterIt=fClusters.begin();
//       tClusterIt!=fClusters.end(); tClusterIt++) {
//     list<OTrHitCluster*>::iterator tClusterIt2 = fClusters.end() ;
//     tClusterIt2--;
//     for( ; tClusterIt2!=tClusterIt; tClusterIt2--) {
//       if( (*tClusterIt)->MergeIfOverlap((*tClusterIt2)) ) {
// 	//cout << "erasing a cluster due to overlap: " << endl;
// 	delete *tClusterIt2;
// 	fClusters.erase(tClusterIt2) ;
// 	//tClusterIt2-- ;
//       };
//     };
//   };
//   //cout << "OTrSlEvent: selected " << fClusters.size() << " clusters." 
//   //<< endl ;
//   return fClusters.size() ;
// >>>>>>> 1.20
// };
/*
   HitContainer::const_iterator hitIt(fHits.begin()), previousHitIt(hitIt), 
    firstHitIt(hitIt) ;
  hitIt++;
  int size=0;
  for( ; hitIt!=fHits.end() ; hitIt++) {
    if(hitIt->GetWire()->GetGede()!=previousHitIt->GetWire()->GetGede() ||
       hitIt->GetWire()->GetWireNumber()!=previousHitIt->GetWire()->GetWireNumber()+1) {
      if(size<=fgMaxNHitsPerSectionCluster)
	switch (firstHitIt->GetWire()->GetGede()->GetEStereoAngle())
	  {
	  case OTrGede::kPlusFiveDegrees :
	    fPlusClusters.push_back(OTrSectionCluster(firstHitIt,hitIt,size)) ; break ;
	  case OTrGede::kMinusFiveDegrees :
	    fMinusClusters.push_back(OTrSectionCluster(firstHitIt,hitIt,size)) ; break ;
	  default :
	    fZeroClusters.push_back(OTrSectionCluster(firstHitIt,hitIt,size)) ; break ;
	  };
      clusterSize->Fill(size) ;
      size=0;
      firstHitIt = hitIt ;
    };
    size++ ;
    previousHitIt = hitIt ;
  };
  // This is for the last cluster   ///I think we miss the last cluster !!
  if(size<=fgMaxNHitsPerSectionCluster)
    switch (firstHitIt->GetWire()->GetGede()->GetEStereoAngle())
      {
      case OTrGede::kPlusFiveDegrees :
	fPlusClusters.push_back(OTrSectionCluster(firstHitIt,fHits.end(),size)) ; break ;
      case OTrGede::kMinusFiveDegrees :
	    fMinusClusters.push_back(OTrSectionCluster(firstHitIt,fHits.end(),size)) ; break ;
      default :
	fZeroClusters.push_back(OTrSectionCluster(firstHitIt,fHits.end(),size)) ; break ;
      };



     float x = It->GetRawZ()*tx ;
      float y = plusIt->GetRawZ()*ty ;
      
      const OTrWire* firstwire = plusIt->begin()->GetWire() ;
      const OTrWire* secondwire = minusIt->begin()->GetWire() ;
      float mu1,mu2 ;
      Point3D p(firstwire->CrossXY(secondwire,mu1)) ;
      Point3D q(secondwire->CrossXY(firstwire,mu2)) ;
      bool withinAcceptance2 = mu1>0 && mu1<1 && mu2>0 && mu2<1 ;
      cout << setw(8) << setprecision(3) << x
	   << setw(8) << setprecision(3) << y
	   << setw(4) << setprecision(1) << withinAcceptance
	   << setw(8) << setprecision(3) << p.x()
	   << setw(8) << setprecision(3) << p.y()
	   << setw(8) << setprecision(3) << mu1
	   << setw(8) << setprecision(3) << mu2
	   << setw(4) << setprecision(1) << withinAcceptance2
	   << endl ;
  // Calculate tx and ty of intersection
      float sindif = sin(plusIt->GetStereoAngle()-minusIt->GetStereoAngle()) ;
      float tx = (sin(plusIt->GetStereoAngle())*minusIt->GetRawTx() - 
		  sin(minusIt->GetStereoAngle())*plusIt->GetRawTx())/sindif ;
      float ty = (cos(plusIt->GetStereoAngle())*minusIt->GetRawTx() - 
		  cos(minusIt->GetStereoAngle())*plusIt->GetRawTx())/sindif ;
      // see wether this is actually in the gede
      if(plusIt->WithinAcceptance(tx,ty) && minusIt->WithinAcceptance(tx,ty) )
	// loop over all zero hits to find clusters
	for(vector<OTrSectionCluster>::iterator zeroIt=fZeroClusters.begin();
	    zeroIt!=fZeroClusters.end(); zeroIt++)
	  if(zeroIt->WithinAcceptance(tx,ty)){
	    float xdif = zeroIt->GetRawX()-zeroIt->GetRawZ()*tx ;
	    cout << zeroIt->GetRawX() << " " << zeroIt->GetRawZ()*tx << endl ;
	    if(fabs(zeroIt->GetRawX()-zeroIt->GetRawZ()*tx)
	       < zeroIt->GetGede()->GetWirePitch()*2) {
	      clus.AddZero(zeroIt) ;
	    };
	  };
      if(clus.Valid()) fClusters.push_back(clus) ;
    };
  OTrDetector::Search3HitClusters( const int suId, const int secId )
  {
  fHitClusters.erase();
  int nmax  = 100;
  int iclus = 0;
  for ( vector<OTrHit>::iterator hit1 = fHits.begin();
	hit1 != fHits.end(); hit1++ ) {
    OTrSection* gede1 = hit1->GetWire()->GetSubSection()->GetSection();
    if ( gede1->layerId() != 1 ) { continue; }
    //    if ( gede1->nhits > nmax ) { continue; }       // testing

    for ( vector<OTrHit>::iterator hit2 = fHits.begin();
	  hit2 != fHits.end(); hit2++ ) {
      OTrSection* gede2 = hit2->GetWire()->GetSubSection()->GetSection();
      if ( gede2->layerId() != 3 ) { continue; }
      if ( gede2->GetSectionId() != gede1->GetSectionId() ) { continue; }
      //if ( gede2->nhits > nmax ) { continue; }       // testing

      if ( hit1->GetWire()->Compatible( hit2->GetWire() ) == 1 ) {
	for ( vector<OTrHit>::iterator hit3 = fHits.begin();
	      hit3 != fHits.end(); hit3++ ) {
          OTrSection* gede3 = hit3->GetWire()->GetSubSection()->GetSection(); 
          if ( gede3->layerId() != 2 ) { continue; }
          if ( gede3->GetSectionId() != gede1->GetSectionId() ) { continue; }
	  //  if ( gede3->nhits > nmax ) { continue; }       // testing

          iclus++;
          //OTrHitCluster* cluster = new OTrHitCluster( hit1, hit2, hit3 );
          //if ( cluster->Compatible( hit3 ) ) {
          //cout << "section = " << gede3->GetSectionId() << endl;
          //fClusterDiff->Fill( cluster->Xdiff() );
          //fClusterDiff2->Fill( 
	  //       hit3->GetWire()->getPoint().x() + cluster->Xdiff(),
	  //       float( hit3->GetWire()->getPoint().x() ) );
          //if ( fabs( cluster->Xdiff() ) < gede3->pitch() ) {
          //  cluster_list.push_back( cluster );
          //} else {
          //  delete cluster;
          }
        }
      }
    }
  }
  return 0;
} 
*/

//*
// int
// OTrSuperLayerEvent::SearchClusters()
// {
//   vector<OTrHit*> tPlusHits ;
//   vector<OTrHit*> tMinusHits ;
//   vector<OTrHit*> tZeroHits ;
//   for(vector<OTrHit>::iterator hitp=fHits.begin();
//       hitp != fHits.end(); hitp++)
//     switch (hitp->GetWire()->GetSubSection()->GetSection()->
// 	    GetGede()->GetEStereoAngle())
//       {
//       case OTrGede::kPlusFiveDegrees :
// 	tPlusHits.push_back(hitp) ; break ;
//       case OTrGede::kMinusFiveDegrees :
// 	tMinusHits.push_back(hitp) ; break ;
//       default :
// 	tZeroHits.push_back(hitp) ; break ;
//       };
//   unsigned int n=10;
//   if(tPlusHits.size()<n && tMinusHits.size()<n && tZeroHits.size()<n) {

//   for(vector<OTrHit*>::iterator plusHitIt = tPlusHits.begin();
//       plusHitIt!=tPlusHits.end(); plusHitIt++)
//     for(vector<OTrHit*>::iterator minusHitIt = tMinusHits.begin();
// 	minusHitIt!=tMinusHits.end(); minusHitIt++)
//       for(vector<OTrHit*>::iterator zeroHitIt = tZeroHits.begin();
// 	  zeroHitIt !=tZeroHits.end() ; zeroHitIt++) {
// 	OTrHitCluster newCluster(*plusHitIt) ;
// 	newCluster.Add(*minusHitIt) ;
// 	newCluster.Add(*zeroHitIt) ;
// 	newCluster.GetXDiff() ;
//       };
//   };
// };

// */

// int
// OTrEvent::SearchTrackCandidates() {
// #ifdef USE_ROOT
//   static TH2F* rawTrackXYH2 = new TH2F("rawTrackXYH2","",100,-100,100,100,-100,100) ;
// #endif
//   if(fSuperLayerEvents.size()<2) { return 0 ; } ;
//   SuperLayerEventContainer::iterator sleIt = fSuperLayerEvents.begin() ;
//   const OTrSuperLayerEvent* sle13 = &((*sleIt).second) ; sleIt++;
//   const OTrSuperLayerEvent* sle14 = &((*sleIt).second) ;

//   // ******** Delete previous *********************
//   fTrackCandidates.erase(fTrackCandidates.begin(),fTrackCandidates.end()) ;

//   // ********* Make all possible candidates *******
//   for(OTrSuperLayerEvent::ClusterContainer::const_iterator cl13=sle13->GetClusters().begin() ;
//       cl13!=sle13->GetClusters().end() ; cl13++)
//     for(OTrSuperLayerEvent::ClusterContainer::const_iterator cl14=sle14->GetClusters().begin() ;
// 	cl14!=sle14->GetClusters().end() ; cl14++) {
//       OTrTrackCandidate* track = new OTrTrackCandidate(&(*cl13),&(*cl14)) ;
// #ifdef USE_ROOT
//       rawTrackXYH2->Fill(track->GetX(),float(track->GetY())) ;
// #endif
//       if(track->Valid()) fTrackCandidates.push_back(track) ;
//       else delete track ;
//     };
//   cout << "Created " << fTrackCandidates.size() << " track candidates " << endl ;
//   // ******** Sort in chi2 ************************
//   fTrackCandidates.sort(CompareTargetCompatibility) ;
//   for(TrackContainer::iterator trackIt=fTrackCandidates.begin() ;
//     trackIt!=fTrackCandidates.end() ; trackIt++) 
//     cout << (*trackIt)->GetX() << " " << (*trackIt)->GetY() << " "
// 	 << (*trackIt)->GetMinusLogTargetCompatibility() << endl ;
//   // Clean-up 
//   for(TrackContainer::iterator trackIt=fTrackCandidates.begin() ;
//       trackIt!=fTrackCandidates.end() ; trackIt++) {
//     TrackContainer::iterator trackIt2 = fTrackCandidates.end() ;
//     trackIt2-- ;
//     for(; trackIt2 != trackIt ; trackIt2--) {
//       (*trackIt2)->CleanUp(*(*trackIt)) ;
//       if(!((*trackIt2)->Valid())) fTrackCandidates.erase( trackIt2 ) ;
//     };
//   };
//   cout << "Left " << fTrackCandidates.size() << " track candidates after clean up." << endl ;
//   return fTrackCandidates.size() ;
// };
