#include "OTrTdcBoard.hh"
#include "OTrFedBuffer.hh"

#include <iostream.h>

#ifdef USE_ROOT
 #include "TProfile.h"
 #include "TH1.h"
#endif

const int OTrTdcBoard::fgDefaultNAsdBoards = 8 ;

OTrTdcBoard::OTrTdcBoard() : OTrDaqComponent(), fAsds(), fNProcessed(0) {
  cout << "OTrTdcBoard::OTrTdcBoard() : I do not think you want to use this." << endl ;
};

OTrTdcBoard::OTrTdcBoard(ComponentId_t id) 
  : OTrDaqComponent(id), fAsds(fgDefaultNAsdBoards), fNProcessed(0)
{
//   cout << "OTrTdcBoard::OTrTdcBoard(....) "
//        << fAsds.capacity()
//        << endl ;
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++)
    fAsds[asdId] = OTrAsdBoard(SetAsdId(GetId(),asdId)) ;
};

OTrTdcBoard::~OTrTdcBoard() {
  //cout << "OTrTdcBoard::~OTrTdcBoard()" << endl ;
};

void
OTrTdcBoard::ProcessFedBuffer(const OTrFedBuffer& fb, OTrEvent *event)
{
  fNProcessed++;
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++)
    fAsds[asdId].
      ProcessAsdBuffer(fb.GetData() + OTrAsdBoard::GetNChannels()*asdId,
		       event) ;
};

void
OTrTdcBoard::Reset()
{
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++)
    fAsds[asdId].Reset() ;
};

#ifdef USE_ROOT
//ClassImp(OTrTdcBoard) ;

TString OTrTdcBoard::GetName() const {
  char name[8];
  sprintf(name,"Tdc_%d",GetTdcId()) ;
  return TString(name) ;
}

TH1*
OTrTdcBoard::GetTimeH1() const
{
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  AsdConstIterator asdIt(*this) ;
  TH1* timeH1 = (*asdIt).GetTimeH1() ;
  timeH1->SetName(timeH1Name) ; 
  for(asdIt.next(); asdIt.valid(); asdIt.next()) {
    TH1* histo = (*asdIt).GetTimeH1() ;
    timeH1->Add(histo,1) ;
    delete histo ;
  }
  return timeH1;
}
#endif

/*
const Text_t* OTrTdcBoard::GetName() const {
  return "ha" ;
};

Int_t OTrTdcBoard::Fill(const OTrTdcBoardDataBuffer* data) {
  //cout << GetName() << "::Fill(..)" << endl; 
  for(int asdId=0; asdId<fgDefaultNAsd16s; asdId++)
    GetAsd16(asdId)->Fill(data + OTHAsd16::GetNChannels()*asdId) ;
};

TH1* OTrTdcBoard::GetTimeProfile() 
{
  TString timeProfileName(GetName());
  timeProfileName.Append("::TimeProfile") ;
  Int_t nbinsx = OTHAsd16::GetNChannels() * (GetLast()+1) ;
  TProfile 
    timeProfile((char*)timeProfileName.Data(),(char*)timeProfileName.Data(),
		nbinsx,0,nbinsx) ;
  //TProfile timeProfile("","",nbinsx,0,nbinsx) ;
  for(Int_t asdId=0; asdId<GetLast()+1 ; asdId++) {
    OTHAsd16* thisAsd = GetAsd(asdId) ;
    for(Int_t channelId=0; channelId<OTHAsd16::GetNChannels() ; channelId++) {
      Int_t xbin = asdId*OTHAsd16::GetNChannels() + channelId +1 ;
      Int_t nHits = thisAsd->GetNHits()[channelId] ;
      if(nHits>0) {
	timeProfile.SetBinEntries(xbin,nHits);
	timeProfile.SetBinContent(xbin,
				  nHits*thisAsd->GetAvTime()[channelId]);
	timeProfile.SetBinError(xbin,
				sqrt(nHits*thisAsd->GetAvTime2()[channelId]));
      }
      //cout << xbin << " " << thisAsd->GetAvTime()[channelId] << " "
      //	   << thisAsd->GetAvTime2()[channelId] << endl;
    };
  };
  return &timeProfile;
};

TH1* OTrTdcBoard::GetOccupancyH1() 
{
  cout << "OTrTdcBoard::GetOccupancyH1()" << endl ;
  TString occupancyH1Name(GetName());
  occupancyH1Name.Append("::OccupancyH1") ;
  Int_t nbinsx = OTHAsd16::GetNChannels() * (GetLast()+1) ;
  TH1S occupancyH1(occupancyH1Name,occupancyH1Name,nbinsx,0,nbinsx) ;
  for(Int_t asdId=0; asdId<GetLast()+1 ; asdId++) {
    OTHAsd16* thisAsd = GetAsd(asdId) ;
    for(Int_t channelId=0; channelId<OTHAsd16::GetNChannels() ; channelId++) {
      Int_t xbin = asdId*OTHAsd::GetNChannels() + channelId +1 ;
      occupancyH1.SetBinContent(xbin,thisAsd->GetNHits()[channelId]);
    };
  };
  return &occupancyH1;
};

TH1* OTrTdcBoard::GetTimeH1() 
{
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  TH1D timeH1(timeH1Name,timeH1Name,OTHAsd16::GetNTimeBins(),
	      OTHAsd16::GetMinimumTime(),OTHAsd16::GetMaximumTime()) ;
  for(Int_t asdId=0; asdId<GetLast()+1 ; asdId++)
    {
      TH1D asdtimeH1(GetAsd16(asdId)->GetTimeH1()) ;
      timeH1.Add(&asdtimeH1,1) ;
    }
  return &timeH1;
};

*/
