#include <iostream.h>
#include <iomanip.h>
 
#include "controller.hh"
#include "OTrGede.hh"
#include "OTrDetector.hh"
#include "OTrDaq.hh"
#include "OTrEventBuffer.hh"
#include "OTrEvent.hh"
#include "OTrView.hh"
#include "OTrSplineRTRelation.hh"
#include "OTrRTResult.hh"
#include "OTrHit.hh"
#include "OTrTZeroCalib.hh"
#include "OTrTrack.hh"
#include "OTrTrackCandidate.hh"
#include "OTrSuperCluster.hh"
#include "OTrRTCalib.hh"
#include "OTrRunSummary.hh"
#include "OTrDirectory.hh"
#include "OTrAlignment.hh"
#include "OTrReconstructedHit.hh"
#include "OTrChannelHistogrammer.hh"
#include "OTrSector.hh"

#include <string>
#include <list>

#ifdef USE_ROOT
 #include <TProfile.h>
 #include <TH2.h>
 #include <TSystem.h>
 #include <TPaveLabel.h>
 #include <TCanvas.h>
 #include <TView.h>
 #include <TNode.h>
 #include <TH1.h>
 #include <TBRIK.h>
 #include <TControlBar.h>
 #include <TGraphErrors.h>
 #include <TFile.h>
 #include <TTree.h>
#endif

#ifdef USE_ARTE
 #include "arte/ArteFlag.h"
 #include <rang/Track.hh>
 #include <rang/Node.hh>
 #include "arte/arte.hh"
#endif

#ifdef USE_ROOT
TNode* gWaxNode = 0;
TPad*  gWaxPad = 0;
//ClassImp(controller) ;
#endif

//__________________________________________________________________
// controller
//
// Provides the main interface to the wax package.
//
// Container for:
// * an OTrAlignment object
// * an OTrDaq object
// * an OTrDetector object
// * an OTrTZeroCalib object
// * an OTrRTCalib
// Functionality:
// * next()
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef USE_ROOT
ClassImp(controller) ;
#endif

extern "C" int next_arte_();

enum EDataType
{
  kArteMC,
  kArteDOTR,
  kDAQFormat
};

controller* gOTr=0;

controller::controller() :
#ifdef USE_ROOT
  fOccCanvas(0), fCombinedCanvas(0),
  fBRIK(0), fWaxView(0), 
  fWaxNode(0), fDisplay(false), 
#endif
  fDataFile(0), fNprocessed(0),
  fDaq(0), fDetector(0), fEventBuffer(0), fEvent(0),
  fSearchClusters(false), fCurrentSuperLayer(0),
  fCombinedLayer(0), fMC(false), fTZeroCalib(0)
#ifdef USE_ROOT
  , fEventBranch(0)
#endif
{
  if(gOTr) {
    cerr << "controller::controller(): Already one controller object created !"
	 << endl ;
    return ;
  } else {
    gOTr = this ;
  };
  
  // The directory where all calibration and geometry is ...
  //string dir = "/afs/desy.de/user/h/hbotr/caldata";
  string dir = "caldata" ;

  fEventBuffer = new OTrEventBuffer() ; 
  fDaq = new OTrDaq() ;
  OTrDaqComponent::SetDaq(fDaq) ;
#ifdef USE_ARTE
  fDetector = new OTrDetector( fDaq );
#else 
  fDetector = 
    new OTrDetector(const_cast<char *>((dir + "/geometry/geometry.txt").c_str()),fDaq);
#endif

  // read in Sergey Solunin's quality mask
  //fDaq->ReadTable( const_cast<char *>((dir + "/masks/big_mask.dat").c_str() ) ;
  //fDaq->ReadTable(const_cast<char *>((dir + "/masks/mask.dat").c_str())) ;

  // read in the tzero values
  //fDaq->ReadTable(const_cast<char *>((dir + "/tzero/tzero.dat").c_str()));
   
  // read in the RT relations
#ifdef USE_ARTE
  if(ArteTable<EVHD>::begin()->exp <= 0) { // MC run
    cout << "Initializing rtrelations for MC." << endl ;
    ifstream rt05mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr05mm.dat").c_str()));
    OTrChannel::SetRTRelation5mm(new OTrSplineRTRelation(rt05mmifs)) ;
    ifstream rt10mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr10mm.dat").c_str()));
    OTrChannel::SetRTRelation10mm(new OTrSplineRTRelation(rt10mmifs)) ;
    // These are used for digitization, so you only need them when reading mc data
    //ifstream mcrt05mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr05mm.dat").c_str()));
    //OTrChannel::SetMCRTRelation5mm(new OTrSplineRTRelation(mcrt05mmifs)) ;
    //ifstream mcrt10mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr10mm.dat").c_str()));
    //OTrChannel::SetMCRTRelation10mm(new OTrSplineRTRelation(mcrt10mmifs)) ;
    OTrChannel::SetMCRTRelation5mm(OTrChannel::GetRTRelation5mm()) ;
    OTrChannel::SetMCRTRelation10mm(OTrChannel::GetRTRelation10mm()) ;
  } else {
#endif USE_ARTE
    ifstream rt05mmifs(const_cast<char *>((dir + "/rtrelation/rtr05mm.dat").c_str()));
    OTrChannel::SetRTRelation5mm(new OTrSplineRTRelation(rt05mmifs)) ;
    ifstream rt10mmifs(const_cast<char *>((dir + "/rtrelation/rtr10mm.dat").c_str()));
    OTrChannel::SetRTRelation10mm(new OTrSplineRTRelation(rt10mmifs)) ;
    // Alignment
    //     cout << "Replacing MSL's." << endl ;
    //     ThreeVector vecMSL1(-1.019,0,0) ; //+-0.007 //837*-1.24E-3
    //     fDetector->FindSuperLayer(31)->Translate(vecMSL1) ;
    //     ThreeVector vecMSL2(-1.416,0,0) ; //+-0.015 //1209*-1.23E-3
    //     fDetector->FindSuperLayer(32)->Translate(vecMSL2) ;
#ifdef USE_ARTE
  }
#endif
  // Allocate calibration classes
  fTZeroCalib = new OTrTZeroCalib( fDaq, fDetector );
  fRTCalib    = 0 ; //new OTrRTCalib() ;
  fRunSummary = 0;//new OTrRunSummary() ;
  fAlignment  = new OTrAlignment() ;
#ifdef USE_ROOT
  gOTrDirectory = new TDirectory("OTrDirectory","OTrDirectory") ;
  gOTrDirectory->cd() ;
  //fSparcifiedOutputFile = new ofstream("asciotrrun.dat") ;
  fSparcifiedOutputRootFile = 0 ;//new TFile("otrrun.root","RECREATE") ;
  fTree = 0 ; //new TTree("OTrTree","OTrTree") ;
  fEventBranch = 0;//fTree->Branch("Events","OTrEvent",&fEvent,64000,0) ;
#endif
  //ThreeVector displacement(1,0,0) ;
  //fDetector->GetSuperLayer(2)->GetLayer(1)->GetSector(4)->Translate(displacement) ;
  // OTrSector* sector = fDetector->GetSuperLayer(2)->GetLayer(1)->GetSector(4) ;
  //   for(OTrSector::SectionContainer::iterator sectionIt=sector->GetSections().begin() ;
  //       sectionIt!=sector->GetSections().end() ; sectionIt++)
  //     for(OTrSection::WireContainer::iterator wireIt=(*sectionIt)->GetWires().begin() ;
  // 	wireIt!=(*sectionIt)->GetWires().end() ; wireIt++) {
  //       OTrChannel* chanIt = wireIt->GetChannel() ;
  //       if(chanIt) chanIt->SetTZero(OTrChannel::TZero_t(20,0)) ;
  //     }

cout << "controller object created." << endl;
} 

controller::~controller()
{
  if(fDaq) delete fDaq ;
  if(fDetector) delete fDetector ;
#ifdef USE_ROOT
  if(fSparcifiedOutputRootFile) {
    fSparcifiedOutputRootFile->Close() ;
    delete fSparcifiedOutputRootFile;
  };
#endif
  gOTr=0;
};

int controller::open(char* filename, int DaqFileType )
{
  if ( fDataFile != 0 ) {
    cout << "closing the data file " << endl;
    fclose(fDataFile);
  }
  fDataFile = fopen(filename,"rb");
  if ( fDataFile ) {
    //if (fEventBuffer) delete fEventBuffer ;
    //fEventBuffer = new OTrEventBuffer(fDataFile) ;
    fEventBuffer->SetFile( fDataFile );
    strcpy(fFileName,filename);
    cout << "successfully opened input file " << fFileName << endl;
    fDaqFileType = DaqFileType;
  } else {
    cout << "No such file \"" << filename << "\"." << endl;
    return 1;
  }
  return 0;
}    

/*
int controller::next( const int n, const int prflag )
{
  cout << "controller::next(..)" << endl ;
  if(GetDataType()==kDAQFormat)
    if ( fDataFile == 0 || fEventBuffer==0) {
      cout << "no datafile open." << endl;
      return 1;
    };
  
  int nClusters=0;
  int nTracks=0;
  for ( int ev = 0; ev < n; ev++ ) {
    if(fEvent) { 
      delete fEvent ;
      fEvent = 0;
    } ;
    switch GetDataType()
      {
      case kDAQFormat:
      // real data run outside ARTE
      if ( feof(fDataFile) ) {
      cout << "at end of datafile" << endl;
      } else {
      if(fEventBuffer->NextEvent( fDaqFileType )) {
      cout << "Received error from eventbuffer" << endl ;
      //fEvent=0 ;
      } else {
      
      };
      
      break ;
      }
      
      fEvent = fDaq->ProcessEvent(fEventBuffer) ;
      fEvent->SortHits();   // Note: not necessary when reading from HITB
      break ;
      case kArteMC:
      {
      if (next_arte_()) break;
      cout << "Now running NEXTMC..." << endl;
      fDaq->IncreaseNProcessedEvents();
      fEvent = fDetector->nextmc() ;
      } else {
      fDetector->ResetGEDEs(); // should not be called when reading from HITB
      }
      }
      break ;
      case kArteDOTR:
      break ;
      };
      };
      
*/


int controller::next( const int n, const int prflag )
{
  cout << "controller::next(..)" << endl ;
#ifdef USE_ARTE
  ArtePointer<EVHD> evhd(1);
  if ( (! fMC) && (evhd->exp > 0) ) {
#else
  if ( ! fMC) {
#endif
    if ( fDataFile == 0 || fEventBuffer==0) {
      cout << "no datafile open." << endl;
      return 1;
    };
  };
  
  int nClusters=0;
  int nTrackCandidates=0;
  for ( int ev = 0; ev < n; ev++ ) {
    if(fEvent) {
      delete fEvent ;
      fEvent = 0;
    }
    //fDetector->EraseHits() ;
#ifdef USE_ARTE
    //    if (fMC) {  // MC run
    if (evhd->exp <= 0) {  // MC run
      if (fMC) {
        if (next_arte_()) break;
      };
      cout << "Now running NEXTMC..." << endl;
      fDaq->IncreaseNProcessedEvents();
      fEvent = fDetector->nextmc() ;
    } else {
      fDetector->ResetGEDEs(); // should not be called when reading from HITB
      //      cout << " bx = " << ArtePointer<DIHD>(1)->bx << endl; // bunch number
    }
#else
                // real data run outside ARTE
    if ( feof(fDataFile) ) {
      cout << "at end of datafile" << endl;
      break;
    }
    
    if(fEventBuffer->NextEvent( fDaqFileType )) {
      cout << "Received error from eventbuffer" << endl ;
      //fEvent=0 ;
      break ;
    }
    
    fEvent = fDaq->ProcessEvent(fEventBuffer) ;
#endif
    fEvent->SortHits(); // Note: not necessary when reading from HITB
                        // That's a pity, this is not true ... ??!!
#ifdef USE_ROOT   
    // I would prefer this call in fDaq->ProcessEvent(...) and fDetector->nextmc(...)
    fEvent->PerformHistogramming() ;
    PerformRawogramming() ;
#endif //USE_ROOT
#ifdef USE_ARTE
    fEvent->AppendHITB();
#endif
    if(fSearchClusters) {
      nClusters += fEvent->SearchClusters() ;
      //nTrackCandidates += fEvent->SearchTrackCandidates() ;
#ifdef USE_ARTE
      fEvent->AppendRSEG();
#endif
      //fEvent->DumpClusters() ;
#ifdef USE_ROOT      
      PerformRecogramming() ;
      gSystem->ProcessEvents() ;
#endif //USE_ROOT
    };
    //  fEvent->Write(*fSparcifiedOutputFile) ;
#ifdef USE_ROOT
    if(fTree) fTree->Fill() ;
#endif
    if (!(fNprocessed%100)) cout << fNprocessed/100 << "|" << flush ;
    fNprocessed++;
  }

#ifdef USE_ROOT 
  if(fEvent) {
    if ( fDisplay ) Display();
    if(fCurrentSuperLayer) plot( fCurrentSuperLayer ); 
  };
#endif //USE_ROOT

  cout << "number of events processed:  " << fNprocessed << endl
       << "number of clusters found:    " << nClusters << endl 
       << "number of track candidates: " << nTrackCandidates << endl ;
  
  return 0;
}

void controller::WriteAsciMask()
{
  fDaq->WriteAsciMask("maskfiletje.dat");
}

#ifdef USE_ARTE
int 
controller::AppendHITB() {
  int rc=0;
  if ( fEvent ) fEvent->AppendHITB();
  return rc;
}

int 
controller::AppendRSEG() {
  int rc=0;
  if ( fEvent ) fEvent->AppendRSEG();
  return rc;
};

int 
controller::EventRowSummary( ostream& os ) const
{
  // get first 4-byte word of table
  ArtePointer<EVHD> evhd( 1 );
  ArtePointer<DIHD> dihd( 1 );
  
  ArtePointer<DDAQ> ddaq( 1 );
  ArtePointer<DSVD> dsvd( 1 );
  //ArtePointer<DITR> ditr( 1 );
  ArtePointer<DPTC> dptc( 1 );
  ArtePointer<DOTR> dotr( 1 );
  ArtePointer<DRIC> dric( 1 );
  //ArtePointer<DTRD> dtrd( 1 );
  ArtePointer<DCAL> dcal( 1 );
  ArtePointer<DMUT> dmut( 1 );
  //ArtePointer<DMUX> dmux( 1 );
  ArtePointer<DMUP> dmup( 1 );
  //ArtePointer<DFLT> dflt( 1 );
  //ArtePointer<DSLT> dslt( 1 );
  //ArtePointer<DTLT> dtlt( 1 );
  //ArtePointer<DTAR> dtar( 1 );
  //ArtePointer<DFCS> dfcs( 1 );
  
  // rows
  os << "event="      << arteflag_.numevt << " "
     << "run="        << arteflag_.numrun << " "
     << "experiment=" << arteflag_.numexp << endl;
  os << "nrow(): "
     << "EVHD=" << evhd.nrow() << " "
     << "DIHD=" << dihd.nrow() << " "
     << "DDAQ=" << ddaq.nrow() << " "
     << "DSVD=" << dsvd.nrow() << " "
    //<< "DITR=" << ditr.nrow() << " "
     << "DPTC=" << dptc.nrow() << " "
     << "DOTR=" << dotr.nrow() << " "
     << "DRIC=" << dric.nrow() << " "
    //<< "DTRD=" << dtrd.nrow() << " "
     << "DCAL=" << dcal.nrow() << " "
     << "DMUT=" << dmut.nrow() << " "
    //<< "DMUX=" << dmux.nrow() << " "
     << "DMUP=" << dmup.nrow() << " "
    //<< "DFLT=" << dflt.nrow() << " "
    //<< "DSLT=" << dslt.nrow() << " "
    //<< "DTLT=" << dtlt.nrow() << " "
    //<< "DTAR=" << dtar.nrow() << " "
    //<< "DFCS=" << dfcs.nrow() 
     << endl; 
  return 0;
}

int
controller::nextDOTR()
{
  //cout << "controller::nextDOTR()" << endl ;
  int nClusters = 0;
  int nTrackCandidates = 0;
  fEventBuffer->DOTRUpdateFedBuffers();
  if (fEvent) { 
    delete fEvent ;
    fEvent = 0;
  }
  //fDetector->EraseHits() ;
  fDetector->ResetGEDEs() ;
  fEvent = fDaq->ProcessEvent(fEventBuffer) ;
  fEvent->SortHits();
  fEvent->AppendHITB() ;
  fNprocessed++; 
  if(fSearchClusters) {
    //cout << "Start reconstruction" << endl ;
    nClusters += fEvent->SearchClusters() ;
    nTrackCandidates += fEvent->SearchTrackCandidates() ;
    //nTracks += fEvent->FitTracks() ;
    fEvent->AppendRSEG() ;
  } ;
#ifdef USE_ROOT
  if ( fDisplay) Display();
  PerformRawogramming() ;
  PerformRecogramming() ;
  gSystem->ProcessEvents() ;
#endif //USE_ROOT

  // a very useful printing message to check the performance:-)
  /*
  cout << "OTr controller: " << fNprocessed << " events processed: Found " 
       << fEvent->GetNHits() << " hits, "
       << nClusters << " clusters, " 
       << nTrackCandidates << " tracks candidates." ;
       */
  return 0;
}
#endif //USE_ARTE


#ifdef USE_ROOT
void
controller::CreateNode()
{  
  float hallsize = 750;
  fBRIK = new TBRIK("HALL","HALL","void",hallsize,hallsize,hallsize);
  fWaxNode = new TNode("waxnode","waxnode",fBRIK);
  fWaxNode->SetPosition( 0, 0, 0 );
  fWaxNode->cd(); 
  //fWaxNode->Draw("same"); 
  gWaxNode = fWaxNode ;
  fWaxNode->cd();

  // now create TNodes for all sections
  for ( vector<OTrSuperLayer>::iterator slIt =  fDetector->GetSuperLayers().begin();
	slIt != fDetector->GetSuperLayers().end(); slIt++ )
    for(OTrSuperLayer::LayerIterator lIt(*slIt) ; lIt.valid(); lIt.next())
      for(OTrLayer::SectionIterator sIt(*lIt) ; sIt.valid() ; sIt.next())
	(*sIt).CreateNode();
}

void controller::PlotAllTdcSpectra() const
{
  cout << "Processed " << fNprocessed << " events. " << endl ;

  TCanvas* mycanvas = new TCanvas("mycanvas","mycanvas") ;
  mycanvas->Divide(4,2) ;
  mycanvas->Show() ;
  vector<TH1*> histos ;
  for(OTrDaq::TdcIterator tdcIt(*fDaq); tdcIt.valid(); tdcIt.next()) {
    int j=0;
    for(OTrTdcBoard::AsdIterator asdIt(*tdcIt); asdIt.valid(); asdIt.next()) {
      mycanvas->cd(++j) ;
      histos.push_back((*asdIt).GetTimeH1()) ;
      histos.back()->Draw() ;
    };    
    mycanvas->Update() ;
    TString name = (*tdcIt).GetName() ;
    name.Append(".ps") ;
    mycanvas->Print(name.Data()) ;
    getchar() ;
    for(vector<TH1*>::iterator histoIt=histos.begin() ; histoIt!=histos.end(); histoIt++)
      delete *histoIt ;
    histos.erase(histos.begin(), histos.end()) ;
  };
  delete mycanvas ;
  //cout << "You have been watching " << n << " tdcspectra." << endl ;
};

void 
controller::PlotChannelTdcSpectra(int tdcId) const
{
  cout << "Processed " << fNprocessed << " events. " << endl ;
  TCanvas* mycanvas = new TCanvas("mycanvas","") ;
  mycanvas->Divide(4,4) ;
  OTrTdcBoard* tdcIt = fDaq->FindTdcBoard(tdcId) ;
  if ( !tdcIt ) {
    cout << "tdc identifier " << tdcId << " not found." << endl;
    return;
  }
  for(OTrTdcBoard::AsdIterator asdIt(*tdcIt); asdIt.valid(); asdIt.next()) {
    int j=0;  
    for(OTrAsdBoard::ChannelConstIterator chanIt(*asdIt) ; chanIt.valid() ;
	chanIt.next() )
      {
	mycanvas->cd(++j) ;
	TH1* histo = new TH1D((*chanIt).GetTimeH1()) ;
	cout << "wire = " << (*chanIt).GetWire()->GetId() << endl;
	if ( (*chanIt).GetWire()->GetSuperLayerId() == 10 ) {
	  cout << "tzero = " << (*chanIt).GetTZero() << endl;
	  cout << "average time = " << (*chanIt).GetAverageTime() << endl;
	  cout << "mask = " << (*chanIt).GetMask() << endl;
	  histo->Draw() ;
	}
	mycanvas->Update();
	getchar();
      };
    mycanvas->Update() ;
    TString name = (*asdIt).GetName() ;
    name.Append(".ps") ;
    mycanvas->Print(name.Data()) ;
    getchar() ;
  };
  delete mycanvas ;
};

void controller::DrawRTRelation() const
{
  static TCanvas* rtcanvas = 0 ;
  if(!rtcanvas) {
    rtcanvas=new TCanvas("drawrtcanvas","") ;
    rtcanvas->Divide(2,2) ;
  };

  rtcanvas->cd(1) ; gPad->DrawFrame(0,0,0.3,256) ;
  TGraph* tgraph05mm = OTrChannel::GetRTRelation5mm()->GetTGraph(50) ; 
  tgraph05mm->Draw("lp") ;
  rtcanvas->cd(2) ; gPad->DrawFrame(0,0,0.6,256) ;
  TGraph* tgraph10mm = OTrChannel::GetRTRelation10mm()->GetTGraph(50) ; 
  tgraph10mm->Draw("lp") ;
  rtcanvas->cd(3) ; gPad->DrawFrame(0,0,0.3,0.2) ;
  TGraph* sgraph05mm = OTrChannel::GetRTRelation5mm()->GetSigmaGraph(50) ; 
  sgraph05mm->Draw("lp") ;
  rtcanvas->cd(4) ; gPad->DrawFrame(0,0,0.6,0.2) ;
  TGraph* sgraph10mm = OTrChannel::GetRTRelation10mm()->GetSigmaGraph(50) ;
  sgraph10mm->Draw("lp") ;

  rtcanvas->Update() ;
}

#include "TGraphErrors.h"
void controller::CreateRTRelation(bool drawFlag) const
{
  // ** FILL REFERENCE SPECTRA ** //
  TH1F* ref5mmH1 = new TH1F("ref5mm","",OTrChannel::GetNTimeBins(),
			    OTrChannel::GetMinimumTimeAxis(),
			    OTrChannel::GetMaximumTimeAxis()) ; 
  ref5mmH1->SetDirectory(0) ;
  TH1F* ref10mmH1 = new TH1F("ref10mm","",OTrChannel::GetNTimeBins(),
			     OTrChannel::GetMinimumTimeAxis(),
			     OTrChannel::GetMaximumTimeAxis()) ;    
  ref10mmH1->SetDirectory(0) ;

  TH1F* refH1 ;
  for ( vector<OTrSuperLayer>::iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ ) { 
    for ( OTrSuperLayer::LayerIterator layIt(*sulIt) ; layIt.valid(); 
	  layIt.next() ) {
      for ( OTrLayer::SectionIterator secIt(*layIt) ; secIt.valid();
	    secIt.next() ) { 
	if ( (*secIt).GetEPitch() == OTrGede::kFiveMM )
	  refH1 = ref5mmH1 ;
	else
	  refH1 = ref10mmH1 ;
	for ( vector<OTrWire>::const_iterator wireIt = 
		(*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  int trbin = -(*chanIt).GetTZeroBin() ;
	  if(trbin<0) trbin+=OTrChannel::GetNTimeBins() ;
	  for(int bin=0; bin<OTrChannel::GetNTimeBins(); bin++,trbin++) {
	    refH1->AddBinContent(bin+1,
	      (*chanIt).GetTimeHistogram()[trbin%OTrChannel::GetNTimeBins()]) ;
	  }
	}
      }
    }
  }

  /** Create RTRelations and set pointers **/
  OTrSplineRTRelation* rt05mm = new OTrSplineRTRelation(10,6,ref5mmH1,0.00125,0.29,250,130,20) ;
  OTrSplineRTRelation* rt10mm = new OTrSplineRTRelation(10,6,ref10mmH1,0.00125,0.58,250,30,20) ;
  OTrChannel::SetRTRelation5mm(rt05mm) ;
  OTrChannel::SetRTRelation10mm(rt10mm) ;
  /** Draw **/
  if(drawFlag) {
    TCanvas rtcanvas("rtcanvas","") ;
    rtcanvas.Divide(2,2) ;
    rtcanvas.cd(1) ; ref5mmH1->Draw() ;
    rtcanvas.cd(3) ; ref10mmH1->Draw() ;
    rtcanvas.cd(2) ; gPad->DrawFrame(0,0,0.3,256) ;
    TGraph* tgraph05mm = rt05mm->GetTGraph(50) ; tgraph05mm->Draw("lp") ;
    rtcanvas.cd(4) ; gPad->DrawFrame(0,0,0.6,256) ;
    TGraph* tgraph10mm = rt10mm->GetTGraph(50) ; tgraph10mm->Draw("lp") ;
    rtcanvas.Update() ;
    getchar() ;
    delete tgraph05mm;
    delete tgraph10mm;
  };
  delete ref5mmH1;
  delete ref10mmH1;
};

void
controller::Reset()
{
  fDaq->Reset(true);
}

void controller::Display() {
  if ( fWaxView == 0 ) {
    CreateNode();
    fWaxView = new OTrView();
  }
  fWaxView->SetEvent( fEvent );
  fWaxView->SetEventNumber( fDaq->GetNProcessedEvents() );
  //TPad* pad = fWaxView->GetDisplayPad();
  //pad->cd();
  fWaxNode->cd();
  fEvent->DrawHits() ;
  fEvent->DrawClusters() ;
  fWaxNode->Modified();
  fWaxView->Draw();
  //fEvent->DrawTrackCandidates( pad );
  //fWaxNode->Draw("same");
  fWaxView->Refresh();  
}

int 
controller::plot( const int sl, const int choice, const bool use_mask )
{
  if ( sl == -1 ) {
    cout << "syntax: plot( int superlayer_id, int choice, bool use_mask ) "
	 << endl;
    cout << "choice = 1 : occupancy" << endl; 
    cout << "choice = 2 : TDC count for each hit channel" << endl;
    cout << "choice = 3 : average TDC counts" << endl;
    cout << "choice = 4 : TDC spectra (summed per section)" << endl;
    cout << "choice = 5 : RMS of TDC counts" << endl;
    cout << "choice = 6 : t0 values" << endl;
    cout << "choice = 7 : occupancy differences with next channel" << endl;
    return 0;
  }
  
  if (!fOccCanvas) {
    fOccCanvas = new TCanvas("occcanvas","OTR superlayer",200,40,700,500);
    fOccCanvas->SetFillColor(50);
  }

  fOccCanvas->cd();

  // print the filename in the canvas:
  TPaveLabel *label = new TPaveLabel(0.05, 0.96, 0.6, 0.99, fFileName );
  label->SetBit(kCanDelete);
  label->SetFillColor(42);
  label->Draw();

  fOccCanvas->cd();
  OTrSuperLayer* sul = fDetector->GetSuperLayer(sl) ;
  if(sul) sul->Plot(fOccCanvas, choice, fEvent, use_mask );
  else cout << "Sorry, no superlayer " << sl << "." << endl ;
  fOccCanvas->Modified();
  fOccCanvas->Update();
  //fOccCanvas->Paint();
  return 0;
}

//   /*
//   if ( choice == 4 && !OTrChannel::fgTimeHistogramming ) {
//     cout << "This option needs OTrChannel::fgTimeHistogramming "
// 	 << "set to true." << endl;
//     return 1;
//   }
//   */
//   static TPad* occpad[14] ; 

//   //   float ymax = 0.95;
//   //   float ymin = 0.00;
//   //   float deltay = (ymax - ymin)/4;

//   if (!fOccCanvas) {
//     fOccCanvas = new TCanvas("occcanvas","OTR superlayer",200,40,700,500);
//     fOccCanvas->SetFillColor(50);
//     float padXmin[] = {0.00,0.00,0.20,0.20,0.20,0.20,0.40,0.40,0.60,0.60,0.60,0.60,0.80,0.80} ;
//     float padXmax[] = {0.20,0.20,0.40,0.40,0.40,0.40,0.60,0.60,0.80,0.80,0.80,0.80,1,00,1,00} ;
//     float padYmin[] = {0.48,0.00,0.72,0.48,0.24,0.00,0.72,0.00,0.72,0.48,0.24,0.00,0.48,0.00} ;
//     float padYmax[] = {0.96,0.48,0.96,0.72,0.48,0.24,0.96,0.72,0.96,0.72,0.48,0.24,0.96,0.48} ;

    
//     for(int i=0; i<14; i++) {
//       char* name = "" ;
//       //sprintf(name,"pad%3d",i) ;
//       occpad[i] = new TPad(name,name,padXmin[i],padYmin[i],padXmax[i],padYmax[i],-1) ;
//       //delete name ;
//       //    float y;
//       //     // 10 mm:
//       //     y = ymin+2*deltay;
//       //     occpad[0] = new TPad("pad 0","pad 0",0.00,y,0.33,y+2*deltay,-1);
//       //     y = ymin;
//       //     occpad[1] = new TPad("pad 1","pad 1",0.00,y,0.33,y+2*deltay,-1);
//       //     occpad[0]->SetLeftMargin(0.09);
//       //     occpad[1]->SetLeftMargin(0.09);
//       //     occpad[0]->SetBottomMargin(0.06);
//       //     occpad[1]->SetBottomMargin(0.06);
//       //     // 5 mm:
//       //     y = ymin+3*deltay;
//       //     occpad[2] = new TPad("pad 2","pad 2",0.33,y,0.68,y+deltay,-1);
//       //     y = ymin+2*deltay;
//       //     occpad[3] = new TPad("pad 3","pad 3",0.33,y,0.68,y+deltay,-1);
//       //     y = ymin+deltay;
//       //     occpad[4] = new TPad("pad 4","pad 4",0.33,y,0.68,y+deltay,-1);
//       //     y = ymin;
//       //     occpad[5] = new TPad("pad 5","pad 5",0.33,y,0.68,y+deltay,-1);
//       //     occpad[2]->SetLeftMargin(0.05);
//       //     occpad[3]->SetLeftMargin(0.05);
//       //     occpad[4]->SetLeftMargin(0.05);
//       //     occpad[5]->SetLeftMargin(0.05);
//       //     y = ymin + 3*deltay;
//       //     occpad[6] = new TPad("pad 6","pad 6",0.68,y,1.00,y+deltay,-1);
//       //     y = ymin;
//       //     occpad[7] = new TPad("pad 7","pad 7",0.68,y,1.00,y+deltay,-1);
//       //     occpad[6]->SetLeftMargin(0.05);
//       //     occpad[7]->SetLeftMargin(0.05);
//       //     for ( int i = 0; i < 8; i++ ) {
//       occpad[i]->SetRightMargin(0.02);
//       occpad[i]->SetTopMargin(0.03);
//       occpad[i]->SetBorderMode(1);
//       occpad[i]->Draw();
//     }
//   }

int 
controller::plot_combined( const int sl ) 
{
  fCombinedCanvas = new TCanvas("combinedcanvas","5 and 10 mm combined",
				250,60,600,750);
  fCombinedCanvas->SetFillColor(36);
  combined_pad[0] = new TPad("comb0","comb0",0.02,0.02,0.98,0.93,-1);
  combined_pad[0]->Draw();

  if ( fCombinedLayer ) {
    fCombinedLayer->PlotCombinedOccupancy( fCombinedCanvas, 
					   combined_pad[0], "" );
  }
  return 0;
}

void
controller::SetCombinedLayer( const int slId, const int layId ) 
{ 
  OTrSuperLayer* sl = fDetector->FindSuperLayer(slId); 
  if ( sl ) {
    OTrLayer* l = sl->FindLayer( layId );
    if ( l ) {
      fCombinedLayer = l;
      fCombinedLayer->BookCombinedOccupancy();
    } else {
      cout << "layer " << layId << "not found." << endl;
      exit(1);
    }
  } else {
    cout << "superlayer " << slId << "not found." << endl;
    exit(1);
  }
}

void
controller::PerformRawogramming()
{
  //cout << "controller::PerformRawogramming()" << endl ;
  if(fRunSummary) fRunSummary->Add(fEvent) ;
  // occupancy plot of one layer in which 5mm cells are OR-ed
  if ( fCombinedLayer ) {
    int sulId = fCombinedLayer->GetSuperLayerId() ;
    const OTrSuperLayerEvent* slEvent = fEvent->FindSuperLayerEvent(sulId) ;
    if ( slEvent ) {
      fCombinedLayer->FillCombinedOccupancy( slEvent );
    } 
  }

//   static TH1F* wireDifMC2H1 = new TH1F("wireDifMC2H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMSL1H1 = new TH1F("wireDifMSL1H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMSL2H1 = new TH1F("wireDifMSL2H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMTC1H1 = new TH1F("wireDifMTC1H1","",128,-0.5,127.5) ;
//   TH1F* wireDifH1 ;
  
//   for(OTrEvent::SuperLayerEventContainer::const_iterator 
//  	sleIt=fEvent->GetSuperLayerEvents().begin();
//       sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++) {
//     int slId = (*sleIt).first ;
//     switch(slId) 
//       {
//       case 2:
// 	wireDifH1 = wireDifMC2H1 ;
// 	break ;
//       case 31:
// 	wireDifH1 = wireDifMSL1H1 ;
// 	break ;
//       case 32:
// 	wireDifH1 = wireDifMSL2H1 ;
// 	break ;
//       case 33:
// 	wireDifH1 = wireDifMTC1H1 ;
// 	break ;
//       default:
// 	wireDifH1 = wireDifMTC1H1 ;
// 	break ;
//       };
//     for(OTrSuperLayerEvent::HitContainer::const_iterator 
// 	  hitIt=(*sleIt).second.GetHits().begin() ;
// 	hitIt!=(*sleIt).second.GetHits().end() ; hitIt++)
//       for(OTrSuperLayerEvent::HitContainer::const_iterator 
// 	    hitIt2=hitIt ;
// 	  hitIt2!=(*sleIt).second.GetHits().end() ; hitIt2++)
// 	if((*hitIt)->GetWire()->GetGede()==
// 	   (*hitIt2)->GetWire()->GetGede())
// 	  wireDifH1->Fill(fabs((*hitIt)->GetWire()->GetPRWireNumber()
// 			       -(*hitIt2)->GetWire()->GetPRWireNumber())) ;
//   } ;
};

extern "C" float prob_(const float&, const int&) ;

void
controller::PerformRecogramming()
{
  //CombineClusters(); //old

  static TH1F* targetCompatH1 = new TH1F("targetCompat","",100,-100,700) ;
  static TH1F* clusterMult13H1 = new TH1F("clusterMultiplicity13","",15,-0.5,14.5) ;
  static TH1F* clusterMult14H1 = new TH1F("clusterMultiplicity14","",15,-0.5,14.5) ;
  static TH1F* trackMultH1   = new TH1F("trackMultiplicity","",15,-0.5,14.5) ;
  static TH2F* targetXYH2    = new TH2F("targetXY","y vs x at z=0",50,-100,100,50,-100,100);
  static TH2F* targetXYbH2   = new TH2F("targetXYb","y vs x zoomed",100,-30,30,100,-30,30) ;
  static TH1F* targetXH1     = new TH1F("targetX","x at z=0",100,-30,30) ;
  static TH1F* targetYH1     = new TH1F("targetY","y at z=0",20,-30,30) ;
  static TH1F* targetXbH1    = new TH1F("targetXb","x at z=0",100,-5,5) ;
  static TH1F* targetYbH1    = new TH1F("targetYb","y at z=0",100,-30,30) ;
  static TH2F* XvsXH2        = new TH2F("XvsX","XvsX",100,-80,30,100,-110,-30) ;
  static TH2F* YvsYH2        = new TH2F("YvsY","YvsY",60,-60,60,60,-60,60) ;
  static TH1F* targetXcH1    = new TH1F("targetXc","x at z=0",100,-30,30) ;
  static TH1F* targetYcH1    = new TH1F("targetYc","y at z=0",100,-30,30) ;
  static TH1F* targetXdH1    = new TH1F("targetXd","x at z=0",100,-30,30) ;
  static TH1F* targetYdH1    = new TH1F("targetYd","y at z=0",100,-30,30) ;
  static TH1F* chi2H1        = new TH1F("chi2","chi2/#dof",100,0,10) ;
  static TH1F* chi2probH1    = new TH1F("chi2probH1","chi2prob",100,0,1) ;
  static TH1F* trackSizeH1   = new TH1F("trackSize","trackSize",15,-0.5,14.5) ;
  static TH1F* xdifH1        = new TH1F("xdif","xdif",20,-3,3) ;
  static TH1F* xdifbH1        = new TH1F("xdifb","xdifb",50,-3,3) ;
  static TH1F* txdifbH1        = new TH1F("txdifb","txdifb",50,-0.01,0.01) ;
  static TH1F* xsigmaH1      = new TH1F("xsigma","xsigma",50,-1,1) ;
  
  const OTrSuperLayerEvent* sl13Event = fEvent->FindSuperLayerEvent(10) ;
  const OTrSuperLayerEvent* sl14Event = fEvent->FindSuperLayerEvent(11) ;

  int n=0; if(sl13Event) n=sl13Event->GetClusters().size() ;
  clusterMult13H1->Fill(n) ;
  n=0; if(sl14Event) n=sl14Event->GetClusters().size() ;
  clusterMult14H1->Fill(n) ;
  
  trackMultH1->Fill(fEvent->GetTrackCandidates().size()) ;

  if(fRTCalib) {
    //OTrEvent::SuperLayerEventContainer::const_iterator 
    //  sleIt = fEvent->GetSuperLayerEvents().find(2) ;
    // I wanna try with mc-02 only.
    for(OTrEvent::SuperLayerEventContainer::const_iterator 
	  sleIt=fEvent->GetSuperLayerEvents().begin();
	sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++)
      if(sleIt->first==2) { // do not take mini tc in rt calibration
	for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	      clIt=(*sleIt).second.GetClusters().begin() ; 
	    clIt!=(*sleIt).second.GetClusters().end() ; clIt++) 
	  fRTCalib->Add((*clIt)) ;
      } 
  }
  for(OTrEvent::TrackContainer::const_iterator trackIt = 
	fEvent->GetTrackCandidates().begin() ;
      trackIt!=fEvent->GetTrackCandidates().end() ; trackIt++) {
    targetCompatH1->Fill((*trackIt)->GetTargetChi2()) ;
    if(fRTCalib) fRTCalib->Add((*trackIt)) ;
    
    OTrReconstructedTrackInterface* track = (*trackIt)->GetTrack() ;
    if(track) {
      Float_t xco = track->GetX() ;
      Float_t yco = track->GetY() ;
      xsigmaH1->Fill(sqrt(track->GetCov()[0][0])) ;
      trackSizeH1->Fill(track->GetHits().size()) ;
      targetXH1->Fill(xco) ;
      targetYH1->Fill(yco) ;
      targetXYH2->Fill(xco,yco) ;
      targetXYbH2->Fill(xco,yco) ;
      if(yco<10 && yco>-10) targetXbH1->Fill(xco);
      if(xco<1.5 && xco>-2.5) targetYbH1->Fill(yco);
      float chi2 = track->GetChi2()/track->GetNDof() ;
      //cout << "chi2/dof" << chi2 << endl ;
      chi2H1->Fill(chi2) ;
      chi2probH1->Fill(prob_(track->GetChi2(),track->GetNDof())) ;
#ifdef USE_ARTE
      static TH1F* lrsignH1 = new TH1F("lrsignH1","",3,-1.5,1.5) ;
      for(OTrTrack::HitContainer::const_iterator hitIt=track->begin() ;
	  hitIt!=track->end() ; hitIt++) {
	ArtePointer<HITB> hitb = (*hitIt)->GetMeasuredHit()->GetHITB() ;
	int reconstructedLRSign = int((*hitIt)->GetLRSign()) ;
	int realLRSign = hitb->q/fabs(hitb->q) ;
	lrsignH1->Fill(reconstructedLRSign*realLRSign) ;
      };
#endif //USE_ARTE
    };
#ifdef USE_ARTE
    rangerTrack* rtrack = (*trackIt)->GetRangerTrack() ;
    if(rtrack) {
      rangerTrack::rangerNodeIterator firstNode = (*rtrack).last() ;
      const rangerState& state = (*firstNode)->getState() ;
      targetXdH1->Fill(state.x()-(*firstNode)->zNode()*state.tx());
      targetYdH1->Fill(state.y()-(*firstNode)->zNode()*state.ty());
      xdifbH1->Fill(state.x()-(*firstNode)->zNode()*state.tx() - track->GetX()) ;
      txdifbH1->Fill(state.tx() - track->GetTx()) ;
    };
#endif
    
  }; 
  if(fAlignment) fAlignment->Add(fEvent) ;
};

void
controller::UpdateRT(bool fitT, bool fitSigma2, bool fit05mm, bool fit10mm)
{
  if(fRTCalib) {
    fRTCalib->Fit(fitT,fitSigma2,fit05mm,fit10mm) ; 
    fRTCalib->Reset() ;
  } ;
  if(fRTCalib && fTZeroCalib) {
    float chi2(0) ;
    int n1(0), n2(0) ;
    for(OTrDaq::TdcContainer::iterator tdcIt=fDaq->GetTdcBoards().begin() ;
	tdcIt!=fDaq->GetTdcBoards().end() ; tdcIt++)
      for(OTrTdcBoard::AsdContainer::iterator asdIt = tdcIt->GetAsdBoards().begin() ;
	  asdIt != tdcIt->GetAsdBoards().end() ; asdIt++)
	for(OTrAsdBoard::ChannelContainer::iterator chanIt= asdIt->GetChannels().begin() ;
	  chanIt !=asdIt->GetChannels().end() ; chanIt++) {
	  float thischi2 = chanIt->GetHistogrammer()->UpdateTZero() ;
	  if(thischi2<0) n2++;
	  else {
	    n1++ ;
	    chi2+= thischi2 ;
	  };
	};
    cout << "NUmber of adjusted/not adjusted tzeros : " << n1 << " " << n2 << endl 
	 << "Average chi2: " <<  chi2/n1 << endl ;
    fTZeroCalib->SubtractAverageMC02() ;
  };
} ;

void
controller::DrawTxTy() const
{
  typedef vector<TGraph*> GraphContainer ;
  static GraphContainer clusterGraphs ;
  static GraphContainer trackGraphs ;
  static TCanvas* txtyCanvas = new TCanvas("txtyCanvas","",2) ;

  /*** cleanup ***/
  for(GraphContainer::iterator grIt=clusterGraphs.begin() ;
      grIt!=clusterGraphs.end() ; grIt++)
    delete *grIt ;
  clusterGraphs.erase(clusterGraphs.begin(),clusterGraphs.end()) ;
  for(GraphContainer::iterator grIt=trackGraphs.begin() ;
      grIt!=trackGraphs.end() ; grIt++)
    delete *grIt ;
  trackGraphs.erase(trackGraphs.begin(),trackGraphs.end()) ;
  txtyCanvas->Clear() ;
  txtyCanvas->DrawFrame(-0.3,-0.3,0.3,0.3) ;

  /*** draw clusters ***/
  int color = 2;
  const int maxnclusters=100;
  float* tx = new float[maxnclusters] ;
  float* txe = new float[maxnclusters] ;
  float* ty = new float[maxnclusters] ;
  float* tye = new float[maxnclusters] ;
  for(OTrEvent::SuperLayerEventContainer::const_iterator 
	sleIt=fEvent->GetSuperLayerEvents().begin() ;
      sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++,color++) {
    int n=0;
    for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	  clIt=(*sleIt).second.GetClusters().begin() ;
	(clIt!=(*sleIt).second.GetClusters().end()) && (n<maxnclusters); clIt++,n++) {
      tx[n] = (*clIt)->GetTx() ;
      ty[n] = (*clIt)->GetTy() ;
      txe[n] = sqrt(((*clIt)->GetCov())[0][0]) ;
      tye[n] = sqrt(((*clIt)->GetCov())[1][1]) ;
    };
    if(n>0) {
      TGraphErrors* graph = new TGraphErrors(n,tx,ty,txe,tye) ;
      graph->SetMarkerColor(color) ;
      graph->SetLineColor(color) ;
      graph->Draw("p") ;
      clusterGraphs.push_back(graph) ;
    } ;
  };
  /*** draw track candidates ***/
  color = 1;
  for(OTrEvent::TrackContainer::const_iterator trackIt=
	fEvent->GetTrackCandidates().begin() ;
      trackIt!=fEvent->GetTrackCandidates().end() ; trackIt++) {
     int n=0;
     for(OTrTrackCandidate::ClusterContainer::const_iterator 
	   clIt=(*trackIt)->GetClusters().begin() ;
	 (clIt!=(*trackIt)->GetClusters().end()) && (n<maxnclusters); clIt++,n++) {
       tx[n] = (*clIt)->GetTx() ;
       ty[n] = (*clIt)->GetTy() ;
     };
     TGraph* graph = new TGraph(n,tx,ty) ;
     graph->SetMarkerColor(color) ;
     graph->SetMarkerSize(0) ;
     graph->SetLineColor(color) ;
     graph->Draw("lp") ;
     trackGraphs.push_back(graph) ;
  };
  txtyCanvas->Update() ;
}

#endif USE_ROOT

/***********************************************************************/
/************* JUNK ****************************************************/
/***********************************************************************/


// TProfile*
// controller::GetNormalizedNoiseProfile()
// {
//  fNormalizedProfile = new TProfile(*fNoiseProfile) ;
//  for(int bin=0; bin<fNormalizedProfile.GetNBins()+2 ; bin++)
//    {
//      double n_1 = fNoiseProfile->GetB()[bin] ;
//      double n_2 = fNormProfile->GetB()[bin] ;
//      double w_1 = fNoiseProfile->GetW()[bin] ;
//      double w_2 = fNormProfile->GetW()[bin] ;
//      double w2_1 = fNoiseProfile->GetW2()[bin] ;
//      double w2_2 = fNormProfile->GetW2()[bin] ;
//      double n_3 = 1 ;
//      double 
//    }
// }


/*
  vector<UShort_t>
  controller::AddTimeHistograms( vector<OTrChannel*>& channel )
{
  vector<OTrChannel::TimeHistogramBin_t> 
    AddedHistogram(OTrChannel::GetNTimeBins()) ;
  for(int i=0; i<OTrChannel::GetNTimeBins(); i++) {
    AddedHistogram[i]=0;
  };

  bool first=true;
  for( vector<OTrChannel*>::iterator chanIt = channel.begin();
       chanIt != channel.end(); chanIt++ ) {
    OTrChannel* chan = *chanIt;
    int trbin = -(*chan).GetTZeroBin() ;
    for (int bin=0; bin<OTrChannel::GetNTimeBins(); bin++,trbin++) {
      (*gReferenceHistogram)[bin]+=
	(*chan).GetTimeHistogram()[trbin%OTrChannel::GetNTimeBins()] ;
      if ((bin-trbin)%OTrChannel::GetNTimeBins() != 
	  (*chan).GetTZeroBin())
	cout << "scheiBe" << endl;
    };
    if(!first) cout << (*chan).GetTZeroBin() << " " << endl ;
    first = false;
  };
  return AddedHistogram;
}
*/

void controller::PrintSummary() const
{ 
  GetDaq()->PrintSummary();
}










