#ifndef __OTR_ASDBOARD_HH__
#define __OTR_ASDBOARD_HH__

#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include "Rtypes.h"
 #endif
#endif

#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef __OTR_CHANNEL_HH__
#include "OTrChannel.hh"
#endif
#ifndef __OTR_ITERATORBASE_HH__
#include "OTrIteratorBase.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

//typedef UChar_t OTrAsdBoardDataBuffer ;
typedef unsigned char OTrAsdBoardDataBuffer ;
class OTrChannel ;
class OTrEvent ;
class TString ;
class TH1 ;

/** The asd class **/

class OTrAsdBoard : public OTrDaqComponent
{
public:
  ///
  OTrAsdBoard();
  ///
  OTrAsdBoard(ComponentId_t) ;
  ///
  virtual ~OTrAsdBoard() ;
  /// The conatiner for channels
  //#if !defined(CINTDICT) && !defined(__CINT__)
  typedef vector<OTrChannel> ChannelContainer ;
  //#else
  //typedef OTrChannel* ChannelContainer ;
  //#endif

  ///
  void ProcessAsdBuffer(const OTrAsdBoardDataBuffer*, OTrEvent*) ; 

#ifdef USE_ROOT
  TH1* GetTimeH1() const ;
  TString GetName() const ;
#endif

  ///
  void Mask(int nEvents) {} ;
  // Setters and getters
  ///
  virtual void Reset(bool resetmask) {} ;
  ///
  inline static int GetNChannels() { return fgNChannels ; } ;
  ///
  inline static int GetNTimeBins() { return fgNTimeBins ; } ;
  ///
  inline static int GetMaximumTime() { return fgMaximumTime ; } ;
  ///
  inline static int GetMinimumTime() { return fgMinimumTime ; } ;
  ///
  inline static float GetMaxOccupancy() { return fgMaxOccupancy; } ;
  ///
  ChannelContainer& GetChannels() { return fChannels ;} ;
  const ChannelContainer& GetChannels() const { return fChannels ;} ;
  ///
  inline OTrChannel* FindChannel(ChannelId_t chanId); 
  ///
  class ChannelIterator; friend class ChannelIterator;
  ///
  class ChannelConstIterator : public OTrConstIteratorBase<OTrChannel>
  { public: ChannelConstIterator(const OTrAsdBoard&) ; };
  friend class ChannelConstIterator ;
  void Reset() ;
private:
  ///
  static int fgNChannels ;
  ///
  static float fgMaxOccupancy ;
  ///
  static int fgNTimeBins ;
  ///
  static int fgMaximumTime ;
  ///
  static int fgMinimumTime ;
  ///
  unsigned short* fTimeHistogram ;
  //  UShort_t* fTimeHistogram ;
  ///
  ChannelContainer fChannels ;
  //OTrChannel* fChannels ;
 
};

inline OTrChannel* 
OTrAsdBoard::FindChannel(ChannelId_t chanId) { 
  if ( chanId >= 0 && chanId < fChannels.size() ) {
    return &(fChannels[chanId]) ; 
  } else {
    cout << "*** ERROR in OTrAsdBoard::FindChannel, chan = " 
	 << chanId << endl;
    exit(1);
  }
} 

/// Implement the constructor for the iterators
#if !defined(CINTDICT) && !defined(__CINT__)
inline
OTrAsdBoard::ChannelConstIterator::ChannelConstIterator(const OTrAsdBoard& asd):
OTrConstIteratorBase(asd.fChannels.begin(),asd.fChannels.end()) {} ;

class OTrAsdBoard::ChannelIterator : public OTrIteratorBase<OTrChannel>
{
public:
  ChannelIterator(OTrAsdBoard& asd):
    OTrIteratorBase(asd.fChannels.begin(),asd.fChannels.end()) {} ;
};

#endif

#endif

