#ifndef __OTR_COMPONENT_HH__
#define __OTR_COMPONENT_HH__

#ifndef _IOSTREAM_H
#include <iostream.h>
#endif

#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include "Rtypes.h"
 #endif
#endif

class OTrDaq ;
class OTrDetector ;

/** This is the OTR virtual base class for detector and daq
    components. Its only data member is the component id. Every
    derived class provides the methods to calculate from this id the
    mother component ids. **/

class OTrComponent
{
public:
  ///
  typedef unsigned int ComponentId_t ;
  typedef unsigned int ItemId_t ;
  ///
  OTrComponent() ;
  ///
  OTrComponent(ComponentId_t) ;
  ///
  virtual ~OTrComponent() ;
  #ifdef USE_ROOT
  ClassDef(OTrComponent,0)
  #endif
  /// Returns the id of this component
  inline ComponentId_t GetId() const ;
  ///
  //virtual ComponentId_t GetItemId() const=0 ;
  ///
  virtual void Dump(ostream& os=cout) const ;

protected:
  ///
  virtual void SetId(ComponentId_t) ;
private:
  ///
  ComponentId_t fId ;
  //OTrComponent* fMother ;
};

/*******************************************************************/
/** This is the OTrDaqComponent class, derived from  OTrComponent **/

class OTrDaqComponent : public OTrComponent
{
public:
  ///
  OTrDaqComponent() ;
  ///
  OTrDaqComponent(ComponentId_t) ;
  ///
  virtual ~OTrDaqComponent() ;
  // Implemented by child class
  ///
  virtual void ProcessEvent(unsigned char* fedDataBuffer) ;
  ///
  virtual void Reset(bool);// {fMask = false ;} ;
  // Local typedefs
#ifdef USE_ROOT
  ClassDef(OTrDaqComponent,0)
#endif
  ///
  typedef ItemId_t ChannelId_t ;
  ///
  typedef ItemId_t AsdId_t ;
  ///
  typedef ItemId_t TdcId_t ;
  ///
  inline ChannelId_t GetChannelId() const  ;
  ///
  inline AsdId_t     GetAsdId() const ;
  ///
  inline TdcId_t     GetTdcId() const ;
  ///
  inline static OTrDaq* GetDaq() ;
  ///
  inline static void SetDaq(OTrDaq* ) ;
  //protected:
  ///
  ///inline void SetId(TdcId_t,AsdId_t,ChannelId_t) ;
  ///
  inline void SetTdcId(TdcId_t) ;
  ///
  inline void SetAsdId(AsdId_t) ;
  ///
  inline void SetChannelId(ChannelId_t) ;
  ///
  //inline virtual void Dump(ostream& os) const ;
  ///
  inline static ChannelId_t GetChannelId(ComponentId_t) ;
  ///
  inline static AsdId_t GetAsdId(ComponentId_t) ;
  ///
  inline static TdcId_t GetTdcId(ComponentId_t) ;
  ///
  inline static ComponentId_t SetDaqComponentId(TdcId_t,AsdId_t,ChannelId_t) ;
  ///
  inline static ComponentId_t SetChannelId(ComponentId_t,ChannelId_t) ;
  ///
  inline static ComponentId_t SetAsdId(ComponentId_t,AsdId_t) ;
  ///
  inline static ComponentId_t SetTdcId(ComponentId_t,TdcId_t) ;
private:
  ///
  static OTrDaq* fgDaq ;
};

/*******************************************************************/
/** This is the OTrDetectorComponent class, derived from  OTrComponent **/
class OTrDetectorComponent : public OTrComponent
{
public:
  // typedefs, enumerates
  typedef ItemId_t WireId_t ;
  typedef ItemId_t SectionId_t ;
  typedef ItemId_t SectorId_t ;
  typedef ItemId_t ModuleId_t ;
  typedef ItemId_t LayerId_t ;
  typedef ItemId_t SuperLayerId_t ;

  // constructors, destructors, assignment
  OTrDetectorComponent() ;
  OTrDetectorComponent(ComponentId_t) ;
  virtual ~OTrDetectorComponent() ;

#ifdef USE_ROOT
  ClassDef(OTrDetectorComponent,0)
#endif

  // Setters and getters
  WireId_t       GetWireId() const ;
  SectionId_t    GetSectionId() const ;
  SectorId_t     GetSectorId() const ;
  ModuleId_t     GetModuleId() const ;
  LayerId_t      GetLayerId() const ;
  SuperLayerId_t GetSuperLayerId() const ;
  
  
  void SetWireId(WireId_t) ;
  void SetSectionId(SectionId_t) ;
  //void SetSectorId(SectorId_t) ;
  //void SetModuleId(ModuleId_t) ;
  void SetLayerId(LayerId_t) ;
  void SetSuperLayerId(SuperLayerId_t) ;

  // Statics
  static WireId_t       GetWireId(ComponentId_t) ;
  static SectionId_t    GetSectionId(ComponentId_t) ;
  static SectorId_t     GetSectorId(ComponentId_t) ;
  static ModuleId_t     GetModuleId(ComponentId_t) ;
  static LayerId_t      GetLayerId(ComponentId_t) ;
  static SuperLayerId_t GetSuperLayerId(ComponentId_t) ;
  static OTrDetector* GetDetector() ;
  static void SetDetector(OTrDetector*) ;

  static ComponentId_t SetDetectorComponentId(SuperLayerId_t, LayerId_t,
					      SectionId_t, WireId_t) ;

  static ComponentId_t SetWireId(ComponentId_t,WireId_t) ;
  static ComponentId_t SetSectionId(ComponentId_t,SectionId_t) ;
  //static ComponentId_t SetSectorId(ComponentId_t,SectorId_t) ;
  //static ComponentId_t SetModuleId(ComponentId_t,ModuleId_t) ;
  static ComponentId_t SetLayerId(ComponentId_t,LayerId_t) ;
  static ComponentId_t SetSuperLayerId(ComponentId_t,SuperLayerId_t) ;
private:
  static OTrDetector* fgDetector ; // Pointer to detector object
} ;

/********************************** inlines implmentations  ******************************/

/********** for OTrComponent **********/
inline void
OTrComponent::SetId(ComponentId_t id) { fId=id; } ;

inline OTrComponent::ComponentId_t
OTrComponent::GetId() const { return fId ; } ;

inline bool operator<(const OTrComponent& lhs,const OTrComponent& rhs) {
  return lhs.GetId() < rhs.GetId() ; } 

/********** for OTrDaqComponent **********/

// Statics
inline OTrDaqComponent::ComponentId_t
OTrDaqComponent::SetDaqComponentId(TdcId_t tdcId, AsdId_t asdId, 
				   ChannelId_t chanId) {
  return chanId + 100*(asdId + tdcId*10) ; };

inline OTrDaqComponent::ChannelId_t
OTrDaqComponent::GetChannelId(ComponentId_t id) {
  return id%100; } ;

inline OTrDaqComponent::AsdId_t
OTrDaqComponent::GetAsdId(ComponentId_t id) {
  return (id/100)%10; } ;

inline OTrDaqComponent::TdcId_t
OTrDaqComponent::GetTdcId(ComponentId_t id) {
  return (id/100)/10; } ;

inline OTrDaqComponent::ComponentId_t
OTrDaqComponent::SetChannelId(ComponentId_t id, ChannelId_t chanId) {
  return SetDaqComponentId(GetTdcId(id),GetAsdId(id),chanId); } ;
  
inline OTrDaqComponent::ComponentId_t
OTrDaqComponent::SetAsdId(ComponentId_t id, AsdId_t asdId) {
  return SetDaqComponentId(GetTdcId(id),asdId,GetChannelId(id)) ; };
 
inline OTrDaqComponent::ComponentId_t
OTrDaqComponent::SetTdcId(ComponentId_t id, TdcId_t tdcId) {
  return SetDaqComponentId(tdcId,GetAsdId(id),GetChannelId(id)) ; } ;

// Non-Statics

inline OTrDaqComponent::ChannelId_t
OTrDaqComponent::GetChannelId() const {
  return GetChannelId(GetId()) ; } ;

inline OTrDaqComponent::AsdId_t
OTrDaqComponent::GetAsdId() const {
  return GetAsdId(GetId()) ; } ;

inline OTrDaqComponent::TdcId_t
OTrDaqComponent::GetTdcId() const {
  return GetTdcId(GetId()) ; } ;

inline OTrDaq*
OTrDaqComponent::GetDaq() { return fgDaq; } ;

inline void
OTrDaqComponent::SetDaq(OTrDaq* daq) { fgDaq = daq ; } ;
 
inline void
OTrDaqComponent::SetTdcId(TdcId_t tdcId) {
  SetId(SetTdcId(GetId(),tdcId)) ; } 
inline void
OTrDaqComponent::SetAsdId(AsdId_t asdId) {
  SetId(SetAsdId(GetId(), asdId)) ; } ;

inline void
OTrDaqComponent::SetChannelId(ChannelId_t chanId) {
  SetId(SetChannelId(GetId(),chanId)) ; };


/********** for OTrDetectorComponent **********/

// statics 
inline OTrComponent::ComponentId_t
OTrDetectorComponent::SetDetectorComponentId(SuperLayerId_t suplayerId, 
					     LayerId_t layerId,
					     SectionId_t secId,
					     WireId_t wireId) 
{
  return wireId + 1000*(secId + 10000*(layerId + 10*suplayerId)) ;
}

inline OTrDetectorComponent::WireId_t
OTrDetectorComponent::GetWireId(ComponentId_t id) { return id%1000 ; }

inline OTrDetectorComponent::SectionId_t
OTrDetectorComponent::GetSectionId(ComponentId_t id) { return (id/1000)%10000 ; } 

inline OTrDetectorComponent::SectorId_t
OTrDetectorComponent::GetSectorId(ComponentId_t id) { return GetSectionId(id)/100 ; } 

inline OTrDetectorComponent::ModuleId_t
OTrDetectorComponent::GetModuleId(ComponentId_t id) { return GetSectionId(id)%100 ; } 

inline OTrDetectorComponent::LayerId_t
OTrDetectorComponent::GetLayerId(ComponentId_t id) { return ((id/1000)/10000)%10 ;} 

inline OTrDetectorComponent::SuperLayerId_t
OTrDetectorComponent::GetSuperLayerId(ComponentId_t id) { return ((id/1000)/10000)/10 ; }

inline OTrComponent::ComponentId_t
OTrDetectorComponent::SetWireId(ComponentId_t id, WireId_t wireId) {
  return SetDetectorComponentId(GetSuperLayerId(id),GetLayerId(id),
				GetSectionId(id), wireId) ; } ;

inline OTrComponent::ComponentId_t
OTrDetectorComponent::SetSectionId(ComponentId_t id, SectionId_t secId) {
  return SetDetectorComponentId(GetSuperLayerId(id),GetLayerId(id),
				secId,GetWireId(id)) ; } ;

inline OTrComponent::ComponentId_t
OTrDetectorComponent::SetLayerId(ComponentId_t id, LayerId_t layId) {
  return SetDetectorComponentId(GetSuperLayerId(id),layId,GetSectionId(id),
				GetWireId(id)) ; } ;

inline OTrComponent::ComponentId_t
OTrDetectorComponent::SetSuperLayerId(ComponentId_t id, SuperLayerId_t sulId) {
  return SetDetectorComponentId(sulId,GetLayerId(id),GetSectionId(id),
				GetWireId(id)) ; } ;

// non-statics

inline OTrDetectorComponent::WireId_t
OTrDetectorComponent::GetWireId() const { return GetWireId(GetId()) ; }

inline OTrDetectorComponent::SectionId_t
OTrDetectorComponent::GetSectionId() const { return GetSectionId(GetId()) ;} 

inline OTrDetectorComponent::SectionId_t
OTrDetectorComponent::GetSectorId() const { return GetSectorId(GetId()) ;} 

inline OTrDetectorComponent::SectionId_t
OTrDetectorComponent::GetModuleId() const { return GetModuleId(GetId()) ;} 

inline OTrDetectorComponent::LayerId_t
OTrDetectorComponent::GetLayerId() const { return GetLayerId(GetId()) ;} 

inline OTrDetectorComponent::SuperLayerId_t
OTrDetectorComponent::GetSuperLayerId() const { return GetSuperLayerId(GetId()) ;} 

inline OTrDetector*
OTrDetectorComponent::GetDetector() { return fgDetector ; } 

inline void
OTrDetectorComponent::SetWireId(WireId_t wireId) { SetId(SetWireId(GetId(),wireId)) ; } 

inline void
OTrDetectorComponent::SetSectionId(SectionId_t secId) { SetId(SetSectionId(GetId(),secId)) ; } 

inline void
OTrDetectorComponent::SetLayerId(LayerId_t layerId) { SetId(SetLayerId(GetId(),layerId)) ; } 

inline void
OTrDetectorComponent::SetSuperLayerId(SuperLayerId_t suplayerId) { SetId(SetSuperLayerId(GetId(),suplayerId)) ; }

inline void
OTrDetectorComponent::SetDetector(OTrDetector* det) { fgDetector = det ; } ;

#endif
