#ifndef __OTR_DAQ_HH__
#define __OTR_DAQ_HH__

#ifndef __OTR_TDCBOARD_HH__
#include "OTrTdcBoard.hh"
#endif
#ifndef __OTR_COMPONENTMOTHER_HH__
#include "OTrComponentMother.hh"
#endif

class OTrRawEvent ;
class OTrTdcBoard ;
class OTrAsdBoard ;
class OTrEventBuffer ;
class OTrEvent ;
#ifdef USE_ROOT
class TH1 ;
#endif
class ostream ;
class istream ;
struct ETB_fed;
struct EVENT_HDR;

/** The OtrDaq class is container and owner of the OtrTdc's. **/

class OTrDaq : public OTrDaqComponent, public OTrComponentMother<OTrTdcBoard>
{
public:
  /// Constructor of fgNTdcsDefault tdcs
  typedef OTrDaqComponent::ItemId_t ItemId_t ;
  OTrDaq() ;

  /// Constructor of nTdcs tdcs
  OTrDaq(int nTdcs) ;
  //OTrDaq(UShort_t nTdcs) ;

  /// Desctructor. The Tdc's are destructed !!!
  virtual ~OTrDaq() ;
  
  /// Get number of processed events
  inline int GetNProcessedEvents() const { return fNProcessedEvents ; } ;
  
  /// Increase the number of processed events (used when reading the HITB)
  inline void IncreaseNProcessedEvents() { fNProcessedEvents++ ; } ;

  /// Process a new event
  OTrEvent* ProcessEvent(OTrEventBuffer*) ;
  ///
  void ReadAsciMask(char *) ;
  ///
  void ReadTable(char *) ;
  ///
  void WriteAsciMask(char *) const ;
  ///
  void WriteTable(char *,int) const ;

  ///
  void PrintSummary() const;

  /// Call Tdc reset method, which resets all histogramming data 
  virtual void Reset(bool);

  ///
  OTrTdcBoard* AddTdcBoard(ItemId_t tdcId) ;
  ///
  OTrTdcBoard* GetTdcBoard(ItemId_t tdcId) { return GetChild(tdcId) ; } ;
  const OTrTdcBoard* GetTdcBoard(int tdcId) const { return GetConstChild(tdcId) ; } ;
  ///
  OTrTdcBoard* FindTdcBoard(ItemId_t tdcId) { return FindChild(tdcId) ; } ;
  ///
  typedef ChildContainer TdcContainer ;
  TdcContainer& GetTdcBoards() { return GetChildren() ; } ;
  const  TdcContainer& GetTdcBoards() const { return GetChildren() ; } ;
  ///
  typedef Iterator TdcIterator ;
  ///
  typedef ConstIterator TdcConstIterator ;
  ///

#ifdef USE_ROOT
  ClassDef(OTrDaq,1) // The Outer Tracker daq class
  /// return time spectrum 
  TH1* GetTimeH1() const;
#endif

#ifdef USE_ARTE
  /// process an event, reading the DOTR bank
  //int ProcessDOTR( bool compressed );
  /// process an event, reading an uncompressed DOTR
  //int ProcessUncompressedDOTR();
#endif

#ifdef USE_ROOT
  virtual TString GetName() const ;
#endif

private:  
  // Container with Tdcs
  int fNProcessedEvents ;
  int fHBComponentId ;
  // Statics

  /// Default number of tdcs
  static int fgDefaultNTdcs ;
  //  static UShort_t fgDefaultNTdcs ;
};



#endif
