#ifndef __OTR_EVENT_HH__
#define __OTR_EVENT_HH__

// #ifndef __SGI_STL_LIST
// #include <list>
// #endif
#include "event.h"

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __SGI_STL_MAP
#include <map>
#endif
#ifndef __OTR_SUPERLAYEREVENT_HH__
#include "OTrSuperLayerEvent.hh"
#endif

#ifdef USE_ROOT
 class TPad ;
 class TBuffer ;
//  #ifndef ROOT_Rtypes
//   #include <Rtypes.h>
//  #endif
 #ifndef ROOT_TObject
  #include <TObject.h>
 #endif
#endif

class OTrEventBuffer ;
class OTrTrackCandidate ;

class OTrEvent
#ifdef USE_ROOT
  : public TObject
#endif
{
public:
  ///
  typedef vector<OTrTrackCandidate*> TrackContainer ;
  typedef map<int, OTrSuperLayerEvent , less<int> > SuperLayerEventContainer ;
  ///
  //OTrEvent() ;
  ///
  //OTrEvent(const OTrEventBuffer*) ;
  OTrEvent(int id=0) ;
  ///
  virtual ~OTrEvent() ;
  ///
  OTrHit* Add( OTrHit* hit ) ;
  ///
  int SearchClusters() ;
  ///
  int SearchTrackCandidates() ;
  ///
  int FitTracks() ;
  const TrackContainer& GetTrackCandidates() const { return fTrackCandidates ; } ;
  ///
  void DumpHits(ostream& os=cout) const ;
  ///
  void DumpClusters(ostream& os=cout) const ;

  /// sort the hits per superlayer according to GEDE index and x
  void SortHits();
  void Write(ostream& ofs) const ;
  ///
#ifdef USE_ROOT
  ClassDef(OTrEvent,1) // The Outer Tracker event class
  // void Streamer(TBuffer&) ;
  /// draw the hits in the event display
  void DrawHits() ;
  /// draw the clusters in the event display
  void DrawClusters() ;
  /// draw the track candidates in the event display
  void DrawTrackCandidates( TPad* pad ) ;
#endif

#ifdef USE_ARTE
  void AppendRSEG(); // append all track candidates to the RSEG bank
  void AppendHITB(); // append all hits to the HITB bank
  static void SetAppendClustersToRSEG(bool on=true) { fgAppendClustersToRSEG=on ; } ;
#endif

  ///
  void PerformHistogramming() ;
  ///
  const OTrSuperLayerEvent* FindSuperLayerEvent(int) const ;
  SuperLayerEventContainer& GetSuperLayerEvents() {
    return fSuperLayerEvents ; }
  ///
  int GetNHits() const ;
  int GetId() const { return fId ; } ; //fEventHdr->sltevt ; } ;
private:
  int fId ; // Event id 
  TrackContainer fTrackCandidates ; // Tracks
  SuperLayerEventContainer fSuperLayerEvents ; // Superlayer events
#ifdef USE_ARTE
  static bool fgAppendClustersToRSEG ;
#endif
} ;

/******************* inlines **********************/

inline const OTrSuperLayerEvent*
OTrEvent::FindSuperLayerEvent(int id) const {
  const SuperLayerEventContainer::const_iterator superLayerEvent = 
    fSuperLayerEvents.find(id) ;
  if(superLayerEvent==fSuperLayerEvents.end()) return 0;
  return &((*superLayerEvent).second) ;
};

#endif
