#ifndef __OTR_EVENTBUFFER_HH__
#define __OTR_EVENTBUFFER_HH__

/*** This is the the event buffer class for the dmon program. On
*construction it reserves space for the ETB fed buffer, with a length
*of fgMaxNFedBuffers. GetEvent is the interface to the ETB_get_feds
*routine.
**/
#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif
#ifndef __OTR_ITERATORBASE_HH__
#include "OTrIteratorBase.hh"
#endif
#ifndef __OTR_FEDBUFFER_HH__
#include "OTrFedBuffer.hh"
#endif
#ifndef CINTDICT
#ifdef USE_ARTE
#include "online/etb.h"
#else
 #include "etb.h"
#endif
#endif

struct ETB_fed;
struct EVENT_HDR;
class OTrEvent ;

class OTrEventBuffer
{
public:
  /// Constructor
  OTrEventBuffer() ; 
  // OTrEventBuffer(FILE *inputfile) ;

  /// Destructor
  ~OTrEventBuffer() ;

  /// Get event with header `event'.
  //void GetEvent(EVENT_HDR *event, int length) ;

  /// Get pointer to first Fed buffer
  const ETB_fed* GetFedBuffers() const { return fFedBuffers ; } ;

  /// Get number of read Fed buffers
  int GetNFedBuffers() const { return fNFedBuffers; } ;

  /// Get maximum number of Fed buffers
  static inline int GetMaxNFedBuffers() {  return fgMaxNFedBuffers ; }
  //static inline UShort_t GetMaxNFedBuffers() { 
  //  return fgMaxNFedBuffers ; }

  /// Get the component id for this event buffer
  inline int GetComponentId() { return fComponentId ;} ; 
  inline const EVENT_HDR* GetEventHdr() const { return fEvtHdr ; } ;
  //inline UShort_t GetComponentId() { return fComponentId ;} ;

  /// Definition of the iterator over FED buffers
  class FedConstIterator : public OTrConstIteratorBase<ETB_fed>
  {
  public:
    FedConstIterator(const OTrEventBuffer&) ;
    inline const OTrFedBuffer operator*() ;
    inline const OTrFedBuffer operator->() ;
  };

#ifdef USE_ARTE
  // #ifdef USE_NEWARTE
  /// Update fed buffers from the ARTE DOTR bank
  int OTrEventBuffer::DOTRUpdateFedBuffers() ;
  //#endif 
#else
  /// Read next event (get header, update fedbuffers)
  int OTrEventBuffer::NextEvent( int DaqFileType = 0 ) ;
#endif

  /// Dump all info in the FedBuffers
  void DumpFedBuffers() ;

  /// set the file pointer
  void SetFile( FILE* inputFile ) {  fInputFile = inputFile; }  

private:
#ifndef USE_ARTE
  /// Get next event header, standard DAQ data format 
  int OTrEventBuffer::NextEventHdr() ;

  /// Get next event header, farm logged format
  int OTrEventBuffer::NextEventHdrFarm() ;

  /// Update fed buffers with current event header (private ?)
  int OTrEventBuffer::UpdateFedBuffers() ;

  /// Update fed buffers using the ETB
  int OTrEventBuffer::ETBUpdateFedBuffers() ;
#endif

  /// The maximum number of Fed buffers in this event buffer
  static const int fgMaxNFedBuffers ;
  //static const Short_t fgMaxNFedBuffers ;

  /// The current number of valid FED buffers
  int fNFedBuffers ;

  /// Pointer to FED buffers. This class is owner (!)
  ETB_fed* fFedBuffers ;

  /// This is the buffer for the event header 
  char* fEvtBuffer ;

  /// Space reserved for the event header
  static const int fgMaxEventLength ;

  /// Pointer to current event header
  EVENT_HDR *fEvtHdr ;

  // Current event length
  //int fLength ;

  /// Input file
  FILE* fInputFile ;

  // Pointer to the current otr event (owner!!!)
  //OTrEvent* fEvent ;

  /// Component mask for FED buffers
  int fComponentId ;
  //Short_t fComponentId ;
};



// Definition of inline functions

inline
OTrEventBuffer::FedConstIterator::FedConstIterator(const OTrEventBuffer& parent) :
  OTrConstIteratorBase(parent.GetFedBuffers(),
		       parent.GetFedBuffers()+parent.GetNFedBuffers())  {} ;

inline const OTrFedBuffer 
OTrEventBuffer::FedConstIterator::operator*(){ 
  return (const OTrFedBuffer)(current()) ; };

inline const OTrFedBuffer
OTrEventBuffer::FedConstIterator::operator->(){
  return (const OTrFedBuffer)(current()) ; };

#endif
