#ifndef __OTR_ATGEDE_HH__
#define __OTR_ATGEDE_HH__

#ifndef _IOSTREAM_H
#include <iostream.h>
#endif
#ifndef _FSTREAM_H
#include <fstream.h>
#endif
#ifndef __STRING__
#include <string>
#endif
#ifndef HepPoint3D_hh
#include <CLHEP/Geometry/Point3D.h>
#endif
typedef HepPoint3D Point3D ;
typedef Hep3Vector ThreeVector ;
//#include <Geometry/Point3D.h>

#ifdef USE_ROOT
class TRotMatrix ;
class TShape ;
class TNode ;
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif 

#if defined(USE_ARTE) && ! defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<GEDE> ArtePointerGEDE ;
#else
 #ifndef _GEDE_HH_
 #include "auto/GEDE.h"
 #endif
 typedef GEDE* ArtePointerGEDE ;
#endif

class OTrWire;

/**  ARTE GEDE modules (detector parts) for the moment presented by a
     geometrical box **/

class OTrGede
{
public:
  // Enumerates and typedefs definitions
  enum EPitch {  // Pitch Type
    kFiveMM = 0,
    kTenMM  = 1
  } ;
  enum EStereoAngle { // Stereoangle type
    kMinusFiveDegrees = 0,
    kZeroDegrees = 1,
    kPlusFiveDegrees = 2
  } ;
  enum EXSide { // Module orientation type
    // EXSide=0 hold for all module on minus X, but e.g. not for MINITC2 !!
    kXSideMinus = 0,
    kXSidePlus  = 1
  } ;
  
  // Constructors, destructor, assignment operator
  OTrGede();                    // default constructor
  OTrGede(const OTrGede&);      // copy constructor
#if defined(USE_ARTE) && ! defined(CINTDICT)
  OTrGede(ArtePointerGEDE gede);// constructor from an ARTE GEDE pointer
#endif
  OTrGede(istream& is);         // constructor reading from an input filestream
  virtual ~OTrGede();
  const OTrGede& operator=(const OTrGede&) ;
  
  // Setters and Getters
  const ArtePointerGEDE& GetGEDE() const { return fGEDE ; }
  int GetGEDEIndex() const { return fGEDEIndex ; } ;
  string GetSuperLayerName() const { return fSuperLayerName; }
  EPitch GetEPitch() const { return fEPitch ; } ;
  EStereoAngle GetEStereoAngle() const { return fEStereoAngle ; } ;
  EXSide GetXSide() const { return fEXSide ; } ;
  int superLayerId() const;
  int layerId() const;
#ifdef USE_ROOT
  const TShape* GetShape() const { return fShape ; } ;
  const TNode* GetNode() const { return fNode ; } ;
  const TRotMatrix* GetWireRotMatrix() const { return fWireTRotMatrix ; } ;
#endif

  Point3D GetPMin() const; // return closest point
  Point3D GetPMax() const; // return furthest point
  Point3D GetMean() const { return 0.5*(GetPMin() + GetPMax()); } ;

  /// long dump
  void LongDump() const;

  // translation
  void Translate( const ThreeVector& p );
  
  int GetNMonoLayers() const { return GetGEDE()->nsl ; } ;
  float GetRawX(int wireNumber=0) const ;
  float GetRawY(int wireNumber=0) const ;
  float GetRawY1(int wireNumber=0) const ;
  float GetRawY2(int wireNumber=0) const ;
  float GetRawZ(int wireNumber=0) const ;
  ///
  int GetPRWireNumber(int wireNumber) const ;
  bool WithinYAcceptance(const float tx, const float ty) const ;
  bool WithinXAcceptance(const float tx, const float ty) const ;
  bool WithinAcceptance(const float tx, const float ty) const ;
  float GetWireLength() const ;
  float GetCentralX(int wireNumber=0) const ;
  float GetCentralY(int wireNumber=0) const ;
  float GetCosStereoAngle() const { return GetGEDE()->rot[0] ; } ;
  float GetSinStereoAngle() const { return GetGEDE()->rot[1] ; } ;
  //float GetWirePitch()   const { return fgWirePitch[fEPitch] ; } ;
  //float GetLayerPitch()  const { return fgLayerPitch[fEPitch] ; } ;
  float GetWirePitch()  const { return GetGEDE()->pit ; } ;
  float GetLayerPitch() const { return 2*(GetGEDE()->zfw[1]-GetGEDE()->zfw[0]) ; } ;

  static float GetWirePitch(EPitch pitch) { return fgWirePitch[pitch] ; } ;
  static float GetLayerPitch(EPitch pitch) { return fgLayerPitch[pitch] ; } ;
  int GetNWires() const;
  //int GetSectorId() const { return GetGEDE()->sec ; }
  int GetSectorNumber() const { return GetGEDE()->sec ; }

#ifdef USE_ROOT
  int CreateNode();   /// create a Tnode
  ClassDef(OTrGede,0)  // The Outer Tracker interface to the GEDE bank
#endif
  
  /// determine the section number from the geometrical plan
  int determineSection() const;
protected:
  /// determine the stereo view and the wire pitch
  void DetermineStereoAndPitch();
private:
  // Methods
  int SetSuperLayerId() ; // set the superlayer id using the superlayer name
private:
  ArtePointerGEDE fGEDE ; // ArtePointer to a GEDE
  int fGEDEIndex ;        // gede index
  int fSuperLayerId;      // OTR superlayer ID
  string fSuperLayerName; // SuperLayername (from GESL)
  
  EStereoAngle fEStereoAngle ; // StereoAngle type
  EPitch fEPitch ;             // Pitch type
  EXSide fEXSide ;             // Orientation type

#ifdef USE_ROOT
  TShape* fShape;                // geometrical shape of the GEDE
  TNode*  fNode;                 // ROOT node
  TRotMatrix* fTRotMatrix ;      // rotation matrix for gede Node
  TRotMatrix* fWireTRotMatrix ;  // rotation matrix for wire node
#endif
  /// These are the ideal pitches
  static const float fgLayerPitch[2] ;
  static const float fgWirePitch[2] ;
  // These are the ideal stereoangles (21/01/999 SHIT : different for TC2 !!)
  // static const float fgStereoAngle[3] ;
};

/***************** inlines **********************************/

ostream & operator << (ostream &, const OTrGede& atgede );

inline float OTrGede::GetRawX(int wireNumber) const
{
  return GetGEDE()->xfw[wireNumber%GetNMonoLayers()] + 
    (wireNumber/GetNMonoLayers())*GetWirePitch() ; 
} 

inline float OTrGede::GetRawY(int wireNumber) const 
{
  int monoLayer = wireNumber%GetNMonoLayers() ;
  return 0.5*(GetGEDE()->y2w[monoLayer] + GetGEDE()->y1w[monoLayer]) ;
};

inline float OTrGede::GetRawY1(int wireNumber) const 
{
  return GetGEDE()->y1w[wireNumber%GetNMonoLayers()] ;
};

inline float OTrGede::GetRawY2(int wireNumber) const 
{
  return GetGEDE()->y2w[wireNumber%GetNMonoLayers()] ;
};

inline float OTrGede::GetRawZ(int wireNumber) const
{
  return GetGEDE()->zfw[wireNumber%GetNMonoLayers()] ;
} 

inline bool OTrGede::WithinYAcceptance(const float tx, const float ty) const
{
  float yraw = GetGEDE()->zmax *
    (ty*GetCosStereoAngle()+tx*GetSinStereoAngle()) ;
  return GetGEDE()->y1w[0] <= yraw && yraw <= GetGEDE()->y2w[0] ;
}

inline bool OTrGede::WithinXAcceptance(const float tx, const float ty) const
{
  float xraw = GetGEDE()->zmax *
    (-ty*GetSinStereoAngle()+tx*GetCosStereoAngle()) ;
  float xmin = GetGEDE()->xfw[GetXSide()*(GetNMonoLayers()-1)] - GetWirePitch()/3 ;
  float xmax = GetGEDE()->xfw[(1-GetXSide())*(GetNMonoLayers()-1)] +
    GetWirePitch() * ( GetGEDE()->nwi[(1-GetXSide())*(GetNMonoLayers()-1)] - 2/3 ) ;
  // There is something strange with the xmax and xmin in gede.
  //   cout << xmin << " " << xmax << " " << GetGEDE()->xfw[0] << " " 
  //        << GetGEDE()->xfw[GetNMonoLayers()-1] << " "
  //        << GetWirePitch() << " "
  //        << GetGEDE()->xmin*GetCosStereoAngle() - GetGEDE()->ymin*GetSinStereoAngle() << " "
  //        << GetGEDE()->xmax*GetCosStereoAngle() - GetGEDE()->ymax*GetSinStereoAngle() << " "
  //        << GetGEDE()->xmin*GetCosStereoAngle() + GetGEDE()->ymin*GetSinStereoAngle() << " "
  //        << GetGEDE()->xmax*GetCosStereoAngle() + GetGEDE()->ymax*GetSinStereoAngle() << " "
  //        << xmax-xmin << " "
  //        << (GetGEDE()->xmax-GetGEDE()->xmin)*GetCosStereoAngle()
  //     - (GetGEDE()->ymax-GetGEDE()->ymin)*GetSinStereoAngle() << " "
  //        << (GetGEDE()->xmax-GetGEDE()->xmin)*GetCosStereoAngle()
  //     + (GetGEDE()->ymax-GetGEDE()->ymin)*GetSinStereoAngle() << " "
  //        << endl ;
  return (xmin <= xraw) && (xraw <= xmax) ;
}

inline bool OTrGede::WithinAcceptance(const float tx, const float ty) const
{
  return WithinYAcceptance(tx,ty) && WithinXAcceptance(tx,ty) ;
}


inline int OTrGede::GetPRWireNumber(int wireNumber) const
{
  switch (GetNMonoLayers()+GetXSide())
    {
    case 2: return 2*wireNumber ;
    case 3: return 2*wireNumber + 4*((wireNumber+1)%2) ;
    case 4: return wireNumber + (3*(wireNumber%4))/2 ;
    case 5: return wireNumber + (16-5*(wireNumber%4))/2 ;
    default: ;
    } ;
  return -1; //error value
} ;

inline float OTrGede::GetWireLength() const 
{
  return fabs(GetGEDE()->y2w[0] - GetGEDE()->y1w[0]) ;
};

inline float OTrGede::GetCentralX(int wireNumber) const
{
  return GetCosStereoAngle()*GetRawX(wireNumber) +
    GetSinStereoAngle()*GetRawY(wireNumber) ;
};

inline float OTrGede::GetCentralY(int wireNumber) const
{
  return GetCosStereoAngle()*GetRawY(wireNumber) -
    GetSinStereoAngle()*GetRawX(wireNumber) ;
};

inline Point3D OTrGede::GetPMin() const
{
  return Point3D(GetGEDE()->xmin,GetGEDE()->ymin,GetGEDE()->zmin) ;
}

inline Point3D OTrGede::GetPMax() const
{
  return Point3D(GetGEDE()->xmax,GetGEDE()->ymax,GetGEDE()->zmax) ;
}


inline int OTrGede::superLayerId() const
{
  return fSuperLayerId;
}

inline int OTrGede::layerId() const
{
  return GetGEDE()->lay ;
}

inline int OTrGede::GetNWires() const
{
  int n = 0;
  for ( int i = 0; i < GetGEDE()->nsl ; i++ ) n += GetGEDE()->nwi[i];
  return n;
}

//typedef OTrGede ATgede ;

#endif
