#ifndef __OTR_HIT_HH__
#define __OTR_HIT_HH__

#ifndef __OTR_CHANNEL_HH__
#include "OTrChannel.hh"
#endif
#ifdef USE_ROOT
#ifndef ROOT_Rtypes
#include "Rtypes.h"
#endif
#endif

class OTrWire ;
class TTUBE ;
class TRotMatrix ;
class TNode ;
class TBuffer ;

#if defined(USE_ARTE) && !defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<HITB> ArtePointerHITB ;
#else
 typedef int ArtePointerHITB ;
#endif

/** OTrHit class */
class OTrHit
{
public:
  ///
  typedef OTrChannel::TdcCount_t TdcCount_t ;

  /// default constructor
  OTrHit() ;

  /// constructor
  OTrHit(OTrChannel* chan, TdcCount_t tdc) ;

  /// copy constructor (not to be used)
  OTrHit( const OTrHit& h );

  /// destructor
  virtual ~OTrHit() ;

  /// assignment operator (not to be used)
  OTrHit& operator=(  const OTrHit& h );

  ///
  inline OTrChannel* GetChannel() const ;
  ///
  inline TdcCount_t GetTdcCounts() const ;
  ///
  inline OTrDObservable GetDriftDistance() const ;
  ///
  inline float GetDriftTime() const ;
  ///
  inline double GetZ() const ;
  ///
  inline const OTrWire* GetWire() const ;
  ///
  inline ArtePointerHITB GetHITB() const { return fHITB; } ;
#ifndef CINTDICT
  inline void SetHITB(ArtePointerHITB hitb) { fHITB = hitb ; } ;
#endif

#ifdef USE_ROOT
  ClassDef(OTrHit,0) // The Outer Tracker hit class
  /// draw the hit as a tube in the event display
  void Draw(int color=1) ;
  //void Streamer(TBuffer &R__b) ;
  ///
  void PerformHistogramming() const ;
#endif

#ifdef USE_ARTE
  /// append this hit to the HITB bank
  ArtePointerHITB AppendHITB();

  /// dump the HITB contents (only makes sense after calling AppendHITB())
  void DumpHITB( ostream& os = cout ) const;
#endif

private:
  ///
  OTrChannel* fChannel ;
  ///
  TdcCount_t fTdcCounts ;
  ///
  TNode *fNode ;
  ///
  ArtePointerHITB fHITB ;
};

inline
OTrHit::OTrHit() : fChannel(0), fTdcCounts(0), fNode(0), fHITB(0) {} ;

inline
OTrHit::OTrHit(OTrChannel* chan, TdcCount_t tdc) : fChannel(chan),
  fTdcCounts(tdc), fNode(0), fHITB(0) {} ;

inline OTrChannel*
OTrHit::GetChannel() const{ return fChannel ; } ;

inline const OTrWire*
OTrHit::GetWire() const{ return fChannel->GetWire() ; } ;

inline OTrChannel::TdcCount_t
OTrHit::GetTdcCounts() const { return fTdcCounts ; } ;

inline float
OTrHit::GetDriftTime() const { 
  return fChannel->GetDriftTime(fTdcCounts) ; } ;

#ifdef USE_ROOT
inline void
OTrHit::PerformHistogramming() const {
  GetChannel()->PerformHistogramming(GetTdcCounts()) ;
} ;
#endif

#endif
