#ifndef __OTR_LAYER_HH__
#define __OTR_LAYER_HH__

#ifndef __OTR_SECTION_HH__
#include "OTrSection.hh"
#endif
#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef __OTR_COMPONENTMOTHER_HH__
#include "OTrComponentMother.hh"
#endif

#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
 #ifndef ROOT_TH1
 #include <TH1.h>
 #endif
#endif

class OTrSuperLayer ;
class OTrHit ;
class OTrSuperlayerEvent ;
class TCanvas ;
class TH1F ;
class OTrSector ;

class OTrLayer : public OTrDetectorComponent,
		 public OTrComponentMother<OTrSection>
{
public:
  // Typedefs, enumerates
  typedef ChildContainer SectionContainer ;
  typedef vector<OTrSector*> SectorContainer ;
  enum EStereoAngle  { minusFiveDegrees, zeroDegrees, plusFiveDegrees } ;
 
  // Constructors, destructor, assigment
  OTrLayer(); 
  OTrLayer(OTrSuperLayer*, ComponentId_t) ; // constructor with identifier
  virtual ~OTrLayer() ; 

  // setters and getters
  OTrSection* GetSection(SectionId_t id) { return GetChild(id) ; } ;
  const OTrSection* GetSection(SectionId_t id) const { return GetConstChild(id) ; } ;
  OTrSection* FindSection(SectionId_t id) { return FindChild(id) ; } ;
  const OTrSection* FindSection(SectionId_t id) const { return FindConstChild(id) ; } ;
  OTrSection* AddSection( OTrGede* gede ) ; // Add a section 
  SectionContainer& GetSections() { return GetChildren() ; } ;
  const SectionContainer& GetSections() const { return GetChildren() ; } ;
  typedef Iterator SectionIterator ;
  //OTrGede::EStereoAngle GetEStereoAngle() { return GetSections().front().GetEStereoAngle() ; }
  EStereoAngle GetEStereoAngle() { return fStereoAngle ; }
  virtual ItemId_t GetItemId() const { return GetLayerId() ; } 
  OTrSuperLayer* GetSuperLayer() const { return fSuperLayer ; }
  SectorContainer& GetSectors() { return fSectors ; }
  const SectorContainer& GetSectors() const { return fSectors ; }
  OTrSector* GetSector(SectorId_t) ; 
 
  // other public methods
  /// print a summary of the occupancies in the different sections
  int OccupancySummary( ostream& ofs, bool useMask = false );
  void Connect(OTrSuperLayer*) ; // Set pointer to superlayer
  void Translate(const ThreeVector& ) ; // Translate over vector
  HepPoint3D GetPMin() const; // return closest point
  HepPoint3D GetPMax() const; // return furthest point
  void CreateSectors() ; // temporary
  
#ifdef USE_ROOT
  ClassDef(OTrLayer,0) // The stereolayer class

  void Plot( vector<TPad*>, const int choice, char* option,
	     const OTrSuperLayerEvent* slEvent, const bool use_mask = true) ;
  // book occupancy plots where the 5 mm cells are OR-ed into 10 mm cells
  int BookCombinedOccupancy();
  // fill the combined occupancy plots
  int FillCombinedOccupancy(const OTrSuperLayerEvent* slEvent, const int use_mask = 1 );
  // plot the combined occupancy plots
  int PlotCombinedOccupancy(  TCanvas* c, TPad* p, char* option );
#endif

private:
  OTrSuperLayer* fSuperLayer ; // pointer to mother superlayer
  EStereoAngle fStereoAngle ;  // StereoAngle
  int fCombinedProcessed;      // number of times that FillCombinedHisto was called
  SectorContainer fSectors ;
  TH1F* fCombinedHistoTop;     // histogram for the combined occupancy (top half)
  TH1F* fCombinedHistoBot;     // histogram for the combined occupancy (bottom half)
  TH1F* fCombinedMC;           // MC histogram for the combined occupancy 
} ;

#endif
