#ifndef __OTR_NODEPROJECTION__
#define __OTR_NODEPROJECTION__

#ifndef _Vector_H_
#include "CLHEP/Matrix/Vector.h"
#endif
#ifndef _SYMMatrix_H_
#include "CLHEP/Matrix/SymMatrix.h"
#endif
#ifndef __OTR_RTRELATION_HH__
#include "OTrRTRelation.hh"
#endif
#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif

class OTrHit ;
class OTrReconstructedHit ;
class OTrTrackDescription ;
class OTrRTRelation ;
class OTrFunctionValue ;
class OTrWire ;

class OTrNodeProjection
{
public:
  OTrNodeProjection(OTrReconstructedHit*) ;
  OTrNodeProjection(const OTrHit*) ;
  OTrNodeProjection(const OTrWire*) ;
  virtual ~OTrNodeProjection() ;
  inline void Init(const OTrHit*) ;
  inline void InitGeometry(const OTrWire*) ;
  void Project(const OTrTrackDescription*, bool updateInternal=false, bool calcDerivatives=true);
  inline double GetRm() const { return fRm ; } ;
  inline double GetD() const { return fD ; } ;
  inline double GetSigmaT2() const { return fSigmaT2 ; } ;
  inline double GetResidualT() const { return fResidualT ; } ;
  inline double GetSigmaR2() const { return fSigmaR2 ; } ;
  inline double GetResidualR() const { return fResidualR ; } ;
  inline double GetResidualD() const { return fResidualD ; } ;
  inline double GetChi2() const { return fChi2 ; } ;
  inline const HepVector GetNormdResdQ() const { return fdDdQ/sqrt(fabs(fSigmaR2)) ; } ;
  inline const HepVector& GetdChi2dQ() const { return fdChi2dQ ; } ;
  inline const HepSymMatrix Getd2Chi2dQ2() const { return fd2Chi2dQ2 ; } ;
  inline bool GetEtaError() const { return fEtaError; } ;
  bool OnTrack() const { return fabs(fD) < fRTRelation->GetRMax() ; } ;
  void Dump() const ;
  const OTrReconstructedHit* GetReconstructedHit() const { return fReconstructedHit ; } ;
#ifdef USE_ROOT
  ClassDef(OTrNodeProjection,0)
#endif
private:
  // These have to be initialised only once
  ///
  OTrReconstructedHit* fReconstructedHit ; // Pointer to mother hit
  /// Wire position
  float fCosTheta, fSinTheta, fZw, fXw ; 
  ///
  OTrRTRelation* fRTRelation ;
  ///
  float fT0 ;
  ///
  float fTm, fRm ;
  // These depend on projection
  ///
  HepDouble fD ;
  HepVector fdDdQ ;
  ///
  HepDouble fResidualT ;
  HepDouble fResidualR ;
  HepDouble fResidualD ;
  ///
  float fChi2 ;
  bool fEtaError ;
  ///
  HepVector fProjVector ;
  HepVector fdChi2dQ ; 
  HepSymMatrix fd2Chi2dQ2 ;
  ///
  OTrFunctionValue fT ;
  ///
  float fSigmaT2 ;
  float fSigmaR2 ;
  float fSigmaC2 ;
 };

#endif
