#ifndef __OTR_RTRELATION_HH__
#define __OTR_RTRELATION_HH__

#ifndef __OTR_FUNCTIONVALUE_HH__
#include "OTrFunctionValue.hh"
#endif
#ifndef __OTR_OBSERVABLE_HH__
#include "OTrObservable.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
class OTrRTResult ;

//class TrackDescription ;
//class MeasuredHit ;
//class WirePosition;
//class TimeValue;

/** Interface base class for the rtrelation. Retrieval via tables for fast access. **/

class OTrRTRelation
{
public:
  //virtual const TimeValue& GetTime(const TrackDescription&,
  //				 const WirePosition&) const ;
  ///
  inline HepDouble GetR(const HepDouble) const ;
  ///
  inline const OTrFunctionValue& GetT(const HepDouble) ;
  ///
  inline const OTrFunctionValue& GetSigma2(const HepDouble) const ;
  ///
  OTrDObservable GetObservedT(const HepDouble) ;
  ///
  inline HepDouble GetT0() const ;
  ///
  inline HepDouble GetRMin() const ;
  ///
  inline HepDouble GetRMax() const ;
  //GetNormalizedResidual2(const TrackDescription&, const MeasuredHit&) ; 
  ///
  virtual void Read(istream&) ;
  ///
  virtual void Write(ostream&) const ;
  void DumpTable() ;
  ///  
protected:
  ///
  OTrRTRelation() ;
  ///
  OTrRTRelation(double rmin, double rmax) ;
  ///
  virtual ~OTrRTRelation() ;
  ///
  inline void SetT0(const HepDouble) ;
  ///
  inline void SetRMax(const HepDouble) ;
  ///
  inline void SetRMin(const HepDouble) ;
  ///
  inline int GetRBin(const HepDouble) const ;
  ///
  void FillTTable() ;
  ///
  void FillSigma2Table() ;
  // void FillRTable() ;
  ///
  inline int GetNoTableBins() const ;
  ///
  inline HepDouble GetR(const int) const ;
  ///
  int tNoTableBins ;
  ///
  HepDouble tDeltaR ;
private:
  ///
  HepDouble fRMin ;
  ///
  HepDouble fRMax ;
  ///
  HepDouble fT0;
  ///
  HepDouble tTRMax ;
  HepDouble tTRMin ;
  HepDouble tDeltaT ;
  /// temporary data
  vector<OTrFunctionValue> tTTable ;
  ///
  vector<OTrFunctionValue> tSigma2Table ;
  ///
  vector<float> tRTable ;
  //Implemented by child class
public:
  ///
  virtual OTrFunctionValue CalcFunctionT(const HepDouble R) const = 0 ;
  ///
  virtual OTrFunctionValue CalcFunctionSigma2(const HepDouble R) const = 0 ;
  ///
  virtual OTrRTResult CalcRTResult(const HepDouble R) const = 0 ;
};

/*************** inlines ********************/

inline HepDouble OTrRTRelation::GetT0() const { return fT0; } ;
inline void OTrRTRelation::SetT0(const HepDouble T0) { fT0 = T0 ; } ; 

inline HepDouble OTrRTRelation::GetRMin() const { return fRMin ; } ;
inline void OTrRTRelation::SetRMin(const HepDouble rmin) { fRMin = rmin ; } ; 

inline HepDouble OTrRTRelation::GetRMax() const { return fRMax ; } ;
inline void OTrRTRelation::SetRMax(const HepDouble rmax) { fRMax = rmax ; } ; 

inline int OTrRTRelation::GetRBin(const HepDouble R) const {
  int rbin = int((R-GetRMin())/tDeltaR) ;
  if(rbin <=0) return 0;
  if(rbin >=tNoTableBins-1) return tNoTableBins-1 ;
  return rbin ;
}

inline int OTrRTRelation::GetNoTableBins() const {
	 return tNoTableBins ; }

inline HepDouble OTrRTRelation::GetR(const int rbin) const {
	 return GetRMin() + tDeltaR*rbin ; } ;

inline HepDouble
OTrRTRelation::GetR(const HepDouble tm) const {
  int tbin = int((tm-tTRMin)/tDeltaT) ;
  if(tbin<=0) return GetRMin() ;
  else if(tbin>=(int)tRTable.size()) return GetRMax() ;
  return tRTable[tbin] ;
};

inline const OTrFunctionValue&
OTrRTRelation::GetT(const HepDouble R) {
  int rbin = int((R-GetRMin())/tDeltaR) ;
  if(rbin <=0) return tTTable[0] ;
  if(rbin >=tNoTableBins) {
      const OTrFunctionValue& validT = tTTable[tNoTableBins-1] ; 
      OTrFunctionValue& T = tTTable[tNoTableBins] ; //overflowbin	
      T.value = validT.value + (R-fRMax)*validT.ddx ;
      //T.ddx = validT.ddx ;
      //T.d2dx2 = 0;
      return T ;
    }
  return tTTable[rbin] ;
};

inline const OTrFunctionValue&
OTrRTRelation::GetSigma2(const HepDouble R) const {
  int rbin = int((R-GetRMin())/tDeltaR) ;
  if(rbin <=0) return tSigma2Table[0] ;
  if(rbin >=(int)tSigma2Table.size()) return tSigma2Table.back() ;
  return tSigma2Table[rbin] ;
};


// inline const TimeValue& 
// ParameterizedOTrRTRelation::GetTime(const TrackDescription& track,
// 				 const WirePosition& wire)
// {
//   const HepDouble d0 = track.GetD0();
//   const HepDouble phi = track.GetPhi();
//   const HepDouble x = wire.GetX();
//   const HepDouble y = wire.GetY();

//   const HepDouble D = d0 + x*sin(phi) - y*cos(phi) ;
//   //const HepDouble dDdD0  = 1 ;
//   const HepDouble dDdPhi = x*cos(phi) + y*sin(phi) ; 
//   const HepDouble d2DdPhi2 = -x*sin(phi) + y*cos(phi) ; // = -D + d0 ;
//   HepDouble signD = 1; if(D<0) signD = -1;
//   const HepDouble R = signD*D;
//   const HepDouble dRdD0 = signD ;
//   const HepDouble dRdPhi = signD* dDdPhi ;
//   const HepDouble d2RdPhi2 = signD*d2DdPhi2 ;

  
//   const FunctionValue& T = GetT(R) ;

//   RTResult& time = GetRTValue() ;

//   time.Time = signD * T.value ;
//   time.dTimedQ[0] = T.ddx * dRdD0 ;
//   time.dTimedQ[1] = T.ddx * dRdPhi ;
//   time.d2TimedQ2[0][0] = T.d2dx2 * dRdD0 * dRdD0    ;
//   time.d2TimedQ2[1][0] = T.d2dx2 * dRdD0 * dRdPhi ;
//   time.d2TimedQ2[1][1] = T.d2dx2 * dRdPhi * dRdPhi + T.ddx * d2RdPhi2  ;

//   return time ;
// } ;














#endif







