#ifndef __OTR_RTRESULT_HH__
#define __OTR_RTRESULT_HH__

#ifndef __OTR_OBSERVABLE_HH__
#include "OTrObservable.hh"
#endif

class OTrRTResult
{
  friend class OTrParameterizedRTRelation ;
  friend class OTrTheoreticRTRelation ;
public:
  OTrDObservable GetT() const ;
  OTrDObservable GetdTdR() const ;
  OTrDObservable Getd2TdR2() const ;
  OTrDObservable GetSigma2() const ;
  OTrDObservable GetSigma() const ;
  OTrDObservable GetSigmaR() const ;
  OTrDObservable GetV() const ;
private:
  OTrDObservable fT;
  OTrDObservable fdTdR;
  OTrDObservable fd2TdR2;
  OTrDObservable fSigma2;
};

inline OTrDObservable OTrRTResult::GetT() const { return fT; } ;
inline OTrDObservable OTrRTResult::GetdTdR() const { return fdTdR; } ;
inline OTrDObservable OTrRTResult::Getd2TdR2() const { return fd2TdR2; } ;
inline OTrDObservable OTrRTResult::GetSigma2() const { return fSigma2; } ;

inline OTrDObservable OTrRTResult::GetSigma() const {
  HepDouble sigma = sqrt(fSigma2.GetValue()) ;
  return OTrDObservable(sigma,fSigma2.GetError()/(2*sigma)) ;
};

inline OTrDObservable OTrRTResult::GetSigmaR() const {
  OTrDObservable sigma = GetSigma() ;
  OTrDObservable sigmaR;
  sigmaR.SetValue(sigma.GetValue() /fdTdR.GetValue()) ;
  sigmaR.SetError(sqrt(fdTdR.GetError()*fdTdR.GetError()*sigmaR.GetValue()*sigmaR.GetValue()+
		      sigma.GetError()*sigma.GetError())/fdTdR.GetValue()) ;
  return sigmaR ;
};

inline OTrDObservable OTrRTResult::GetV() const {
  OTrDObservable dTdR= GetdTdR() ;
  HepDouble v = 1/dTdR.GetValue() ;
  return OTrDObservable(v,dTdR.GetError()*v*v) ;
}

inline ostream& operator<<(ostream& os, const OTrRTResult& f) {
  os << f.GetT() << f.GetdTdR() << f.Getd2TdR2() ;
  return os ;
};

#endif




