#ifndef __OTR_RECONSTRUCTEDHIT_HH__
#define __OTR_RECONSTRUCTEDHIT_HH__

#ifndef __OTR_OBSERVABLE_HH__
#include "OTrObservable.hh"
#endif

#if defined(USE_ARTE) && !defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<RHIT> ArtePointerRHIT ;
#else
 typedef char ArtePointerRHIT ;
#endif //USE_ARTE

#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif

class OTrHit ;
class OTrTrackDescription ;
class OTrNodeProjection ;

/** The ReconstructedHit class describes a reconstructed hit. A
    reconstructed hit contains a pointer to a measured hit and a value
    for the ambiguity. **/

class OTrReconstructedHit
{
public:
  ///
  typedef char LRSign_t ;
  ///
  OTrReconstructedHit(const OTrReconstructedHit& rhs) ;
  ///
  OTrReconstructedHit() ;
  ///
  OTrReconstructedHit(const OTrHit* hit, LRSign_t lrSign=0) ;
  ///
  virtual ~OTrReconstructedHit() ;
  ///
  inline LRSign_t GetLRSign() const { return fLRSign ; } ;
  ///
  inline bool GetLRSignIsFixed() const { return fLRSignIsFixed; } ;
  inline void FixLRSign() { fLRSignIsFixed=true ; } ;
  inline void ReleaseLRSign() { fLRSignIsFixed=false; } ;
  inline void SetLRSign(double s) ;
  inline void SetLRSign(char s) ;
  bool UpdateInternal() ;
  ///
  inline void FlipLRSign() { fLRSign *= -1 ; } ;
  ///
  inline const OTrHit* GetMeasuredHit() const { return fHit ; }
  ///
  inline OTrNodeProjection* GetNodeProjection() ;
  /// Use this to clean up memory
  inline void DeleteNodeProjection() ;
  inline double GetChi2() const ;
  //inline double GetZ() { return GetMeasuredHit()->GetZ() ; } ;
  ///
#ifdef USE_ARTE
  ArtePointerRHIT AppendRHIT(const OTrTrackDescription*) ;
  ArtePointerRHIT GetRHIT() const { return fRHIT ; } ;
#endif
#ifdef USE_ROOT
  //ClassDef(OTrReconstructedHit,0)
#endif
private:
  const OTrHit* fHit ;  // Pointer to corresponding OTrHit
  LRSign_t fLRSign ;    // Left-right sign (ambiguity)
  bool fLRSignIsFixed ; // Status flag for left-right sign
  OTrNodeProjection* fProjection ; //! Corresponding projection to track
#ifdef USE_ARTE
  ArtePointerRHIT fRHIT ; // ArtePointer to RHIT bank
#endif
} ;


inline void OTrReconstructedHit::SetLRSign(double s) 
{ 
  if(!GetLRSignIsFixed()) fLRSign  = (s>0) ? 1 : -1; 
} ;

inline void OTrReconstructedHit::SetLRSign(char s) 
{ 
  if(!GetLRSignIsFixed()) {
    if(s<0) fLRSign = -1 ;
    else if(s>0) fLRSign = 1;
    else fLRSign =0 ;
    //fLRSign  = (s>0) ? 1 :-1;
    if(abs(s)>=2) FixLRSign() ;
  };
} 

#ifndef __OTR_NODEPROJECTION_HH__
#include "OTrNodeProjection.hh"
#endif

inline OTrNodeProjection* 
OTrReconstructedHit::GetNodeProjection()
{
  if(!fProjection) {
    fProjection = new OTrNodeProjection(this) ;
  };
  return fProjection ;
} ;

inline void 
OTrReconstructedHit::DeleteNodeProjection()
{
  if(fProjection) {
    delete fProjection ;
    fProjection=0;
  };
} ;

inline double
OTrReconstructedHit::GetChi2() const { return fProjection->GetChi2() ; } ;
#endif
