#ifndef __OTR_RECONSTRUCTEDTRACKINTERFACE_HH__
#define __OTR_RECONSTRUCTEDTRACKINTERFACE_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __OTR_TRACKDESCRIPTION_HH__
#include "OTrTrackDescription.hh"
#endif

#if defined(USE_ARTE) && !defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<RSEG> ArtePointerRSEG ;
#else
 typedef int ArtePointerRSEG ;
#endif

class OTrReconstructedHit ;
class OTrHit ;

extern "C" float prob_(const float&, const int&) ;
extern "C" float chisin_(const float&, const int&) ;

class OTrReconstructedTrackInterface 
  : public virtual OTrTrackDescription
{
public:
  ///
  OTrReconstructedTrackInterface() ;
  OTrReconstructedTrackInterface(const OTrReconstructedTrackInterface&) ;
  virtual ~OTrReconstructedTrackInterface() ; 
  OTrReconstructedTrackInterface& operator=(const OTrReconstructedTrackInterface&) ;
  ///
  typedef vector<OTrReconstructedHit*> HitContainer ;
  typedef HitContainer::iterator HitIterator ;
  virtual inline HitIterator begin() { return fHits.begin() ; } ;
  virtual inline HitIterator end() { return fHits.end() ; } ;
  //
  virtual const HitContainer& GetHits() const { return fHits ; } ;
  virtual HitContainer& GetHits() { return fHits ; } ;
  virtual HitContainer& GetHitVector() { return fHits ; } ;
  virtual void EraseHits() ;
  //static float GetMinChi2Probability() { return fgMinChi2Probability ; }
  //static void  SetMinChi2Probability(float x) { fgMinChi2Probability=x ; }
  virtual float GetMinChi2Probability() const = 0 ;
  virtual float GetChi2() const { return fChi2 ; } ;
  virtual void  SetChi2(float chi2) { fChi2 = chi2 ; } ;
  virtual int   GetError() const { return fError ; } ;
  virtual void  SetError(int error) { fError = error ; } ;
  virtual ArtePointerRSEG GetRSEG() const { return fRSEG ; } ;
  virtual int   GetNDof() const { return fHits.size()-GetDim() ; } ;
  virtual float GetChi2OverNDof() const { return fChi2/GetNDof() ; } ;
  virtual float GetChi2Prob() const { return prob_(GetChi2(),GetNDof()) ; } ;
  virtual bool ValidChi2(float chi2 = -1 ) const ;
  virtual const OTrReconstructedHit* FindHit(const OTrHit* ) const ;
#ifdef USE_ARTE
  ArtePointerRSEG AppendRSEG() ;
#endif
private:
  HitContainer fHits ;
  float fChi2 ;
  int fError ;
  static float fgMinChi2Probability = 0;
  ArtePointerRSEG fRSEG ;
};

inline bool
OTrReconstructedTrackInterface::ValidChi2(float chi2) const {
  if(chi2<0) chi2=GetChi2() ; 
  return chi2 < chisin_(1-GetMinChi2Probability(),GetNDof()) ; 
} ;

// Oke. So we once need o write a base class here.

// inline bool operator<(const OTrReconstructedTrackInterface& lhs,
// 		      const OTrReconstructedTrackInterface& rhs)
// {
//   if(rhs.GetHits().size()==lhs.GetHits().size())
//     return lhs.GetChi2() < rhs.GetChi2() ;
//   return rhs.GetHits().size() < lhs.GetHits().size() ;
// }

#endif
