#ifndef __OTR_SECTOR_HH__
#define __OTR_SECTOR_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef HepPoint3D_hh
#include <CLHEP/Geometry/Point3D.h>
typedef HepPoint3D Point3D ;
typedef Hep3Vector ThreeVector ;
#endif

#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
 #ifndef ROOT_TString
 #include <TString.h>
 #endif
 class TH1 ;
 class TH1D ;
 class TPad ;
#endif

class OTrSection ;
class OTrSuperLayerEvent ;
class OTrLayer ;
class OTrChannel ;
class OTrTdcHistogram ;

class OTrSector : public OTrDetectorComponent
{
public:
  // typedefs, enumerates 
  typedef vector<const OTrSection*> SectionContainer ;

  // constructors, destructors, assignment
  OTrSector();
  OTrSector( OTrLayer* parent, ComponentId_t id ) ;
  virtual ~OTrSector() ;
  
#ifdef USE_ROOT
  //ClassDef(OTrSector,0) // The Outer Track sector class
  TH1* GetOccupancyH1(bool useMask=false) const  ; // return occupancy plot
  TH1D GetTimeH1() const ; // return time spectrum 
  TH1* GetTZeroH1(bool useMask=false) const ;  // histogram with the t0's
  TH1* GetSigmaTimeH1( bool useHistogram = false, bool useMask = false ) ; // histogram with the RMS TDC count for each channel
  TH1* GetAvTimeH1( bool useHistogram = false, bool useMask = false ) ; // histogram with the average TDC count for each channel
  virtual TString GetName() const ;
  TH1* GetChannelH1(bool useMask, float (OTrChannel::* channelFunc)() const ) const ;

  TH1* GetOccupancyVariationH1( bool useMask = false ) const;
  
  /// make a one dimensional plot 
  int Plot(TPad* p, const int choice, char* option, const int cl,
	   const OTrSuperLayerEvent* slEvent, const bool use_mask = true);
#endif
  OTrLayer* GetLayer() const { return fLayer ; }
  HepPoint3D GetPMin() const; // return closest point
  HepPoint3D GetPMax() const; // return furthest point
  void Translate(const ThreeVector&) ; 

  const SectionContainer& GetSections() const { return fSections ; }
  SectionContainer& GetSections() { return fSections ; } 
  
  // the occupancy as averaged over all wires
  float OccupancySummary( ostream& ofs, const bool usemask = false );
  int GetNWires() const ;
  
private:
  SectionContainer fSections ; // gedes (modules)
  OTrLayer* fLayer ;     // pointer to parent layer
  
#ifdef USE_ROOT
  TH1* fPlot; // a one dimensional histogram used for various purposes
  OTrTdcHistogram* fTdcHisto; // summed TDC spectra of all channels
#endif
} ;

#endif
