#ifndef __OTR_SUPERCLUSTER_HH__
#define __OTR_SUPERCLUSTER_HH__

#ifndef __OTR_SECTIONCLUSTER_HH__
#include "OTrSectionCluster.hh"
#endif
#ifndef __OTR_RECONSTRUCTEDTRACKINTERFACE_HH__
#include "OTrReconstructedTrackInterface.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
// #ifdef USE_ROOT
//  #ifndef ROOT_Rtypes
//  #include <Rtypes.h>
//  #endif
// #endif

class TPolyMarker3D ;

class OTrSuperCluster : public OTrReconstructedTrackInterface,
			private OTrTrackDescription2D
{
public:
  typedef vector<OTrSectionCluster*> ClusterContainer ;
  OTrSuperCluster(OTrSectionCluster* plusClus, OTrSectionCluster* minusClus) ;
  OTrSuperCluster(const OTrSuperCluster&) ;
  virtual ~OTrSuperCluster() ;
  bool AddZeroIfValid(OTrSectionCluster* zeroClus) ;
  bool WithinAcceptance() const ;  
  float GetXDif() const ;
  float GetXDif(const OTrSectionCluster* ) const ;
  float CalcXDif() ;
  bool Valid() const ;
  bool Merge(OTrSuperCluster& rhs) ;
  const ClusterContainer& GetPlusClusters() const { return fPlusClusters ; } ;
  const ClusterContainer& GetMinusClusters() const { return fMinusClusters ;} ;
  const ClusterContainer& GetZeroClusters() const { return fZeroClusters ; } ;
  const ClusterContainer& GetSectionClusters() const { return fSectionClusters ; } ;
  float GetCentralZ() const { return fZeroClusters[0]->GetRawZ() ; } ;
  float GetCentralX() const { return GetTx()*(GetCentralZ()-GetZ())+GetX(); } 
  float GetCentralY() const { return GetTy()*(GetCentralZ()-GetZ())+GetY(); } 
  void RawWireFit() ;
  void Connect() ;
  void PerformHistogramming() const ;
  void Dump() const ;
  void CreateReconstructedHits() ;
#ifdef USE_ROOT
  //ClassDef(OTrSuperCluster,0)
  void Draw(int color) ;
#endif 
  using OTrTrackDescription2D::GetX ;  
  using OTrTrackDescription2D::GetY ;  
  using OTrTrackDescription2D::GetZ ;  
  using OTrTrackDescription2D::GetTx ;  
  using OTrTrackDescription2D::GetTy ;
  using OTrTrackDescription2D::GetCov ;  
  using OTrTrackDescription2D::GetQ ;
  virtual float GetMinChi2Probability() const { return fgMinChi2Probability ; }
private:
  inline bool OneFitIteration(int fitMode=0) ;
  bool AddPlus(OTrSectionCluster* plusClus) ;
  bool AddMinus(OTrSectionCluster* minusClus) ;
  bool AddZero(OTrSectionCluster* zeroClus) ;
  float fXDif ;
  ClusterContainer fPlusClusters ;
  ClusterContainer fMinusClusters ;
  ClusterContainer fZeroClusters ;
  ClusterContainer fSectionClusters ;
  TPolyMarker3D* fClosestPointMarker ;
  static float fgMinChi2Probability ;
};

inline float
OTrSuperCluster::GetXDif() const { 
  return fXDif ; } ;

inline float 
OTrSuperCluster::GetXDif(const OTrSectionCluster* zeroClus) const {
  return zeroClus->GetXDif(GetTx()) ;
};

inline bool
OTrSuperCluster::OTrSuperCluster::Valid() const { 
  return fZeroClusters.size()>0 && 
    fPlusClusters.size()>0 && fMinusClusters.size()>0; } ;

inline bool
OTrSuperCluster::WithinAcceptance() const {
  return fPlusClusters.front()->WithinYAcceptance(GetTx(),GetTy()) &&
    fMinusClusters.front()->WithinYAcceptance(GetTx(),GetTy()) ;
};

#endif








