#ifndef __OTR_TZEROCALIB_HH__
#define __OTR_TZEROCALIB_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
class OTrDaq;
class OTrDetector;
class TH1F;

class OTrTZeroCalib
{
public:
  /// default constructor
  OTrTZeroCalib();

  /// constructor
  OTrTZeroCalib( OTrDaq* daq, OTrDetector* det );
 
  /// calibration by different methods (average, fit to a function etc.)
  int Calibration( const int method = 0, bool inter = false );

  /// compare different methods of tzero calibration
  void Compare();

  /// compare two calibration files
  void CompareCalibrationFiles(char* file1, char* file2 );  

  /// subtract an average T0 shift
  void SubtractAverage( const double av5mm, const double av10mm );
  void SubtractAverageMC02() ;

  /// set the superlayer ID of the superlayer to calibrate
  inline void SetSuperLayerId( const int id ) { fSuperLayerId = id; };

  /// get the superlayer id
  inline int GetSuperLayerId() const { return fSuperLayerId; }

private:
  /// fill reference spectra
  void FillReferenceSpectra( const int nSteps );

  /// create a ROOT reference histogram
  TH1F* CreateReferenceHistogram( vector<double>& ref, char* name,
				  const int color = 4);
 
  /// pointer to DAQ object
  OTrDaq* fDaq;

  /// pointer to detector object
  OTrDetector* fDetector;

  /// superlayer ID, of the superlayer to calibrate
  int fSuperLayerId;

  /// 5mm reference histogram used for the reference histogram method
  vector<double> fReferenceHistogram5mm;

  /// 10mm reference histogram used for the reference histogram method
  vector<double> fReferenceHistogram10mm;   

  ///
  TH1F* fRef5mmH1;

  ///
  TH1F* fRef10mmH1;
  
  
};

#endif
