#ifndef __OTR_TDCBOARD_HH__
#define __OTR_TDCBOARD_HH__

#ifndef __OTR_ASDBOARD_HH__
#include "OTrAsdBoard.hh"
#endif
#ifdef USE_ROOT
 #ifndef ROOT_Rtypes
 #include "Rtypes.h"
 #endif
#endif

class OTrFedBuffer ;
class OTrEvent ;

/** This class describes the Outer Tracker tdc boards. A tdc board is
    owner of Asd boards **/

class OTrTdcBoard : public OTrDaqComponent
{
public:
  /// Default constructor: creates NO asd boards
  OTrTdcBoard() ;
  /// This is the constructor you are supposed to use
  OTrTdcBoard(ComponentId_t) ;
  /// Destructor
  virtual ~OTrTdcBoard() ;
  
  /// Process an eventbuffer
  void ProcessFedBuffer(const OTrFedBuffer&, OTrEvent*) ;

  ///
  int GetNProcessed() const { return fNProcessed; } ;

#ifdef USE_ROOT
  TH1* GetTimeH1() const ;
  TString GetName() const ;
#endif

  typedef vector<OTrAsdBoard> AsdContainer ;
  class AsdIterator : public OTrIteratorBase<OTrAsdBoard>
  { public: AsdIterator(OTrTdcBoard&) ; };
  friend class AsdIterator ;
  ///
  class AsdConstIterator : public OTrConstIteratorBase<OTrAsdBoard>
  { public: AsdConstIterator(const OTrTdcBoard&) ; };
  friend class AsdConstIterator ;
  ///
  inline static GetNAsdBoards() { return fgDefaultNAsdBoards ; } ;
  ///
  inline OTrAsdBoard* FindAsdBoard(AsdId_t id) ;
  ///
  inline OTrAsdBoard& GetAsdBoard(AsdId_t id) ;
  ///
  AsdContainer& GetAsdBoards() { return fAsds ; } ;
  const AsdContainer& GetAsdBoards() const { return fAsds ; } ;
  ///
  virtual ItemId_t GetItemId() const { return GetTdcId() ; } ;
  ///
  void Reset() ;
private:
  static const int fgDefaultNAsdBoards ; // The number of asd boards per tdc board
  AsdContainer fAsds ; // Container of asd boards connected to this tdc

  int fNProcessed; // total number of times this TDC was processed
};

/// Implement the constructor for the iterators

inline
OTrTdcBoard::AsdIterator::AsdIterator(OTrTdcBoard& tdc) : 
  OTrIteratorBase(tdc.fAsds.begin(),tdc.fAsds.end()) {} ;
inline
OTrTdcBoard::AsdConstIterator::AsdConstIterator(const OTrTdcBoard& tdc) : 
  OTrConstIteratorBase(tdc.fAsds.begin(),tdc.fAsds.end()) {} ;

inline OTrAsdBoard&
OTrTdcBoard::GetAsdBoard(AsdId_t id) {
  if ( id >= 0 && id < fAsds.size() ) {
    return fAsds[id] ; 
  } else {
    cout << "** ERROR in OTrTdcBoard::GetAsdBoard, id = " << id << endl; 
    exit(1);
  }
}

inline OTrAsdBoard*
OTrTdcBoard::FindAsdBoard(AsdId_t id) {  
  if ( id >= 0 && id < fAsds.size() ) {
    return &(fAsds[id]) ; 
  } else {
    exit(1);
  }
}

#endif
