#ifndef _OTrView__
#define _OTrView__

#ifdef USE_ROOT

#include <TView.h>
#include <TCanvas.h>

class TRint;
class TControlBar;
class OTrEvent;

enum ViewMode {END, RSIDE, LSIDE,  TOP,  BOT,  ALL};

class OTrView : public TObject
{ 
public:
  /// default constructor
  OTrView();

  /// destructor
  ~OTrView();

  /// zoom function (still being tested)
  Zoom( const float z );

  /// refresh the canvas
  void Refresh() const;

  /// display the buttons
  void DisplayButtons();

  /// set the current event
  void SetEvent( OTrEvent* event ) { fEvent = event; }

  /// number of processed events
  void SetEventNumber( const int eventnr ) { fEventNumber = eventnr; }

  /// set the range
  int SetRange( const float x1, const float y1, const float z1, 
		const float x2, const float y2, const float z2 );

  /// set the view orientation
  void SetAngles( const float longitude, const float latitude,
	       const float psi );

  /// set to one of the standard views
  void SetStandardView(char mode);

  /// toggle zoom mode
  void ToggleZoomMode();

  /// return the zoom mode
  bool GetZoomMode() { return fZoomMode; }

  /// draw current view using OPENGL
  void DrawViewGL();

  /// draw current view using X3D
  void DrawViewX3D();

  /// set the view ranges
  void SetGeometry(float rin);

  /// draw a view
  void DrawView( float theta, float phi, float psi );

  /// draw a title in a pad
  void DrawTitle(Option_t *option);

  /// draw all views
  void DrawAllViews();

  /// return AllViews flag
  virtual bool AllViews() { return fDrawAllViews; }

  /// zoom in
  void ZoomIn();

  /// zoom out
  void ZoomOut();

  ///
  void Pan(char direction);

  /// execute an event
  virtual void ExecuteEvent(Int_t event, Int_t px, Int_t py);

  /// Distance to primitive
  Int_t DistancetoPrimitive(Int_t px, Int_t py);

  /// Draw
  void Draw(Option_t *option="");

  ///
  virtual void ToggleDisplayAttribute(char attribute); 

  /// draw the beampipe
  int DrawBeamPipe();  

  // return the display pad
  inline TPad* GetDisplayPad() { return fDisplayPad; }

protected:    
  // create the menubar of buttons, used in the constructor
  //CreateMenuBar();

  /// viewing mode
  ViewMode fViewMode;

  /// flag true if AllViews selected
  bool fDrawAllViews;

  // view
  //TView* fView;

  /// canvas
  TCanvas* fCanvas;

  /// display pad
  TPad* fDisplayPad;

  /// buttons pad
  TPad* fButtonsPad;

  /// pointer to the current event
  OTrEvent* fEvent; 

  /// pointer to the daq
  int fEventNumber;
 
  // minimum point of the range of the detector display view
  //Point3D* fRangeMin;

  // maximum point of the range of the detector display view 
  //Point3D* fRangeMax;

  /// longitudinal angle of the detector display view
  float fLongitude;

  /// latitudinal angle of the detector display view
  float fLatitude;
  
  /// psi angle of the detector display view
  float fPsi;

  /// X center
  float           fXcenter;

  /// Y center
  float           fYcenter;

  /// Z Center
  float           fZcenter;

  /// X range
  float           fXin;

  /// Y range
  float           fYin;

  /// Outer radius
  float           fRout; 

  /// Inner length along Z
  float           fZmin;

  /// Inner length along Z
  float           fZmax;

  /// Outer length along Z
  float           fZout; 

  /// flag true in zoom mode
  bool fZoomMode;

  /// zoom factor
  float fZoomFactor;

  // menubar of buttons
  //TControlBar* fMenuBar;

  bool m_DrawAxis;
  bool m_DrawTracks;
  bool m_DrawChambers;
  bool m_DrawWireHits;
  bool m_DrawHitPoints;
  bool m_CurrentSlides[2];
};

extern OTrView* gWaxView;
extern TRint* myApp;

#endif

#endif








