#ifndef _controller_
#define _controller_

#include <iostream>
#include <fstream>
#include <stdio.h>
#ifdef USE_ROOT
  class TH2F ;
  class TCanvas;
  class TPad;
  class TProfile ;
  class TView ;
  class TNode ;
  class TH1 ;
  class TBRIK ;
  class TControlBar ;
  class TFile ;
  class TTree ;
  class TBranch ;
  #ifndef ROOT_Rtypes
  #include <Rtypes.h>
  #endif
#endif

class controller ;
extern controller* gOTr ;

class OTrChannel;
class OTrEvent ;
class OTrDetector ;
class OTrEventBuffer ;
class OTrDaq ;
class OTrView;
class OTrLayer;
class OTrTZeroCalib ;
class controller ;
class OTrAlignment ;
 
//extern controller* gOTr ;

class OTrRTCalib ;
class OTrRunSummary ;

class controller {
public:
  /// default constructor
  controller();
  ///
  virtual ~controller() ;

  /// loop over the next event(s)
  next( const int n = 1, const int prflag = 0 );

  /// open a datafile  
  open( char* filename = "/hb/otr2/run98/data/otr_run_0074.dat",
	int DaqFileType = 0);

#ifdef USE_ARTE
  //#ifdef USE_NEWARTE
  /// loop over the next DOTR event
  int nextDOTR();
  /// append all hits to the HITB bank
  int AppendHITB(); 
 /// append all track candidates to the RSEG bank
  int AppendRSEG();
  /// print a short ROW summary
  int EventRowSummary( ostream& os = cout ) const;
#endif

#ifdef USE_ROOT
  ClassDef(controller,0)

  /// display the event
  void Display();

  /// create the TNodes
  void CreateNode();

  /// plot histograms for each section of a certain superlayer
  int plot( const int sl = -1, const int choice = 2, const bool use_mask = 1 );

  /// plot the combined occupancy plots (5mm cells OR-ed)
  int plot_combined( const int sl = 13 );

  ///
  void PlotChannelTdcSpectra(int tdcId) const ;

  ///
  void PlotAllTdcSpectra() const ;

  /// create rtrelation with current channel tdc spectra
  void CreateRTRelation(bool drawFlag=true) const ;

  ///
  void DrawRTRelation() const ;

  /// Here you can put all the histogramming you want to do with the raw event
  void PerformRawogramming() ;
  /// Here you can put everything you want to do with clusters and tracks
  void PerformRecogramming() ;
  /// set fCombinedLayer
  void DrawTxTy() const ;
  void SetCombinedLayer( const int slId, const int layId );
  ///
  OTrRunSummary* GetRunSummary() const { return fRunSummary ; } ; 

#endif //USE_ROOT
  /// print a short summary, used to identify missing FED problems
  void PrintSummary() const;

  ///
  void UpdateRT(bool,bool,bool,bool) ;
  
  /// set the current superlayer number
  inline void SetCurrentSuperLayer( const int s ) { fCurrentSuperLayer = s; }
  
  /// set the event display on
  inline void SetDisplay( const bool b = true ) { fDisplay = b; }

  /// switch on/off the cluster search 
  void SetSearchClusters(bool k=true) { fSearchClusters=k ; } ;

  /// set MC events read from ARTE
  void SetMC(bool k=true) { fMC=k ; } ;

  /// return pointer to Daq
  OTrDaq* GetDaq() const { return fDaq ; } ;

  /// Write an ASCII mask file
  void WriteAsciMask();

  /// get the tzero calibration object
  OTrTZeroCalib* GetTZeroCalib() { return fTZeroCalib; } ;

  /// reset the histograms for each channel
  void Reset();

private:
#ifdef USE_ROOT
  /// canvas for plots of one superlayer, different sections
  TCanvas* fOccCanvas;

  /// canvas for plots with 5 and 10mm cells combined 
  TCanvas* fCombinedCanvas; 

  TPad* waxpad;
  TPad* combined_pad[3];

  /// cube for detector display
  TBRIK* fBRIK;

  /// view for the ROOT detector display
  OTrView* fWaxView;

  /// main node for the ROOT detector display
  TNode* fWaxNode;
#endif
 
  /// event display on/off
  bool fDisplay;

  /// pointer to the currently opened datafile
  FILE* fDataFile;

  /// total number of processed events
  int fNprocessed;

  ///
  OTrDaq* fDaq ;

  ///
  OTrDetector* fDetector ;

  ///
  OTrEventBuffer* fEventBuffer ;

  ///
  OTrEvent* fEvent;

  /// true if cluster are to be searched 
  bool fSearchClusters ;

  /// current superlayer number
  int fCurrentSuperLayer;

  /// the layer for which combined occupancy plots are to be filled
  OTrLayer* fCombinedLayer;

  /// true if reading MC data
  bool fMC ;

  /// name of the currently opened data file
  char fFileName[80];

  /// type of DAQ file ( 0=standard, 1=farm logged)
  int fDaqFileType;

  /// tzero calibration
  OTrTZeroCalib* fTZeroCalib;

  ///
  OTrRTCalib* fRTCalib ;

  ///
  OTrRunSummary* fRunSummary ;
  OTrAlignment* fAlignment ;

  ofstream* fSparcifiedOutputFile ;
#ifdef USE_ROOT
  TFile* fSparcifiedOutputRootFile ;
  TTree* fTree ;
  TBranch* fEventBranch ;
#endif
};

#endif



